/*!
 * (C) Emu Analytics
 */
import { Component, Host, h, Prop, State, Watch } from '@stencil/core';
import { globalContextRegistry } from '../../core/context/context-registry';
import { createColorClasses } from '../../utils/theme';
/**
 * @since 1.1
 * @status stable
 * @reactive
 *
 * @slot - The content to overlay
 */
export class BusyOverlay {
  constructor() {
    this.showProgress = false;
    /**
     * The size of the progress ring.
     */
    this.size = 'medium';
    /**
     * Set to `true` to display the busy overlay. Specify a Flo.w reactive expression or a boolean value.
     *
     * @reactive
     */
    this.busy = 'false';
    /**
     * Set to `true` to overlay the component's parent element rather than its content.
     */
    this.overlayParent = false;
    /**
     * The Flo.w context ID to use for application state.
     */
    this.flowContext = 'default';
    this.handleTransitionEnd = this.handleTransitionEnd.bind(this);
  }
  connectedCallback() {
    this.subscribe();
  }
  disconnectedCallback() {
    this.unsubscribe();
  }
  async subscribe() {
    var _a;
    this.unsubscribe();
    const context = await globalContextRegistry.getContext(this.flowContext);
    const parsedDataExpression = context.parser.parseExpression((_a = this.busy) !== null && _a !== void 0 ? _a : 'false');
    if (parsedDataExpression.status) {
      this.autorunDisposer = context.state.autorun(() => {
        try {
          this._busy = context.parser.evaluateExpression(parsedDataExpression.value);
          if (this._busy) {
            this.showProgress = true;
          }
        }
        catch (e) {
          context.emitError(e);
        }
      });
    }
    else {
      context.emitError(new Error(`flow-busy-overlay: Malformed busy expression '${this.busy}'`));
    }
  }
  unsubscribe() {
    if (this.autorunDisposer) {
      this.autorunDisposer();
      this.autorunDisposer = undefined;
    }
  }
  handleTransitionEnd(event) {
    const target = event.target;
    // Ensure we only emit one event when the target element is no longer visible
    if (event.propertyName === 'opacity' && target.classList.contains('flow-busy-overlay')) {
      this.showProgress = this._busy;
    }
  }
  render() {
    const { _busy, overlayParent, showProgress } = this;
    const hostClasses = Object.assign(Object.assign({}, createColorClasses(this.color)), { 'flow-busy-overlay--overlay-parent': overlayParent });
    const overlayClasses = {
      'flow-busy-overlay': true,
      'flow-busy-overlay--busy': _busy
    };
    return (h(Host, { class: hostClasses },
      h("slot", null),
      h("div", { class: overlayClasses, onTransitionEnd: this.handleTransitionEnd }, showProgress && (h("flow-progress-ring", { class: "flow-busy-overlay__progress", indeterminate: true, size: this.size })))));
  }
  static get is() { return "flow-busy-overlay"; }
  static get encapsulation() { return "shadow"; }
  static get originalStyleUrls() { return {
    "$": ["busy-overlay.scss"]
  }; }
  static get styleUrls() { return {
    "$": ["busy-overlay.css"]
  }; }
  static get properties() { return {
    "color": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "PredefinedColor",
        "resolved": "\"danger\" | \"gray\" | \"primary\" | \"secondary\" | \"success\" | \"warning\"",
        "references": {
          "PredefinedColor": {
            "location": "import",
            "path": "../../utils/theme"
          }
        }
      },
      "required": false,
      "optional": true,
      "docs": {
        "tags": [],
        "text": "The color of the component. Colors are taken from the application palette."
      },
      "attribute": "color",
      "reflect": false
    },
    "size": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "'small' | 'medium' | 'large' | 'x-large' | 'xx-large'",
        "resolved": "\"large\" | \"medium\" | \"small\" | \"x-large\" | \"xx-large\"",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "The size of the progress ring."
      },
      "attribute": "size",
      "reflect": false,
      "defaultValue": "'medium'"
    },
    "busy": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [{
            "name": "reactive",
            "text": undefined
          }],
        "text": "Set to `true` to display the busy overlay. Specify a Flo.w reactive expression or a boolean value."
      },
      "attribute": "busy",
      "reflect": true,
      "defaultValue": "'false'"
    },
    "overlayParent": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Set to `true` to overlay the component's parent element rather than its content."
      },
      "attribute": "overlay-parent",
      "reflect": false,
      "defaultValue": "false"
    },
    "flowContext": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "The Flo.w context ID to use for application state."
      },
      "attribute": "flow-context",
      "reflect": false,
      "defaultValue": "'default'"
    }
  }; }
  static get states() { return {
    "_busy": {},
    "showProgress": {}
  }; }
  static get watchers() { return [{
      "propName": "busy",
      "methodName": "subscribe"
    }]; }
}
