/*!
 * (C) Emu Analytics
 */
import { Component, Element, Host, h, Prop } from '@stencil/core';
import { MDCRipple } from '@material/ripple';
import { createColorClasses } from '../../utils/theme';
/**
 * @status stable
 *
 * @slot - The button label. Alternatively, specify using the `label` attribute.
 *
 * @part icon - The button icon.
 * @part label - The button label.
 */
export class Button {
  constructor() {
    /**
     * Set to `true` to display a smaller-sized button with reduced margins.
     */
    this.compact = false;
    /**
     * Set to `true` to disable interaction with the button.
     */
    this.disabled = false;
    /**
     * Set to `true` to display an 'icon-only' button with no button body or label. Note that
     * the default color of an 'icon-only' button is the current text color. Use the `color` property
     * to select a different color from the application palette.
     */
    this.iconButton = false;
    /**
     * Set to display an 'outlined' button.
     */
    this.outlined = false;
    /**
     * Set to display a 'solid' button.
     */
    this.solid = false;
    /**
     * Set to `true` to display the icon *after* the label (to the right in LTR mode).
     */
    this.trailingIcon = false;
    /** Set to 'true' to indicate that activating the button should submit the form. Ignored when `href` is set. */
    this.submit = false;
  }
  componentDidLoad() {
    this.ripple = new MDCRipple(this.button);
    this.ripple.unbounded = this.iconButton;
    // Workaround for Firefox - doesn't support delegatesFocus
    this.el.focus = () => this.button.focus();
  }
  handleClick(ev) {
    // Prevent propagation of event if button is disabled
    if (this.disabled) {
      ev.preventDefault();
      ev.stopImmediatePropagation();
    }
  }
  render() {
    const { label, color, disabled, icon, trailingIcon } = this;
    const isLink = !!this.href;
    const Button = isLink ? 'a' : 'button';
    const hostClasses = Object.assign({}, createColorClasses(color));
    const nativeClasses = {
      'mdc-button': !this.iconButton,
      'mdc-icon-button': this.iconButton,
      'mdc-button--unelevated': this.solid,
      'mdc-button--outlined': this.outlined,
      'mdc-button--compact': this.compact
    };
    const iconClasses = {
      'mdc-button__icon': !this.iconButton,
      'mdc-icon-button__icon': this.iconButton
    };
    const iconDom = h("flow-icon", { part: "icon", class: iconClasses, icon: this.icon });
    return (h(Host, { class: hostClasses, onClick: this.handleClick },
      h(Button, { ref: el => (this.button = el), class: nativeClasses, type: isLink ? null : this.submit ? 'submit' : 'button', disabled: disabled, href: isLink && this.href, target: isLink && this.target ? this.target : null, download: isLink && this.download ? this.download : null, rel: isLink && this.target ? 'noreferrer noopener' : null },
        h("div", { class: "mdc-button__ripple" }),
        icon && !trailingIcon ? iconDom : undefined,
        h("span", { class: "mdc-button__label", part: "label" }, label ? label : h("slot", null)),
        icon && trailingIcon ? iconDom : undefined)));
  }
  static get is() { return "flow-button"; }
  static get encapsulation() { return "shadow"; }
  static get delegatesFocus() { return true; }
  static get originalStyleUrls() { return {
    "$": ["button.scss"]
  }; }
  static get styleUrls() { return {
    "$": ["button.css"]
  }; }
  static get properties() { return {
    "color": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "PredefinedColor",
        "resolved": "\"danger\" | \"gray\" | \"primary\" | \"secondary\" | \"success\" | \"warning\"",
        "references": {
          "PredefinedColor": {
            "location": "import",
            "path": "../../utils/theme"
          }
        }
      },
      "required": false,
      "optional": true,
      "docs": {
        "tags": [],
        "text": "The color of the component. Colors are taken from the application palette."
      },
      "attribute": "color",
      "reflect": false
    },
    "compact": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Set to `true` to display a smaller-sized button with reduced margins."
      },
      "attribute": "compact",
      "reflect": false,
      "defaultValue": "false"
    },
    "disabled": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Set to `true` to disable interaction with the button."
      },
      "attribute": "disabled",
      "reflect": false,
      "defaultValue": "false"
    },
    "label": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "If set, specifies the button label text. Otherwise, the\ntext content of the element is used as the button label text."
      },
      "attribute": "label",
      "reflect": false
    },
    "icon": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Set to display an icon on the button. The property can be the name of\na built-in icon, a URL of an SVG file, or an SVG data URI."
      },
      "attribute": "icon",
      "reflect": false
    },
    "iconButton": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Set to `true` to display an 'icon-only' button with no button body or label. Note that\nthe default color of an 'icon-only' button is the current text color. Use the `color` property\nto select a different color from the application palette."
      },
      "attribute": "icon-button",
      "reflect": false,
      "defaultValue": "false"
    },
    "outlined": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Set to display an 'outlined' button."
      },
      "attribute": "outlined",
      "reflect": false,
      "defaultValue": "false"
    },
    "solid": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Set to display a 'solid' button."
      },
      "attribute": "solid",
      "reflect": false,
      "defaultValue": "false"
    },
    "trailingIcon": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Set to `true` to display the icon *after* the label (to the right in LTR mode)."
      },
      "attribute": "trailing-icon",
      "reflect": false,
      "defaultValue": "false"
    },
    "href": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "When set, the underlying button will be rendered as an `<a>` with this `href` instead of a `<button>`."
      },
      "attribute": "href",
      "reflect": false
    },
    "target": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "'_blank' | '_parent' | '_self' | '_top'",
        "resolved": "\"_blank\" | \"_parent\" | \"_self\" | \"_top\"",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Specify a target for opening the link. Only used when `href` is set."
      },
      "attribute": "target",
      "reflect": false
    },
    "download": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Specify that the linked file should be downloaded using this filename. Only used when `href` is set."
      },
      "attribute": "download",
      "reflect": false
    },
    "submit": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Set to 'true' to indicate that activating the button should submit the form. Ignored when `href` is set."
      },
      "attribute": "submit",
      "reflect": false,
      "defaultValue": "false"
    }
  }; }
  static get elementRef() { return "el"; }
}
