/*!
 * (C) Emu Analytics
 */
import { Component, Element, Event, h, Prop, Watch } from '@stencil/core';
import { randomId } from '../../../utils/random-id';
import { debugLifecycle, formatEl } from '../../../utils/debug';
import { FlowCoreChart } from '../../../core/charts/flow-core-chart';
import { globalContextRegistry } from '../../../core/context/context-registry';
/**
 * @status stable
 */
export class FlowChart {
  constructor() {
    /**
     * The Flo.w context ID to use for application state.
     */
    this.flowContext = 'default';
    /** The height of the chart in pixels. */
    this.height = 200;
    /** The width of the chart in pixels. */
    this.width = 300;
    /** Set to enable the Vega Tooltip plugin. */
    this.tooltip = false;
  }
  async componentWillLoad() {
    if (!this.el.id) {
      this.el.setAttribute('id', randomId('chart'));
    }
    debugLifecycle(`${formatEl(this.el)} will load`);
    this._chart = new FlowCoreChart(this.el.id, this.flowContext);
    this._chart.on('signalChanged', (details) => {
      this.flowChartSignalChanged.emit(details);
    });
  }
  componentDidLoad() {
    return this.initializeChart();
  }
  disconnectedCallback() {
    if (this._chart) {
      this._chart.uninitialize();
    }
    debugLifecycle(`${formatEl(this.el)} did unload`);
  }
  async updateSize() {
    this._chart.setSignal('width', this.width.toString(10), false);
    this._chart.setSignal('height', this.height.toString(10));
  }
  async initializeChart() {
    const context = await globalContextRegistry.getContext(this.flowContext);
    context.chartSpecs.getSpec(this.chartSpec).then(spec => {
      this._chart.initialize({
        container: this.chartContainerEl,
        spec,
        width: this.width,
        height: this.height,
        tooltip: this.tooltip
      });
    });
  }
  render() {
    return h("div", { class: "vega-container", ref: el => (this.chartContainerEl = el) });
  }
  static get is() { return "flow-chart"; }
  static get encapsulation() { return "shadow"; }
  static get originalStyleUrls() { return {
    "$": ["chart.scss"]
  }; }
  static get styleUrls() { return {
    "$": ["chart.css"]
  }; }
  static get properties() { return {
    "_chart": {
      "type": "unknown",
      "mutable": false,
      "complexType": {
        "original": "FlowCoreChart",
        "resolved": "FlowCoreChart",
        "references": {
          "FlowCoreChart": {
            "location": "import",
            "path": "../../../core/charts/flow-core-chart"
          }
        }
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [{
            "name": "internal",
            "text": undefined
          }],
        "text": ""
      }
    },
    "chartSpec": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "The registered chart specification to display."
      },
      "attribute": "chart-spec",
      "reflect": false
    },
    "flowContext": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "The Flo.w context ID to use for application state."
      },
      "attribute": "flow-context",
      "reflect": false,
      "defaultValue": "'default'"
    },
    "height": {
      "type": "number",
      "mutable": false,
      "complexType": {
        "original": "number",
        "resolved": "number",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "The height of the chart in pixels."
      },
      "attribute": "height",
      "reflect": false,
      "defaultValue": "200"
    },
    "width": {
      "type": "number",
      "mutable": false,
      "complexType": {
        "original": "number",
        "resolved": "number",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "The width of the chart in pixels."
      },
      "attribute": "width",
      "reflect": false,
      "defaultValue": "300"
    },
    "tooltip": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Set to enable the Vega Tooltip plugin."
      },
      "attribute": "tooltip",
      "reflect": false,
      "defaultValue": "false"
    }
  }; }
  static get events() { return [{
      "method": "flowChartSignalChanged",
      "name": "flowChartSignalChanged",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": "Emitted when a chart signal has changed."
      },
      "complexType": {
        "original": "FlowChartSignalChangeDetails",
        "resolved": "FlowChartSignalChangeDetails",
        "references": {
          "FlowChartSignalChangeDetails": {
            "location": "local"
          }
        }
      }
    }]; }
  static get elementRef() { return "el"; }
  static get watchers() { return [{
      "propName": "width",
      "methodName": "updateSize"
    }, {
      "propName": "height",
      "methodName": "updateSize"
    }]; }
}
