/*!
 * (C) Emu Analytics
 */
import { Component, Host, Element, Event, Prop, h, Watch, Listen } from '@stencil/core';
import { globalContextRegistry } from '../../core/context/context-registry';
import { isArray } from '../../utils/checks.util';
/**
 * @since 2.0
 * @status stable
 * @reactive
 *
 * @slot - Child checkboxes to place in the checkbox group.
 *
 */
export class CheckboxGroup {
  constructor() {
    this.initialized = false;
    /** The value attribute of the checkbox group. */
    this.value = [];
    /**
     * Set to `true` to disable interaction with the checkbox group.
     */
    this.disabled = false;
    /**
     * The Flo.w context ID to use for application state.
     */
    this.flowContext = 'default';
  }
  async updateValue(value, oldValue) {
    if (!this.valuesEqual(oldValue, value)) {
      if (!isArray(value)) {
        this.getAllCheckboxes().forEach(radio => (radio.checked = false));
      }
      else {
        // Select radio button matching value
        this.getAllCheckboxes().forEach(radio => (radio.checked = this.value.includes(radio.value)));
      }
      // Update state if subscribed
      if (this.selectionState) {
        const context = await globalContextRegistry.getContext(this.flowContext);
        context.state.set(this.selectionState, value);
      }
    }
  }
  updateDisabled(disabled) {
    this.getAllCheckboxes().forEach(checkbox => (checkbox.disabled = disabled));
  }
  connectedCallback() {
    if (this.initialized) {
      // Handle re-connect to DOM
      this.connect();
    }
  }
  componentDidLoad() {
    this.initialized = true;
    this.connect();
  }
  disconnectedCallback() {
    this.disconnect();
  }
  connect() {
    this.updateDisabled(this.disabled);
    this.subscribe();
  }
  disconnect() {
    this.unsubscribe();
  }
  /**
   * Update group value when radio is clicked
   */
  onFlowChange(ev) {
    const target = ev.target;
    if (target.tagName === 'FLOW-CHECKBOX') {
      this.value = this.getAllCheckboxes()
        .map(checkbox => (checkbox.checked ? checkbox.value : null))
        .filter(val => val !== null);
      this.flowCheckboxGroupChange.emit();
    }
  }
  async subscribe() {
    this.unsubscribe();
    if (this.selectionState) {
      const context = await globalContextRegistry.getContext(this.flowContext);
      this.subscription = context.state.toStream(this.selectionState, true).subscribe(value => {
        this.value = value;
      });
    }
  }
  unsubscribe() {
    if (this.subscription) {
      this.subscription.unsubscribe();
      this.subscription = undefined;
    }
  }
  /**
   * Return all child radio elements
   */
  getAllCheckboxes() {
    const radios = this.host.querySelectorAll('flow-checkbox');
    return Array.from(radios);
  }
  valuesEqual(val1, val2) {
    if (!isArray(val1) || !isArray(val2)) {
      return false;
    }
    return val1.length === val2.length && val1.every((value, index) => value === val2[index]);
  }
  render() {
    const hostClasses = {
      'flow-checkbox-group': true
    };
    return (h(Host, { class: hostClasses },
      h("slot", null)));
  }
  static get is() { return "flow-checkbox-group"; }
  static get encapsulation() { return "shadow"; }
  static get originalStyleUrls() { return {
    "$": ["checkbox-group.scss"]
  }; }
  static get styleUrls() { return {
    "$": ["checkbox-group.css"]
  }; }
  static get properties() { return {
    "value": {
      "type": "unknown",
      "mutable": true,
      "complexType": {
        "original": "string[]",
        "resolved": "string[]",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "The value attribute of the checkbox group."
      },
      "defaultValue": "[]"
    },
    "disabled": {
      "type": "boolean",
      "mutable": true,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Set to `true` to disable interaction with the checkbox group."
      },
      "attribute": "disabled",
      "reflect": true,
      "defaultValue": "false"
    },
    "selectionState": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [{
            "name": "reactive",
            "text": undefined
          }],
        "text": "Set to bind a Flo.w application state property to the group. The state property\nis an array of `value` attributes corresponding to checked checkboxes."
      },
      "attribute": "selection-state",
      "reflect": false
    },
    "flowContext": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "The Flo.w context ID to use for application state."
      },
      "attribute": "flow-context",
      "reflect": false,
      "defaultValue": "'default'"
    }
  }; }
  static get events() { return [{
      "method": "flowCheckboxGroupChange",
      "name": "flowCheckboxGroupChange",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": "Emitted when the control's checked state changes."
      },
      "complexType": {
        "original": "any",
        "resolved": "any",
        "references": {}
      }
    }]; }
  static get elementRef() { return "host"; }
  static get watchers() { return [{
      "propName": "value",
      "methodName": "updateValue"
    }, {
      "propName": "disabled",
      "methodName": "updateDisabled"
    }, {
      "propName": "selectionState",
      "methodName": "subscribe"
    }]; }
  static get listeners() { return [{
      "name": "flowChange",
      "method": "onFlowChange",
      "target": undefined,
      "capture": false,
      "passive": false
    }]; }
}
