/*!
 * (C) Emu Analytics
 */
import { Component, Host, Event, Prop, h, Watch, Method } from '@stencil/core';
import { createColorClasses } from '../../utils/theme';
import { MDCFormField } from '@material/form-field';
import { MDCCheckbox } from '@material/checkbox';
let id = 0;
/**
 * @since 1.0
 * @status stable
 *
 * @slot - The checkbox label. Alternatively, set using the `label` attribute.
 *
 * @part label - The checkbox label.
 */
export class Checkbox {
  constructor() {
    this.initialized = false;
    this.inputId = `checkbox-${++id}`;
    /**
     * The checkbox position relative to its label.
     */
    this.align = 'start';
    /** The checked state of the checkbox.  */
    this.checked = false;
    /** Set to `true` to disable interaction with the checkbox. */
    this.disabled = false;
    /** Set to `true` to enable the indeterminate checkbox state. */
    this.indeterminate = false;
    this.handleChange = this.handleChange.bind(this);
    this.handleFocus = this.handleFocus.bind(this);
    this.handleBlur = this.handleBlur.bind(this);
    this.handleKeyDown = this.handleKeyDown.bind(this);
  }
  updateChecked(val) {
    if (this.mdcCheckbox) {
      this.mdcCheckbox.checked = val;
    }
  }
  updateDisabled(val) {
    if (this.mdcCheckbox) {
      this.mdcCheckbox.disabled = val;
    }
  }
  updateIndeterminate(val) {
    if (this.mdcCheckbox) {
      this.mdcCheckbox.indeterminate = val;
    }
  }
  connectedCallback() {
    if (this.initialized) {
      // Handle re-connect to DOM
      this.connect();
    }
  }
  componentDidLoad() {
    this.initialized = true;
    this.connect();
  }
  disconnectedCallback() {
    this.disconnect();
  }
  connect() {
    this.mdcCheckbox = new MDCCheckbox(this.mdcCheckboxEl);
    this.mdcFormField = new MDCFormField(this.mdcFormFieldEl);
    this.mdcFormField.input = this.mdcCheckbox;
    this.updateChecked(this.checked);
    this.updateDisabled(this.disabled);
    this.updateIndeterminate(this.indeterminate);
  }
  disconnect() {
    if (this.mdcCheckbox) {
      this.mdcCheckbox.destroy();
      this.mdcCheckbox = undefined;
    }
    if (this.mdcFormField) {
      this.mdcFormField.destroy();
      this.mdcFormField = undefined;
    }
  }
  /** Set focus on the checkbox. */
  async setFocus() {
    this.inputEl.focus();
  }
  /** Remove focus from the checkbox. */
  async removeFocus() {
    this.inputEl.blur();
  }
  handleChange() {
    this.checked = this.mdcCheckbox.checked;
    this.indeterminate = this.mdcCheckbox.indeterminate;
    this.flowChange.emit();
  }
  handleFocus(e) {
    this.flowFocus.emit();
  }
  handleBlur(e) {
    this.flowBlur.emit();
  }
  handleKeyDown(e) {
    if (e.key === ' ' || e.key === 'Enter') {
      // Prevent focus state getting stuck
      e.preventDefault();
      this.checked = !this.checked;
      this.indeterminate = false;
      this.flowChange.emit();
    }
  }
  handleLabelMouseDown(e) {
    // Prevent clicks on the label from briefly blurring the input
    e.preventDefault();
  }
  render() {
    const { label, disabled, name, align, checked, inputId, color, value } = this;
    const hostClasses = Object.assign({ 'flow-checkbox': true }, createColorClasses(color));
    const formFieldClasses = {
      'mdc-form-field': true,
      'mdc-form-field--align-end': align === 'end'
    };
    return (h(Host, { class: hostClasses },
      h("div", { class: formFieldClasses, ref: (el) => (this.mdcFormFieldEl = el) },
        h("div", { class: "mdc-checkbox", ref: (el) => (this.mdcCheckboxEl = el) },
          h("input", { type: "checkbox", class: "mdc-checkbox__native-control", id: inputId, name: name, value: value, ref: (el) => (this.inputEl = el), checked: checked, onChange: this.handleChange, onFocus: this.handleFocus, onBlur: this.handleBlur, onKeyDown: this.handleKeyDown, disabled: disabled }),
          h("div", { class: "mdc-checkbox__background" },
            h("svg", { class: "mdc-checkbox__checkmark", viewBox: "0 0 24 24" },
              h("path", { class: "mdc-checkbox__checkmark-path", fill: "none", d: "M1.73,12.91 8.1,19.28 22.79,4.59" })),
            h("div", { class: "mdc-checkbox__mixedmark" })),
          h("div", { class: "mdc-checkbox__ripple" })),
        h("label", { htmlFor: inputId, onMouseDown: this.handleLabelMouseDown, part: "label" }, label ? label : h("slot", null)))));
  }
  static get is() { return "flow-checkbox"; }
  static get encapsulation() { return "shadow"; }
  static get originalStyleUrls() { return {
    "$": ["checkbox.scss"]
  }; }
  static get styleUrls() { return {
    "$": ["checkbox.css"]
  }; }
  static get properties() { return {
    "color": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "PredefinedColor",
        "resolved": "\"danger\" | \"gray\" | \"primary\" | \"secondary\" | \"success\" | \"warning\"",
        "references": {
          "PredefinedColor": {
            "location": "import",
            "path": "../../utils/theme"
          }
        }
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "The color of the checkbox. Colors are taken from the application palette."
      },
      "attribute": "color",
      "reflect": false
    },
    "align": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "'start' | 'end'",
        "resolved": "\"end\" | \"start\"",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "The checkbox position relative to its label."
      },
      "attribute": "align",
      "reflect": false,
      "defaultValue": "'start'"
    },
    "label": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "If set, specifies the checkbox label text. Otherwise, the\ntext content of the element is used as the label text."
      },
      "attribute": "label",
      "reflect": false
    },
    "value": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "The value attribute of the checkbox."
      },
      "attribute": "value",
      "reflect": false
    },
    "checked": {
      "type": "boolean",
      "mutable": true,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "The checked state of the checkbox."
      },
      "attribute": "checked",
      "reflect": true,
      "defaultValue": "false"
    },
    "name": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "The name attribute of the checkbox."
      },
      "attribute": "name",
      "reflect": false
    },
    "disabled": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Set to `true` to disable interaction with the checkbox."
      },
      "attribute": "disabled",
      "reflect": true,
      "defaultValue": "false"
    },
    "indeterminate": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Set to `true` to enable the indeterminate checkbox state."
      },
      "attribute": "indeterminate",
      "reflect": true,
      "defaultValue": "false"
    }
  }; }
  static get events() { return [{
      "method": "flowBlur",
      "name": "flowBlur",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": "Emitted when the checkbox loses focus."
      },
      "complexType": {
        "original": "any",
        "resolved": "any",
        "references": {}
      }
    }, {
      "method": "flowChange",
      "name": "flowChange",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": "Emitted when the checked state changes."
      },
      "complexType": {
        "original": "any",
        "resolved": "any",
        "references": {}
      }
    }, {
      "method": "flowFocus",
      "name": "flowFocus",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": "Emitted when the checkbox gains focus."
      },
      "complexType": {
        "original": "any",
        "resolved": "any",
        "references": {}
      }
    }]; }
  static get methods() { return {
    "setFocus": {
      "complexType": {
        "signature": "() => Promise<void>",
        "parameters": [],
        "references": {
          "Promise": {
            "location": "global"
          }
        },
        "return": "Promise<void>"
      },
      "docs": {
        "text": "Set focus on the checkbox.",
        "tags": []
      }
    },
    "removeFocus": {
      "complexType": {
        "signature": "() => Promise<void>",
        "parameters": [],
        "references": {
          "Promise": {
            "location": "global"
          }
        },
        "return": "Promise<void>"
      },
      "docs": {
        "text": "Remove focus from the checkbox.",
        "tags": []
      }
    }
  }; }
  static get watchers() { return [{
      "propName": "checked",
      "methodName": "updateChecked"
    }, {
      "propName": "disabled",
      "methodName": "updateDisabled"
    }, {
      "propName": "indeterminate",
      "methodName": "updateIndeterminate"
    }]; }
}
