/*!
 * (C) Emu Analytics
 */
import { Component, Element, Host, h, Event, Prop, Watch, State, Method } from '@stencil/core';
import { FLOW_DATA_COLUMN_TAG } from '../constants';
import { isObjectFull } from '../../../utils/checks.util';
import { globalContextRegistry } from '../../../core/context/context-registry';
import * as d3 from 'd3';
import { MDCDataTable, events } from '@material/data-table';
import sanitizeHtml from 'sanitize-html';
/**
 * @since 1.1
 * @status stable
 * @reactive
 *
 * @slot - Table column definitions.
 *
 * @part table - The table container element (for styling the external border).
 * @part header - The <thead> element of the table.
 * @part header-row - The table header row element (<tr>).
 * @part header-cell - Table header cell element (<th>).
 * @part column-<n> - Applied to all table cells (<th>, <td>) where `n` is zero-based column index.
 * @part body - The <tbody> element of the table.
 * @part row - Table data row elements (<tr>).
 * @part row-even - Applied to even-numbered table rows (<tr>).
 * @part row-odd - Applied to odd-numbered table rows (<tr>).
 * @part row-<type> - Applied to data table rows (<tr>) where `type` is row type.
 * @part cell - Table data cell elements (<td>).
 * @part icon - Icon elements (<flow-icon>) in icon-formatted columns.
 * @part icon-<name> - Applied to Icon elements (<flow-icon>) in icon-formatted columns where `name` is the name of the icon.
 */
export class DataTable {
  constructor() {
    this.pendingLayout = false;
    /**
     * The Flo.w context ID to use for application state.
     */
    this.flowContext = 'default';
    /**
     * Set to `true` to enable row selection.
     */
    this.rowSelect = false;
    /**
     * Set to `true` to enable full row selection (requires rowSelect).
     */
    this.fullRowSelect = false;
    /**
     * Set to `true` to display a compact table with reduced font size and padding.
     */
    this.compact = false;
    /**
     * Set to `true` to keep the header row in place when scrolling the table data.
     */
    this.stickyHeader = false;
    /**
     * Set to `false` to hide the header row.
     */
    this.showHeader = true;
    /**
     * Set to `true` to disable horizontal scrolling of table data.
     */
    this.noScroll = false;
    /**
     * Set to `true` to allow wrapping of table cell contents.
     */
    this.wrap = false;
  }
  /**
   * Returns the selected row indexes, or row IDs if `rowId` has been set.
   */
  async selectedRows() {
    return this.mdcDataTable.getSelectedRowIds();
  }
  /**
   * Set the selected rows by index, or row ID if `rowId` has been set.
   */
  async setSelectedRows(rowIds) {
    var _a;
    if ((_a = this.mdcDataTable) === null || _a === void 0 ? void 0 : _a.foundation.adapter.isRowsSelectable()) {
      this.mdcDataTable.setSelectedRowIds(rowIds);
    }
  }
  connectedCallback() {
    this.handleRowClick = this.handleRowClick.bind(this);
  }
  componentWillLoad() {
    this.updateColumns();
    this.subscribe();
    this.mutationObserver = new MutationObserver(() => {
      this.updateColumns();
    });
    this.mutationObserver.observe(this.el, { childList: true });
  }
  componentDidLoad() {
    this.mdcDataTable = new MDCDataTable(this.mdcDataTableEl);
    this.mdcDataTable.listen(events.ROW_SELECTION_CHANGED, (event) => {
      this.flowRowSelectionChanged.emit(event.detail);
    });
    this.mdcDataTable.listen(events.SELECTED_ALL, () => this.flowRowSelectedAll.emit());
    this.mdcDataTable.listen(events.UNSELECTED_ALL, () => this.flowRowSelectedNone.emit());
    this.mdcDataTable.listen(events.SORTED, (event) => this.flowSortAction.emit(event.detail));
    this.mdcDataTable.layout();
  }
  componentDidUpdate() {
    if (this.pendingLayout && this.mdcDataTable) {
      this.mdcDataTable.layout();
      this.pendingLayout = false;
    }
  }
  disconnectedCallback() {
    if (this.mutationObserver) {
      this.mutationObserver.disconnect();
      this.mutationObserver = undefined;
    }
    this.unsubscribe();
    try {
      this.mdcDataTable.destroy();
    }
    catch (e) {
      // Ignore error from MDC
    }
  }
  async subscribe() {
    this.unsubscribe();
    if (typeof this.data === 'string') {
      // Reactive data
      const context = await globalContextRegistry.getContext(this.flowContext);
      this.subscription = context.state.toStream(this.data, true).subscribe((data) => {
        this.updateData(data);
      });
    }
    else {
      // Static data
      this.updateData(this.data);
    }
  }
  unsubscribe() {
    if (this.subscription) {
      this.subscription.unsubscribe();
      this.subscription = undefined;
    }
  }
  updateData(data) {
    if (Array.isArray(data)) {
      // Array data
      this._data = data;
    }
    else if (isObjectFull(data)) {
      // Object data - create array of key/value pairs
      this._data = Object.keys(data).map((key) => ({ key, value: data[key] }));
    }
    else {
      // No data
      this._data = null;
    }
    this.pendingLayout = true;
  }
  updateColumns() {
    const columnElements = this.el.getElementsByTagName(FLOW_DATA_COLUMN_TAG);
    this.columnDefs = Array.from(columnElements).map((col) => ({
      name: col.name,
      property: col.property,
      width: col.width,
      align: col.align,
      sortable: col.sortable,
      allowHtml: col.allowHtml,
      formatter: this.cellFormatter(col.format)
    }));
  }
  cellFormatter(format) {
    if (!format) {
      // Default formatter
      return (val) => (typeof val === 'object' ? JSON.stringify(val) : val);
    }
    if (format === 'icon') {
      // No formatter
      return (val) => h("flow-icon", { part: `icon icon-${val}`, icon: val });
    }
    if (typeof format === 'string') {
      // D3 number formatter
      const numberFormatter = d3.format(format);
      return (val) => (typeof val === 'number' ? numberFormatter(val) : val);
    }
    else {
      // Explicit formatter function
      return format;
    }
  }
  handleRowClick(ev) {
    if (!this.rowSelect) {
      return;
    }
    const clickedCell = ev.composedPath()[0].closest('.mdc-data-table__cell:not(.mdc-data-table__cell--checkbox)');
    if (clickedCell) {
      const clickedRow = clickedCell.closest('.mdc-data-table__row');
      const clickedRowId = clickedRow.getAttribute('data-row-id');
      const selectedRows = this.mdcDataTable.getSelectedRowIds();
      const clickedIndex = selectedRows.indexOf(clickedRowId);
      if (clickedIndex === -1) {
        selectedRows.push(clickedRowId);
      }
      else {
        selectedRows.splice(clickedIndex, 1);
      }
      this.mdcDataTable.setSelectedRowIds(selectedRows);
      this.flowRowSelectionChanged.emit({
        row: clickedRow,
        rowIndex: clickedRow.rowIndex,
        rowId: clickedRow.dataset.rowId,
        selected: clickedIndex === -1
      });
    }
  }
  renderTableHeader() {
    return (h("thead", { part: "header" },
      h("tr", { class: "mdc-data-table__header-row", part: "header-row" },
        this.rowSelect && (h("th", { class: "mdc-data-table__header-cell mdc-data-table__header-cell--checkbox", role: "columnheader", scope: "col" },
          h("div", { class: "mdc-checkbox mdc-data-table__header-row-checkbox mdc-checkbox--selected" },
            h("input", { type: "checkbox", class: "mdc-checkbox__native-control", "aria-label": "Toggle all rows" }),
            h("div", { class: "mdc-checkbox__background" },
              h("svg", { class: "mdc-checkbox__checkmark", viewBox: "0 0 24 24" },
                h("path", { class: "mdc-checkbox__checkmark-path", fill: "none", d: "M1.73,12.91 8.1,19.28 22.79,4.59" })),
              h("div", { class: "mdc-checkbox__mixedmark" })),
            h("div", { class: "mdc-checkbox__ripple" })))),
        this.columnDefs.map((colDef, colIndex) => {
          const colClasses = {
            'mdc-data-table__header-cell': true,
            'cell--align-left': colDef.align === 'left',
            'cell--align-center': colDef.align === 'center',
            'cell--align-right': colDef.align === 'right',
            'mdc-data-table__header-cell--with-sort': colDef.sortable
          };
          const colStyles = {
            width: colDef.width || 'auto'
          };
          if (colDef.sortable) {
            return (h("th", { class: colClasses, style: colStyles, role: "columnheader", scope: "col", part: `header-cell column-${colIndex}`, "data-column-id": colDef.property },
              h("div", { class: "mdc-data-table__header-cell-wrapper" },
                h("flow-icon", { class: "mdc-data-table__sort-icon-button", icon: "fas-arrow-up" }),
                h("div", { class: "mdc-data-table__header-cell-label" }, colDef.name))));
          }
          else
            return (h("th", { class: colClasses, style: colStyles, role: "columnheader", scope: "col", part: `header-cell column-${colIndex}` }, colDef.name));
        }))));
  }
  renderTableBody() {
    var _a;
    const bodyClasses = {
      'mdc-data-table__content': true
    };
    return (h("tbody", { part: "body", class: bodyClasses, onClick: (ev) => this.handleRowClick(ev) }, (_a = this._data) === null || _a === void 0 ? void 0 : _a.map((rowData, index) => this.renderTableRow(rowData, index))));
  }
  renderTableRow(rowData, index) {
    const rowClasses = {
      'mdc-data-table__row': true
    };
    const rowType = this.rowType ? rowData[this.rowType] : null;
    const part = `row row-${index % 2 ? 'even' : 'odd'}${rowType ? ' row-' + rowType : ''}`;
    return (h("tr", { class: rowClasses, "data-row-id": this.rowId ? rowData[this.rowId] : index, part: part },
      this.rowSelect && (h("td", { class: "mdc-data-table__cell mdc-data-table__cell--checkbox" },
        h("div", { class: "mdc-checkbox mdc-data-table__row-checkbox" },
          h("input", { type: "checkbox", class: "mdc-checkbox__native-control", "aria-labelledby": "u0" }),
          h("div", { class: "mdc-checkbox__background" },
            h("svg", { class: "mdc-checkbox__checkmark", viewBox: "0 0 24 24" },
              h("path", { class: "mdc-checkbox__checkmark-path", fill: "none", d: "M1.73,12.91 8.1,19.28 22.79,4.59" })),
            h("div", { class: "mdc-checkbox__mixedmark" })),
          h("div", { class: "mdc-checkbox__ripple" })))),
      this.columnDefs.map((colDef, colIndex) => this.renderTableCell(rowData, colDef, colIndex))));
  }
  renderTableCell(rowData, colDef, index) {
    const cellClasses = {
      'mdc-data-table__cell': true,
      'cell--align-left': colDef.align === 'left',
      'cell--align-center': colDef.align === 'center',
      'cell--align-right': colDef.align === 'right'
    };
    const cellStyles = {
      width: colDef.width || 'auto'
    };
    if (colDef.allowHtml) {
      const sanitizedHtml = sanitizeHtml(colDef.formatter(rowData[colDef.property]), {
        allowedTags: sanitizeHtml.defaults.allowedTags.concat(['flow-tooltip', 'flow-icon', 'flow-badge']),
        allowedAttributes: false // allow all
      });
      return h("td", { class: cellClasses, part: `cell column-${index}`, style: cellStyles, innerHTML: sanitizedHtml });
    }
    else {
      return (h("td", { class: cellClasses, part: `cell column-${index}`, style: cellStyles }, colDef.formatter(rowData[colDef.property])));
    }
  }
  render() {
    const dataTableClasses = {
      'mdc-data-table': true,
      'mdc-data-table--sticky-header': this.stickyHeader,
      'mdc-data-table--compact': this.compact,
      'mdc-data-table--no-scroll': this.noScroll,
      'mdc-data-table--wrap': this.wrap
    };
    return (h(Host, { onFlowChange: () => this.updateColumns() },
      h("div", { class: dataTableClasses, part: "table", ref: (el) => (this.mdcDataTableEl = el) },
        h("div", { class: "mdc-data-table__table-container" },
          h("table", { class: "mdc-data-table__table" },
            this.showHeader && this.renderTableHeader(),
            this.renderTableBody())))));
  }
  static get is() { return "flow-data-table"; }
  static get encapsulation() { return "shadow"; }
  static get originalStyleUrls() { return {
    "$": ["data-table.scss"]
  }; }
  static get styleUrls() { return {
    "$": ["data-table.css"]
  }; }
  static get properties() { return {
    "data": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string | object[] | object",
        "resolved": "object | object[] | string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [{
            "name": "reactive",
            "text": undefined
          }],
        "text": "Set to bind data to the table. May be a Flo.w reactive expression, an array of data objects\nor a single data object."
      },
      "attribute": "data",
      "reflect": false
    },
    "flowContext": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "The Flo.w context ID to use for application state."
      },
      "attribute": "flow-context",
      "reflect": false,
      "defaultValue": "'default'"
    },
    "rowSelect": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Set to `true` to enable row selection."
      },
      "attribute": "row-select",
      "reflect": false,
      "defaultValue": "false"
    },
    "fullRowSelect": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Set to `true` to enable full row selection (requires rowSelect)."
      },
      "attribute": "full-row-select",
      "reflect": false,
      "defaultValue": "false"
    },
    "rowId": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Specify a data object property to use as a row ID."
      },
      "attribute": "row-id",
      "reflect": false
    },
    "rowType": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Specify a data object property to use as a row type. The row type can be used\nto style table rows based on row data."
      },
      "attribute": "row-type",
      "reflect": false
    },
    "compact": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Set to `true` to display a compact table with reduced font size and padding."
      },
      "attribute": "compact",
      "reflect": false,
      "defaultValue": "false"
    },
    "stickyHeader": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Set to `true` to keep the header row in place when scrolling the table data."
      },
      "attribute": "sticky-header",
      "reflect": false,
      "defaultValue": "false"
    },
    "showHeader": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Set to `false` to hide the header row."
      },
      "attribute": "show-header",
      "reflect": false,
      "defaultValue": "true"
    },
    "noScroll": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Set to `true` to disable horizontal scrolling of table data."
      },
      "attribute": "no-scroll",
      "reflect": false,
      "defaultValue": "false"
    },
    "wrap": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Set to `true` to allow wrapping of table cell contents."
      },
      "attribute": "wrap",
      "reflect": false,
      "defaultValue": "false"
    }
  }; }
  static get states() { return {
    "columnDefs": {},
    "_data": {}
  }; }
  static get events() { return [{
      "method": "flowRowSelectionChanged",
      "name": "flowRowSelectionChanged",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": "Emitted when the row selection state changes. Only applicable when `rowSelect` is enabled."
      },
      "complexType": {
        "original": "RowSelectionChangedEventDetail",
        "resolved": "RowSelectionChangedEventDetail",
        "references": {
          "RowSelectionChangedEventDetail": {
            "location": "import",
            "path": "../types"
          }
        }
      }
    }, {
      "method": "flowSortAction",
      "name": "flowSortAction",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": "Emitted when a column sort action is performed. Only applicable when sorting is enabled for a table column."
      },
      "complexType": {
        "original": "SortActionEventDetail",
        "resolved": "SortActionEventDetail",
        "references": {
          "SortActionEventDetail": {
            "location": "import",
            "path": "../types"
          }
        }
      }
    }, {
      "method": "flowRowSelectedAll",
      "name": "flowRowSelectedAll",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": "Emitted when all rows are selected using the row selection controls.\nOnly applicable when `rowSelect` is enabled."
      },
      "complexType": {
        "original": "any",
        "resolved": "any",
        "references": {}
      }
    }, {
      "method": "flowRowSelectedNone",
      "name": "flowRowSelectedNone",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": "Emitted when all rows are deselected using the row selection controls.\nOnly applicable when `rowSelect` is enabled."
      },
      "complexType": {
        "original": "any",
        "resolved": "any",
        "references": {}
      }
    }]; }
  static get methods() { return {
    "selectedRows": {
      "complexType": {
        "signature": "() => Promise<string[]>",
        "parameters": [],
        "references": {
          "Promise": {
            "location": "global"
          }
        },
        "return": "Promise<string[]>"
      },
      "docs": {
        "text": "Returns the selected row indexes, or row IDs if `rowId` has been set.",
        "tags": []
      }
    },
    "setSelectedRows": {
      "complexType": {
        "signature": "(rowIds: string[]) => Promise<void>",
        "parameters": [{
            "tags": [],
            "text": ""
          }],
        "references": {
          "Promise": {
            "location": "global"
          }
        },
        "return": "Promise<void>"
      },
      "docs": {
        "text": "Set the selected rows by index, or row ID if `rowId` has been set.",
        "tags": []
      }
    }
  }; }
  static get elementRef() { return "el"; }
  static get watchers() { return [{
      "propName": "data",
      "methodName": "subscribe"
    }]; }
}
