/*!
 * (C) Emu Analytics
 */
import { Component, Element, Host, h, Prop, Event, Watch } from '@stencil/core';
import { randomId } from '../../utils/random-id';
import { formatEl, debugLifecycle, } from '../../utils/debug';
import * as d3 from 'd3';
import { globalContextRegistry } from '../../core/context/context-registry';
import { isNull, isUndefined } from '../../utils/checks.util';
/**
 * @since 1.0
 * @status stable
 * @reactive
 */
export class FlowValue {
  constructor() {
    this.initialized = false;
    /**
     * Specifies the transition duration in milliseconds for the animation between numeric values. Set
     * to zero to disable animation.
     */
    this.duration = 250;
    /**
     * The Flo.w context ID to use for application state.
     */
    this.flowContext = 'default';
  }
  async connectedCallback() {
    debugLifecycle(`${formatEl(this.el)} connectedCallback`);
    if (this.initialized) {
      // Handle reconnect
      this.connect();
    }
  }
  async componentDidLoad() {
    debugLifecycle(`${formatEl(this.el)} did load`);
    if (!this.el.id) {
      this.el.setAttribute('id', randomId('value'));
    }
    this.initialized = true;
    this.connect();
  }
  async disconnectedCallback() {
    debugLifecycle(`${formatEl(this.el)} disconnectedCallback`);
    this.disconnect();
  }
  async connect() {
    this.context = await globalContextRegistry.getContext(this.flowContext);
    this.updateNumberFormatter();
    this.subscribe();
  }
  async disconnect() {
    this.unsubscribe();
  }
  updateNumberFormatter() {
    if (!this.format || this.format === 'icon') {
      this.numberFormatter = n => n.toString(10);
    }
    else {
      this.numberFormatter = d3.format(this.format);
    }
    // update display
    this.setValue(this.internalValue);
  }
  subscribe() {
    var _a;
    this.unsubscribe();
    if (isUndefined(this.value)) {
      this.context.emitWarning(`${formatEl(this.el)} value attribute undefined`);
    }
    else if (isNull(this.value)) {
      // Don't attempt to evaluate null value
      this.setValue(this.value);
      this.flowDataValueChanged.emit(this.value);
    }
    else {
      const parsedDataExpression = this.context.parser.parseExpression(this.value);
      if (parsedDataExpression.status) {
        this.autorunDisposer = this.context.state.autorun(() => {
          try {
            const newValue = this.context.parser.evaluateExpression(parsedDataExpression.value);
            this.setValue(newValue);
            this.flowDataValueChanged.emit(newValue);
          }
          catch (e) {
            this.context.emitError(e);
          }
        }, `autorun: update(FlowDataValue.${(_a = this.el) === null || _a === void 0 ? void 0 : _a.id})`);
      }
      else {
        this.context.emitError(new Error(`Malformed expression - ${parsedDataExpression}`));
      }
    }
  }
  unsubscribe() {
    if (this.autorunDisposer) {
      this.autorunDisposer();
    }
  }
  setValue(newValue) {
    const selection = d3.select(this.el.shadowRoot).select('span');
    const node = selection.node();
    selection.interrupt(this.el.id);
    if (!this.duration || typeof newValue !== 'number' || typeof this.internalValue !== 'number') {
      // Cannot animate so just set value
      this.internalValue = newValue;
      node.innerHTML = this.formatValue(newValue);
    }
    else {
      const oldValue = isNaN(this.internalValue) ? 0 : this.internalValue;
      this.internalValue = newValue;
      // Going from number to number - animate
      selection
        .transition(this.el.id)
        .duration(this.duration)
        .ease(d3.easeQuadInOut)
        .tween('update', () => {
        const interp = d3.interpolateNumber(oldValue, newValue);
        return (t) => {
          node.innerHTML = this.formatValue(interp(t)).replace(' ', '&nbsp;');
        };
      });
    }
  }
  formatValue(value) {
    if (this.format === 'icon') {
      return `<flow-icon icon="${value || 'far-circle-question'}"></flow-icon>`;
    }
    if (typeof value === 'number') {
      return this.numberFormatter(value);
    }
    if (typeof value === 'undefined' || value === null) {
      return '--';
    }
    return value;
  }
  render() {
    return (h(Host, { class: "flow-value" },
      h("span", { part: "content" })));
  }
  static get is() { return "flow-value"; }
  static get encapsulation() { return "shadow"; }
  static get originalStyleUrls() { return {
    "$": ["data-value.scss"]
  }; }
  static get styleUrls() { return {
    "$": ["data-value.css"]
  }; }
  static get properties() { return {
    "duration": {
      "type": "number",
      "mutable": false,
      "complexType": {
        "original": "number",
        "resolved": "number",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Specifies the transition duration in milliseconds for the animation between numeric values. Set\nto zero to disable animation."
      },
      "attribute": "duration",
      "reflect": false,
      "defaultValue": "250"
    },
    "flowContext": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "The Flo.w context ID to use for application state."
      },
      "attribute": "flow-context",
      "reflect": false,
      "defaultValue": "'default'"
    },
    "format": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Specifies the display format. Use a D3 format string for numeric values. Use `\"icon\"` to\ntreat `value` as an icon name."
      },
      "attribute": "format",
      "reflect": false
    },
    "value": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [{
            "name": "reactive",
            "text": undefined
          }],
        "text": "Specifies the value to display as a Flo.w reactive expression."
      },
      "attribute": "value",
      "reflect": false
    }
  }; }
  static get events() { return [{
      "method": "flowDataValueChanged",
      "name": "flowDataValueChanged",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": "Emitted when the value changes."
      },
      "complexType": {
        "original": "number | string",
        "resolved": "number | string",
        "references": {}
      }
    }]; }
  static get elementRef() { return "el"; }
  static get watchers() { return [{
      "propName": "format",
      "methodName": "updateNumberFormatter"
    }, {
      "propName": "value",
      "methodName": "subscribe"
    }]; }
}
