/*!
 * (C) Emu Analytics
 */
// Polyfills for blocking elements
import 'wicg-inert';
import 'blocking-elements';
import { Component, Element, Event, h, Host, Method, Prop, Watch } from '@stencil/core';
import { MDCDialogFoundation } from '@material/dialog';
import { closest, matches } from '@material/dom/ponyfill';
import { applyPassive } from '@material/dom/events';
import { selectors, attributes, events } from './constants';
/**
 * @since 1.0
 * @status stable
 *
 * @slot - The main dialog body content.
 * @slot title - The dialog heading content.
 * @slot actions - The dialog footer content. Used for dialog action buttons.
 *
 * @part dialog - The full-screen dialog container (including scrim).
 * @part surface - The visible dialog box container.
 * @part content - The dialog content container (excluding header and actions).
 *
 */
export class FlowDialog {
  constructor() {
    /**
     * Specifies the dialog action to emit when closing the dialog via the `escape` key. Set to
     * the empty string to disable closing by this method.
     */
    this.escapeKeyAction = 'close';
    /**
     * Specifies the dialog action to emit when closing the dialog by clicking on the scrim. Set to
     * the empty string to disable closing by this method.
     */
    this.scrimClickAction = 'close';
  }
  updateEscapeKeyAction(newAction) {
    this.dialogFoundation.setEscapeKeyAction(newAction);
  }
  updateScrimClickAction(newAction) {
    this.dialogFoundation.setScrimClickAction(newAction);
  }
  componentDidLoad() {
    this.dialogFoundation = this.createFoundation();
    this.dialogFoundation.init();
    this.dialogFoundation.setEscapeKeyAction(this.escapeKeyAction);
    this.dialogFoundation.setScrimClickAction(this.scrimClickAction);
  }
  disconnectedCallback() {
    this.removeEventListeners();
    if (this.dialogFoundation) {
      this.dialogFoundation.destroy();
      this.dialogFoundation = undefined;
    }
  }
  /**
   * Open the dialog.
   */
  async open() {
    return new Promise((resolve) => {
      const openedHandler = (ev) => {
        this.el.removeEventListener(events.DIALOG_OPENED_EVENT, openedHandler);
        resolve();
      };
      this.el.addEventListener(events.DIALOG_OPENED_EVENT, openedHandler);
      this.setEventListeners();
      this.dialogFoundation.open();
    });
  }
  /**
   * Close the dialog.
   *
   * @param action The dialog `action` to return in the closing/closed events.
   */
  async close(action) {
    return new Promise((resolve) => {
      const closedHandler = (ev) => {
        this.el.removeEventListener(events.DIALOG_CLOSED_EVENT, closedHandler);
        resolve();
      };
      this.el.addEventListener(events.DIALOG_CLOSED_EVENT, closedHandler);
      this.dialogFoundation.close();
    });
  }
  /**
   * Returns a promise that resolves with the dialog `action` when the dialog is closed.
   */
  async waitUntilClosed() {
    return new Promise((resolve) => {
      const closedHandler = (ev) => {
        this.el.removeEventListener(events.DIALOG_CLOSED_EVENT, closedHandler);
        resolve(ev.detail.action);
      };
      this.el.addEventListener(events.DIALOG_CLOSED_EVENT, closedHandler);
    });
  }
  get hasTitle() {
    return !!this.el.querySelector(selectors.TITLE_SELECTOR);
  }
  createFoundation() {
    const adapter = {
      addBodyClass: (className) => document.body.classList.add(className),
      removeBodyClass: (className) => document.body.classList.remove(className),
      addClass: (className) => this.dialogEl.classList.add(className),
      removeClass: (className) => this.dialogEl.classList.remove(className),
      hasClass: (className) => this.dialogEl.classList.contains(className),
      areButtonsStacked: () => false,
      clickDefaultButton: () => this.clickDefaultButton(),
      eventTargetMatches: (target, selector) => (target ? matches(target, selector) : false),
      getActionFromEvent: (e) => {
        if (!e.target) {
          return '';
        }
        const element = closest(e.target, `[${attributes.FLOW_DIALOG_ACTION_ATTR}]`);
        const action = element && element.getAttribute(attributes.FLOW_DIALOG_ACTION_ATTR);
        return action;
      },
      getInitialFocusEl: () => this.getInitialFocus(),
      isContentScrollable: () => false,
      notifyClosed: (action) => {
        this.removeEventListeners();
        this.dialogClosed.emit(action ? { action } : {});
      },
      notifyClosing: (action) => this.dialogClosing.emit(action ? { action } : {}),
      notifyOpened: () => this.dialogOpened.emit(),
      notifyOpening: () => this.dialogOpening.emit(),
      releaseFocus: () => this.releaseFocus(),
      reverseButtons: () => { },
      trapFocus: (el) => this.trapFocus(el)
    };
    return new MDCDialogFoundation(adapter);
  }
  setEventListeners() {
    this.boundClickHandler = this.dialogFoundation.handleClick.bind(this.dialogFoundation);
    this.boundKeydownHandler = this.dialogFoundation.handleKeydown.bind(this.dialogFoundation);
    this.boundDocumentKeydownHandler = this.dialogFoundation.handleDocumentKeydown.bind(this.dialogFoundation);
    this.boundLayoutHandler = this.dialogFoundation.layout.bind(this.dialogFoundation);
    this.dialogEl.addEventListener('click', this.boundClickHandler);
    this.dialogEl.addEventListener('keydown', this.boundKeydownHandler, applyPassive());
    document.addEventListener('keydown', this.boundDocumentKeydownHandler, applyPassive());
    window.addEventListener('resize', this.boundLayoutHandler, applyPassive());
    window.addEventListener('orientationchange', this.boundLayoutHandler, applyPassive());
  }
  removeEventListeners() {
    if (this.boundClickHandler) {
      this.dialogEl.removeEventListener('click', this.boundClickHandler);
      this.boundClickHandler = null;
    }
    if (this.boundKeydownHandler) {
      this.dialogEl.removeEventListener('keydown', this.boundKeydownHandler);
      this.boundKeydownHandler = null;
    }
    if (this.boundDocumentKeydownHandler) {
      document.removeEventListener('keydown', this.boundDocumentKeydownHandler);
      this.boundDocumentKeydownHandler = null;
    }
    if (this.boundLayoutHandler) {
      window.removeEventListener('resize', this.boundLayoutHandler);
      window.removeEventListener('orientationchange', this.boundLayoutHandler);
      this.boundLayoutHandler = null;
    }
  }
  getInitialFocus() {
    let initialEl = this.el.querySelector(selectors.INITIAL_FOCUS_SELECTOR);
    if (!initialEl) {
      // Find last flow-button
      initialEl = this.el.querySelector(selectors.LAST_FLOW_BUTTON_SELECTOR);
    }
    return initialEl;
  }
  clickDefaultButton() {
    const defaultButton = this.el.querySelector(selectors.DEFAULT_FLOW_BUTTON_SELECTOR);
    if (defaultButton) {
      defaultButton.click();
    }
  }
  trapFocus(el) {
    document.$blockingElements.push(this.el);
    if (el) {
      el.focus();
    }
  }
  releaseFocus() {
    document.$blockingElements.remove(this.el);
  }
  render() {
    const titleDom = h("slot", { name: "title" });
    return (h(Host, null,
      h("div", { part: "dialog", ref: (el) => (this.dialogEl = el), class: "mdc-dialog", role: "alertdialog", "aria-modal": "true", "aria-labelledby": "my-dialog-title", "aria-describedby": "my-dialog-content" },
        h("div", { class: "mdc-dialog__container" },
          h("div", { class: "mdc-dialog__surface", part: "surface" },
            this.hasTitle && titleDom,
            h("div", { class: "mdc-dialog__content", part: "content" },
              h("slot", null)),
            h("slot", { name: "actions" }))),
        h("div", { class: "mdc-dialog__scrim" }))));
  }
  static get is() { return "flow-dialog"; }
  static get encapsulation() { return "shadow"; }
  static get originalStyleUrls() { return {
    "$": ["dialog.scss"]
  }; }
  static get styleUrls() { return {
    "$": ["dialog.css"]
  }; }
  static get properties() { return {
    "escapeKeyAction": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Specifies the dialog action to emit when closing the dialog via the `escape` key. Set to\nthe empty string to disable closing by this method."
      },
      "attribute": "escape-key-action",
      "reflect": false,
      "defaultValue": "'close'"
    },
    "scrimClickAction": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Specifies the dialog action to emit when closing the dialog by clicking on the scrim. Set to\nthe empty string to disable closing by this method."
      },
      "attribute": "scrim-click-action",
      "reflect": false,
      "defaultValue": "'close'"
    }
  }; }
  static get events() { return [{
      "method": "dialogOpened",
      "name": "flowDialogOpened",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": "Emitted after the dialog has been opened and is visible."
      },
      "complexType": {
        "original": "any",
        "resolved": "any",
        "references": {}
      }
    }, {
      "method": "dialogClosed",
      "name": "flowDialogClosed",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": "Emitted after the dialog has been closed. The dialog action is specified in the event detail."
      },
      "complexType": {
        "original": "any",
        "resolved": "any",
        "references": {}
      }
    }, {
      "method": "dialogOpening",
      "name": "flowDialogOpening",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": "Emitted when the dialog is opening."
      },
      "complexType": {
        "original": "any",
        "resolved": "any",
        "references": {}
      }
    }, {
      "method": "dialogClosing",
      "name": "flowDialogClosing",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": "Emitted when the dialog has been closing. The dialog action is specified in the event detail."
      },
      "complexType": {
        "original": "any",
        "resolved": "any",
        "references": {}
      }
    }]; }
  static get methods() { return {
    "open": {
      "complexType": {
        "signature": "() => Promise<void>",
        "parameters": [],
        "references": {
          "Promise": {
            "location": "global"
          },
          "CustomEvent": {
            "location": "global"
          }
        },
        "return": "Promise<void>"
      },
      "docs": {
        "text": "Open the dialog.",
        "tags": []
      }
    },
    "close": {
      "complexType": {
        "signature": "(action?: string) => Promise<void>",
        "parameters": [{
            "tags": [{
                "name": "param",
                "text": "action The dialog `action` to return in the closing/closed events."
              }],
            "text": "The dialog `action` to return in the closing/closed events."
          }],
        "references": {
          "Promise": {
            "location": "global"
          },
          "CustomEvent": {
            "location": "global"
          }
        },
        "return": "Promise<void>"
      },
      "docs": {
        "text": "Close the dialog.",
        "tags": [{
            "name": "param",
            "text": "action The dialog `action` to return in the closing/closed events."
          }]
      }
    },
    "waitUntilClosed": {
      "complexType": {
        "signature": "() => Promise<string>",
        "parameters": [],
        "references": {
          "Promise": {
            "location": "global"
          },
          "CustomEvent": {
            "location": "global"
          }
        },
        "return": "Promise<string>"
      },
      "docs": {
        "text": "Returns a promise that resolves with the dialog `action` when the dialog is closed.",
        "tags": []
      }
    }
  }; }
  static get elementRef() { return "el"; }
  static get watchers() { return [{
      "propName": "escapeKeyAction",
      "methodName": "updateEscapeKeyAction"
    }, {
      "propName": "scrimClickAction",
      "methodName": "updateScrimClickAction"
    }]; }
}
