/*!
 * (C) Emu Analytics
 */
import { Component, Element, Host, h, Prop, Watch, Event } from '@stencil/core';
import { MDCDrawer } from '@material/drawer';
import { randomId } from '../../../utils/random-id';
import { debugLifecycle, formatEl, debug } from '../../../utils/debug';
/**
 * @since 1.0
 * @status stable
 *
 * @slot - The main body content of drawer.
 * @slot header - The drawer header content.
 * @slot footer - The drawer footer content.
 *
 * @part drawer - The drawer container.
 */
export class FlowDrawer {
  constructor() {
    this.initialized = false;
    this.hasHeader = false;
    this.hasFooter = false;
    /**
     * Set to `true` to open the drawer.
     */
    this.open = false;
    /**
     * The side of the parent element to which the drawer is attached.
     */
    this.position = 'start';
    /**
     * Set to `true` to display a 'side tab', which can be used to toggle the open state of the drawer.
     */
    this.sideTab = false;
    this.openHandler = ev => {
      this.drawerOpened.emit({ position: this.position });
    };
    this.closeHandler = ev => {
      this.drawerClosed.emit({ position: this.position });
    };
  }
  updateOpen(newVal, oldVal) {
    if (this.drawer && this.open !== this.drawer.open) {
      const eventProp = this.open ? 'drawerOpening' : 'drawerClosing';
      this[eventProp].emit({ position: this.position });
      this.drawer.open = this.open;
    }
  }
  connectedCallback() {
    if (this.initialized) {
      // Handle re-connect to DOM
      this.connect();
    }
  }
  componentWillLoad() {
    if (!this.el.id) {
      this.el.setAttribute('id', randomId('drawer'));
    }
    debugLifecycle(`${formatEl(this.el)} will load`);
    this.hasHeader = !!this.el.querySelector('[slot="header"]');
    this.hasFooter = !!this.el.querySelector('[slot="footer"]');
  }
  componentDidLoad() {
    this.initialized = true;
    this.connect();
    debug(`Initial setting of [open] for ${formatEl(this.el)}: ${this.open}`);
    if (this.open) {
      // Immediately set drawer open
      this.drawerEl.classList.add('mdc-drawer--open');
      this.drawerOpened.emit({ position: this.position });
    }
  }
  disconnectedCallback() {
    debugLifecycle(`${formatEl(this.el)} did unload`);
    this.disconnect();
  }
  connect() {
    this.drawer = MDCDrawer.attachTo(this.drawerEl);
    this.drawer.listen('MDCDrawer:opened', this.openHandler);
    this.drawer.listen('MDCDrawer:closed', this.closeHandler);
  }
  disconnect() {
    this.drawer.unlisten('MDCDrawer:opened', this.openHandler);
    this.drawer.unlisten('MDCDrawer:closed', this.closeHandler);
    this.drawer.destroy();
  }
  render() {
    const nativeClasses = {
      ['mdc-drawer']: true,
      ['mdc-drawer--dismissible']: true,
      ['flow-drawer--position-start']: this.position !== 'end',
      ['flow-drawer--position-end']: this.position === 'end'
    };
    const sideTabDom = (h("div", { class: "flow-drawer__side-tab" },
      h("flow-button", { class: "flow-drawer__side-tab-button", compact: true, iconButton: true, icon: this.position === 'start' ? 'fas-chevron-left' : 'fas-chevron-right', onClick: () => (this.open = !this.open) })));
    return (h(Host, null,
      h("aside", { part: "drawer", ref: el => (this.drawerEl = el), class: nativeClasses, dir: this.position === 'end' ? 'rtl' : 'ltr' },
        this.sideTab && sideTabDom,
        this.hasHeader && (h("div", { class: "mdc-drawer__header", dir: "ltr" },
          h("slot", { name: "header" }))),
        h("div", { class: "mdc-drawer__content", dir: "ltr" },
          h("slot", null)),
        this.hasFooter && (h("div", { class: "mdc-drawer__footer", dir: "ltr" },
          h("slot", { name: "footer" }))))));
  }
  static get is() { return "flow-drawer"; }
  static get encapsulation() { return "shadow"; }
  static get originalStyleUrls() { return {
    "$": ["drawer.scss"]
  }; }
  static get styleUrls() { return {
    "$": ["drawer.css"]
  }; }
  static get properties() { return {
    "open": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Set to `true` to open the drawer."
      },
      "attribute": "open",
      "reflect": true,
      "defaultValue": "false"
    },
    "position": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "'start' | 'end'",
        "resolved": "\"end\" | \"start\"",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "The side of the parent element to which the drawer is attached."
      },
      "attribute": "position",
      "reflect": false,
      "defaultValue": "'start'"
    },
    "sideTab": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Set to `true` to display a 'side tab', which can be used to toggle the open state of the drawer."
      },
      "attribute": "side-tab",
      "reflect": false,
      "defaultValue": "false"
    }
  }; }
  static get events() { return [{
      "method": "drawerOpening",
      "name": "flowDrawerOpening",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": "Emitted when the drawer is opening."
      },
      "complexType": {
        "original": "any",
        "resolved": "any",
        "references": {}
      }
    }, {
      "method": "drawerClosing",
      "name": "flowDrawerClosing",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": "Emitted when the drawer is closing."
      },
      "complexType": {
        "original": "any",
        "resolved": "any",
        "references": {}
      }
    }, {
      "method": "drawerOpened",
      "name": "flowDrawerOpened",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": "Emitted when the drawer has opened."
      },
      "complexType": {
        "original": "any",
        "resolved": "any",
        "references": {}
      }
    }, {
      "method": "drawerClosed",
      "name": "flowDrawerClosed",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": "Emitted when the drawer has closed."
      },
      "complexType": {
        "original": "any",
        "resolved": "any",
        "references": {}
      }
    }]; }
  static get elementRef() { return "el"; }
  static get watchers() { return [{
      "propName": "open",
      "methodName": "updateOpen"
    }]; }
}
