/*!
 * (C) Emu Analytics
 */
import { Component, Host, h, Prop, Watch } from '@stencil/core';
import { globalContextRegistry } from '../../../core/context/context-registry';
/**
 * @since 1.1
 * @status stable
 * @reactive
 *
 * @slot - Child FABs to place in the FAB group.
 */
export class FabGroup {
  constructor() {
    this.initialized = false;
    /**
     * Set this attribute to `true` to force the FAB group to open. The open state will
     * be toggled automatically when the main FAB is clicked.
     */
    this.open = false;
    /**
     * Specify the direction in which to expand buttons when the FAB group is opened.
     */
    this.direction = 'up';
    /**
     * The Flo.w context ID to use for application state.
     */
    this.flowContext = 'default';
    this.handleClick = this.handleClick.bind(this);
  }
  connectedCallback() {
    if (this.initialized) {
      // Handle re-connect to DOM
      this.connect();
    }
  }
  componentDidLoad() {
    this.initialized = true;
    this.connect();
  }
  disconnectedCallback() {
    this.disconnect();
  }
  connect() {
    this.subscribe();
  }
  disconnect() {
    this.unsubscribe();
  }
  async subscribe() {
    this.unsubscribe();
    if (this.selectionState) {
      const context = await globalContextRegistry.getContext(this.flowContext);
      this.subscription = context.state.toStream(this.selectionState, true).subscribe(value => {
        if (Array.isArray(value)) {
          this.getAllSubFABs().forEach(fab => (fab.checked = value.indexOf(fab.value) > -1));
        }
      });
    }
  }
  unsubscribe() {
    if (this.subscription) {
      this.subscription.unsubscribe();
      this.subscription = undefined;
    }
  }
  async handleClick(event) {
    const target = event.target;
    if (target.classList.contains('main-fab')) {
      // Click on main FAB - toggle open state
      this.open = !this.open;
    }
    else {
      // Click on sub FAB - update checked status
      const subFABs = this.getAllSubFABs();
      const state = subFABs.reduce((accum, fab) => {
        if (fab.checked) {
          accum.push(fab.value);
        }
        return accum;
      }, []);
      if (this.selectionState) {
        const context = await globalContextRegistry.getContext(this.flowContext);
        let stateValues = context.state.get(this.selectionState) || [];
        // Remove all values for this FAB group
        const fabValues = subFABs.map(el => el.value);
        stateValues = stateValues.filter(val => fabValues.indexOf(val) === -1);
        // Add selected values for this FAB group
        stateValues = stateValues.concat(state);
        context.state.set(this.selectionState, stateValues);
      }
    }
  }
  getAllSubFABs() {
    var _a;
    const slot = (_a = this.subFABContainer) === null || _a === void 0 ? void 0 : _a.querySelector('slot');
    return slot ? slot.assignedElements({ flatten: true }).map(el => this.findFab(el)).filter(el => !!el) : [];
  }
  findFab(el) {
    var _a;
    const selector = 'flow-fab';
    // The button could be the target element or a child of it (e.g. a dropdown or tooltip anchor)
    return (_a = el.closest(selector)) !== null && _a !== void 0 ? _a : el.querySelector(selector);
  }
  render() {
    const containerClasses = {
      'fab-container': true,
      'fab-container--open': this.open,
      // Directions
      'fab-container--up': this.direction === 'up',
      'fab-container--down': this.direction === 'down',
      'fab-container--left': this.direction === 'left',
      'fab-container--right': this.direction === 'right'
    };
    return (h(Host, null,
      h("div", { class: containerClasses, onClick: this.handleClick },
        h("flow-fab", { class: "main-fab", icon: this.icon }),
        h("div", { ref: el => (this.subFABContainer = el), class: "fab-sub-container" },
          h("slot", null)))));
  }
  static get is() { return "flow-fab-group"; }
  static get encapsulation() { return "shadow"; }
  static get originalStyleUrls() { return {
    "$": ["fab-group.scss"]
  }; }
  static get styleUrls() { return {
    "$": ["fab-group.css"]
  }; }
  static get properties() { return {
    "icon": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "The icon of the main FAB group button. The property can be the name of\na built-in icon, a URL of an SVG file, or an SVG data URI."
      },
      "attribute": "icon",
      "reflect": false
    },
    "open": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Set this attribute to `true` to force the FAB group to open. The open state will\nbe toggled automatically when the main FAB is clicked."
      },
      "attribute": "open",
      "reflect": true,
      "defaultValue": "false"
    },
    "direction": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "'up' | 'down' | 'left' | 'right'",
        "resolved": "\"down\" | \"left\" | \"right\" | \"up\"",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Specify the direction in which to expand buttons when the FAB group is opened."
      },
      "attribute": "direction",
      "reflect": false,
      "defaultValue": "'up'"
    },
    "selectionState": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [{
            "name": "reactive",
            "text": undefined
          }],
        "text": "Set to bind a Flo.w application state property to the group. The state property\nis an array of `value` attributes corresponding to checked FABs."
      },
      "attribute": "selection-state",
      "reflect": false
    },
    "flowContext": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "The Flo.w context ID to use for application state."
      },
      "attribute": "flow-context",
      "reflect": false,
      "defaultValue": "'default'"
    }
  }; }
  static get watchers() { return [{
      "propName": "selectionState",
      "methodName": "subscribe"
    }]; }
}
