/*!
 * (C) Emu Analytics
 */
import { Component, Host, h, Prop } from '@stencil/core';
import { MDCRipple } from '@material/ripple';
import { createColorClasses } from '../../../utils/theme';
/**
 * @since 1.0
 * @status stable
 */
export class FlowFabButton {
  constructor() {
    this.initialized = false;
    /**
     * Set to `true` to display a smaller FAB.
     */
    this.compact = false;
    /**
     * Set to `true` to display the FAB using the 'square' style. By default,
     * FABs are round.
     */
    this.square = false;
    /**
     * Set to `true` for the FAB to act like a checkbox.
     */
    this.checkbox = false;
    /**
     * The checked state of the FAB. Only applicable if `checkbox` is set to `true`.
     */
    this.checked = false;
    this.handleClick = this.handleClick.bind(this);
  }
  connectedCallback() {
    if (this.initialized) {
      // Handle re-connect to DOM
      this.connect();
    }
  }
  componentDidLoad() {
    this.initialized = true;
    this.connect();
  }
  disconnectedCallback() {
    this.disconnect();
  }
  connect() {
    this.ripple = new MDCRipple(this.button);
    this.ripple.unbounded = true;
  }
  disconnect() {
    var _a;
    (_a = this.ripple) === null || _a === void 0 ? void 0 : _a.destroy();
  }
  handleClick() {
    this.checked = this.checkbox && !this.checked;
  }
  render() {
    const { label, icon, color, compact, square, checkbox, checked } = this;
    const hostClasses = Object.assign({}, createColorClasses(color));
    const classes = {
      'mdc-fab': true,
      'mdc-fab--mini': compact,
      'mdc-fab--square': square,
      'mdc-fab--extended': !!label,
      'mdc-fab--checkbox': checkbox,
      'mdc-fab--checked': checked
    };
    return (h(Host, { class: hostClasses },
      h("button", { ref: (el) => (this.button = el), class: classes, onClick: this.handleClick },
        h("div", { class: "mdc-fab__ripple" }),
        h("flow-icon", { class: "mdc-fab__icon", icon: icon }),
        label ? h("span", { class: "mdc-fab__label" }, label) : '')));
  }
  static get is() { return "flow-fab"; }
  static get encapsulation() { return "shadow"; }
  static get originalStyleUrls() { return {
    "$": ["fab.scss"]
  }; }
  static get styleUrls() { return {
    "$": ["fab.css"]
  }; }
  static get properties() { return {
    "color": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "PredefinedColor",
        "resolved": "\"danger\" | \"gray\" | \"primary\" | \"secondary\" | \"success\" | \"warning\"",
        "references": {
          "PredefinedColor": {
            "location": "import",
            "path": "../../../utils/theme"
          }
        }
      },
      "required": false,
      "optional": true,
      "docs": {
        "tags": [],
        "text": "The color of the FAB. Colors are taken from the application palette."
      },
      "attribute": "color",
      "reflect": false
    },
    "compact": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Set to `true` to display a smaller FAB."
      },
      "attribute": "compact",
      "reflect": false,
      "defaultValue": "false"
    },
    "icon": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": true,
      "docs": {
        "tags": [],
        "text": "Set to display an icon on the button. The property can be the name of\na built-in icon, a URL of an SVG file, or an SVG data URI."
      },
      "attribute": "icon",
      "reflect": false
    },
    "label": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": true,
      "docs": {
        "tags": [],
        "text": "Set to add an optional label to the FAB."
      },
      "attribute": "label",
      "reflect": false
    },
    "square": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Set to `true` to display the FAB using the 'square' style. By default,\nFABs are round."
      },
      "attribute": "square",
      "reflect": false,
      "defaultValue": "false"
    },
    "checkbox": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Set to `true` for the FAB to act like a checkbox."
      },
      "attribute": "checkbox",
      "reflect": false,
      "defaultValue": "false"
    },
    "checked": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "The checked state of the FAB. Only applicable if `checkbox` is set to `true`."
      },
      "attribute": "checked",
      "reflect": true,
      "defaultValue": "false"
    },
    "value": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": true,
      "docs": {
        "tags": [],
        "text": "The value attribute of the FAB. Only applicable if `checkbox` is set to `true`."
      },
      "attribute": "value",
      "reflect": true
    }
  }; }
}
