/*!
 * (C) Emu Analytics
 */
import { Component, Element, Host, h, Prop, State } from '@stencil/core';
import { randomId } from '../../utils/random-id';
import { formatEl, debugLifecycle } from '../../utils/debug';
import { createColorClasses } from '../../utils/theme';
/**
 * @since 1.0
 * @status stable
 * @reactive
 *
 * @slot info - Content to place in the gauge's information dialog. Information button is added to the gauge. (optional)
 *
 * @part header - The gauge header.
 * @part header-icon - The gauge header icon.
 * @part info-button - The gauge header info button.
 * @part value - The gauge value.
 * @part units - The gauge units.
 */
export class FlowGauge {
  constructor() {
    /**
     * The transition duration in milliseconds for the animation between numeric values.
     * Set to zero to disable animation.
     */
    this.duration = 250;
    /**
     * The Flo.w context ID to use for application state.
     */
    this.flowContext = 'default';
  }
  async componentDidLoad() {
    if (!this.el.id) {
      this.el.setAttribute('id', randomId('gauge'));
    }
    debugLifecycle(`${formatEl(this.el)} did load`);
  }
  infoClicked() {
    this.infoDialog.open();
  }
  handleDataChanged(event) {
    if (this.format === 'icon') {
      // Current icon is reflected as CSS class on host element to allow color styling
      // based on icon
      this.currentIcon = event.detail;
    }
  }
  render() {
    const hasInfo = !!this.el.querySelector('[slot="info"]');
    const headerIconDom = h("flow-icon", { part: "header-icon", class: "flow-gauge__header-icon", icon: this.icon });
    const infoDialogIconDom = h("flow-icon", { class: "flow-gauge__info-icon", icon: this.icon });
    const unitsDom = h("div", { part: "units", class: "flow-gauge__units" }, this.units);
    const infoButtonDom = (h("flow-button", { part: "info-button", onClick: ev => this.infoClicked(), class: "flow-gauge__header-button", compact: true, "icon-button": true, icon: "fas-circle-info" }));
    const infoDialog = (h("flow-dialog", { ref: el => (this.infoDialog = el) },
      h("div", { slot: "title" },
        this.icon && infoDialogIconDom,
        this.heading),
      h("slot", { name: "info" }),
      h("div", { slot: "actions" },
        h("flow-button", { class: "mdc-dialog__button", "dialog-action": "ok" }, "OK"))));
    const classes = Object.assign({ 'flow-gauge': true, [this.currentIcon]: !!this.currentIcon }, createColorClasses(this.color));
    return (h(Host, { class: classes },
      h("div", { part: "header", class: "flow-gauge__header" },
        this.icon && headerIconDom,
        h("span", { class: "flow-gauge__header-text" }, this.heading),
        hasInfo && infoButtonDom),
      h("div", { class: "flow-gauge__body" },
        h("flow-value", { part: "value", "flow-context": this.flowContext, class: "flow-gauge__data-value", value: this.value, format: this.format, onFlowDataValueChanged: event => this.handleDataChanged(event), duration: this.duration }),
        this.units && unitsDom),
      hasInfo && infoDialog));
  }
  static get is() { return "flow-gauge"; }
  static get encapsulation() { return "shadow"; }
  static get originalStyleUrls() { return {
    "$": ["gauge.scss"]
  }; }
  static get styleUrls() { return {
    "$": ["gauge.css"]
  }; }
  static get properties() { return {
    "color": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "PredefinedColor",
        "resolved": "\"danger\" | \"gray\" | \"primary\" | \"secondary\" | \"success\" | \"warning\"",
        "references": {
          "PredefinedColor": {
            "location": "import",
            "path": "../../utils/theme"
          }
        }
      },
      "required": false,
      "optional": true,
      "docs": {
        "tags": [],
        "text": "The color of the gauge value. Colors are taken from the application palette."
      },
      "attribute": "color",
      "reflect": false
    },
    "duration": {
      "type": "number",
      "mutable": false,
      "complexType": {
        "original": "number",
        "resolved": "number",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "The transition duration in milliseconds for the animation between numeric values.\nSet to zero to disable animation."
      },
      "attribute": "duration",
      "reflect": false,
      "defaultValue": "250"
    },
    "flowContext": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "The Flo.w context ID to use for application state."
      },
      "attribute": "flow-context",
      "reflect": false,
      "defaultValue": "'default'"
    },
    "format": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": true,
      "docs": {
        "tags": [],
        "text": "The display format of the gauge value. Use a D3 format string for numeric values. Use `\"icon\"` to\ntreat `value` as an icon name."
      },
      "attribute": "format",
      "reflect": false
    },
    "heading": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": true,
      "docs": {
        "tags": [],
        "text": "The gauge heading text."
      },
      "attribute": "heading",
      "reflect": false
    },
    "icon": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": true,
      "docs": {
        "tags": [],
        "text": "Set to display an icon in the gauge heading. The property can be the name of\na built-in icon, a URL to an SVG file, or an SVG data URI."
      },
      "attribute": "icon",
      "reflect": false
    },
    "units": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": true,
      "docs": {
        "tags": [],
        "text": "The units text to display under the gauge value."
      },
      "attribute": "units",
      "reflect": false
    },
    "value": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": true,
      "docs": {
        "tags": [{
            "name": "reactive",
            "text": undefined
          }],
        "text": "The gauge value specified as a Flo.w reactive expression."
      },
      "attribute": "value",
      "reflect": false
    }
  }; }
  static get states() { return {
    "currentIcon": {}
  }; }
  static get elementRef() { return "el"; }
}
