/*!
 * (C) Emu Analytics
 */
import { Component, Host, Element, Event, Prop, h, Watch } from '@stencil/core';
import { createColorClasses } from '../../utils/theme';
import { MDCTextField } from '@material/textfield';
let id = 0;
/**
 * @since 1.1
 * @status stable
 *
 * @part label - The floating label.
 * @part icon - The icon.
 * @part trailing-icon - The trailing icon button.
 * @part prefix - The input's prefix span.
 * @part suffix - The input's suffix span.
 * @part input - The native input control.
 */
export class TextInput {
  constructor() {
    this.labelId = `input-label-${id++}`;
    /**
     * The style of the input control.
     */
    this.type = 'filled';
    /** The value of the input control. */
    this.value = '';
    /**
     * Set to `true` to disable interaction with the input control.
     */
    this.disabled = false;
    this.handleChange = this.handleChange.bind(this);
    this.handleInput = this.handleInput.bind(this);
    this.handleAction = this.handleAction.bind(this);
  }
  updateDisabled(val) {
    this.mdcTextField.disabled = val;
  }
  componentDidLoad() {
    this.mdcTextField = new MDCTextField(this.mdcTextFieldEl);
    this.updateDisabled(this.disabled);
    // Workaround for Firefox - doesn't support delegatesFocus
    this.host.focus = () => this.mdcInputEl.focus();
  }
  disconnectedCallback() { }
  handleChange() {
    this.flowChange.emit();
  }
  handleInput() {
    this.value = this.mdcInputEl.value;
    this.flowInput.emit();
  }
  handleAction(ev) {
    this.flowInputAction.emit();
  }
  renderFloatingLabel() {
    const { label, labelId } = this;
    return (label && (h("span", { part: "label", class: "mdc-floating-label", id: labelId }, label)));
  }
  renderPrefix() {
    const { prefixText } = this;
    return (prefixText && (h("span", { part: "prefix", class: "mdc-text-field__affix mdc-text-field__affix--prefix" }, prefixText)));
  }
  renderSuffix() {
    const { suffixText } = this;
    return (suffixText && (h("span", { part: "suffix", class: "mdc-text-field__affix mdc-text-field__affix--suffix" }, suffixText)));
  }
  renderIcon() {
    const { icon } = this;
    return (icon && h("flow-icon", { part: "icon", icon: icon, class: "mdc-text-field__icon mdc-text-field__icon--leading" }));
  }
  renderTrailingIcon() {
    const { trailingIcon } = this;
    const style = {
      '--color': 'current'
    };
    return (trailingIcon && (h("flow-button", { part: "trailing-icon", style: style, tabindex: "0", "icon-button": true, icon: trailingIcon, class: "mdc-text-field__icon mdc-text-field__icon--trailing", onClick: this.handleAction })));
  }
  renderFilledInput() {
    const { label, labelId, icon, trailingIcon } = this;
    const mdcClasses = {
      'mdc-text-field': true,
      'mdc-text-field--filled': true,
      'mdc-text-field--no-label': !label,
      'mdc-text-field--with-leading-icon': !!icon,
      'mdc-text-field--with-trailing-icon': !!trailingIcon
    };
    return (h("label", { ref: el => (this.mdcTextFieldEl = el), class: mdcClasses },
      this.renderIcon(),
      h("span", { class: "mdc-text-field__ripple" }),
      this.renderFloatingLabel(),
      this.renderPrefix(),
      h("input", { ref: el => (this.mdcInputEl = el), part: "input", class: "mdc-text-field__input", type: "text", "aria-labelledby": labelId, placeholder: " ", value: this.value, onChange: ev => this.handleChange(), onInput: ev => this.handleInput() }),
      this.renderSuffix(),
      h("span", { class: "mdc-line-ripple" }),
      this.renderTrailingIcon()));
  }
  renderOutlinedInput() {
    const { label, labelId, icon, trailingIcon } = this;
    const mdcClasses = {
      'mdc-text-field': true,
      'mdc-text-field--outlined': true,
      'mdc-text-field--no-label': !label,
      'mdc-text-field--with-leading-icon': !!icon,
      'mdc-text-field--with-trailing-icon': !!trailingIcon
    };
    return (h("label", { ref: el => (this.mdcTextFieldEl = el), class: mdcClasses },
      this.renderIcon(),
      h("span", { class: "mdc-notched-outline" },
        h("span", { class: "mdc-notched-outline__leading" }),
        h("span", { class: "mdc-notched-outline__notch" }, label && this.renderFloatingLabel()),
        h("span", { class: "mdc-notched-outline__trailing" })),
      this.renderPrefix(),
      h("input", { ref: el => (this.mdcInputEl = el), part: "input", type: "text", class: "mdc-text-field__input", "aria-labelledby": labelId, value: this.value, onChange: ev => this.handleChange(), onInput: ev => this.handleInput() }),
      this.renderSuffix(),
      this.renderTrailingIcon()));
  }
  renderPlainInput() {
    const { label, labelId } = this;
    const mdcClasses = {
      'mdc-text-field': true,
      'mdc-text-field--filled': true,
      'mdc-text-field--fullwidth': true
    };
    return (h("label", { ref: el => (this.mdcTextFieldEl = el), class: mdcClasses },
      h("span", { class: "mdc-text-field__ripple" }),
      h("input", { ref: el => (this.mdcInputEl = el), part: "input", class: "mdc-text-field__input", type: "text", placeholder: label, "aria-label": label, value: this.value, onChange: ev => this.handleChange(), onInput: ev => this.handleInput() }),
      h("span", { class: "mdc-line-ripple" })));
  }
  render() {
    const { color, label, type } = this;
    const hostClasses = Object.assign({ 'flow-input': true }, createColorClasses(color));
    return (h(Host, { class: hostClasses }, type === 'filled'
      ? this.renderFilledInput()
      : type === 'plain'
        ? this.renderPlainInput()
        : this.renderOutlinedInput()));
  }
  static get is() { return "flow-input"; }
  static get encapsulation() { return "shadow"; }
  static get delegatesFocus() { return true; }
  static get originalStyleUrls() { return {
    "$": ["input.scss"]
  }; }
  static get styleUrls() { return {
    "$": ["input.css"]
  }; }
  static get properties() { return {
    "color": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "PredefinedColor",
        "resolved": "\"danger\" | \"gray\" | \"primary\" | \"secondary\" | \"success\" | \"warning\"",
        "references": {
          "PredefinedColor": {
            "location": "import",
            "path": "../../utils/theme"
          }
        }
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "The color of the control. Colors are taken from the application palette."
      },
      "attribute": "color",
      "reflect": false
    },
    "type": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "'filled' | 'outlined' | 'plain'",
        "resolved": "\"filled\" | \"outlined\" | \"plain\"",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "The style of the input control."
      },
      "attribute": "type",
      "reflect": false,
      "defaultValue": "'filled'"
    },
    "label": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "The label text."
      },
      "attribute": "label",
      "reflect": false
    },
    "value": {
      "type": "string",
      "mutable": true,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "The value of the input control."
      },
      "attribute": "value",
      "reflect": false,
      "defaultValue": "''"
    },
    "prefixText": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Prefix text to place before the input."
      },
      "attribute": "prefix-text",
      "reflect": false
    },
    "suffixText": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Suffix text to place after the input."
      },
      "attribute": "suffix-text",
      "reflect": false
    },
    "icon": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "An icon to place before the input. The property can be the name of\na built-in icon, a URL of an SVG file, or an SVG data URI."
      },
      "attribute": "icon",
      "reflect": false
    },
    "trailingIcon": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "An icon action button to place after the input. The property can be the name of\na built-in icon, a URL of an SVG file, or an SVG data URI."
      },
      "attribute": "trailing-icon",
      "reflect": false
    },
    "name": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "The name attribute of the native input control"
      },
      "attribute": "name",
      "reflect": true
    },
    "disabled": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Set to `true` to disable interaction with the input control."
      },
      "attribute": "disabled",
      "reflect": true,
      "defaultValue": "false"
    }
  }; }
  static get events() { return [{
      "method": "flowChange",
      "name": "flowChange",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": "Emitted when the control's value changes."
      },
      "complexType": {
        "original": "any",
        "resolved": "any",
        "references": {}
      }
    }, {
      "method": "flowInput",
      "name": "flowInput",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": "Emitted when the control receives input."
      },
      "complexType": {
        "original": "any",
        "resolved": "any",
        "references": {}
      }
    }, {
      "method": "flowInputAction",
      "name": "flowInputAction",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": "Emitted when trailing action icon is clicked."
      },
      "complexType": {
        "original": "any",
        "resolved": "any",
        "references": {}
      }
    }]; }
  static get elementRef() { return "host"; }
  static get watchers() { return [{
      "propName": "disabled",
      "methodName": "updateDisabled"
    }]; }
}
