/*!
 * (C) Emu Analytics
 */
import { Component, Host, h, Element, Prop, Watch } from '@stencil/core';
import { createColorClasses } from '../../../utils/theme';
/**
 * @since 1.0
 * @status stable
 *
 * @slot - The content of the header.
 */
export class FlowListHeader {
  constructor() {
    /**
     * Set to `true` to display an expand/collapse button and make the following `flow-list`
     * collapsible.
     */
    this.collapsible = false;
    /**
     * Set to open or close the following `flow-list`. Only applicable if `collapsible` is `true`.
     */
    this.open = true;
  }
  async updateOpen() {
    const listEl = this.getList();
    if (!listEl) {
      return;
    }
    if (this.open) {
      listEl.style.height = `${listEl.scrollHeight}px`;
    }
    else {
      listEl.style.height = `${listEl.scrollHeight}px`;
      requestAnimationFrame(() => {
        // tslint:disable-next-line: no-unused-expression
        listEl.clientWidth; // force a reflow
        listEl.style.overflow = 'hidden';
        listEl.style.height = '0';
      });
    }
  }
  componentWillLoad() { }
  componentDidLoad() {
    if (this.collapsible) {
      const listEl = this.getList();
      if (listEl) {
        this.boundTransitionHandler = this.transitionHandler.bind(this);
        listEl.addEventListener('transitionend', this.boundTransitionHandler);
        listEl.style.height = this.open ? 'auto' : '0';
        listEl.style.overflow = this.open ? 'visible' : 'hidden';
      }
    }
  }
  disconnectedCallback() {
    if (this.collapsible) {
      const listEl = this.getList();
      if (listEl) {
        listEl.removeEventListener('transitionend', this.boundTransitionHandler);
      }
    }
  }
  transitionHandler(event) {
    const target = event.target;
    // Ensure we only emit one event when the target element is no longer visible
    if (target.nodeName === 'FLOW-LIST' && event.propertyName === 'height') {
      target.style.height = this.open ? null : '0';
      target.style.overflow = this.open ? null : 'hidden';
    }
  }
  getList() {
    const nextEl = this.el.nextElementSibling;
    return (nextEl === null || nextEl === void 0 ? void 0 : nextEl.nodeName) === 'FLOW-LIST' ? nextEl : null;
  }
  render() {
    const { color, open, icon, collapsible } = this;
    const classes = Object.assign({ ['mdc-list-group__subheader']: true, ['mdc-list-group__subheader--open']: open, ['mdc-list-group__subheader--closed']: !open }, createColorClasses(color));
    const iconClasses = {
      'list-header-icon': true
    };
    const buttonClasses = {
      'list-header-button ': true
    };
    const iconDom = h("flow-icon", { class: iconClasses, icon: icon });
    const buttonDom = (h("flow-button", { compact: true, "icon-button": true, class: buttonClasses, icon: "fas-chevron-right" }));
    return (h(Host, { class: classes, onClick: () => this.open = this.collapsible ? !this.open : true },
      icon ? iconDom : undefined,
      h("slot", null),
      collapsible ? buttonDom : undefined));
  }
  static get is() { return "flow-list-header"; }
  static get originalStyleUrls() { return {
    "$": ["list-header.scss"]
  }; }
  static get styleUrls() { return {
    "$": ["list-header.css"]
  }; }
  static get properties() { return {
    "color": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "PredefinedColor",
        "resolved": "\"danger\" | \"gray\" | \"primary\" | \"secondary\" | \"success\" | \"warning\"",
        "references": {
          "PredefinedColor": {
            "location": "import",
            "path": "../../../utils/theme"
          }
        }
      },
      "required": false,
      "optional": true,
      "docs": {
        "tags": [],
        "text": "The color of list header text and optional icon. Colors are taken from the application palette."
      },
      "attribute": "color",
      "reflect": false
    },
    "icon": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Set to display an icon before the title of the header. The property can be the name of\na built-in icon, a URL to an SVG file, or an SVG data URI."
      },
      "attribute": "icon",
      "reflect": false
    },
    "collapsible": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Set to `true` to display an expand/collapse button and make the following `flow-list`\ncollapsible."
      },
      "attribute": "collapsible",
      "reflect": false,
      "defaultValue": "false"
    },
    "open": {
      "type": "boolean",
      "mutable": true,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Set to open or close the following `flow-list`. Only applicable if `collapsible` is `true`."
      },
      "attribute": "open",
      "reflect": true,
      "defaultValue": "true"
    }
  }; }
  static get elementRef() { return "el"; }
  static get watchers() { return [{
      "propName": "open",
      "methodName": "updateOpen"
    }]; }
}
