/*!
 * (C) Emu Analytics
 */
import { Component, Element, Event, Host, h, Prop, Method, Watch } from '@stencil/core';
import { MDCList } from '@material/list';
import { ensureArray } from '../../../utils/ensure-array';
import { globalContextRegistry } from '../../../core/context/context-registry';
import { randomId } from '../../../utils/random-id';
import { debugLifecycle, formatEl } from '../../../utils/debug';
/**
 * @since 1.0
 * @status stable
 * @reactive
 */
export class FlowList {
  constructor() {
    this.initialized = false;
    /**
     * Set to `true` to enable 'single selection' mode. The list will have a
     * single selected item displayed in a distinct style.
     */
    this.singleSelection = false;
    /**
     * Set to `true` to support two-line list items.
     */
    this.twoLine = false;
    /**
     * The Flo.w context ID to use for application state.
     */
    this.flowContext = 'default';
    this.actionHandler = async (ev) => {
      const actionIndex = ev.detail.index;
      const actionItem = this.list.listElements[actionIndex];
      const selectedItems = ensureArray(this.list.selectedIndex).map((i) => this.list.listElements[i]);
      // Update context state based on selection
      if (this.subscription) {
        const context = await globalContextRegistry.getContext(this.flowContext);
        const selectedValues = selectedItems.map((item) => item.value);
        if (this.isCheckboxList) {
          // Checkbox-type list uses arrays for indexes
          let stateValues = context.state.get(this.selectionState) || [];
          // Remove all values for this list
          const listValues = this.list.listElements.map((el) => el.value);
          stateValues = stateValues.filter((val) => listValues.indexOf(val) === -1);
          // Add selected values for this list
          stateValues = stateValues.concat(selectedValues);
          context.state.set(this.selectionState, stateValues);
        }
        else {
          context.state.set(this.selectionState, selectedValues[0]);
        }
      }
      // Fire flow:list-action event
      this.listAction.emit({
        originalEvent: ev,
        actionIndex,
        actionItem,
        selectedItems
      });
    };
  }
  connectedCallback() {
    if (this.initialized) {
      // Handle re-connect to DOM
      this.connect();
    }
  }
  async componentWillLoad() {
    if (!this.el.id) {
      this.el.setAttribute('id', randomId('list'));
    }
    debugLifecycle(`${formatEl(this.el)} will load`);
  }
  async componentDidLoad() {
    this.mutationObserver = new MutationObserver((mutations) => {
      // Allow list to re-initialize when child items are changed
      setTimeout(() => {
        this.list.layout();
        this.list.initializeListType();
        if (this.selectionState) {
          this.subscribe();
        }
      }, 100);
    });
    this.initialized = true;
    this.connect();
  }
  disconnectedCallback() {
    this.disconnect();
  }
  connect() {
    this.list = MDCList.attachTo(this.el);
    this.list.singleSelection = this.singleSelection;
    this.list.listen('MDCList:action', this.actionHandler);
    if (this.selectionState) {
      this.subscribe();
    }
    this.mutationObserver.observe(this.el, { childList: true });
  }
  disconnect() {
    if (this.mutationObserver) {
      this.mutationObserver.disconnect();
      this.mutationObserver = undefined;
    }
    this.unsubscribe();
    if (this.list) {
      this.list.unlisten('MDCList:action', this.actionHandler);
      this.list.destroy();
      this.list = null;
    }
  }
  /**
   * Set the selected list item by index.
   *
   * @param index
   */
  async setSelectedIndex(index) {
    this.list.selectedIndex = index;
    const selectedItems = ensureArray(this.list.selectedIndex).map((i) => this.list.listElements[i]);
    return selectedItems;
  }
  /**
   * Set the selected list item by value.
   *
   * @param val
   */
  async setSelected(val) {
    val = ensureArray(val);
    const elements = this.list.listElements;
    const indexes = elements
      .map((item, index) => (val.indexOf(item.value) !== -1 ? index : -1))
      .filter((index) => index >= 0);
    if (this.isCheckboxList) {
      // Checkbox-type list uses arrays for indexes
      this.list.selectedIndex = indexes;
    }
    else {
      this.list.selectedIndex = indexes[0];
    }
    const selectedItems = ensureArray(this.list.selectedIndex).map((i) => this.list.listElements[i]);
    return selectedItems;
  }
  async subscribe() {
    this.unsubscribe();
    const context = await globalContextRegistry.getContext(this.flowContext);
    this.subscription = context.state.toStream(this.selectionState, true).subscribe((value) => {
      this.setSelected(value);
    });
  }
  unsubscribe() {
    if (this.subscription) {
      this.subscription.unsubscribe();
    }
  }
  get isCheckboxList() {
    // Checkbox lists use arrays for selectedIndex
    return Array.isArray(this.list.selectedIndex);
  }
  render() {
    const nativeClasses = {
      'mdc-list': true,
      'mdc-list--two-line': this.twoLine
    };
    return (h(Host, { class: nativeClasses },
      h("slot", null)));
  }
  static get is() { return "flow-list"; }
  static get originalStyleUrls() { return {
    "$": ["list.scss"]
  }; }
  static get styleUrls() { return {
    "$": ["list.css"]
  }; }
  static get properties() { return {
    "singleSelection": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Set to `true` to enable 'single selection' mode. The list will have a\nsingle selected item displayed in a distinct style."
      },
      "attribute": "single-selection",
      "reflect": false,
      "defaultValue": "false"
    },
    "twoLine": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Set to `true` to support two-line list items."
      },
      "attribute": "two-line",
      "reflect": false,
      "defaultValue": "false"
    },
    "selectionState": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [{
            "name": "reactive",
            "text": undefined
          }],
        "text": "Set to bind a Flo.w application state property to this list. Radio or check-box list\nchanges will be reflected in state property changes and vice versa."
      },
      "attribute": "selection-state",
      "reflect": false
    },
    "flowContext": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "The Flo.w context ID to use for application state."
      },
      "attribute": "flow-context",
      "reflect": false,
      "defaultValue": "'default'"
    }
  }; }
  static get events() { return [{
      "method": "listAction",
      "name": "flowListAction",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": "Emitted when a list action occurs."
      },
      "complexType": {
        "original": "FlowListActionEventDetail",
        "resolved": "FlowListActionEventDetail",
        "references": {
          "FlowListActionEventDetail": {
            "location": "local"
          }
        }
      }
    }]; }
  static get methods() { return {
    "setSelectedIndex": {
      "complexType": {
        "signature": "(index: number | number[]) => Promise<HTMLFlowListItemElement[]>",
        "parameters": [{
            "tags": [{
                "name": "param",
                "text": "index"
              }],
            "text": ""
          }],
        "references": {
          "Promise": {
            "location": "global"
          },
          "HTMLFlowListItemElement": {
            "location": "global"
          }
        },
        "return": "Promise<HTMLFlowListItemElement[]>"
      },
      "docs": {
        "text": "Set the selected list item by index.",
        "tags": [{
            "name": "param",
            "text": "index"
          }]
      }
    },
    "setSelected": {
      "complexType": {
        "signature": "(val: any | any[]) => Promise<HTMLFlowListItemElement[]>",
        "parameters": [{
            "tags": [{
                "name": "param",
                "text": "val"
              }],
            "text": ""
          }],
        "references": {
          "Promise": {
            "location": "global"
          },
          "HTMLFlowListItemElement": {
            "location": "global"
          }
        },
        "return": "Promise<HTMLFlowListItemElement[]>"
      },
      "docs": {
        "text": "Set the selected list item by value.",
        "tags": [{
            "name": "param",
            "text": "val"
          }]
      }
    }
  }; }
  static get elementRef() { return "el"; }
  static get watchers() { return [{
      "propName": "selectionState",
      "methodName": "subscribe"
    }]; }
}
