/*!
 * (C) Emu Analytics
 */
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
  var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
  if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
  else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
  return c > 3 && r && Object.defineProperty(target, key, r), r;
};
import { Component, Element, h, Host, Prop, Method, State, Watch } from '@stencil/core';
import { Event } from '@stencil/core';
import { FlowCoreMap } from '../../../core/flow-core-map/flow-core-map';
import { capitalize } from '../../../utils/capitalize';
import { randomId } from '../../../utils/random-id';
import { formatEl, debugLifecycle, debug } from '../../../utils/debug';
import { FLOW_MAP_LAYER_TAG, FLOW_MAP_DATA_SOURCE_TAG, FLOW_MAP_POPUP_TAG } from '../constants';
import { globalContextRegistry } from '../../../core/context/context-registry';
import { AncestorElement } from '../../../utils/component.utils';
import { FLOW_DRAWER_CHANGED_EVENT } from '../../drawer/drawer-container/constants';
import { isStringFull, isBoolean, isString } from '../../../utils/checks.util';
import { action } from 'mobx';
/**
 * @since 1.0
 * @status stable
 *
 * @reactive
 *
 * @slot - Specify map data sources, layers and icons.
 * @slot header - The map header content.
 * @slot footer - The map footer content.
 * @slot top-left - The top-left control content.
 * @slot top-right - The top-right control content.
 * @slot bottom-left - The bottom-left control content.
 * @slot bottom-right - The bottom-right control content.
 *
 * @slot popup - Default map popup content.
 * @slot <popup_name> - Content for additional map popups.
 */
export class FlowMap {
  constructor() {
    this.initialized = false;
    // private context: FlowCoreContext;
    this.mapLoading = false;
    this.isFeatureHovered = false;
    this.drawerClasses = {};
    this.showBusyIndicator = false;
    /**
     * Set to `true` to activate the map busy indicator.
     */
    this.busyIndicator = false;
    /**
     * The Flo.w context ID to use for application state.
     */
    this.flowContext = 'default';
    /**
     * Set to true to sync the map position, zoom, bearing and pitch with the browser URL.
     */
    this.sync = false;
    /**
     * Position the map canvas using fixed positioning to take up the entire window.
     * UI elements that should appear on top of the map (sidebars, drawers, etc.) must
     * have z-index set to 1 or higher to appear above the rendered map.
     */
    this.fixedPosition = false;
    /**
     * Set to true to automatically resize the map when its container is resized.
     */
    this.trackResize = true;
    /**
     * Specify radius in pixels of area around clicked point to search for map features.
     */
    this.featureClickTolerance = 0;
  }
  updateMapStyle(styleUrlOrId) {
    var _a;
    (_a = this._map) === null || _a === void 0 ? void 0 : _a.updateBaseStyle(styleUrlOrId);
  }
  async connectedCallback() {
    debugLifecycle(`${formatEl(this.el)} connectedCallback`);
    if (this.initialized) {
      this.connect();
    }
  }
  async componentWillLoad() {
    if (!this.el.id) {
      this.el.setAttribute('id', randomId('map'));
    }
    debugLifecycle(`${formatEl(this.el)} will load`);
    // this.context = await globalContextRegistry.getContext<any>(this.flowContext);
    this._map = new FlowCoreMap(this.el.id, this.flowContext);
    this._map.on('mousemove', this.mouseMoveHandler.bind(this));
    this._map.on('maploading', this.mapLoadingHandler.bind(this));
    this.forwardMapEvents();
  }
  async componentDidLoad() {
    debugLifecycle(`${formatEl(this.el)} did load`);
    const mapOptions = {
      container: this.mapContainerEl,
      featureClickTolerance: this.featureClickTolerance,
      trackResize: this.trackResize
    };
    if (this.center) {
      const center = this.center.split(',').map((val) => +val);
      if (center.length !== 3) {
        const context = await globalContextRegistry.getContext(this.flowContext);
        context.emitError(new Error(`[center] must be 'lng,lat,zoom'`));
      }
      mapOptions.center = [center[0], center[1]];
      mapOptions.zoom = center[2];
    }
    await this._map.initialize(this.mapStyle, this.sync, mapOptions);
    // Sanity check - component may have been destroyed while map was initializing
    if (this._map) {
      this.setMapDataSources();
      this.setMapLayers();
      this.setMapPopups();
      this._map.maplibreMap.resize();
      // Add mutation observer to set sources, layers and popups when child elements are added or removed
      this.mutationObserver = new MutationObserver(() => {
        debug(`${formatEl(this.el)} running mutation observer in 100ms`);
        setTimeout(this.updateMapComponents.bind(this), 100);
      });
      this.initialized = true;
      this.connect();
    }
  }
  async disconnectedCallback() {
    debugLifecycle(`${formatEl(this.el)} did unload`);
    this.disconnect();
    if (this._map) {
      this._map.uninitialize();
      this._map = undefined;
    }
  }
  connect() {
    this.mutationObserver.observe(this.el, { childList: true });
    // Bind to drawer container events if present
    // This is a workaround for lack of :host-context on Safari and Firefox
    // Drawer classes are effectively copied from the drawer container to the map so that
    // CSS styling of the map overlays can use the :host selector
    if (this.flowDrawerContainer) {
      this.boundDrawerChangedHandler = () => (this.drawerClasses = this.flowDrawerContainer._drawerClasses);
      this.flowDrawerContainer.addEventListener(FLOW_DRAWER_CHANGED_EVENT, this.boundDrawerChangedHandler);
      this.boundDrawerChangedHandler();
    }
    this._map.connect();
  }
  disconnect() {
    if (this.mutationObserver) {
      this.mutationObserver.disconnect();
      this.mutationObserver = undefined;
    }
    if (this.flowDrawerContainer && this.boundDrawerChangedHandler) {
      this.flowDrawerContainer.removeEventListener(FLOW_DRAWER_CHANGED_EVENT, this.boundDrawerChangedHandler);
    }
  }
  updateBusyIndicator() {
    this.showBusyIndicator = this.busyIndicator || this.mapLoading;
  }
  /**
   * Returns the underlying JavaScript map object.
   */
  async getMap() {
    return this._map;
  }
  /**
   * Zoom the map by the specified increment. Use a positive increment to zoom in
   * and a negative increment to zoom out.
   * @param increment +/- zoom level
   */
  async zoomBy(increment) {
    this._map.zoomBy(increment);
  }
  /**
   * Export the map to PNG format.
   *
   * @param options
   */
  async export(options) {
    return this._map.export(options);
  }
  /**
   * Set feature state on a single map feature.
   *
   * @param feature
   * @param state
   */
  async setFeatureState(feature, state) {
    return this._map.setFeatureState(feature, state);
  }
  /**
   * Remove feature state from a single map feature.
   * @param feature
   * @param key
   */
  async removeFeatureState(feature, key) {
    return this._map.removeFeatureState(feature, key);
  }
  /**
   * Open a map popup.
   *
   * @param lngLat
   * @param options
   */
  async openPopup(lngLat, options) {
    // const popup = new FlowCoreMapPopup(this._map, options);
    // popup.open(lngLat);
    return this._map.openPopup(lngLat, options);
  }
  async closeAllPopups() {
    this._map.closeAllPopups();
  }
  forwardMapEvents() {
    FlowMap.FORWARDED_MAP_EVENTS.forEach((e) => {
      const deprecatedEvent = 'map' + capitalize(e);
      const newEvent = 'flowMap' + capitalize(e);
      if (this[deprecatedEvent]) {
        this._map.on(e, (details) => {
          this[deprecatedEvent].emit(details);
        });
      }
      if (this[newEvent]) {
        this._map.on(e, (details) => {
          this[newEvent].emit(details);
        });
      }
    });
  }
  updateMapComponents() {
    debug(`${formatEl(this.el)} running mutation observer`);
    this.setMapDataSources();
    this.setMapLayers();
    this.setMapPopups();
  }
  setMapLayers() {
    const layerElements = this.el.getElementsByTagName(FLOW_MAP_LAYER_TAG);
    const layers = Array.prototype.map.call(layerElements, (element) => element._layer);
    this._map.setLayers(layers);
  }
  setMapDataSources() {
    const dataSourceElements = this.el.getElementsByTagName(FLOW_MAP_DATA_SOURCE_TAG);
    const dataSources = Array.prototype.map.call(dataSourceElements, (element) => element._datasource);
    this._map.setDataSources(dataSources);
  }
  setMapPopups() {
    const popupElements = this.el.getElementsByTagName(FLOW_MAP_POPUP_TAG);
    const popups = Array.prototype.map.call(popupElements, (element) => ({
      id: element.id,
      closeButton: element.closeButton,
      closeOnClick: element.closeOnClick,
      closeOnMove: element.closeOnMove,
      maxWidth: element.maxWidth,
      class: element.popupClass,
      trackLocation: this.convertTrackLocation(element.trackLocation),
      fireTrackImmediately: element.fireTrackImmediately,
      content: element.innerHTML
    }));
    this._map.registerPopupOptions(popups);
  }
  mouseMoveHandler(_event, features) {
    this.isFeatureHovered = features.length > 0;
  }
  mapLoadingHandler(busy) {
    this.mapLoading = busy;
    this.updateBusyIndicator();
  }
  convertTrackLocation(trackLocation) {
    if (isBoolean(trackLocation)) {
      return trackLocation;
    }
    else if (isStringFull(trackLocation)) {
      switch (trackLocation.toLowerCase()) {
        case 'true':
          return true;
        case 'false':
          return false;
        default:
          return trackLocation;
      }
    }
    else if (isString(trackLocation)) {
      // empty string indicates bare attribute
      return true;
    }
    return false;
  }
  render() {
    const progressBarClasses = {
      'map__progress-bar': true,
      'map__progress-bar--top': true,
      active: this.showBusyIndicator
    };
    const hostClasses = Object.assign({ 'flow-map': true, 'flow-map--fixed': this.fixedPosition }, this.drawerClasses);
    return (h(Host, { class: hostClasses },
      h("div", { class: { 'feature-hovered': this.isFeatureHovered }, ref: (el) => (this.mapContainerEl = el) }),
      h("div", { class: "map__overlay" },
        h("div", { class: "map__header" },
          h("slot", { name: "header" })),
        h("flow-progress-bar", { class: progressBarClasses, indeterminate: true }),
        h("div", { class: "map__control-containers" },
          h("div", { class: "map__control-container map__control-container--top-left" },
            h("slot", { name: "top-left" })),
          h("div", { class: "map__control-container map__control-container--top-right" },
            h("slot", { name: "top-right" })),
          h("div", { class: "map__control-container map__control-container--bottom-left" },
            h("slot", { name: "bottom-left" })),
          h("div", { class: "map__control-container map__control-container--bottom-right" },
            h("slot", { name: "bottom-right" }))),
        h("div", { class: "map__footer" },
          h("slot", { name: "footer" })))));
  }
  static get is() { return "flow-map"; }
  static get originalStyleUrls() { return {
    "$": ["map.scss"]
  }; }
  static get styleUrls() { return {
    "$": ["map.css"]
  }; }
  static get properties() { return {
    "_map": {
      "type": "unknown",
      "mutable": true,
      "complexType": {
        "original": "FlowCoreMap",
        "resolved": "FlowCoreMap",
        "references": {
          "FlowCoreMap": {
            "location": "import",
            "path": "../../../core/flow-core-map/flow-core-map"
          }
        }
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [{
            "name": "internal",
            "text": undefined
          }],
        "text": ""
      }
    },
    "busyIndicator": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Set to `true` to activate the map busy indicator."
      },
      "attribute": "busy-indicator",
      "reflect": false,
      "defaultValue": "false"
    },
    "center": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "The initial position and zoom for the map (format: `\"lng,lat,zoom\"`). If unset\nthe default position and zoom from the map style will be used."
      },
      "attribute": "center",
      "reflect": false
    },
    "flowContext": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "The Flo.w context ID to use for application state."
      },
      "attribute": "flow-context",
      "reflect": false,
      "defaultValue": "'default'"
    },
    "mapStyle": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "The base map style ID."
      },
      "attribute": "map-style",
      "reflect": true
    },
    "sync": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Set to true to sync the map position, zoom, bearing and pitch with the browser URL."
      },
      "attribute": "sync",
      "reflect": false,
      "defaultValue": "false"
    },
    "fixedPosition": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Position the map canvas using fixed positioning to take up the entire window.\nUI elements that should appear on top of the map (sidebars, drawers, etc.) must\nhave z-index set to 1 or higher to appear above the rendered map."
      },
      "attribute": "fixed-position",
      "reflect": false,
      "defaultValue": "false"
    },
    "trackResize": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Set to true to automatically resize the map when its container is resized."
      },
      "attribute": "track-resize",
      "reflect": false,
      "defaultValue": "true"
    },
    "featureClickTolerance": {
      "type": "number",
      "mutable": false,
      "complexType": {
        "original": "number",
        "resolved": "number",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Specify radius in pixels of area around clicked point to search for map features."
      },
      "attribute": "feature-click-tolerance",
      "reflect": false,
      "defaultValue": "0"
    }
  }; }
  static get states() { return {
    "isFeatureHovered": {},
    "drawerClasses": {},
    "showBusyIndicator": {}
  }; }
  static get events() { return [{
      "method": "flowMapLoad",
      "name": "flowMapLoad",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": "Emitted when the initial map style has loaded."
      },
      "complexType": {
        "original": "any",
        "resolved": "any",
        "references": {}
      }
    }, {
      "method": "mapLoad",
      "name": "mapLoad",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [{
            "name": "deprecated",
            "text": "Use flowMapLoad."
          }],
        "text": "Emitted when the initial map style has loaded."
      },
      "complexType": {
        "original": "any",
        "resolved": "any",
        "references": {}
      }
    }, {
      "method": "flowMapClick",
      "name": "flowMapClick",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": "Emitted when the user clicks on the map."
      },
      "complexType": {
        "original": "any",
        "resolved": "any",
        "references": {}
      }
    }, {
      "method": "flowMapDblClick",
      "name": "flowMapDblClick",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": "Emitted when the user double-clicks on the map."
      },
      "complexType": {
        "original": "any",
        "resolved": "any",
        "references": {}
      }
    }, {
      "method": "flowMapSelectionStateChanged",
      "name": "flowMapSelectionStateChanged",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": "Emitted when the selection state of a data source changes."
      },
      "complexType": {
        "original": "any",
        "resolved": "any",
        "references": {}
      }
    }, {
      "method": "flowMapSelectionFeatureStateChanged",
      "name": "flowMapSelectionFeatureStateChanged",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": "Emitted when the selection feature state of a data source changes."
      },
      "complexType": {
        "original": "any",
        "resolved": "any",
        "references": {}
      }
    }, {
      "method": "mapClick",
      "name": "mapClick",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [{
            "name": "deprecated",
            "text": "Use flowMapClick."
          }],
        "text": "Emitted when the user clicks on the map."
      },
      "complexType": {
        "original": "any",
        "resolved": "any",
        "references": {}
      }
    }, {
      "method": "mapDblClick",
      "name": "mapDblClick",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [{
            "name": "deprecated",
            "text": "Use flowMapDblClick."
          }],
        "text": "Emitted when the user double-clicks on the map."
      },
      "complexType": {
        "original": "any",
        "resolved": "any",
        "references": {}
      }
    }, {
      "method": "flowMapMoveEnd",
      "name": "flowMapMoveEnd",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": "Emitted when the map has finished zooming or panning."
      },
      "complexType": {
        "original": "any",
        "resolved": "any",
        "references": {}
      }
    }, {
      "method": "mapMoveEnd",
      "name": "mapMoveEnd",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [{
            "name": "deprecated",
            "text": "Use flowMapMoveEnd."
          }],
        "text": "Emitted when the map has finished zooming or panning."
      },
      "complexType": {
        "original": "any",
        "resolved": "any",
        "references": {}
      }
    }]; }
  static get methods() { return {
    "getMap": {
      "complexType": {
        "signature": "() => Promise<FlowCoreMap>",
        "parameters": [],
        "references": {
          "Promise": {
            "location": "global"
          },
          "FlowCoreMap": {
            "location": "import",
            "path": "../../../core/flow-core-map/flow-core-map"
          }
        },
        "return": "Promise<FlowCoreMap>"
      },
      "docs": {
        "text": "Returns the underlying JavaScript map object.",
        "tags": []
      }
    },
    "zoomBy": {
      "complexType": {
        "signature": "(increment: number) => Promise<void>",
        "parameters": [{
            "tags": [{
                "name": "param",
                "text": "increment +/- zoom level"
              }],
            "text": "+/- zoom level"
          }],
        "references": {
          "Promise": {
            "location": "global"
          }
        },
        "return": "Promise<void>"
      },
      "docs": {
        "text": "Zoom the map by the specified increment. Use a positive increment to zoom in\nand a negative increment to zoom out.",
        "tags": [{
            "name": "param",
            "text": "increment +/- zoom level"
          }]
      }
    },
    "export": {
      "complexType": {
        "signature": "(options: FlowMapExportOptions) => Promise<string>",
        "parameters": [{
            "tags": [{
                "name": "param",
                "text": "options"
              }],
            "text": ""
          }],
        "references": {
          "Promise": {
            "location": "global"
          },
          "FlowMapExportOptions": {
            "location": "import",
            "path": "../../../core/flow-core-map/map-exporter"
          }
        },
        "return": "Promise<string>"
      },
      "docs": {
        "text": "Export the map to PNG format.",
        "tags": [{
            "name": "param",
            "text": "options"
          }]
      }
    },
    "setFeatureState": {
      "complexType": {
        "signature": "(feature: FlowMapFeatureIdentifier | FlowMapGeoJSONFeature, state: { [key: string]: any; }) => Promise<void>",
        "parameters": [{
            "tags": [{
                "name": "param",
                "text": "feature"
              }],
            "text": ""
          }, {
            "tags": [{
                "name": "param",
                "text": "state"
              }],
            "text": ""
          }],
        "references": {
          "Promise": {
            "location": "global"
          },
          "FlowMapFeatureIdentifier": {
            "location": "import",
            "path": "../../../core/flow-core-map/flow-core-map"
          },
          "FlowMapGeoJSONFeature": {
            "location": "import",
            "path": "../../../core/flow-core-map/flow-core-map"
          }
        },
        "return": "Promise<void>"
      },
      "docs": {
        "text": "Set feature state on a single map feature.",
        "tags": [{
            "name": "param",
            "text": "feature"
          }, {
            "name": "param",
            "text": "state"
          }]
      }
    },
    "removeFeatureState": {
      "complexType": {
        "signature": "(feature: maplibregl.FeatureIdentifier | FlowMapGeoJSONFeature, key?: string) => Promise<void>",
        "parameters": [{
            "tags": [{
                "name": "param",
                "text": "feature"
              }],
            "text": ""
          }, {
            "tags": [{
                "name": "param",
                "text": "key"
              }],
            "text": ""
          }],
        "references": {
          "Promise": {
            "location": "global"
          },
          "maplibregl": {
            "location": "global"
          },
          "FlowMapGeoJSONFeature": {
            "location": "import",
            "path": "../../../core/flow-core-map/flow-core-map"
          }
        },
        "return": "Promise<void>"
      },
      "docs": {
        "text": "Remove feature state from a single map feature.",
        "tags": [{
            "name": "param",
            "text": "feature"
          }, {
            "name": "param",
            "text": "key"
          }]
      }
    },
    "openPopup": {
      "complexType": {
        "signature": "(lngLat: FlowMapLngLatLike, options?: FlowCoreMapPopupOptions) => Promise<FlowCoreMapPopup>",
        "parameters": [{
            "tags": [{
                "name": "param",
                "text": "lngLat"
              }],
            "text": ""
          }, {
            "tags": [{
                "name": "param",
                "text": "options"
              }],
            "text": ""
          }],
        "references": {
          "Promise": {
            "location": "global"
          },
          "FlowCoreMapPopup": {
            "location": "import",
            "path": "../../../core/flow-core-map/flow-core-map-popup"
          },
          "FlowMapLngLatLike": {
            "location": "import",
            "path": "../../../core/flow-core-map/flow-core-map"
          },
          "FlowCoreMapPopupOptions": {
            "location": "import",
            "path": "../../../core/flow-core-map/flow-core-map-popup"
          }
        },
        "return": "Promise<FlowCoreMapPopup>"
      },
      "docs": {
        "text": "Open a map popup.",
        "tags": [{
            "name": "param",
            "text": "lngLat"
          }, {
            "name": "param",
            "text": "options"
          }]
      }
    },
    "closeAllPopups": {
      "complexType": {
        "signature": "() => Promise<void>",
        "parameters": [],
        "references": {
          "Promise": {
            "location": "global"
          }
        },
        "return": "Promise<void>"
      },
      "docs": {
        "text": "",
        "tags": []
      }
    }
  }; }
  static get elementRef() { return "el"; }
  static get watchers() { return [{
      "propName": "mapStyle",
      "methodName": "updateMapStyle"
    }, {
      "propName": "busyIndicator",
      "methodName": "updateBusyIndicator"
    }]; }
}
FlowMap.FORWARDED_MAP_EVENTS = [
  'load',
  'click',
  'dblClick',
  'moveEnd',
  'selectionStateChanged',
  'selectionFeatureStateChanged'
];
__decorate([
  AncestorElement({ optional: true, supportAttr: false })
], FlowMap.prototype, "flowDrawerContainer", void 0);
__decorate([
  action
], FlowMap.prototype, "updateMapComponents", null);
