/*!
 * (C) Emu Analytics
 */
import { Component, Host, h, Prop, Watch, State } from '@stencil/core';
import { createColorClasses } from '../../utils/theme';
import { MDCCircularProgress } from '@material/circular-progress';
import { clamp } from '../../utils/clamp';
/**
 * @since 1.1
 * @status stable
 */
export class ProgressRing {
  constructor() {
    /**
     * Set to `true` to show an indeterminate circular progress indicator.
     */
    this.indeterminate = false;
    /**
     * Set to `true` to automatically show the progress indicator when progress is between 0 and 1.
     */
    this.autoShow = false;
    /**
     * Specify the size of the circular progress indicator.
     */
    this.size = 'medium';
    /**
     * Set to a value between 0 and 1 to indicate current progress.
     */
    this.value = 0;
  }
  updateSize() {
    switch (this.size) {
      case 'small':
        this.width = 24;
        this.radius = 8.75;
        this.strokeWidth = 2.5;
        break;
      case 'large':
        this.width = 48;
        this.radius = 18;
        this.strokeWidth = 4;
        break;
      case 'x-large':
        this.width = 60;
        this.radius = 26;
        this.strokeWidth = 5;
        break;
      case 'xx-large':
        this.width = 72;
        this.radius = 33;
        this.strokeWidth = 6;
        break;
      case 'medium':
      default:
        this.width = 36;
        this.radius = 12.5;
        this.strokeWidth = 3;
    }
    this.circum = this.radius * 2 * Math.PI;
  }
  updateProgress() {
    this.mdcProgress.progress = clamp(this.value, 0, 1);
  }
  componentWillLoad() {
    this.updateSize();
  }
  componentDidLoad() {
    this.mdcProgress = new MDCCircularProgress(this.mdcProgressEl);
    this.updateProgress();
  }
  renderDeterminateContainer() {
    const { width, radius, circum, strokeWidth } = this;
    return (h("div", { class: "mdc-circular-progress__determinate-container" },
      h("svg", { class: "mdc-circular-progress__determinate-circle-graphic", viewBox: `0 0 ${width} ${width}`, xmlns: "http://www.w3.org/2000/svg" },
        h("circle", { class: "mdc-circular-progress__determinate-track", cx: width / 2, cy: width / 2, r: radius, "stroke-width": strokeWidth }),
        h("circle", { class: "mdc-circular-progress__determinate-circle", cx: width / 2, cy: width / 2, r: radius, "stroke-dasharray": circum, "stroke-dashoffset": circum, "stroke-width": strokeWidth }))));
  }
  renderIndeterminateContainer() {
    const { width, radius, circum, strokeWidth } = this;
    return (h("div", { class: "mdc-circular-progress__indeterminate-container" },
      h("div", { class: "mdc-circular-progress__spinner-layer" },
        h("div", { class: "mdc-circular-progress__circle-clipper mdc-circular-progress__circle-left" },
          h("svg", { class: "mdc-circular-progress__indeterminate-circle-graphic", viewBox: `0 0 ${width} ${width}`, xmlns: "http://www.w3.org/2000/svg" },
            h("circle", { cx: width / 2, cy: width / 2, r: radius, "stroke-dasharray": circum, "stroke-dashoffset": circum / 2, "stroke-width": strokeWidth }))),
        h("div", { class: "mdc-circular-progress__gap-patch" },
          h("svg", { class: "mdc-circular-progress__indeterminate-circle-graphic", viewBox: `0 0 ${width} ${width}`, xmlns: "http://www.w3.org/2000/svg" },
            h("circle", { cx: width / 2, cy: width / 2, r: radius, "stroke-dasharray": circum, "stroke-dashoffset": circum / 2, "stroke-width": strokeWidth * 0.8 }))),
        h("div", { class: "mdc-circular-progress__circle-clipper mdc-circular-progress__circle-right" },
          h("svg", { class: "mdc-circular-progress__indeterminate-circle-graphic", viewBox: `0 0 ${width} ${width}`, xmlns: "http://www.w3.org/2000/svg" },
            h("circle", { cx: width / 2, cy: width / 2, r: radius, "stroke-dasharray": circum, "stroke-dashoffset": circum / 2, "stroke-width": strokeWidth }))))));
  }
  render() {
    const { value, width, indeterminate, autoShow, color } = this;
    const hostClasses = Object.assign({}, createColorClasses(color));
    const baseClasses = {
      'mdc-circular-progress': true,
      'mdc-circular-progress--indeterminate': indeterminate,
      'mdc-circular-progress--closed': autoShow && value >= 1
    };
    return (h(Host, { class: hostClasses },
      h("div", { ref: el => (this.mdcProgressEl = el), class: baseClasses, style: { width: `${width}px`, height: `${width}px` }, role: "progressbar", "aria-label": "Progress Bar", "aria-valuemin": "0", "aria-valuemax": "1" },
        this.renderIndeterminateContainer(),
        this.renderDeterminateContainer())));
  }
  static get is() { return "flow-progress-ring"; }
  static get encapsulation() { return "shadow"; }
  static get originalStyleUrls() { return {
    "$": ["progress-ring.scss"]
  }; }
  static get styleUrls() { return {
    "$": ["progress-ring.css"]
  }; }
  static get properties() { return {
    "color": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "PredefinedColor",
        "resolved": "\"danger\" | \"gray\" | \"primary\" | \"secondary\" | \"success\" | \"warning\"",
        "references": {
          "PredefinedColor": {
            "location": "import",
            "path": "../../utils/theme"
          }
        }
      },
      "required": false,
      "optional": true,
      "docs": {
        "tags": [],
        "text": "The color of the progress indicator. Colors are taken from the application palette."
      },
      "attribute": "color",
      "reflect": false
    },
    "indeterminate": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Set to `true` to show an indeterminate circular progress indicator."
      },
      "attribute": "indeterminate",
      "reflect": false,
      "defaultValue": "false"
    },
    "autoShow": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Set to `true` to automatically show the progress indicator when progress is between 0 and 1."
      },
      "attribute": "auto-show",
      "reflect": false,
      "defaultValue": "false"
    },
    "size": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "'small' | 'medium' | 'large' | 'x-large' | 'xx-large'",
        "resolved": "\"large\" | \"medium\" | \"small\" | \"x-large\" | \"xx-large\"",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Specify the size of the circular progress indicator."
      },
      "attribute": "size",
      "reflect": false,
      "defaultValue": "'medium'"
    },
    "value": {
      "type": "number",
      "mutable": false,
      "complexType": {
        "original": "number",
        "resolved": "number",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Set to a value between 0 and 1 to indicate current progress."
      },
      "attribute": "value",
      "reflect": false,
      "defaultValue": "0"
    }
  }; }
  static get states() { return {
    "width": {},
    "radius": {},
    "circum": {},
    "strokeWidth": {}
  }; }
  static get watchers() { return [{
      "propName": "size",
      "methodName": "updateSize"
    }, {
      "propName": "value",
      "methodName": "updateProgress"
    }]; }
}
