/*!
 * (C) Emu Analytics
 */
import { Component, Host, Element, Event, Prop, h, Watch, Listen } from '@stencil/core';
import { globalContextRegistry } from '../../core/context/context-registry';
let groupId = 0;
/**
 * @since 1.1
 * @status stable
 * @reactive
 *
 * @slot - The tag's content.
 *
 * @part base - The component's base wrapper.
 * @part content - The tag content.
 * @part clear-button - The clear button.
 */
export class RadioGroup {
  constructor() {
    this.initialized = false;
    this.groupName = `radio-group-${groupId++}`;
    /**
     * Set to `true` to disable interaction with the radio button group.
     */
    this.disabled = false;
    /**
     * The Flo.w context ID to use for application state.
     */
    this.flowContext = 'default';
  }
  async updateValue(value, oldValue) {
    // Select radio button matching value
    this.getAllRadios().forEach(radio => (radio.checked = radio.value === value));
    // Update state if subscribed
    if (this.selectionState) {
      const context = await globalContextRegistry.getContext(this.flowContext);
      context.state.set(this.selectionState, value);
    }
  }
  updateDisabled(disabled) {
    this.getAllRadios().forEach(radio => (radio.disabled = disabled));
  }
  connectedCallback() {
    if (this.initialized) {
      // Handle re-connect to DOM
      this.connect();
    }
  }
  componentDidLoad() {
    this.initialized = true;
    this.connect();
  }
  disconnectedCallback() {
    this.disconnect();
  }
  connect() {
    this.getAllRadios().forEach(radio => (radio.name = this.groupName));
    this.updateDisabled(this.disabled);
    this.subscribe();
  }
  disconnect() {
    this.unsubscribe();
  }
  /**
   * Update group value when radio is clicked
   */
  onFlowChange(ev) {
    const target = ev.target;
    if (target.tagName === 'FLOW-RADIO' && target.value !== this.value) {
      this.value = target.value;
      this.flowRadioGroupChange.emit();
    }
  }
  async subscribe() {
    this.unsubscribe();
    if (this.selectionState) {
      const context = await globalContextRegistry.getContext(this.flowContext);
      this.subscription = context.state.toStream(this.selectionState, true).subscribe(value => {
        this.value = value;
      });
    }
  }
  unsubscribe() {
    if (this.subscription) {
      this.subscription.unsubscribe();
      this.subscription = undefined;
    }
  }
  /**
   * Return all child radio elements
   */
  getAllRadios() {
    const radios = this.host.querySelectorAll('flow-radio');
    return Array.from(radios);
  }
  render() {
    const hostClasses = {
      'flow-radio-group': true
    };
    return (h(Host, { class: hostClasses },
      h("slot", null)));
  }
  static get is() { return "flow-radio-group"; }
  static get encapsulation() { return "shadow"; }
  static get originalStyleUrls() { return {
    "$": ["radio-group.scss"]
  }; }
  static get styleUrls() { return {
    "$": ["radio-group.css"]
  }; }
  static get properties() { return {
    "value": {
      "type": "string",
      "mutable": true,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "The value attribute of the radio button group."
      },
      "attribute": "value",
      "reflect": true
    },
    "disabled": {
      "type": "boolean",
      "mutable": true,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Set to `true` to disable interaction with the radio button group."
      },
      "attribute": "disabled",
      "reflect": true,
      "defaultValue": "false"
    },
    "selectionState": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [{
            "name": "reactive",
            "text": undefined
          }],
        "text": "Set to bind a Flo.w application state property to this list. The state property\nvalue reflects the value of the selected radio button."
      },
      "attribute": "selection-state",
      "reflect": false
    },
    "flowContext": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "The Flo.w context ID to use for application state."
      },
      "attribute": "flow-context",
      "reflect": false,
      "defaultValue": "'default'"
    }
  }; }
  static get events() { return [{
      "method": "flowRadioGroupChange",
      "name": "flowRadioGroupChange",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": "Emitted when the control's checked state changes."
      },
      "complexType": {
        "original": "any",
        "resolved": "any",
        "references": {}
      }
    }]; }
  static get elementRef() { return "host"; }
  static get watchers() { return [{
      "propName": "value",
      "methodName": "updateValue"
    }, {
      "propName": "disabled",
      "methodName": "updateDisabled"
    }, {
      "propName": "selectionState",
      "methodName": "subscribe"
    }]; }
  static get listeners() { return [{
      "name": "flowChange",
      "method": "onFlowChange",
      "target": undefined,
      "capture": false,
      "passive": false
    }]; }
}
