/*!
 * (C) Emu Analytics
 */
import { Component, Host, Element, Event, Prop, h, Watch, Method } from '@stencil/core';
import { createColorClasses } from '../../utils/theme';
import { MDCFormField } from '@material/form-field';
import { MDCRadio } from '@material/radio';
let id = 0;
/**
 * @since 1.1
 * @status stable
 * @reactive
 *
 * @slot - Child radio buttons to place in the radio button group.
 */
export class Radio {
  constructor() {
    this.initialized = false;
    this.inputId = `radio-${++id}`;
    /**
     * The radio button position relative to its label.
     */
    this.align = 'start';
    /** The checked state of the radio button  */
    this.checked = false;
    /** Set to `true` to disable interaction with the checkbox. */
    this.disabled = false;
    this.handleChange = this.handleChange.bind(this);
    this.handleFocus = this.handleFocus.bind(this);
    this.handleBlur = this.handleBlur.bind(this);
    this.handleKeyDown = this.handleKeyDown.bind(this);
  }
  updateChecked(val) {
    if (this.mdcRadio) {
      this.mdcRadio.checked = val;
    }
    if (val) {
      this.getSiblingRadios().map((radio) => (radio.checked = false));
    }
  }
  updateDisabled(val) {
    if (this.mdcRadio) {
      this.mdcRadio.disabled = val;
    }
  }
  connectedCallback() {
    if (this.initialized) {
      // Handle re-connect to DOM
      this.connect();
    }
  }
  componentDidLoad() {
    this.initialized = true;
    this.connect();
  }
  disconnectedCallback() {
    this.disconnect();
  }
  connect() {
    this.mdcRadio = new MDCRadio(this.mdcRadioEl);
    this.mdcFormField = new MDCFormField(this.mdcFormFieldEl);
    this.mdcFormField.input = this.mdcRadio;
    this.updateDisabled(this.disabled);
  }
  disconnect() {
    if (this.mdcRadio) {
      this.mdcRadio.destroy();
      this.mdcRadio = undefined;
    }
    if (this.mdcFormField) {
      this.mdcFormField.destroy();
      this.mdcFormField = undefined;
    }
  }
  /** Set focus on the checkbox. */
  async setFocus() {
    this.inputEl.focus();
  }
  /** Remove focus from the checkbox. */
  async removeFocus() {
    this.inputEl.blur();
  }
  handleChange() {
    this.checked = this.mdcRadio.checked;
    if (this.checked) {
      this.getSiblingRadios().map((radio) => (radio.checked = false));
    }
    this.flowChange.emit();
  }
  handleFocus(e) {
    this.flowFocus.emit();
  }
  handleBlur(e) {
    this.flowBlur.emit();
  }
  handleKeyDown(e) {
    if (e.key === ' ' || e.key === 'Enter') {
      // Prevent focus state getting stuck
      e.preventDefault();
      this.checked = !this.checked;
      this.flowChange.emit();
    }
    if (['ArrowUp', 'ArrowDown', 'ArrowLeft', 'ArrowRight'].includes(e.key)) {
      const radios = this.getAllRadios().filter((radio) => !radio.disabled);
      const incr = ['ArrowUp', 'ArrowLeft'].includes(e.key) ? -1 : 1;
      let index = radios.indexOf(this.host) + incr;
      if (index < 0) {
        index = radios.length - 1;
      }
      if (index > radios.length - 1) {
        index = 0;
      }
      radios[index].setFocus();
      e.preventDefault();
    }
  }
  handleLabelMouseDown(e) {
    // Prevent clicks on the label from briefly blurring the input
    e.preventDefault();
  }
  getAllRadios() {
    const container = this.host.closest('flow-radio-group, form') || document.body;
    if (!this.name) {
      return [];
    }
    return Array.from(container.querySelectorAll('flow-radio')).filter((radio) => radio.name === this.name);
  }
  getSiblingRadios() {
    return this.getAllRadios().filter((radio) => radio !== this.host);
  }
  render() {
    const { label, name, disabled, align, inputId, value, checked } = this;
    const hostClasses = Object.assign({ 'flow-radio': true }, createColorClasses(this.color));
    const formFieldClasses = {
      'mdc-form-field': true,
      'mdc-form-field--align-end': align === 'end'
    };
    return (h(Host, { class: hostClasses },
      h("div", { class: formFieldClasses, ref: (el) => (this.mdcFormFieldEl = el) },
        h("div", { class: "mdc-radio", ref: (el) => (this.mdcRadioEl = el) },
          h("input", { class: "mdc-radio__native-control", type: "radio", id: inputId, name: name, value: value, disabled: disabled, ref: (el) => (this.inputEl = el), checked: checked, onChange: this.handleChange, onFocus: this.handleFocus, onBlur: this.handleBlur, onKeyDown: this.handleKeyDown }),
          h("div", { class: "mdc-radio__background" },
            h("div", { class: "mdc-radio__outer-circle" }),
            h("div", { class: "mdc-radio__inner-circle" })),
          h("div", { class: "mdc-radio__ripple" })),
        h("label", { htmlFor: inputId, part: "label", onMouseDown: this.handleLabelMouseDown }, label ? label : h("slot", null)))));
  }
  static get is() { return "flow-radio"; }
  static get encapsulation() { return "shadow"; }
  static get originalStyleUrls() { return {
    "$": ["radio.scss"]
  }; }
  static get styleUrls() { return {
    "$": ["radio.css"]
  }; }
  static get properties() { return {
    "color": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "PredefinedColor",
        "resolved": "\"danger\" | \"gray\" | \"primary\" | \"secondary\" | \"success\" | \"warning\"",
        "references": {
          "PredefinedColor": {
            "location": "import",
            "path": "../../utils/theme"
          }
        }
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "The color of the radio button. Colors are taken from the application palette."
      },
      "attribute": "color",
      "reflect": false
    },
    "align": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "'start' | 'end'",
        "resolved": "\"end\" | \"start\"",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "The radio button position relative to its label."
      },
      "attribute": "align",
      "reflect": false,
      "defaultValue": "'start'"
    },
    "label": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "If set, specifies the radio button label text. Otherwise, the\ntext content of the element is used as the label text."
      },
      "attribute": "label",
      "reflect": false
    },
    "value": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "The value attribute of the radio button."
      },
      "attribute": "value",
      "reflect": false
    },
    "name": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "The name attribute of the radio button."
      },
      "attribute": "name",
      "reflect": true
    },
    "checked": {
      "type": "boolean",
      "mutable": true,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "The checked state of the radio button"
      },
      "attribute": "checked",
      "reflect": true,
      "defaultValue": "false"
    },
    "disabled": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Set to `true` to disable interaction with the checkbox."
      },
      "attribute": "disabled",
      "reflect": true,
      "defaultValue": "false"
    }
  }; }
  static get events() { return [{
      "method": "flowBlur",
      "name": "flowBlur",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": "Emitted when the control loses focus."
      },
      "complexType": {
        "original": "any",
        "resolved": "any",
        "references": {}
      }
    }, {
      "method": "flowChange",
      "name": "flowChange",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": "Emitted when the control's checked state changes."
      },
      "complexType": {
        "original": "any",
        "resolved": "any",
        "references": {}
      }
    }, {
      "method": "flowFocus",
      "name": "flowFocus",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": "Emitted when the control gains focus."
      },
      "complexType": {
        "original": "any",
        "resolved": "any",
        "references": {}
      }
    }]; }
  static get methods() { return {
    "setFocus": {
      "complexType": {
        "signature": "() => Promise<void>",
        "parameters": [],
        "references": {
          "Promise": {
            "location": "global"
          }
        },
        "return": "Promise<void>"
      },
      "docs": {
        "text": "Set focus on the checkbox.",
        "tags": []
      }
    },
    "removeFocus": {
      "complexType": {
        "signature": "() => Promise<void>",
        "parameters": [],
        "references": {
          "Promise": {
            "location": "global"
          }
        },
        "return": "Promise<void>"
      },
      "docs": {
        "text": "Remove focus from the checkbox.",
        "tags": []
      }
    }
  }; }
  static get elementRef() { return "host"; }
  static get watchers() { return [{
      "propName": "checked",
      "methodName": "updateChecked"
    }, {
      "propName": "disabled",
      "methodName": "updateDisabled"
    }]; }
}
