/*!
 * (C) Emu Analytics
 */
import { Component, Element, Event, Method, Prop, State, Watch, h } from '@stencil/core';
import { getOffset } from '../../../utils/offset';
import { scrollIntoView } from '../../../utils/scroll';
import { focusVisible } from '../../../utils/focus-visible';
/**
 * @since 1.1
 * @status stable
 *
 * @slot nav - Place <flow-tab> elements in this slot.
 * @slot - Place <flow-tab-panel> elements in this slot.
 *
 * @part nav - The tab group navigation container.
 * @part tabs - The container that wraps the slotted tabs.
 * @part active-tab-indicator - An element that displays the currently selected tab. This is a child of the tabs container.
 * @part body - The tab group body where tab panels are displayed.
 */
export class TabGroup {
  constructor() {
    this.initialized = false;
    this.canScrollHorizontally = false;
    /** The placement of the tab navigation buttons. */
    this.placement = 'top';
    this.handleClick = this.handleClick.bind(this);
    this.handleKeyDown = this.handleKeyDown.bind(this);
    this.handleScrollLeft = this.handleScrollLeft.bind(this);
    this.handleScrollRight = this.handleScrollRight.bind(this);
  }
  handlePlacementChange() {
    this.syncActiveTabIndicator();
  }
  connectedCallback() {
    if (this.initialized) {
      // Handle re-connect to DOM
      this.connect();
    }
  }
  componentDidLoad() {
    // Set initial tab state when the tabs first become visible
    const observer = new IntersectionObserver((entries, observer) => {
      if (entries[0].intersectionRatio > 0) {
        this.setAriaLabels();
        this.setActiveTab(this.getActiveTab() || this.getAllTabs()[0], false);
        observer.unobserve(entries[0].target);
      }
    });
    observer.observe(this.host);
    this.resizeObserver = new ResizeObserver(() => this.syncHorizontalScroll());
    // Update aria labels if the DOM changes
    this.mutationObserver = new MutationObserver((mutations) => {
      if (mutations.some((mutation) => {
        return !['aria-labelledby', 'aria-controls'].includes(mutation.attributeName);
      })) {
        setTimeout(() => this.setAriaLabels());
      }
    });
    this.initialized = true;
    this.connect();
    requestAnimationFrame(() => this.syncHorizontalScroll());
  }
  disconnectedCallback() {
    this.disconnect();
  }
  connect() {
    this.mutationObserver.observe(this.host, { attributes: true, childList: true, subtree: true });
    this.resizeObserver.observe(this.nav);
    focusVisible.observe(this.tabGroup);
  }
  disconnect() {
    if (this.mutationObserver) {
      this.mutationObserver.disconnect();
      this.mutationObserver = undefined;
    }
    if (this.resizeObserver) {
      this.resizeObserver.unobserve(this.nav);
      this.resizeObserver = undefined;
    }
    focusVisible.unobserve(this.tabGroup);
  }
  /** Show the specified tab panel. */
  async show(panel) {
    const tabs = this.getAllTabs();
    const tab = tabs.find((el) => el.panel === panel);
    if (tab) {
      this.setActiveTab(tab);
    }
  }
  getAllTabs(includeDisabled = false) {
    const slot = this.tabs.querySelector('slot');
    return [...slot.assignedElements()].filter((el) => {
      return includeDisabled
        ? el.tagName.toLowerCase() === 'flow-tab'
        : el.tagName.toLowerCase() === 'flow-tab' && !el.disabled;
    });
  }
  getAllPanels() {
    const slot = this.body.querySelector('slot');
    return [...slot.assignedElements()].filter((el) => el.tagName.toLowerCase() === 'flow-tab-panel');
  }
  getActiveTab() {
    return this.getAllTabs().find((el) => el.active);
  }
  handleClick(event) {
    const target = event.target;
    const tab = target.closest('flow-tab');
    if (tab) {
      this.setActiveTab(tab);
    }
  }
  handleKeyDown(event) {
    // Activate a tab
    if (['Enter', ' '].includes(event.key)) {
      const target = event.target;
      const tab = target.closest('flow-tab');
      if (tab) {
        this.setActiveTab(tab);
        event.preventDefault();
      }
    }
    // Move focus left or right
    if (['ArrowLeft', 'ArrowRight', 'ArrowUp', 'ArrowDown', 'Home', 'End'].includes(event.key)) {
      const activeEl = document.activeElement;
      if (activeEl && activeEl.tagName.toLowerCase() === 'flow-tab') {
        const tabs = this.getAllTabs();
        let index = tabs.indexOf(activeEl);
        if (event.key === 'Home') {
          index = 0;
        }
        else if (event.key === 'End') {
          index = tabs.length - 1;
        }
        else if (event.key === 'ArrowLeft') {
          index = Math.max(0, index - 1);
        }
        else if (event.key === 'ArrowRight') {
          index = Math.min(tabs.length - 1, index + 1);
        }
        tabs[index].setFocus();
        if (['top', 'bottom'].includes(this.placement)) {
          scrollIntoView(tabs[index], this.nav, 'horizontal');
        }
        event.preventDefault();
      }
    }
  }
  handleScrollLeft() {
    this.nav.scroll({
      left: this.nav.scrollLeft - this.nav.clientWidth,
      behavior: 'smooth'
    });
  }
  handleScrollRight() {
    this.nav.scroll({
      left: this.nav.scrollLeft + this.nav.clientWidth,
      behavior: 'smooth'
    });
  }
  setActiveTab(tab, emitEvents = true) {
    if (tab && tab !== this.activeTab && !tab.disabled) {
      const previousTab = this.activeTab;
      this.activeTab = tab;
      // Sync tabs and panels
      this.getAllTabs().map((el) => (el.active = el === this.activeTab));
      this.getAllPanels().map((el) => (el.active = el.name === this.activeTab.panel));
      this.syncActiveTabIndicator();
      if (['top', 'bottom'].includes(this.placement)) {
        scrollIntoView(this.activeTab, this.nav, 'horizontal');
      }
      // Emit events
      if (emitEvents) {
        if (previousTab) {
          this.flowTabHide.emit({ name: previousTab.panel });
        }
        this.flowTabShow.emit({ name: this.activeTab.panel });
      }
    }
  }
  setAriaLabels() {
    const tabs = this.getAllTabs();
    const panels = this.getAllPanels();
    // Link each tab with its corresponding panel
    tabs.map((tab) => {
      const panel = panels.find((el) => el.name === tab.panel);
      if (panel) {
        tab.setAttribute('aria-controls', panel.getAttribute('id'));
        panel.setAttribute('aria-labelledby', tab.getAttribute('id'));
      }
    });
  }
  syncActiveTabIndicator() {
    const tab = this.getActiveTab();
    const width = tab.clientWidth;
    const height = tab.clientHeight;
    const offset = getOffset(tab, this.nav);
    const offsetTop = offset.top + this.nav.scrollTop;
    const offsetLeft = offset.left + this.nav.scrollLeft;
    switch (this.placement) {
      case 'top':
      case 'bottom':
        this.activeTabIndicator.style.width = `${width}px`;
        this.activeTabIndicator.style.height = null;
        this.activeTabIndicator.style.transform = `translateX(${offsetLeft}px)`;
        break;
      case 'left':
      case 'right':
        this.activeTabIndicator.style.width = null;
        this.activeTabIndicator.style.height = `${height}px`;
        this.activeTabIndicator.style.transform = `translateY(${offsetTop}px)`;
        break;
    }
  }
  syncHorizontalScroll() {
    this.canScrollHorizontally =
      ['top', 'bottom'].includes(this.placement) && this.nav.scrollWidth > this.nav.clientWidth;
  }
  render() {
    return (h("div", { ref: (el) => (this.tabGroup = el), class: {
        'tab-group': true,
        // Placements
        'tab-group--top': this.placement === 'top',
        'tab-group--bottom': this.placement === 'bottom',
        'tab-group--left': this.placement === 'left',
        'tab-group--right': this.placement === 'right',
        'tab-group--horizontal-scroll': this.canScrollHorizontally
      }, onClick: this.handleClick, onKeyDown: this.handleKeyDown },
      h("div", { class: "tab-group__nav-container" },
        this.canScrollHorizontally && (h("flow-button", { class: "tab-group__scroll-button tab-group__scroll-button--left", "icon-button": true, compact: true, icon: "fas-chevron-left", onClick: this.handleScrollLeft })),
        h("div", { ref: (el) => (this.nav = el), key: "nav", part: "nav", class: "tab-group__nav", tabindex: "-1" },
          h("div", { ref: (el) => (this.tabs = el), part: "tabs", class: "tab-group__tabs", role: "tablist" },
            h("div", { ref: (el) => (this.activeTabIndicator = el), part: "active-tab-indicator", class: "tab-group__active-tab-indicator" }),
            h("slot", { name: "nav" }))),
        this.canScrollHorizontally && (h("flow-button", { class: "tab-group__scroll-button tab-group__scroll-button--right", "icon-button": true, compact: true, icon: "fas-chevron-right", onClick: this.handleScrollRight }))),
      h("div", { ref: (el) => (this.body = el), part: "body", class: "tab-group__body" },
        h("slot", null))));
  }
  static get is() { return "flow-tab-group"; }
  static get encapsulation() { return "shadow"; }
  static get originalStyleUrls() { return {
    "$": ["tab-group.scss"]
  }; }
  static get styleUrls() { return {
    "$": ["tab-group.css"]
  }; }
  static get properties() { return {
    "placement": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "'top' | 'bottom' | 'left' | 'right'",
        "resolved": "\"bottom\" | \"left\" | \"right\" | \"top\"",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "The placement of the tab navigation buttons."
      },
      "attribute": "placement",
      "reflect": false,
      "defaultValue": "'top'"
    }
  }; }
  static get states() { return {
    "canScrollHorizontally": {}
  }; }
  static get events() { return [{
      "method": "flowTabShow",
      "name": "flowTabShow",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": "Emitted when a tab is shown."
      },
      "complexType": {
        "original": "any",
        "resolved": "any",
        "references": {}
      }
    }, {
      "method": "flowTabHide",
      "name": "flowTabHide",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": "Emitted when a tab is hidden."
      },
      "complexType": {
        "original": "any",
        "resolved": "any",
        "references": {}
      }
    }]; }
  static get methods() { return {
    "show": {
      "complexType": {
        "signature": "(panel: string) => Promise<void>",
        "parameters": [{
            "tags": [],
            "text": ""
          }],
        "references": {
          "Promise": {
            "location": "global"
          }
        },
        "return": "Promise<void>"
      },
      "docs": {
        "text": "Show the specified tab panel.",
        "tags": []
      }
    }
  }; }
  static get elementRef() { return "host"; }
  static get watchers() { return [{
      "propName": "placement",
      "methodName": "handlePlacementChange"
    }]; }
}
