/*!
 * (C) Emu Analytics
 */
import { Component, Element, Event, Host, h, Prop, State, Watch } from '@stencil/core';
import { randomId } from '../../utils/random-id';
import { formatEl, debugLifecycle, debug } from '../../utils/debug';
import { globalContextRegistry } from '../../core/context/context-registry';
import debounce from 'lodash-es/debounce';
import moment from '../../utils/moment-round';
import { parse as parseDuration } from '../../utils/parse-duration';
const TIMER_INTERVAL = 100; // ms
const DEBOUNCE_INTERVAL = 300; // ms
/**
 * @since 1.0
 * @status stable
 * @reactive
 *
 * @part date-button: The date select button.
 * @part time-button: The time select button.
 * @part control-button: Applied to each of the time control buttons (back, play, forward etc.).
 */
export class FlowTimer {
  constructor() {
    this.initialized = false;
    this.runOnReconnect = false;
    this.nextUpdate = moment(0);
    this.internalFlooredMoment = moment();
    this.currentMoment = moment();
    /**
     * Set to enable/disable the play button.
     */
    this.enablePlay = true;
    /**
     * The Flo.w context ID to use for application state.
     */
    this.flowContext = 'default';
    /**
     * The time interval for the major step buttons.
     */
    this.majorStep = '1 day';
    /**
     * The time interval for the minor step buttons.
     */
    this.minorStep = '1 hour';
    /**
     * Set to start/stop the timer. Read to retrieve the current running state.
     */
    this.running = true;
    /**
     * Specifies the format of the Flo.w state property bound to the timer.
     */
    this.timerStateFormat = 'iso';
    /**
     * Set to delay scheduled updates when the timer is running. This can be used to allow time
     * for data to arrive in real-time use cases.
     */
    this.updateDelay = '0 seconds';
    /**
     * Specifies the frequency at which the timer emits updates when running.
     * All emitted values are also rounded down using this interval. This guarantees
     * that emitted values are always aligned to the interval specified.
     */
    this.updateInterval = '10 seconds';
    /**
     * Set to `true` to disable interaction with the timer.
     */
    this.disabled = false;
    /**
     * Set display mode to UTC (note that setting this to `true` overrides the `timezone` attribute)
     */
    this.utc = false;
    /**
     * Set to `true` to enable static positioning of popup calendars. By default
     * calendars are positioned absolutely using body coordinates.
     */
    this.static = false;
    /**
     * Set to `true` to enable fixed positioning of popup calendars. By default
     * calendars are positioned absolutely using body coordinates.
     */
    this.fixed = false;
    /**
     * The position of the popup date/time selectors relative to the timer control.
     */
    this.position = 'auto';
  }
  async connectedCallback() {
    if (this.initialized) {
      // Handle re-connect
      this.connect(this.runOnReconnect);
    }
  }
  async componentDidLoad() {
    if (!this.el.id) {
      this.el.setAttribute('id', randomId('timer'));
    }
    debugLifecycle(`${formatEl(this.el)} did load`);
    this.context = await globalContextRegistry.getContext(this.flowContext);
    this.debouncedEmitChange = debounce(this.emitChange.bind(this), DEBOUNCE_INTERVAL, {
      leading: true,
      trailing: true
    });
    this.updateUpdateDelay(this.updateDelay);
    this.updateUpdateInterval(this.updateInterval);
    this.updateMinorStep(this.minorStep);
    this.updateMajorStep(this.majorStep);
    this.initialized = true;
    this.connect(this.running);
    this.currentMoment = this.value ? moment(this.value) : moment();
    this.updateValue();
  }
  async disconnectedCallback() {
    debugLifecycle(`${formatEl(this.el)} did unload`);
    this.disconnect();
  }
  connect(shouldStart) {
    this.subscribe();
    if (shouldStart) {
      this.start();
    }
  }
  disconnect() {
    // Store the current running state before stopping so that it can be restored on reconnect
    this.runOnReconnect = this.running;
    this.stop();
    this.unsubscribe();
  }
  subscribe() {
    this.unsubscribe();
    if (this.timerState) {
      this.subscription = this.context.state.toStream(this.timerState, true).subscribe((value) => {
        if (value) {
          this.value = value;
        }
      });
    }
  }
  updateUpdateDelay(val) {
    this.parsedUpdateDelay = parseDuration(val);
    if (!this.parsedUpdateDelay.unit || typeof this.parsedUpdateDelay.duration !== 'number') {
      this.context.emitError(new Error(`Badly formed duration for update-delay'${val}'`));
    }
  }
  updateUpdateInterval(val) {
    this.parsedUpdateInterval = parseDuration(val);
    if (!this.parsedUpdateInterval.unit || !this.parsedUpdateInterval.duration) {
      this.context.emitError(new Error(`Badly formed duration for update-interval'${val}'`));
    }
  }
  updateMinorStep(val) {
    this.parsedMinorStep = parseDuration(val);
    if (!this.parsedMinorStep.unit || !this.parsedMinorStep.duration) {
      this.context.emitError(new Error(`Badly formed duration for minor-step'${val}'`));
    }
  }
  updateMajorStep(val) {
    this.parsedMajorStep = parseDuration(val);
    if (!this.parsedMajorStep.unit || !this.parsedMajorStep.duration) {
      this.context.emitError(new Error(`Badly formed duration for major-step'${val}'`));
    }
  }
  valueUpdated(val) {
    if (val !== this.internalFlooredMoment.toISOString()) {
      debug('External time value update', val);
      // value attribute has changed from outside - stop timer and update
      this.stop();
      this.currentMoment = moment(val);
      this.updateValue();
    }
  }
  updateRunning(running) {
    if (running && !this.timer) {
      this.start();
    }
    else if (!running && this.timer) {
      this.stop();
    }
  }
  async updateDisabled(disabled) {
    if (disabled) {
      this.stop();
    }
  }
  start() {
    if (this.timer) {
      return;
    }
    this.stop();
    this.nextUpdate = moment(0);
    this.timer = setInterval(this.update.bind(this), TIMER_INTERVAL);
    this.running = true;
    this.update();
  }
  stop() {
    if (!this.timer) {
      return;
    }
    if (this.timer) {
      clearInterval(this.timer);
      this.timer = null;
    }
    this.running = false;
  }
  update() {
    const now = moment();
    this.currentMoment = moment(now).millisecond(0);
    if (now
      .subtract(this.parsedUpdateDelay.duration, this.parsedUpdateDelay.unit)
      .isAfter(this.nextUpdate)) {
      this.nextUpdate = moment(now)
        .add(this.parsedUpdateInterval.duration, this.parsedUpdateInterval.unit)
        .floor(this.parsedUpdateInterval.duration, this.parsedUpdateInterval.unit);
      this.updateValue();
    }
  }
  onFastRewind() {
    this.stop();
    this.currentMoment = moment(this.currentMoment).subtract(this.parsedMajorStep.duration, this.parsedMajorStep.unit);
    this.updateValue();
  }
  onRewind() {
    this.stop();
    this.currentMoment = moment(this.currentMoment)
      .subtract(this.parsedMinorStep.duration, this.parsedMinorStep.unit)
      .ceil(this.parsedMinorStep.duration, this.parsedMinorStep.unit);
    this.updateValue();
  }
  onPlayPause() {
    if (!this.enablePlay) {
      return;
    }
    if (this.running) {
      this.stop();
      this.debouncedEmitChange();
    }
    else {
      this.start();
    }
  }
  onForward() {
    this.stop();
    this.currentMoment = moment(this.currentMoment)
      .add(this.parsedMinorStep.duration, this.parsedMinorStep.unit)
      .floor(this.parsedMinorStep.duration, this.parsedMinorStep.unit);
    this.updateValue();
  }
  onFastForward() {
    this.stop();
    this.currentMoment = moment(this.currentMoment).add(this.parsedMajorStep.duration, this.parsedMajorStep.unit);
    this.updateValue();
  }
  datePickerChanged(ev, picker) {
    this.stop();
    if (picker === 'time') {
      // Set new date and time
      this.currentMoment = moment(ev.detail.value);
    }
    else {
      // Set new date but retain current time
      const newMoment = moment(ev.detail.value);
      newMoment.hours(this.currentMoment.hours());
      newMoment.minutes(this.currentMoment.minutes());
      newMoment.seconds(this.currentMoment.seconds());
      this.currentMoment = newMoment;
    }
    this.updateValue();
  }
  unsubscribe() {
    if (this.subscription) {
      this.subscription.unsubscribe();
    }
  }
  updateValue(floor = true) {
    const m = floor
      ? moment(this.currentMoment)
        .subtract(this.parsedUpdateDelay.duration, this.parsedUpdateDelay.unit)
        .floor(this.parsedUpdateInterval.duration, this.parsedUpdateInterval.unit)
      : this.currentMoment;
    if (!m.isSame(this.internalFlooredMoment)) {
      this.internalFlooredMoment = m;
      this.value = m.toISOString();
      this.debouncedEmitChange();
    }
  }
  emitChange() {
    debug('Emitting timer change event', this.value);
    this.flowChange.emit({ value: this.value, millis: moment(this.value).valueOf(), running: this.running });
    if (this.context && this.timerState) {
      if (this.timerStateFormat === 'iso') {
        this.context.state.set(this.timerState, this.value);
      }
      else {
        this.context.state.set(this.timerState, moment(this.value).valueOf());
      }
    }
  }
  render() {
    const hostClasses = {};
    return (h(Host, { class: hostClasses },
      h("div", { class: "date-time-container" },
        h("div", { class: "date-container" },
          h("flow-button", { part: "date-button", compact: true, "icon-button": true, icon: "fas-calendar", onClick: (ev) => setTimeout(() => this.dayPicker.open(), 0) }),
          h("div", { class: "date-parts" },
            h("flow-date-time-picker", { utc: this.utc, timezone: this.timezone, static: this.static, fixed: this.fixed, position: this.position, ref: (el) => (this.dayPicker = el), class: "day-part", border: false, format: "ddd", minuteIncrement: 1, value: this.currentMoment.toISOString(), onFlowChange: (ev) => this.datePickerChanged(ev, 'date'), onFlowDateTimeOpened: (ev) => this.stop() }),
            h("flow-date-time-picker", { utc: this.utc, timezone: this.timezone, static: this.static, fixed: this.fixed, position: this.position, ref: (el) => (this.datePicker = el), class: "date-part", border: false, format: "DD/MM/YYYY", minuteIncrement: 1, value: this.currentMoment.toISOString(), onFlowChange: (ev) => this.datePickerChanged(ev, 'date'), onFlowDateTimeOpened: (ev) => this.stop() }))),
        h("div", { class: "spacer" }),
        h("div", { class: "time-container" },
          h("flow-button", { part: "time-button", compact: true, "icon-button": true, icon: "fas-clock", onClick: (ev) => setTimeout(() => this.timePicker.open(), 0) }),
          h("flow-date-time-picker", { utc: this.utc, timezone: this.timezone, static: this.static, fixed: this.fixed, position: this.position, ref: (el) => (this.timePicker = el), border: false, mode: "time", format: "HH:mm:ss", minuteIncrement: 1, enableSeconds: true, value: this.currentMoment.toISOString(), onFlowChange: (ev) => this.datePickerChanged(ev, 'time'), onFlowDateTimeOpened: (ev) => this.stop(), style: { width: '200px' } }))),
      h("div", { class: "vcr-button-container" },
        h("flow-button", { part: "control-button", compact: true, "icon-button": true, icon: "fas-backward-step", onClick: () => this.onFastRewind() }),
        h("flow-button", { part: "control-button", compact: true, "icon-button": true, icon: "fas-backward", onClick: () => this.onRewind() }),
        h("flow-button", { part: "control-button", compact: true, "icon-button": true, icon: this.timer ? 'fas-pause' : 'fas-play', onClick: () => this.onPlayPause(), disabled: !this.enablePlay }),
        h("flow-button", { part: "control-button", compact: true, "icon-button": true, icon: "fas-forward", onClick: () => this.onForward() }),
        h("flow-button", { part: "control-button", compact: true, "icon-button": true, icon: "fas-forward-step", onClick: () => this.onFastForward() }))));
  }
  static get is() { return "flow-timer"; }
  static get encapsulation() { return "shadow"; }
  static get originalStyleUrls() { return {
    "$": ["timer.scss"]
  }; }
  static get styleUrls() { return {
    "$": ["timer.css"]
  }; }
  static get properties() { return {
    "enablePlay": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Set to enable/disable the play button."
      },
      "attribute": "enable-play",
      "reflect": false,
      "defaultValue": "true"
    },
    "flowContext": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "The Flo.w context ID to use for application state."
      },
      "attribute": "flow-context",
      "reflect": false,
      "defaultValue": "'default'"
    },
    "majorStep": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "The time interval for the major step buttons."
      },
      "attribute": "major-step",
      "reflect": false,
      "defaultValue": "'1 day'"
    },
    "minorStep": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "The time interval for the minor step buttons."
      },
      "attribute": "minor-step",
      "reflect": false,
      "defaultValue": "'1 hour'"
    },
    "running": {
      "type": "boolean",
      "mutable": true,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Set to start/stop the timer. Read to retrieve the current running state."
      },
      "attribute": "running",
      "reflect": true,
      "defaultValue": "true"
    },
    "timerState": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": true,
      "docs": {
        "tags": [{
            "name": "reactive",
            "text": undefined
          }],
        "text": "The Flo.w state property to bind to the timer. The state property will\nbe synchronized with the value of the timer."
      },
      "attribute": "timer-state",
      "reflect": false
    },
    "timerStateFormat": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "'iso' | 'millis'",
        "resolved": "\"iso\" | \"millis\"",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Specifies the format of the Flo.w state property bound to the timer."
      },
      "attribute": "timer-state-format",
      "reflect": false,
      "defaultValue": "'iso'"
    },
    "updateDelay": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Set to delay scheduled updates when the timer is running. This can be used to allow time\nfor data to arrive in real-time use cases."
      },
      "attribute": "update-delay",
      "reflect": false,
      "defaultValue": "'0 seconds'"
    },
    "updateInterval": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Specifies the frequency at which the timer emits updates when running.\nAll emitted values are also rounded down using this interval. This guarantees\nthat emitted values are always aligned to the interval specified."
      },
      "attribute": "update-interval",
      "reflect": false,
      "defaultValue": "'10 seconds'"
    },
    "value": {
      "type": "string",
      "mutable": true,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": true,
      "docs": {
        "tags": [],
        "text": "Set to update the current date/time. Read to retrieve the current date/time."
      },
      "attribute": "value",
      "reflect": true
    },
    "disabled": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Set to `true` to disable interaction with the timer."
      },
      "attribute": "disabled",
      "reflect": true,
      "defaultValue": "false"
    },
    "utc": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Set display mode to UTC (note that setting this to `true` overrides the `timezone` attribute)"
      },
      "attribute": "utc",
      "reflect": false,
      "defaultValue": "false"
    },
    "timezone": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": true,
      "docs": {
        "tags": [],
        "text": "Set display timezone using TZ identifier (e.g. Europe/London, default is local time)"
      },
      "attribute": "timezone",
      "reflect": false
    },
    "static": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Set to `true` to enable static positioning of popup calendars. By default\ncalendars are positioned absolutely using body coordinates."
      },
      "attribute": "static",
      "reflect": false,
      "defaultValue": "false"
    },
    "fixed": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Set to `true` to enable fixed positioning of popup calendars. By default\ncalendars are positioned absolutely using body coordinates."
      },
      "attribute": "fixed",
      "reflect": false,
      "defaultValue": "false"
    },
    "position": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "'above' | 'below' | 'auto'",
        "resolved": "\"above\" | \"auto\" | \"below\"",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "The position of the popup date/time selectors relative to the timer control."
      },
      "attribute": "position",
      "reflect": false,
      "defaultValue": "'auto'"
    }
  }; }
  static get states() { return {
    "currentMoment": {}
  }; }
  static get events() { return [{
      "method": "flowChange",
      "name": "flowChange",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": "Emitted when the timer value has changed."
      },
      "complexType": {
        "original": "TimerChangeEventDetail",
        "resolved": "TimerChangeEventDetail",
        "references": {
          "TimerChangeEventDetail": {
            "location": "import",
            "path": "./interface"
          }
        }
      }
    }]; }
  static get elementRef() { return "el"; }
  static get watchers() { return [{
      "propName": "timerState",
      "methodName": "subscribe"
    }, {
      "propName": "updateDelay",
      "methodName": "updateUpdateDelay"
    }, {
      "propName": "updateInterval",
      "methodName": "updateUpdateInterval"
    }, {
      "propName": "minorStep",
      "methodName": "updateMinorStep"
    }, {
      "propName": "majorStep",
      "methodName": "updateMajorStep"
    }, {
      "propName": "value",
      "methodName": "valueUpdated"
    }, {
      "propName": "running",
      "methodName": "updateRunning"
    }, {
      "propName": "disabled",
      "methodName": "updateDisabled"
    }]; }
}
