/*!
 * (C) Emu Analytics
 */
import { Component, Element, Host, Method, Prop, Watch, h } from '@stencil/core';
import Popover from '../../utils/popover';
let id = 0;
/**
 * @since 2.0
 * @status stable
 *
 * @slot - The tooltip's target element. Only the first element will be used as the target. Alternatively
 * specify the `for` attribute to attach the tooltip to its target element.
 *
 * @part tooltip - The tooltip container DIV.
 */
export class Tooltip {
  constructor() {
    this.initialized = false;
    this.componentId = `tooltip-${++id}`;
    this.isShowing = false;
    /**
     * The tooltip text.
     */
    this.tipText = '';
    /**
     * The preferred placement of the tooltip. Note that the actual placement may vary as needed to keep the tooltip
     * inside of the viewport.
     */
    this.position = 'top';
    /**
     * Set to `true` to disable triggering of the tooltip.
     */
    this.disabled = false;
    /**
     * The distance in pixels from which to offset the tooltip away from its target.
     */
    this.offset = 10;
    /**
     * Set to `true` to force the tooltip to display.
     */
    this.open = false;
    /**
     * Controls how the tooltip is activated. Possible options include `click`, `hover`, `focus`, and `manual`. Multiple
     * options can be passed by separating them with a space. When manual is used, the tooltip must be activated
     * programmatically.
     */
    this.trigger = 'hover focus';
    this.handleBlur = this.handleBlur.bind(this);
    this.handleClick = this.handleClick.bind(this);
    this.handleFocus = this.handleFocus.bind(this);
    this.handleMouseOver = this.handleMouseOver.bind(this);
    this.handleMouseOut = this.handleMouseOut.bind(this);
    this.handleSlotChange = this.handleSlotChange.bind(this);
  }
  handleOpenChange() {
    this.open ? this.show() : this.hide();
  }
  connectedCallback() {
    if (this.initialized) {
      // Handle re-connect to DOM
      this.connect();
    }
  }
  async componentDidLoad() {
    this.target = this.getTarget();
    if (!this.target) {
      throw new Error(`No target #${this.for} for tooltip`);
    }
    this.initialized = true;
    this.connect();
  }
  componentDidUpdate() {
    this.syncOptions();
  }
  disconnectedCallback() {
    var _a, _b, _c, _d, _e, _f;
    (_a = this.popover) === null || _a === void 0 ? void 0 : _a.destroy();
    (_b = this.target) === null || _b === void 0 ? void 0 : _b.removeEventListener('mouseover', this.handleMouseOver, true);
    (_c = this.target) === null || _c === void 0 ? void 0 : _c.removeEventListener('mouseout', this.handleMouseOut, true);
    (_d = this.target) === null || _d === void 0 ? void 0 : _d.removeEventListener('blur', this.handleBlur, true);
    (_e = this.target) === null || _e === void 0 ? void 0 : _e.removeEventListener('click', this.handleClick, true);
    (_f = this.target) === null || _f === void 0 ? void 0 : _f.removeEventListener('focus', this.handleFocus, true);
  }
  connect() {
    this.popover = new Popover(this.target, this.tooltipPositioner);
    this.syncOptions();
    this.target.addEventListener('mouseover', this.handleMouseOver, true);
    this.target.addEventListener('mouseout', this.handleMouseOut, true);
    this.target.addEventListener('blur', this.handleBlur, true);
    this.target.addEventListener('click', this.handleClick, true);
    this.target.addEventListener('focus', this.handleFocus, true);
    this.syncOptions();
    // Show on init if open
    this.tooltipPositioner.hidden = !this.open;
    if (this.open) {
      this.show();
    }
  }
  disconnect() {
    var _a, _b, _c, _d, _e, _f;
    (_a = this.popover) === null || _a === void 0 ? void 0 : _a.destroy();
    (_b = this.target) === null || _b === void 0 ? void 0 : _b.removeEventListener('mouseover', this.handleMouseOver, true);
    (_c = this.target) === null || _c === void 0 ? void 0 : _c.removeEventListener('mouseout', this.handleMouseOut, true);
    (_d = this.target) === null || _d === void 0 ? void 0 : _d.removeEventListener('blur', this.handleBlur, true);
    (_e = this.target) === null || _e === void 0 ? void 0 : _e.removeEventListener('click', this.handleClick, true);
    (_f = this.target) === null || _f === void 0 ? void 0 : _f.removeEventListener('focus', this.handleFocus, true);
  }
  /** Show the tooltip. */
  async show() {
    // Prevent subsequent calls to the method, whether manually or triggered by the `open` watcher
    if (this.isShowing) {
      return;
    }
    this.isShowing = true;
    this.open = true;
    this.popover.show();
  }
  /** Hide the tooltip. */
  async hide() {
    // Prevent subsequent calls to the method, whether manually or triggered by the `open` watcher
    if (!this.isShowing) {
      return;
    }
    this.isShowing = false;
    this.open = false;
    this.popover.hide();
  }
  getTarget() {
    const target = this.for ? document.getElementById(this.for) : this.host.querySelector('*:not(style)');
    if (!target) {
      throw new Error('Invalid tooltip target: no child element was found.');
    }
    return target;
  }
  handleBlur() {
    if (this.hasTrigger('focus')) {
      this.hide();
    }
  }
  handleClick() {
    if (this.hasTrigger('click')) {
      this.open ? this.hide() : this.show();
    }
  }
  handleFocus() {
    if (this.hasTrigger('focus')) {
      this.show();
    }
  }
  handleMouseOver() {
    if (this.hasTrigger('hover')) {
      this.show();
    }
  }
  handleMouseOut() {
    if (this.hasTrigger('hover')) {
      this.hide();
    }
  }
  handleSlotChange() {
    const oldTarget = this.target;
    const newTarget = this.getTarget();
    if (newTarget !== oldTarget) {
      oldTarget === null || oldTarget === void 0 ? void 0 : oldTarget.removeAttribute('aria-describedby');
      newTarget.setAttribute('aria-describedby', this.componentId);
    }
  }
  hasTrigger(triggerType) {
    const triggers = this.trigger.split(' ');
    return triggers.includes(triggerType);
  }
  syncOptions() {
    this.popover.setOptions({
      placement: this.position,
      distance: this.offset,
      transitionElement: this.tooltip
    });
  }
  render() {
    return (h(Host, null,
      h("slot", { "aria-describedby": this.componentId, onSlotchange: this.handleSlotChange }),
      !this.disabled && (h("div", { ref: el => (this.tooltipPositioner = el), class: "tooltip-positioner" },
        h("div", { part: "tooltip", ref: el => (this.tooltip = el), id: this.componentId, class: {
            tooltip: true,
            'tooltip--open': this.open
          }, role: "tooltip", "aria-hidden": !this.open }, this.tipText)))));
  }
  static get is() { return "flow-tooltip"; }
  static get encapsulation() { return "shadow"; }
  static get originalStyleUrls() { return {
    "$": ["tooltip.scss"]
  }; }
  static get styleUrls() { return {
    "$": ["tooltip.css"]
  }; }
  static get properties() { return {
    "tipText": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "The tooltip text."
      },
      "attribute": "tip-text",
      "reflect": false,
      "defaultValue": "''"
    },
    "position": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "| 'top'\n    | 'top-start'\n    | 'top-end'\n    | 'right'\n    | 'right-start'\n    | 'right-end'\n    | 'bottom'\n    | 'bottom-start'\n    | 'bottom-end'\n    | 'left'\n    | 'left-start'\n    | 'left-end'",
        "resolved": "\"bottom\" | \"bottom-end\" | \"bottom-start\" | \"left\" | \"left-end\" | \"left-start\" | \"right\" | \"right-end\" | \"right-start\" | \"top\" | \"top-end\" | \"top-start\"",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "The preferred placement of the tooltip. Note that the actual placement may vary as needed to keep the tooltip\ninside of the viewport."
      },
      "attribute": "position",
      "reflect": false,
      "defaultValue": "'top'"
    },
    "disabled": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Set to `true` to disable triggering of the tooltip."
      },
      "attribute": "disabled",
      "reflect": false,
      "defaultValue": "false"
    },
    "offset": {
      "type": "number",
      "mutable": false,
      "complexType": {
        "original": "number",
        "resolved": "number",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "The distance in pixels from which to offset the tooltip away from its target."
      },
      "attribute": "offset",
      "reflect": false,
      "defaultValue": "10"
    },
    "open": {
      "type": "boolean",
      "mutable": true,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Set to `true` to force the tooltip to display."
      },
      "attribute": "open",
      "reflect": true,
      "defaultValue": "false"
    },
    "trigger": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Controls how the tooltip is activated. Possible options include `click`, `hover`, `focus`, and `manual`. Multiple\noptions can be passed by separating them with a space. When manual is used, the tooltip must be activated\nprogrammatically."
      },
      "attribute": "trigger",
      "reflect": false,
      "defaultValue": "'hover focus'"
    },
    "for": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Specify the target element ID of the tooltip. This is an alternative to wrapping the target element\nwith the tooltip."
      },
      "attribute": "for",
      "reflect": false
    }
  }; }
  static get methods() { return {
    "show": {
      "complexType": {
        "signature": "() => Promise<void>",
        "parameters": [],
        "references": {
          "Promise": {
            "location": "global"
          }
        },
        "return": "Promise<void>"
      },
      "docs": {
        "text": "Show the tooltip.",
        "tags": []
      }
    },
    "hide": {
      "complexType": {
        "signature": "() => Promise<void>",
        "parameters": [],
        "references": {
          "Promise": {
            "location": "global"
          }
        },
        "return": "Promise<void>"
      },
      "docs": {
        "text": "Hide the tooltip.",
        "tags": []
      }
    }
  }; }
  static get elementRef() { return "host"; }
  static get watchers() { return [{
      "propName": "open",
      "methodName": "handleOpenChange"
    }]; }
}
