/*!
 * (C) Emu Analytics
 */
import Debug from 'debug';
import { deferred } from '../../utils/deferred';
const debug = Debug('flow:chart-spec-registry');
/**
 * The `ChartSpecRegistry` is a container for [Vega/Vega-Lite](https://vega.github.io/)
 * chart specifications.
 *
 * Chart specifications can be registered and retrieved by ID.
 */
export class ChartSpecRegistry {
  /**
   * Constructs a new chart specification registry instance.
   */
  constructor() {
    this.specs = new Map();
  }
  /**
   * Register a chart specification.
   *
   * @param id Chart specification ID.
   * @param spec Chart specification.
   */
  registerSpec(id, spec) {
    debug(`Registering chart spec '${id}'`);
    if (!this.specs.get(id)) {
      this.specs.set(id, deferred());
    }
    const deferredSpec = this.specs.get(id);
    deferredSpec.resolve(spec);
  }
  /**
   * Unregister a chart specification.
   *
   * @param id Chart specification ID.
   */
  unregisterSpec(id) {
    debug(`Unregistering chart spec '${id}'`);
    this.specs.delete(id);
  }
  /**
   * Get a chart specification.
   *
   * This method returns a promise that will resolve when the specification has been registered.
   *
   * @param id Chart specification ID.
   */
  getSpec(id) {
    if (!this.specs.get(id)) {
      // Spec hasn't been registered yet. A promise is created that will
      // eventually resolve when the spec is registered.
      this.specs.set(id, deferred());
    }
    const deferredSpec = this.specs.get(id);
    return deferredSpec;
  }
}
