/*!
 * (C) Emu Analytics
 */
import { globalContextRegistry } from '../context/context-registry';
import { isArray, isNil } from '../../utils/checks.util';
/**
 * Core functionality for chart data. This class is wrapped by the `<flow-chart-data>` web component.
 */
export class FlowCoreChartData {
  /**
   * Constructs a new `FlowCoreChartData` instance.
   *
   * @param flowChart Parent Flo.w chart.
   */
  constructor(flowChart) {
    this.flowChart = flowChart;
  }
  /**
   * Subscribe to reactive data updates.
   *
   * This method is called internally by the `<flow-chart-data>` component.
   *
   * @param options Chart data options
   */
  async subscribe(options) {
    this.unsubscribe();
    if (isNil(options.data)) {
      // set data to empty array
      this.flowChart.setData(options.name, [], options.xProperty, options.yProperty);
    }
    else if (isArray(options.data)) {
      this.flowChart.setData(options.name, options.data, options.xProperty, options.yProperty);
    }
    else {
      const context = await globalContextRegistry.getContext(this.flowChart.contextId);
      const parsedDataExpression = context.parser.parseExpression(options.data);
      if (parsedDataExpression.status) {
        this.autorunDisposer = context.state.autorun(() => {
          try {
            const data = context.parser.evaluateExpression(parsedDataExpression.value);
            this.flowChart.setData(options.name, data, options.xProperty, options.yProperty);
          }
          catch (e) {
            context.emitError(e);
          }
        }, `autorun: update(FlowChartData.${this.flowChart.id}.${options.name})`);
      }
      else {
        context.emitError(new Error(`Malformed expression - ${parsedDataExpression}`));
      }
    }
  }
  /**
   * Unsubscribe from reactive data updates.
   *
   * This method is called internally by the `<flow-chart-data>` component.
   */
  unsubscribe() {
    if (this.autorunDisposer) {
      this.autorunDisposer();
    }
  }
}
