/*!
 * (C) Emu Analytics
 */
import EventEmitter from 'wolfy87-eventemitter';
import vegaEmbed from 'vega-embed';
import { changeset } from 'vega';
import Debug from 'debug';
const debug = Debug('flow:flow-core-chart');
/**
 * Core functionality for Flo.w charts. This class is wrapped by the `<flow-chart>` web component.
 *
 * Charts are defined using [Vega/Vega-Lite](https://vega.github.io/) chart specifications.
 *
 * @noInheritDoc
 */
export class FlowCoreChart extends EventEmitter {
  /**
   * Constructs a new `FlowCoreChart` instance.
   *
   * @param id Chart ID.
   * @param contextId Associated Flo.w context ID.
   */
  constructor(id, contextId) {
    super();
    this.id = id;
    this.contextId = contextId;
    this.pendingSignals = [];
    this.pendingData = {};
    this._initialized = false;
  }
  /**
   * Returns `true` if the chart has been initialized.
   */
  get initialized() {
    return this._initialized;
  }
  /**
   * Initialize the chart.
   *
   * This method is called internally by the `<flow-chart>` web component.
   *
   * @param options Chart options.
   */
  initialize(options) {
    this.options = options;
    const textColor = getComputedStyle(options.container).getPropertyValue('--flow-color-text') || '#000';
    const surfaceColor = getComputedStyle(options.container).getPropertyValue('--flow-color-surface') || '#fff';
    const gridColor = getComputedStyle(options.container).getPropertyValue('--flow-color-step-500') || '#ccc';
    const primaryColor = getComputedStyle(options.container).getPropertyValue('--flow-color-primary') || '#ccc';
    vegaEmbed(options.container, options.spec, {
      defaultStyle: '',
      actions: { export: { png: true, svg: true }, source: false, editor: false, compiled: false },
      scaleFactor: 3,
      tooltip: !!options.tooltip,
      width: options.width,
      height: options.height,
      config: {
        autosize: {
          type: 'fit',
          resize: false,
          contains: 'padding'
        },
        background: surfaceColor,
        bar: { continuousBandSize: 18 },
        padding: { left: 5, top: 5, right: 10, bottom: 5 },
        title: {
          color: textColor,
          anchor: 'start'
        },
        axis: {
          gridColor,
          domainColor: gridColor,
          tickColor: gridColor,
          labelColor: textColor,
          titleColor: textColor,
          titleOpacity: 0.6
        },
        legend: {
          titleColor: textColor,
          labelColor: textColor
        },
        view: { stroke: gridColor },
        mark: { color: primaryColor }
      }
    }).then(({ view, finalize }) => {
      this.vegaView = view;
      this.vegaFinalizeFn = finalize;
      this.initializeSignals();
      this.initializeData();
      this._initialized = true;
    });
  }
  /**
   * Uninitialize the chart.
   *
   * This method is called internally by the `<flow-chart>` component. If instantiated programmatically,
   * calling this method is the responsibility of the developer.
   */
  uninitialize() {
    this.cleanUpSignals();
    if (this.vegaFinalizeFn) {
      this.vegaFinalizeFn();
    }
  }
  /**
   * Set a Vega chart signal.
   *
   * @param signal Signal name.
   * @param value Signal value.
   * @param run Run chart update immediately.
   */
  async setSignal(signal, value, run = true) {
    if (!this.vegaView) {
      this.pendingSignals.push({ signal, value });
      return;
    }
    debug(`Setting signal '${signal}'`, value);
    this.vegaView.signal(signal, value);
    if (run) {
      return this.vegaView.runAsync().then();
    }
  }
  /**
   * Set chart data.
   *
   * This method is called internally by the `<flow-chart>` web component.
   *
   * @param name Data name.
   * @param vals Data values.
   * @param xProp Name of X property in data.
   * @param yProp Name of Y property in data.
   * @param run Run update immediately.
   */
  async setData(name, vals, xProp, yProp, run = true) {
    debug(`Setting chart data: ${name}`, vals, xProp, yProp);
    if (!this.vegaView) {
      this.pendingData[name] = vals;
      return;
    }
    const changes = changeset()
      .remove(() => true)
      .insert(vals);
    this.vegaView.change(name, changes);
    if (xProp) {
      this.setSignal(`${name}X`, xProp, false);
    }
    if (yProp) {
      this.setSignal(`${name}Y`, yProp, false);
    }
    if (run) {
      return this.vegaView
        .resize()
        .runAsync()
        .then();
    }
  }
  /**
   * Trigger a resize of the chart.
   */
  resize() {
    return this.vegaView.resize().runAsync();
  }
  initializeSignals() {
    this.boundSignalHandler = this.signalHandler.bind(this);
    const signals = this.vegaView.getState().signals;
    Object.keys(signals).forEach(key => {
      this.vegaView.addSignalListener(key, this.boundSignalHandler);
    });
    if (this.pendingSignals) {
      this.pendingSignals.forEach(s => this.setSignal(s.signal, s.value));
    }
    this.pendingSignals = [];
  }
  initializeData() {
    Object.keys(this.pendingData).forEach(key => {
      this.setData(key, this.pendingData[key]);
      delete this.pendingData[key];
    });
  }
  cleanUpSignals() {
    const signals = this.vegaView.getState().signals;
    Object.keys(signals).forEach(key => {
      this.vegaView.removeSignalListener(key, this.boundSignalHandler);
    });
  }
  signalHandler(name, value) {
    debug(`Got signal '${name}'`, value);
    this.emit('signalChanged', { name, value });
  }
}
