/*!
 * (C) Emu Analytics
 */
import { FlowCoreContext } from './flow-core-context';
import { debugContext } from '../../utils/debug';
import { deferred } from '../../utils/deferred';
/** @internal */
export const DEFAULT_CONTEXT_ID = 'default';
/**
 * A `ContextRegistry` is a container for [[FlowCoreContext]] instances.
 *
 * Context instances can be registered and retrieved by ID.
 *
 * A global instance of the context registry is created by the Flo.w RDF library and exported
 * as `flow`. This should be used by client code as the entry point to the Flo.w RDF JavaScript API.
 */
export class ContextRegistry {
  /**
   * Constructs a new context registry instance.
   */
  constructor() {
    this.contexts = new Map();
  }
  /**
   * Initialize a new Flo.w context.
   *
   * @param options Context options.
   */
  initializeContext(options) {
    options = Object.assign(Object.assign({}, ContextRegistry.defaultOptions), options);
    debugContext(`Initializing context '${options.id}'`, options);
    if (!this.contexts.get(options.id)) {
      this.contexts.set(options.id, deferred());
    }
    const deferredContext = this.contexts.get(options.id);
    const context = new FlowCoreContext(options.id, options.host, options.apiKey, options.enableDebug);
    if (options.initialState) {
      context.state.initialize(options.initialState);
    }
    deferredContext.resolve(context);
    return context;
  }
  /**
   * Uninitialize the specified Flo.w context.
   *
   * @param contextId Context ID.
   */
  uninitializeContext(contextId = DEFAULT_CONTEXT_ID) {
    this.contexts.delete(contextId);
  }
  /**
   * Get the specified Flo.w context.
   *
   * This method returns a promise that will resolve when the context has been registered.
   *
   * @param id Context ID.
   */
  getContext(id) {
    id = id || DEFAULT_CONTEXT_ID;
    if (!this.contexts.get(id)) {
      // Context hasn't been initialized yet. A promise is created that will
      // eventually resolve when the context is initialized.
      this.contexts.set(id, deferred());
    }
    const deferredContext = this.contexts.get(id);
    return deferredContext;
  }
}
ContextRegistry.defaultOptions = {
  id: DEFAULT_CONTEXT_ID,
  host: 'https://flow.emu-analytics.net'
};
/** @internal */
export const globalContextRegistry = new ContextRegistry();
