/*!
 * (C) Emu Analytics
 */
import { Client as FlowClient } from '@emuanalytics/flow-engine-client';
import { MapRegistry } from '../map-registry';
import { ImageRegistry } from '../image-registry';
import { ImageLoader } from '../image-loader';
import { StateStore } from '../state-store';
import EventEmitter from 'wolfy87-eventemitter';
import { PaletteRegistry } from '../palette-registry';
import { StyleParser } from '../parser/style-parser';
import { ChartSpecRegistry } from '../charts/char-spec-registry';
import { Debugger, installGlobalDebugger } from '../debugger/debugger';
/** @internal */
export const FLOW_CONTEXT_TAG = 'FLOW-CONTEXT';
/** @internal */
export const FLOW_CONTEXT_ATTR = 'context';
/**
 * `FlowCoreContext` if the root of the Flo.w RDF JavaScript API.
 *
 * Properties of this class expose functionality to consumers of the Flo.w RDF library.
 *
 * Use the global [[ContextRegistry]] (exposed as `flow` by the library) to initialize
 * a Flo.w RDF context in your application start-up code.
 *
 * @typeParam TState  Type of the reactive application state object.
 * @noInheritDoc
 */
export class FlowCoreContext extends EventEmitter {
  /**
   * Constructs a new `FlowCoreContext`.
   *
   * Use [[ContextRegistry.initializeContext]] to create an instance of `FlowCoreContext`.
   *
   * @param id The ID of the context.
   * @param host The Flo.w Engine host URL.
   * @param apiKey The Flo.w Engine API key.
   * @param enableDebug Set to `true` to enable the debugger.
   */
  constructor(id, host, apiKey, enableDebug = false) {
    super();
    this.id = id;
    this.host = host;
    this.apiKey = apiKey;
    this.flowClient = new FlowClient(this.host, this.apiKey, {
      withCredentials: true,
      transports: ['polling', 'websocket']
    });
    this.maps = new MapRegistry();
    this.images = new ImageRegistry(this);
    this.imageLoader = new ImageLoader();
    this.state = new StateStore(this);
    this.palettes = new PaletteRegistry();
    this.parser = new StyleParser(this);
    this.chartSpecs = new ChartSpecRegistry();
    if (enableDebug) {
      this.debugger = new Debugger(this);
      installGlobalDebugger(this.debugger);
    }
  }
  /**
   * Emit an error.
   *
   * Attach an error handler to respond to errors.
   *
   * ```javascript
   * context.on('error', (error: Error) => {...handle error...});
   * ```
   *
   * If no error handler is attached an exception will be thrown.
   */
  emitError(error) {
    if (!this.getListeners('error').length) {
      // Throw an exception if no error listener is connected
      throw error;
    }
    this.emit('error', error);
  }
  /**
   * Emit a warning
   *
   * Attach a warning handler to respond to warnings.
   *
   * ```javascript
   * context.on('warn', (warning: string) => {...handle warning...});
   * ```
   *
   * If no warning handler is attached, the warning will be logged to the console.
   */
  emitWarning(warning) {
    if (!this.getListeners('warn').length) {
      // tslint:disable
      console.warn(warning);
    }
    this.emit('warn', warning);
  }
}
