/*!
 * (C) Emu Analytics
 */
import * as mobx from 'mobx';
import { isComputed, isObservable } from 'mobx';
import { isObservableArray, isObservableMap, isObservableSet } from 'mobx';
import { isUndefined } from '../../utils/checks.util';
import { FlowQuery } from '../flow-queries/flow-query';
/** @internal */
const FLOW_GLOBAL = 'FDBG';
/** @internal */
const CONSOLE_HIGHLIGHT_STYLE = `
  padding: 2px 4px;
  border-radius: 2px;
  background: #389DFF;
  color: white;`;
/** @internal */
const CONSOLE_BOLD_STYLE = `font-weight: bold;`;
/**
 * Flo.w context debugger. Provides debugging methods available in the browser web developer console.
 *
 * If the `enableDebug` is specified when initializing a Flo.w context a `Debugger` instance is available
 * as the global property `window.FDBG`.
 */
export class Debugger {
  /**
   * Constructs a new Flo.w debugger instance.
   *
   * A new Flo.w debugger will be created automatically if the `enableDebug` option is
   * specified when initializing a Flo.w context.
   *
   * @param context Flo.w context ID.
   */
  constructor(context) {
    this.context = context;
    this.watchDisposers = {};
  }
  /**
   * Install a 'watch' for the specified context state property path.
   *
   * @param path Context state property path.
   */
  watch(path) {
    if (!this.watchDisposers[path]) {
      this.watchDisposers[path] = this.context.state.autorun(() => this.logStateValue(path));
    }
  }
  /**
   * Remove a  previously-installed 'watch' for the specified context state property path.
   *
   * @param path Context state property path.
   */
  unwatch(path = null) {
    if (path) {
      if (this.watchDisposers[path]) {
        this.watchDisposers[path]();
        delete this.watchDisposers[path];
      }
    }
    else {
      // Unwatch all
      Object.keys(this.watchDisposers).forEach(key => this.watchDisposers[key]());
      this.watchDisposers = {};
    }
  }
  /**
   * Set a Flo.w context state property.
   *
   * @param path Context state property path
   * @param value New value to set
   */
  set(path, value) {
    this.context.state.set(path, value);
  }
  /**
   * Read the value of a Flo.w context state property.
   *
   * @param keyOrPath Context state property path.
   * @returns The context state property value.
   */
  get(keyOrPath) {
    if (isUndefined(keyOrPath)) {
      keyOrPath = [];
    }
    else if (typeof keyOrPath === 'string') {
      if (keyOrPath.startsWith('$')) {
        keyOrPath = keyOrPath.slice(1);
      }
      return this.get(keyOrPath.split('.'));
    }
    let value = this.context.state.state;
    const path = [...keyOrPath];
    while (path.length > 0) {
      if (value == null) {
        value = undefined;
        break;
      }
      const key = path.shift();
      if (value instanceof FlowQuery && key === 'results') {
        // Special case to get results of FlowQuery outside of reactive context
        value = value.resultsNonReactive;
      }
      else {
        value = value[key];
      }
    }
    return toJSHelper(value, {});
  }
  /**
   * Enable verbose debugging of all reactive updates.
   */
  spy() {
    this.unspy();
    this.spyDisposer = mobx.spy(this.spyListener.bind(this));
  }
  /**
   * Disable verbose debugging of all reactive updates.
   */
  unspy() {
    if (this.spyDisposer) {
      this.spyDisposer();
      this.spyDisposer = undefined;
    }
  }
  spyListener(event) {
    console.log(`%cspy`, CONSOLE_HIGHLIGHT_STYLE, event);
  }
  logStateValue(path) {
    const value = mobx.toJS(this.context.state.get(path), { recurseEverything: false });
    console.log(`%cwatch%c %s`, CONSOLE_HIGHLIGHT_STYLE, CONSOLE_BOLD_STYLE, path, value);
  }
}
/**
 * Install the specified [[Debugger]] instance as `window.FDBG`.
 *
 * @param dbg Debugger instance
 */
export function installGlobalDebugger(dbg) {
  console.log(`%cFlow.w RDF Debbugger`, CONSOLE_HIGHLIGHT_STYLE, `Enabled for context '${dbg.context.id}' - access using ${FLOW_GLOBAL} in web developer console.`);
  const g = typeof global !== 'undefined' ? global : window;
  g[FLOW_GLOBAL] = g[FLOW_GLOBAL] || {
    watch: dbg.watch.bind(dbg),
    unwatch: dbg.unwatch.bind(dbg),
    set: dbg.set.bind(dbg),
    get: dbg.get.bind(dbg),
    spy: dbg.spy.bind(dbg),
    unspy: dbg.unspy.bind(dbg)
  };
}
// tslint:disable: curly
/** @internal */
function cache(map, key, value, options) {
  if (options.detectCycles)
    map.set(key, value);
  return value;
}
/** @internal */
function getPlainObjectKeys(object) {
  const enumerables = new Set();
  // tslint:disable-next-line: forin
  for (const key in object)
    enumerables.add(key); // *all* enumerables
  Object.getOwnPropertySymbols(object).forEach(function (k) {
    if (Object.getOwnPropertyDescriptor(object, k).enumerable)
      enumerables.add(k);
  }); // *own* symbols
  // Note: this implementation is missing enumerable, inherited, symbolic property names! That would however pretty expensive to add,
  // as there is no efficient iterator that returns *all* properties
  return Array.from(enumerables);
}
/** @internal */
function toJSHelper(source, options, __alreadySeen = null) {
  if (!options.recurseEverything && !isObservable(source))
    return source;
  if (typeof source !== 'object')
    return source;
  // Directly return null if source is null
  if (source === null)
    return null;
  // Directly return the Date object itself if contained in the observable
  if (source instanceof Date)
    return source;
  if (isComputed(source)) {
    try {
      const value = source.get();
      return { computedValue: value };
    }
    catch (e) {
      return { computedValueException: e.message };
    }
  }
  const detectCycles = options.detectCycles === true;
  if (detectCycles && source !== null && __alreadySeen.has(source)) {
    return __alreadySeen.get(source);
  }
  if (isObservableArray(source) || Array.isArray(source)) {
    const res_1 = cache(__alreadySeen, source, [], options);
    const toAdd = source.map(function (value) {
      return toJSHelper(value, options, __alreadySeen);
    });
    res_1.length = toAdd.length;
    for (let i = 0, l = toAdd.length; i < l; i++)
      res_1[i] = toAdd[i];
    return res_1;
  }
  if (isObservableSet(source) || Object.getPrototypeOf(source) === Set.prototype) {
    if (options.exportMapsAsObjects === false) {
      const res_2 = cache(__alreadySeen, source, new Set(), options);
      source.forEach(function (value) {
        res_2.add(toJSHelper(value, options, __alreadySeen));
      });
      return res_2;
    }
    else {
      const res_3 = cache(__alreadySeen, source, [], options);
      source.forEach(function (value) {
        res_3.push(toJSHelper(value, options, __alreadySeen));
      });
      return res_3;
    }
  }
  if (isObservableMap(source) || Object.getPrototypeOf(source) === Map.prototype) {
    if (options.exportMapsAsObjects === false) {
      const res_4 = cache(__alreadySeen, source, new Map(), options);
      source.forEach(function (value, key) {
        res_4.set(key, toJSHelper(value, options, __alreadySeen));
      });
      return res_4;
    }
    else {
      const res_5 = cache(__alreadySeen, source, {}, options);
      source.forEach(function (value, key) {
        res_5[key] = toJSHelper(value, options, __alreadySeen);
      });
      return res_5;
    }
  }
  // Fallback to the situation that source is an ObservableObject or a plain object
  const res = cache(__alreadySeen, source, {}, options);
  getPlainObjectKeys(source).forEach(function (key) {
    res[key] = toJSHelper(source[key], options, __alreadySeen);
  });
  return res;
}
