/*!
 * (C) Emu Analytics
 */
import { isBoolean, isDefined, isNumber, isPlainObject, isString, isStringFull } from '../../../utils/type-checks';
/* Minimal example of a composite layer definition - no datasources and single layer */
const testSimpleLayer = {
  id: 'airquality_layer',
  layers: [
    {
      // Layer 1
      id: 'test',
      source: 'airquality',
      type: 'circle',
      style: {
        type: 'flow',
        fillColor: '#222222',
        strokeColor: 'white',
        strokeWidth: 2,
        labelText: '@name',
        labelColor: 'white',
        radius: 10
      }
    }
  ]
};
const testSimpleLayer2 = {
  id: 'airquality_layer',
  dataSources: [
    {
      id: 'airquality',
      type: 'vector',
      src: 'airquality'
    }
  ],
  layers: [
    {
      // Layer 1
      id: 'test',
      source: 'airquality',
      type: 'circle',
      style: {
        type: 'flow',
        fillColor: {
          type: 'expression',
          value: 'rgba(0, 0, 0, 0.5)'
        },
        strokeColor: {
          type: 'data-driven',
          attribute: 'name',
          default: 'gray',
          values: [
            ['^A', 'red', { regexp: true }],
            ['^B', 'green']
          ]
        },
        radius: {
          type: 'data-driven',
          attribute: 'no2',
          default: 10,
          values: [
            [0, 10],
            [5, 15],
            [100, 20]
          ]
        }
      }
    }
  ]
};
/**
 * Takes a single composite layer definition and expands it into a list of Maplibre-style layers.
 *
 * @param compositeLayer
 */
export function expandCompositeLayer(compositeLayer, layerPrefix) {
  // Style is already a Maplibre style definition
  if (compositeLayer.style.type === 'maplibre') {
    return [compositeLayer];
  }
  else if (compositeLayer.style.type === 'flow') {
    const expandedLayers = [];
    switch (compositeLayer.type) {
      case 'circle':
        expandedLayers.push(...expandCompositeCircleLayer(compositeLayer, layerPrefix));
        break;
      case 'fill':
        expandedLayers.push(...expandCompositeFillLayer(compositeLayer, layerPrefix));
        break;
      case 'line':
        expandedLayers.push(...expandCompositeLineLayer(compositeLayer, layerPrefix));
        break;
      default:
        throw new Error('expandCompositeLayer: Unsupported layer type');
    }
    if (isStringFull(compositeLayer.style.labelText)) {
      expandedLayers.push(labelLayerForCompositeLayer(compositeLayer, layerPrefix));
    }
    return expandedLayers;
  }
  throw new Error(`expandCompositeLayer: Unsupported style type '${compositeLayer.style.type}}'`);
}
function addLayerPrefix(layerId, layerPrefix) {
  return layerPrefix ? `${layerPrefix}__${layerId}` : layerId;
}
export function expandCompositeCircleLayer(layer, layerPrefix) {
  if (layer.style.type !== 'flow') {
    throw new Error('expandCompositeCircleLayer: Unsupported style type');
  }
  const styleRules = [];
  if (layer.style.fillColor) {
    styleRules.push(`circle-color: ${expandPropertyValue(layer.style.fillColor)};`);
  }
  if (layer.style.strokeColor) {
    styleRules.push(`circle-stroke-color: ${expandPropertyValue(layer.style.strokeColor)};`);
  }
  if (layer.style.strokeWidth) {
    styleRules.push(`circle-stroke-width: ${expandStrokeWidth(layer.style.strokeWidth)};`);
  }
  if (layer.style.radius) {
    styleRules.push(`circle-radius: ${expandCircleRadius(layer.style.radius)};`);
  }
  if (layer.style.opacity) {
    styleRules.push(`circle-opacity: ${expandPropertyValue(layer.style.opacity)};`);
    styleRules.push(`circle-stroke-opacity: ${expandPropertyValue(layer.style.opacity)};`);
  }
  if (isDefined(layer.style.visibility)) {
    styleRules.push(`visibility: ${expandPropertyValue(layer.style.visibility)};`);
  }
  const maplibreLayer = Object.assign(Object.assign({}, layer), { id: addLayerPrefix(layer.id, layerPrefix), interactive: isBoolean(layer.interactive) ? layer.interactive : true, type: 'circle', style: {
      type: 'maplibre',
      style: styleRules.join('\n')
    } });
  return [maplibreLayer];
}
export function expandCompositeFillLayer(layer, layerPrefix) {
  if (layer.style.type !== 'flow' || layer.type !== 'fill') {
    throw new Error('expandCompositeFillLayer: Unsupported style type');
  }
  // Fill layer
  let fillStyleRules = [];
  if (layer.style.fillColor) {
    fillStyleRules.push(`fill-color: ${expandPropertyValue(layer.style.fillColor)};`);
  }
  if (layer.style.opacity) {
    fillStyleRules.push(`fill-opacity: ${expandPropertyValue(layer.style.opacity)};`);
  }
  if (isDefined(layer.style.visibility)) {
    fillStyleRules.push(`visibility: ${expandPropertyValue(layer.style.visibility)};`);
  }
  const fillLayer = Object.assign(Object.assign({}, layer), { id: addLayerPrefix(layer.id, layerPrefix), type: 'fill', interactive: isBoolean(layer.interactive) ? layer.interactive : true, style: {
      type: 'maplibre',
      style: fillStyleRules.join('\n')
    } });
  // Line layer (outline of polygon)
  let lineStyleRules = [];
  if (layer.style.strokeColor) {
    lineStyleRules.push(`line-color: ${expandPropertyValue(layer.style.strokeColor)};`);
  }
  if (layer.style.strokeWidth) {
    lineStyleRules.push(`line-width: ${expandStrokeWidth(layer.style.strokeWidth)};`);
  }
  if (layer.style.opacity) {
    lineStyleRules.push(`line-opacity: ${expandPropertyValue(layer.style.opacity)};`);
  }
  if (isDefined(layer.style.visibility)) {
    lineStyleRules.push(`visibility: ${expandPropertyValue(layer.style.visibility)};`);
  }
  const lineLayer = Object.assign(Object.assign({}, layer), { id: `${addLayerPrefix(layer.id, layerPrefix)}__stroke`, type: 'line', interactive: isBoolean(layer.interactive) ? layer.interactive : true, style: {
      type: 'maplibre',
      style: lineStyleRules.join('\n')
    } });
  return [fillLayer, lineLayer];
}
export function expandCompositeLineLayer(layer, layerPrefix) {
  if (layer.style.type !== 'flow' || layer.type !== 'line') {
    throw new Error('expandCompositeLineLayer: Unsupported style type');
  }
  const styleRules = [];
  if (layer.style.strokeColor) {
    styleRules.push(`line-color: ${expandPropertyValue(layer.style.strokeColor)};`);
  }
  if (layer.style.strokeWidth) {
    styleRules.push(`line-width: ${expandStrokeWidth(layer.style.strokeWidth)};`);
  }
  if (isDefined(layer.style.visibility)) {
    styleRules.push(`visibility: ${expandPropertyValue(layer.style.visibility)};`);
  }
  const maplibreLayer = Object.assign(Object.assign({}, layer), { id: addLayerPrefix(layer.id, layerPrefix), type: 'line', interactive: isBoolean(layer.interactive) ? layer.interactive : true, style: {
      type: 'maplibre',
      style: styleRules.join('\n')
    } });
  return [maplibreLayer];
}
export function expandCompositeLayerFlowStyle(layerType, style) {
  let styleRules = [];
  switch (layerType) {
    case 'circle':
      if (style.fillColor) {
        styleRules.push(`circle-color: ${expandPropertyValue(style.fillColor)};`);
      }
      if (style.strokeColor) {
        styleRules.push(`circle-stroke-color: ${expandPropertyValue(style.strokeColor)};`);
      }
      if (style.strokeWidth) {
        styleRules.push(`circle-stroke-width: ${expandStrokeWidth(style.strokeWidth)};`);
      }
      if (style.radius) {
        styleRules.push(`circle-radius: ${expandCircleRadius(style.radius)};`);
      }
      if (style.opacity) {
        styleRules.push(`circle-opacity: ${expandPropertyValue(style.opacity)};`);
      }
      break;
    case 'line':
      if (style.strokeColor) {
        styleRules.push(`line-color: ${expandPropertyValue(style.strokeColor)};`);
      }
      if (style.strokeWidth) {
        styleRules.push(`line-width: ${expandStrokeWidth(style.strokeWidth)};`);
      }
      if (style.opacity) {
        styleRules.push(`line-opacity: ${expandPropertyValue(style.opacity)};`);
      }
      break;
    case 'fill':
      if (style.fillColor) {
        styleRules.push(`fill-color: ${expandPropertyValue(style.fillColor)};`);
      }
      if (style.opacity) {
        styleRules.push(`fill-opacity: ${expandPropertyValue(style.opacity)};`);
      }
      break;
    default:
      throw new Error('expandCompositeLayerFlowStyle: Unsupported layer type');
  }
  if (isDefined(style.visibility)) {
    styleRules.push(`visibility: ${expandPropertyValue(style.visibility)};`);
  }
  return {
    type: 'maplibre',
    style: styleRules.join('\n')
  };
}
export function labelLayerForCompositeLayer(layer, layerPrefix) {
  if (layer.style.type !== 'flow' || !isStringFull(layer.style.labelText)) {
    return;
  }
  const styleRules = [];
  styleRules.push(`text-field: ${expandPropertyValue(layer.style.labelText)};`);
  if (layer.style.labelColor) {
    styleRules.push(`text-color: ${expandPropertyValue(layer.style.labelColor)};`);
  }
  if (layer.style.labelHaloColor) {
    styleRules.push(`text-halo-color: ${expandPropertyValue(layer.style.labelHaloColor)};`);
    styleRules.push(`text-halo-blur: 1;`);
    styleRules.push(`text-halo-width: 2;`);
  }
  if (layer.style.labelSize) {
    styleRules.push(`text-size: ${expandLabelSize(layer.style.labelSize)};`);
  }
  if (layer.style.opacity) {
    styleRules.push(`text-opacity: ${expandPropertyValue(layer.style.opacity)};`);
  }
  if (isDefined(layer.style.visibility)) {
    styleRules.push(`visibility: ${expandPropertyValue(layer.style.visibility)};`);
  }
  if (layer.type === 'circle') {
    styleRules.push(`text-variable-anchor: ['top'];`);
    styleRules.push(`text-radial-offset: 1.2;`);
  }
  const labelLayer = Object.assign(Object.assign({}, layer), { id: `${addLayerPrefix(layer.id, layerPrefix)}__label`, interactive: isBoolean(layer.interactive) ? layer.interactive : false, type: 'symbol', style: {
      type: 'maplibre',
      style: styleRules.join('\n')
    } });
  return labelLayer;
}
export function expandPropertyValue(value) {
  if (isString(value)) {
    // Support short-hand feature property expression
    return value.startsWith('@') ? value : `'${value}'`;
  }
  if (isNumber(value) || isBoolean(value)) {
    return value.toString();
  }
  if (isLiteralProperty(value)) {
    if (isString(value.value)) {
      return `'${value.value}'`;
    }
    else {
      return value.value.toString();
    }
  }
  if (isExpressionProperty(value)) {
    return value.value;
  }
  if (isDataDrivenProperty(value)) {
    return expandDataDrivenPropertyValue(value);
  }
  throw new Error('expandPropertyValue: Unsupported property value type');
}
export function isLiteralProperty(value) {
  return isPlainObject(value) && value.type === 'literal';
}
export function isExpressionProperty(value) {
  return isPlainObject(value) && value.type === 'expression';
}
export function isDataDrivenProperty(value) {
  return isPlainObject(value) && value.type === 'data-driven';
}
export function isSupportedLayerType(type) {
  return ['fill', 'line', 'circle'].includes(type);
}
function expandDataDrivenPropertyValue(value) {
  // check values
  const types = value.values.map((v) => typeof v[0]);
  // Check if all values are of the same type
  if (!types.every((t) => t === types[0])) {
    throw new Error('expandPropertyValue: All case values in a data-driven property must be of the same type');
  }
  switch (value.mode) {
    case 'off':
      // return fixed default value
      return expandPropertyValue(value.default);
    case 'interpolate':
      return expandDataDrivenInterpolateModePropertyValue(value);
    default:
      return expandDataDrivenMatchModePropertyValue(value);
  }
}
function expandDataDrivenInterpolateModePropertyValue(value) {
  const cases = value.values
    .map((v) => {
    const inputVal = v[0];
    const outputVal = v[1];
    if (!isNumber(inputVal)) {
      throw new Error('expandPropertyValue: Input values for interpolated data-driven property must be numeric');
    }
    return `${expandPropertyValue(inputVal)}, ${expandPropertyValue(outputVal)}`;
  })
    .join(', ');
  return `typeof(get("${value.attribute}")) == 'number' ? interpolate(linear(), get("${value.attribute}"), ${cases}) : ${expandPropertyValue(value.default)}`;
}
function expandDataDrivenMatchModePropertyValue(value) {
  const propertyType = typeof value.values[0][0];
  if (propertyType === 'string') {
    const cases = value.values
      .map((v) => {
      let operator;
      const caseMatch = v[0];
      const caseValue = v[1];
      const options = v[2];
      if (options === null || options === void 0 ? void 0 : options.regexp) {
        operator = options.caseSensitive ? '~' : '~*';
        // Check if the regex is valid
        try {
          new RegExp(caseValue);
        }
        catch (e) {
          // Invalid regex
          throw new Error('expandPropertyValue: Invalid regex in data-driven property');
        }
      }
      else {
        operator = (options === null || options === void 0 ? void 0 : options.caseSensitive) ? '==' : '=*';
      }
      return `get("${value.attribute}") ${operator} '${escapeMaplibreString(caseMatch)}', ${expandPropertyValue(caseValue)}`;
    })
      .join(', ');
    return `case(${cases}, ${expandPropertyValue(value.default)})`;
  }
  else if (propertyType === 'number') {
    const cases = orderByArrayIndex(value.values, [0], ['desc'])
      .map((v) => {
      const caseMatch = v[0];
      const caseValue = v[1];
      return `get("${value.attribute}") >= ${caseMatch}, ${expandPropertyValue(caseValue)}`;
    })
      .join(', ');
    return `case(get("${value.attribute}") == null, ${expandPropertyValue(value.default)}, ${cases}, ${expandPropertyValue(value.default)})`;
  }
  else {
    return expandPropertyValue(value.default);
  }
}
export function escapeMaplibreString(value) {
  return value.replace(/['\\]/g, '\\$&'); // $& means the whole matched string
}
export function expandStrokeWidth(strokeWidth) {
  if (isString(strokeWidth) && !strokeWidth.startsWith('$')) {
    switch (strokeWidth) {
      case 'x-small':
        // Hairline
        strokeWidth = 0.5;
        break;
      case 'small':
        strokeWidth = { type: 'expression', value: 'interpolate(linear(), zoom(), 0, 1, 10, 1, 20, 2)' };
        break;
      case 'medium':
        strokeWidth = { type: 'expression', value: 'interpolate(linear(), zoom(), 0, 1, 10, 2, 20, 4)' };
        break;
      case 'large':
        strokeWidth = { type: 'expression', value: 'interpolate(linear(), zoom(), 0, 2, 10, 4, 20, 8)' };
        break;
      case 'x-large':
        strokeWidth = { type: 'expression', value: 'interpolate(linear(), zoom(), 0, 4, 10, 8, 20, 16)' };
        break;
      default:
        throw new Error(`expandStrokeWidth: Unsupported stroke width '${strokeWidth}'`);
    }
  }
  return expandPropertyValue(strokeWidth);
}
export function expandLabelSize(labelSize) {
  if (isString(labelSize) && !labelSize.startsWith('$')) {
    switch (labelSize) {
      case 'x-small':
        labelSize = 8;
        break;
      case 'small':
        labelSize = { type: 'expression', value: 'interpolate(linear(), zoom(), 0, 4, 10, 8, 20, 15)' };
        break;
      case 'medium':
        labelSize = { type: 'expression', value: 'interpolate(linear(), zoom(), 0, 5, 10, 10, 20, 20)' };
        break;
      case 'large':
        labelSize = { type: 'expression', value: 'interpolate(linear(), zoom(), 0, 8, 10, 14, 20, 30)' };
        break;
      case 'x-large':
        labelSize = { type: 'expression', value: 'interpolate(linear(), zoom(), 0, 10, 10, 18, 20, 36)' };
        break;
      default:
        throw new Error(`expandStrokeWidth: Unsupported label size '${labelSize}'`);
    }
  }
  return expandPropertyValue(labelSize);
}
export function expandCircleRadius(circleRadius) {
  if (isString(circleRadius) && !circleRadius.startsWith('$')) {
    switch (circleRadius) {
      case 'x-small':
        circleRadius = { type: 'expression', value: 'interpolate(linear(), zoom(), 0, 1, 10, 2, 20, 8)' };
        break;
      case 'small':
        circleRadius = { type: 'expression', value: 'interpolate(linear(), zoom(), 0, 1, 10, 3, 20, 12)' };
        break;
      case 'medium':
        circleRadius = { type: 'expression', value: 'interpolate(linear(), zoom(), 0, 2, 10, 5, 20, 20)' };
        break;
      case 'large':
        circleRadius = { type: 'expression', value: 'interpolate(linear(), zoom(), 0, 3, 10, 7, 20, 28)' };
        break;
      case 'x-large':
        circleRadius = { type: 'expression', value: 'interpolate(linear(), zoom(), 0, 4, 10, 10, 20, 40)' };
        break;
      default:
        throw new Error(`expandStrokeWidth: Unsupported stroke width '${circleRadius}'`);
    }
  }
  return expandPropertyValue(circleRadius);
}
export function orderByArrayIndex(array, indexes, directions) {
  const sorted = [...array];
  sorted.sort((a, b) => {
    for (let i = 0; i < indexes.length; i++) {
      const index = indexes[i];
      const direction = directions[i];
      if (a[index] < b[index]) {
        return direction === 'asc' ? -1 : 1;
      }
      else if (a[index] > b[index]) {
        return direction === 'asc' ? 1 : -1;
      }
    }
    return 0;
  });
  return sorted;
}
