/*!
 * (C) Emu Analytics
 */
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
  var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
  if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
  else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
  return c > 3 && r && Object.defineProperty(target, key, r), r;
};
import { computed, observable, toJS, autorun } from 'mobx';
import { FlowCoreDataSource } from './flow-core-data-source';
import { debugDataJoin } from '../../utils/debug';
import { isNil, isURL } from '../../utils/checks.util';
/**
 * Core functionality for Flo.w map data joins. Wrapped by the `<flow-map-data-join>` web component.
 *
 * @noInheritDoc
 */
export class FlowCoreDataJoin {
  /**
   * Constructs a new `FlowCoreDataJoin` instance.
   *
   * @param options Map data join options.
   */
  constructor(options) {
    this.options = Object.assign(Object.assign({}, FlowCoreDataSource.defaultOptions), options);
    this.id = options.id;
    this.src = options.src;
  }
  /**
   * Return the data source for the map data join.
   *
   * If `src` is a Flo.w reactive expression, the expression will be expanded to
   * return an array of data objects.
   *
   * <flow-badge pill>Reactive</flow-badge>
   *
   * @returns array of data, dataset ID or external URL.
   */
  get dataSrc() {
    if (typeof this._src === 'string' && this._src.startsWith('$')) {
      // src is a state variable - should contain an array of data objects
      const context = this.options.flowMap.context;
      return toJS(context.state.get(this._src));
    }
    else {
      // src is an array, dataset ID or external URL
      return this._src;
    }
  }
  /**
   * Return the parameters associated with the map data join.
   *
   * <flow-badge pill>Reactive</flow-badge>
   */
  get parameters() {
    return this._parameters.map(p => {
      return p.parameterDef;
    });
  }
  /**
   * Start the reactive map data join.
   */
  start() {
    debugDataJoin(`Starting data join: ${this.id}`);
    this.stop();
    this.disposer = autorun(async () => {
      const src = this.dataSrc;
      const params = this.parameters;
      const missingParams = !!params.find(p => p.required && isNil(p.value));
      if (src === null || missingParams) {
        // Null source or required params missing
        debugDataJoin(`Skipping data join: ${this.id} - src ${src} missing ${missingParams}`);
        return;
      }
      debugDataJoin(`Performing data join: ${this.id}`);
      let data;
      if (typeof src === 'string') {
        // Load data from dataset or url
        try {
          data = await this.loadData(src, params);
        }
        catch (e) {
          this.options.flowMap.context.emitWarning(`Failed to load data for data join '${this.id}': ${e.message}`);
          data = [];
        }
      }
      else {
        data = src;
      }
      this.doDataJoin(data);
    }, { name: `autorun: dataJoin(FlowDataJoin.${this.options.flowMap.id}.${this.id})` });
  }
  /**
   * Stop the reactive map data join.
   */
  stop() {
    if (this.disposer) {
      debugDataJoin(`Stopping data join: ${this.id}`);
      this.disposer();
      this.disposer = null;
    }
  }
  /**
   * Set the `src` property. One of
   *
   * - array of data objects.
   * - Flo.w reactive expression.
   * - dataset ID.
   * - external URL.
   */
  set src(src) {
    this._src = src;
  }
  /**
   * Get the `src` property.
   */
  get src() {
    return this._src;
  }
  /**
   * Set parameters associated with this data join.
   *
   * Called internally by the `flow-map-data-join` web component. If set programmatically,
   * it will be overwritten by the `flow-map-data-join` web component.
   *
   * @param parameters Data join parameters.
   */
  setParameters(parameters) {
    this._parameters = parameters;
  }
  loadData(datasetOrUrl, paramDefs) {
    if (isURL(datasetOrUrl)) {
      const url = new URL(datasetOrUrl);
      // Add datajoin parameters to querystring
      const params = new URLSearchParams(url.searchParams);
      paramDefs.forEach(p => {
        params.append(p.name, p.value);
      });
      url.search = params.toString();
      const headers = new Headers({ Accept: 'application/json' });
      const request = new Request(url.toString(), {
        method: 'GET',
        mode: 'cors',
        headers,
        cache: 'default'
      });
      return fetch(request).then(response => {
        if (!response.ok) {
          throw new Error(`status=${response.status} ${response.statusText}`);
        }
        return response.json();
      });
    }
    else {
      // Dataset - run query using flowClient
      const context = this.options.flowMap.context;
      const params = paramDefs.reduce((accum, p) => {
        accum[p.name] = p.value;
        return accum;
      }, {});
      return context.flowClient.datasets.query(datasetOrUrl, {}, params);
    }
  }
  doDataJoin(data) {
    const { id, foreignKey, clear, flowDataSource, flowMap } = this.options;
    const source = flowDataSource.id;
    const sourceLayer = flowDataSource.type === 'vector' ? this.options.sourceLayer || 'geometry' : null;
    if (clear) {
      flowMap.removeFeatureState({ source, sourceLayer }, id);
    }
    if (data) {
      data.forEach(d => flowMap.setFeatureState({ id: d[foreignKey], source, sourceLayer }, { [id]: d }));
    }
  }
}
FlowCoreDataJoin.defaultOptions = {
  foreignKey: 'id'
};
__decorate([
  observable
], FlowCoreDataJoin.prototype, "_parameters", void 0);
__decorate([
  observable
], FlowCoreDataJoin.prototype, "_src", void 0);
__decorate([
  computed
], FlowCoreDataJoin.prototype, "dataSrc", null);
__decorate([
  computed
], FlowCoreDataJoin.prototype, "parameters", null);
