/*!
 * (C) Emu Analytics
 */
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
  var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
  if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
  else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
  return c > 3 && r && Object.defineProperty(target, key, r), r;
};
import centroid from '@turf/centroid';
import { getType, getType as turfGetType } from '@turf/invariant';
import nearestPointOnLine from '@turf/nearest-point-on-line';
import Debug from 'debug';
import deepEqual from 'fast-deep-equal';
import { computed, observable, toJS } from 'mobx';
import EventEmitter from 'wolfy87-eventemitter';
import { isArray, isNil, isObject, isStringFull, isValue } from '../../utils/checks.util';
import { ensureArray } from '../../utils/ensure-array';
/** @internal */
const debug = Debug('flow:flow-core-data-source');
/** @internal */
const areSetsEqual = (a, b) => a.size === b.size && [...a].every((value) => b.has(value));
/** @internal */
const shallowCompareObjects = (obj1, obj2) => isObject(obj1) &&
  isObject(obj2) &&
  Object.keys(obj1).length === Object.keys(obj2).length &&
  Object.keys(obj1).every((key) => obj2.hasOwnProperty(key) && obj1[key] === obj2[key]);
/** @internal */
const shallowCompareArrays = (arr1, arr2) => isArray(arr1) && isArray(arr2) && arr1.length === arr2.length && arr1.every((val) => arr2.includes(val));
export class FlowCoreDataSource extends EventEmitter {
  constructor(options) {
    super();
    this.selectedFeatureIDs = [];
    this._selectedFeatures = new Map();
    this.dataLoading = false;
    // Currently open popups managed by this datasource
    this.popups = new Map();
    this.parameters = [];
    this.updateOptions(options);
  }
  get options() {
    return this._options;
  }
  /* @internal */
  get maplibreSourceSpecification() {
    const context = this._options.flowMap.context;
    const apiKey = context.apiKey;
    let src = this._src;
    if (typeof src === 'string' && src.startsWith('$')) {
      // src is a state variable - resolve it to get value before continuing
      src = toJS(context.state.get(src));
    }
    if (typeof src === 'string') {
      // src is a datasetID/tileSourceID or URL
      // Expand to full URL
      const urlObj = new URL(this.expandSrc(src, this._options.type));
      const combinedParams = new URLSearchParams(urlObj.searchParams);
      // Add datasource parameters to querystring
      this.parameters
        .filter((p) => !isNil(p.parameterDef.value))
        .forEach((p) => combinedParams.append(p.parameterDef.name, p.parameterDef.value));
      // Add apikey to querystring if not already specified in URL
      // Datasource-specific API key takes precedence over application-wide API key
      if (!combinedParams.has('apikey') && (this._options.apiKey || apiKey)) {
        combinedParams.append('apikey', this._options.apiKey || apiKey);
      }
      // Add limit to querystring if set
      if (this._options.limit) {
        combinedParams.append('limit', this._options.limit.toString(10));
      }
      // Add format to querystring if geojson
      if (this._options.type === 'geojson') {
        combinedParams.append('format', 'geojson');
      }
      urlObj.search = combinedParams.toString();
      src = urlObj.toString();
    }
    switch (this._options.type) {
      case 'vector':
        return isStringFull(src)
          ? {
            type: 'vector',
            url: src,
            promoteId: this._options.featureId
          }
          : // For null src use an empty geojson datasource as a placeholder
            {
              type: 'geojson',
              data: { type: 'FeatureCollection', features: [] }
            };
      case 'raster':
        return isStringFull(src)
          ? {
            type: 'raster',
            url: src
          }
          : // For null src use an empty geojson datasource as a placeholder
            {
              type: 'geojson',
              data: { type: 'FeatureCollection', features: [] }
            };
      case 'geojson':
        return {
          type: 'geojson',
          data: src || { type: 'FeatureCollection', features: [] },
          cluster: this._options.cluster,
          clusterRadius: this._options.clusterRadius,
          clusterMaxZoom: this._options.clusterMaxZoom,
          promoteId: this._options.featureId
        };
      default:
        context.emitError(new Error(`<flow-map-data-source id="${this.id}">: invalid type '${this._options.type}'`));
    }
  }
  set src(src) {
    this._src = src;
  }
  get src() {
    return this._src;
  }
  /**
   * @internal
   *
   * Called internally from MapDataSource to update options
   *
   * @param parameters Data source parameters
   */
  setParameters(parameters) {
    this.parameters = parameters;
  }
  /**
   * @internal
   *
   * Called internally from MapDataSource to update options
   *
   * @param options Data source options
   */
  updateOptions(options) {
    this._options = Object.assign(Object.assign({}, FlowCoreDataSource.defaultOptions), options);
    this.id = options.id;
    this.type = options.type;
    this.src = options.src;
    // validate options for raster-type
    if (this.type === 'raster') {
      if (this._options.selectionMode !== 'none') {
        this._options.flowMap.context.emitError(new Error(`<flow-map-data-source id="${this.id}">: selection is not supported for type 'raster'`));
      }
    }
    // Validate options
    if (this._options.selectionMode !== 'none' && !this._options.featureId) {
      this._options.flowMap.context.emitError(new Error(`<flow-map-data-source id="${this.id}">: feature-id attribute must be specified`));
    }
  }
  /**
   * Called internally to attach a data source to its parent map.
   *
   * @param map Parent map
   */
  attach(map) {
    debug(`Attaching ${this.id} to ${map.id}`);
    // Forward selection events
    this.on('selectionStateChanged', (ev) => map.emit('selectionStateChanged', ev));
    this.on('selectionFeatureStateChanged', (ev) => map.emit('selectionFeatureStateChanged', ev));
    // Attach map data events
    this.boundMapSourceDataHandler = this.handleMapSourceData.bind(this);
    this.boundMapSourceDataLoadingHandler = this.handleMapSourceDataLoading.bind(this);
    map.maplibreMap.on('sourcedata', this.boundMapSourceDataHandler);
    map.maplibreMap.on('sourcedataloading', this.boundMapSourceDataHandler);
    // Allow map to update style and settle before attempting to subscribe
    // (Otherwise setting selected map feature state will fail)
    map.maplibreMap.once('idle', () => {
      this.subscribe();
    });
  }
  /**
   * Called internally to detach a data source from its parent map.
   */
  detach() {
    debug(`Detaching ${this.id} from ${this._options.flowMap.id}`);
    this.closeAllPopups();
    this.removeAllListeners();
    this._options.flowMap.maplibreMap.off('sourcedata', this.boundMapSourceDataHandler);
    this._options.flowMap.maplibreMap.off('sourcedataloading', this.boundMapSourceDataHandler);
    this.unsubscribe();
  }
  /**
   * @internal
   *
   * Subscribe to reactive updates from application state
   */
  subscribe() {
    if (this._options.selectionMode === 'none' || !this._options.selectionState) {
      return;
    }
    this.unsubscribe();
    const context = this._options.flowMap.context;
    this.subscription = context.state.toStream(this._options.selectionState, true).subscribe((value) => {
      let selectedFeatureIDs;
      switch (this._options.selectionMode) {
        case 'single':
          selectedFeatureIDs = isValue(value) ? [value] : [];
          break;
        case 'multi':
          selectedFeatureIDs = isArray(value) ? value : [];
          break;
      }
      if (!areSetsEqual(new Set(selectedFeatureIDs), new Set(this.selectedFeatureIDs))) {
        // Incoming selected features are different - need to update map state internals
        const deselectedFeatures = this.selectedFeatureIDs.filter((id) => !selectedFeatureIDs.includes(id));
        this.selectedFeatureIDs = selectedFeatureIDs;
        this._selectedFeatures.clear();
        this.queryMapFeatures(selectedFeatureIDs).forEach((feature) => {
          this._selectedFeatures.set(feature.id, feature.properties);
        });
        this.updateMapSelectionState();
        this.updateApplicationFeatureSelectionState();
        deselectedFeatures.forEach((id) => this.closePopup(id));
      }
    });
  }
  /**
   * @internal
   *
   * Unsubscribe from reactive updates from application state
   */
  unsubscribe() {
    if (this.subscription) {
      this.subscription.unsubscribe();
      this.subscription = undefined;
    }
  }
  /**
   * @internal
   *
   * Register a click on a feature. Called internally by FlowCoreMap to support automatic feature selection and popups.
   *
   * @param feature Clicked feature
   * @param multiSelect True if alt/ctrl/meta click
   * @param clickedPoint Lng/Lat of clicked point
   */
  featureClicked(feature, multiSelect, clickedGroup, clickedPoint) {
    if (this._options.selectionMode === 'none') {
      // Do not handle automatic selection state
      return;
    }
    const selectedFeatureIDs = new Set(this.selectedFeatureIDs);
    let clearSelection = false;
    let toggleFeature = false;
    let showPopup = false;
    // A feature from this data source has been clicked
    if (feature && feature.source === this.id) {
      if (this._options.selectionMode === 'single') {
        // Single-selection mode
        //
        if (selectedFeatureIDs.has(feature.id)) {
          // Toggle feature off
          toggleFeature = true;
        }
        else {
          // Select single feature
          clearSelection = true;
          toggleFeature = true;
        }
      }
      else {
        // Multi-selection mode
        if (!multiSelect) {
          clearSelection = true;
        }
        toggleFeature = true;
      }
    }
    else if (feature && feature.source !== this.id) {
      // Feature from another data source clicked - clear selection only if selectionClear option set
      clearSelection =
        this._options.selectionClear === 'group'
          ? this._options.selectionGroup === clickedGroup
          : !!this._options.selectionClear;
    }
    else {
      // Empty map clicked - always clear unless it is a ctrl-click
      clearSelection = !multiSelect;
    }
    if (clearSelection) {
      this.closeAllPopups();
      selectedFeatureIDs.clear();
      this._selectedFeatures.clear();
    }
    if (toggleFeature) {
      if (selectedFeatureIDs.has(feature.id) && multiSelect) {
        // Already selected - deselect feature
        selectedFeatureIDs.delete(feature.id);
        this._selectedFeatures.delete(feature.id);
        this.closePopup(feature.id);
      }
      else {
        // Select feature
        selectedFeatureIDs.add(feature.id);
        this._selectedFeatures.set(feature.id, feature.properties);
        showPopup = true;
      }
    }
    // Update selection
    if (!areSetsEqual(selectedFeatureIDs, new Set(this.selectedFeatureIDs))) {
      // Selection has changed
      this.selectedFeatureIDs = Array.from(selectedFeatureIDs);
      this.updateMapSelectionState();
      this.updateApplicationSelectionState();
      this.updateApplicationFeatureSelectionState();
    }
    // Show popup if not already showing
    if (showPopup && !this.popups.has(feature.id)) {
      this.openPopup(feature, clickedPoint);
    }
  }
  /**
   * Programmatically select one or more features.
   *
   * Call with an empty array to clear selection.
   *
   * @param features Single feature ID or array of feature IDs to select
   * @returns
   */
  selectFeatures(features) {
    if (this._options.selectionMode === 'none') {
      // Do not handle automatic selection state
      return;
    }
    this.selectedFeatureIDs = ensureArray(features);
    this.queryMapFeatures(this.selectedFeatureIDs).forEach((feature) => {
      this._selectedFeatures.set(feature.id, feature.properties);
    });
    this.updateMapSelectionState();
    this.updateApplicationSelectionState();
    this.updateApplicationFeatureSelectionState();
  }
  /**
   * Open a selection popup for the specified feature.
   *
   * For point features, clickedPoint is ignored and the popup is opened at the feature point.
   *
   * For LineStrings, the popup is opened at the closest point on the line to the clicked point.
   *
   * For Polygons, the popup is opened at the clicked point.
   *
   * If clicked point is not specified then the feature centroid is used.
   *
   * @param feature Map feature
   * @param clickedPoint Lng/Lat of clicked point.
   */
  openPopup(feature, clickedPoint) {
    if (!this._options.selectionPopup) {
      return;
    }
    // Close popup if already open
    if (this.popups.has(feature.id)) {
      this.closePopup(feature.id);
    }
    // If clicked point is not provided then use centroid of feature for polygons
    clickedPoint = clickedPoint !== null && clickedPoint !== void 0 ? clickedPoint : centroid(feature).geometry.coordinates;
    const map = this._options.flowMap;
    let popupOptions;
    if (typeof this._options.selectionPopup === 'function') {
      try {
        popupOptions = this._options.selectionPopup(feature);
      }
      catch (e) {
        map.context.emitError(e);
        return;
      }
    }
    else {
      popupOptions = this._options.selectionPopup;
    }
    if (popupOptions) {
      let coords;
      switch (turfGetType(feature)) {
        case 'Point':
          coords = feature.geometry.coordinates;
          break;
        case 'MultiPoint':
          coords = feature.geometry.coordinates[0];
          break;
        case 'LineString':
        case 'MultiLineString':
          coords = nearestPointOnLine(feature.geometry, clickedPoint).geometry.coordinates;
          break;
        default:
          coords = clickedPoint;
          break;
      }
      const popup = map.openPopup(coords, popupOptions);
      this.popups.set(feature.id, popup);
      popup.once('close', () => {
        this.popups.delete(feature.id);
      });
    }
  }
  /**
   * Close the selection popup for the specified feature.
   *
   * @param featureId Feature ID
   */
  closePopup(featureId) {
    var _a;
    (_a = this.popups.get(featureId)) === null || _a === void 0 ? void 0 : _a.close();
  }
  /**
   * Close all selection popups.
   */
  closeAllPopups() {
    for (const popup of this.popups.values()) {
      popup.close();
    }
  }
  /**
   * Convert a src parameter to final URL
   *
   * @param src Data source url/dataset/tilesource ID
   * @param type Data source type
   * @returns URL
   */
  expandSrc(src, type) {
    if (src.startsWith('http')) {
      return src;
    }
    const context = this._options.flowMap.context;
    const host = context.host;
    switch (type) {
      case 'vector':
      case 'raster':
        return `${host}/tiles/${src}/index.json`;
      case 'geojson':
        return `${host}/api/v1/datasets/${src}/query`;
      default:
        context.emitError(new Error(`<flow-map-data-source id="${this.id}">: invalid type '${this._options.type}'`));
    }
  }
  /**
   * Query map source data for specified features.
   *
   * @param featureIDs Array of features IDs to query
   * @returns Array of features
   */
  queryMapFeatures(featureIDs) {
    const sourceLayer = this._options.type === 'vector' ? this._options.sourceLayer || 'geometry' : null;
    let features = this._options.flowMap.maplibreMap.querySourceFeatures(this.id, {
      sourceLayer,
      filter: ['in', ['get', this._options.featureId], ['literal', featureIDs]]
    });
    // Dedupe features
    const ids = new Set();
    features = features.filter((feature) => (ids.has(feature.id) ? false : ids.add(feature.id) && true));
    return features;
  }
  /**
   * Update map `selected` feature state to reflect current selection
   */
  updateMapSelectionState() {
    const sourceLayer = this._options.type === 'vector' ? this._options.sourceLayer || 'geometry' : null;
    // Remove selected feature state from all features for this data source
    this._options.flowMap.removeFeatureState({ source: this.id, sourceLayer }, 'selected');
    // Set feature state for selected features
    this.selectedFeatureIDs.forEach((id) => this._options.flowMap.setFeatureState({ id, source: this.id, sourceLayer }, { selected: true }));
  }
  /**
   * Update bound app state property from current selection. This will trigger reactive updates.
   *
   * selectionMode determines how property is updated:
   *
   * - single-id: Feature ID of selected features
   * - multi-id: Array of selected feature IDs
   * - single-feature: Object containing feature properties
   * - multi-feature: Map of feature IDs to feature properties
   */
  updateApplicationSelectionState() {
    const context = this._options.flowMap.context;
    switch (this._options.selectionMode) {
      case 'single':
        if (this._options.selectionState) {
          context.state.set(this._options.selectionState, this.selectedFeatureIDs[0]);
        }
        this.emit('selectionStateChanged', { dataSourceId: this.id, selection: this.selectedFeatureIDs[0] });
        break;
      case 'multi':
        {
          if (this._options.selectionState) {
            // Only update if new state is different
            const oldState = context.state.get(this._options.selectionState);
            if (!deepEqual(oldState, this.selectedFeatureIDs)) {
              if (this._options.selectionState) {
                context.state.set(this._options.selectionState, this.selectedFeatureIDs);
              }
            }
          }
          this.emit('selectionStateChanged', { dataSourceId: this.id, selection: this.selectedFeatureIDs });
        }
        break;
      default:
        context.emitError(new Error(`<flow-map-data-source id="${this.id}">: Unrecognized selection-mode`));
        return;
    }
  }
  updateApplicationFeatureSelectionState() {
    const context = this._options.flowMap.context;
    switch (this._options.selectionMode) {
      case 'single':
        {
          const newState = this._selectedFeatures.get(this.selectedFeatureIDs[0]);
          if (this._options.selectionFeatureState) {
            const oldState = context.state.get(this._options.selectionFeatureState);
            // Only update if new state is different
            if (!deepEqual(oldState, newState)) {
              context.state.set(this._options.selectionFeatureState, newState);
            }
          }
          this.emit('selectionFeatureStateChanged', { dataSourceId: this.id, selection: newState });
        }
        break;
      case 'multi':
        {
          const newState = this.selectedFeatureIDs.reduce((accum, val) => {
            accum[val] = this._selectedFeatures.get(val);
            return accum;
          }, {});
          if (this._options.selectionFeatureState) {
            // Only update if new state is different
            const oldState = context.state.get(this._options.selectionFeatureState);
            if (!deepEqual(oldState, newState)) {
              context.state.set(this._options.selectionFeatureState, newState);
            }
          }
          this.emit('selectionFeatureStateChanged', { dataSourceId: this.id, selection: newState });
        }
        break;
      default:
        context.emitError(new Error(`<flow-map-data-source id="${this.id}">: Unrecognized selection-mode`));
        return;
    }
  }
  /**
   * Attach map idle handler when source data starts loading as a fail-safe in case
   * sourceLoaded is not detected properly. This ensures selected feature state will be updated in all cases.
   *
   * (sourcedata event firing is unreliable)
   */
  handleMapSourceDataLoading(ev) {
    if (ev.sourceId !== this.id) {
      return;
    }
    if (!this.dataLoading) {
      this.dataLoading = true;
      this._options.flowMap.maplibreMap.once('idle', () => {
        if (this.dataLoading) {
          this.dataLoading = false;
          this.updateFromMapData('idle');
        }
      });
    }
  }
  /**
   * Handle sourcedata events:
   *
   * - 'visibility' events generated when source is no longer visible (all associated layers hidden)
   * - 'tile' events signalling that data is loaded.
   *
   * Note that the last 'tile' event should be sent with isSourceLoaded=true signalling that the source is fully loaded.
   * However, it seems that this is not always the case. Handling the 'idle' event above is a fail-safe to ensure that
   * feature data is always updated. (Updating via 'idle' introduces a visible delay for moving popups so updating on a 'tile'
   * event is preferable if it is actually sent).
   */
  handleMapSourceData(ev) {
    if (ev.sourceId !== this.id) {
      return;
    }
    if (ev.sourceDataType === 'visibility' && ev.isSourceLoaded === false) {
      // All layers associated with data source have been hidden
      this.updateFromMapData('Source hidden');
      this.closeAllPopups();
    }
    else if (ev.tile && ev.isSourceLoaded) {
      // Data has finished loading
      this.dataLoading = false;
      this.updateFromMapData('Source loaded');
    }
  }
  /**
   * Perform updates when data source has finished loading data
   *
   * - update bound app state property with new feature properties
   * - move popup to new location if required
   * - deselect features that are no longer available in map data
   */
  updateFromMapData(reason) {
    if (!this._options.selectionState && !this._options.selectionFeatureState && !this._options.selectionPopup) {
      // Nothing to do
      return;
    }
    if (this.selectedFeatureIDs.length === 0) {
      // No features selected - nothing to do
      return;
    }
    debug(`Data Source ${this.id}: updating selected feature data (reason = ${reason})`);
    const features = this.queryMapFeatures(this.selectedFeatureIDs);
    // Some selected features may no longer be available in map data - track which ones need deselecting
    const availableFeatures = Array.from(new Set(features.map((feature) => feature.id)));
    const unavailableFeatures = this.selectedFeatureIDs.filter((id) => !availableFeatures.includes(id));
    // Close popups for unavailable features
    unavailableFeatures.forEach((id) => this.closePopup(id));
    // Update selected feature properties using new map data
    this._selectedFeatures.clear();
    features.forEach((feature) => {
      this._selectedFeatures.set(feature.id, feature.properties);
      // Reposition popup if feature is point-like
      const popup = this.popups.get(feature.id);
      const featureType = getType(feature);
      if (popup && popup.options.trackLocation === true && (featureType === 'Point' || featureType === 'MultiPoint')) {
        popup.moveTo(centroid(feature).geometry.coordinates);
      }
    });
    this.updateApplicationFeatureSelectionState();
  }
}
FlowCoreDataSource.defaultOptions = {
  type: 'vector',
  cluster: false,
  clusterRadius: 50,
  clusterMaxZoom: 20,
  selectionMode: 'none',
  selectionClear: false
};
__decorate([
  observable
], FlowCoreDataSource.prototype, "parameters", void 0);
__decorate([
  observable
], FlowCoreDataSource.prototype, "_src", void 0);
__decorate([
  computed
], FlowCoreDataSource.prototype, "maplibreSourceSpecification", null);
