/*!
 * (C) Emu Analytics
 */
import maplibregl from '@emuanalytics/maplibre-gl';
import EventEmitter from 'wolfy87-eventemitter';
import { isStringFull } from '../../utils/checks.util';
import Debug from 'debug';
/** @internal */
const debug = Debug('flow:flow-core-map-popup');
/**
 * Core functionality for Flo.w map popups.
 *
 * @noInheritDoc
 */
export class FlowCoreMapPopup extends EventEmitter {
  /**
   * Constructs a new `FlowCoreMapPopup` instance.
   *
   * This is called by the `FlowCoreMap::openPopup` method.
   *
   * @param map Parent Flo.w map.
   * @param options Popup options.
   */
  constructor(map, options) {
    super();
    this.map = map;
    this.options = options;
    this.options = Object.assign({ 
      // Defaults
      closeButton: true, closeOnClick: false, closeOnMove: false, maxWidth: '300px', trackLocation: false }, this.options);
  }
  /**
   * Open the popup at the specified location.
   *
   * @param lngLat Popup location.
   */
  open(lngLat) {
    if (this.maplibrePopup) {
      // Already open
      return;
    }
    const classes = ['flow-map-popup'];
    if (isStringFull(this.options.class)) {
      classes.push(this.options.class);
    }
    if (this.options.closeButton) {
      classes.push('flow-map-popup--with-close');
    }
    this.maplibrePopup = new maplibregl.Popup({
      className: classes.join(' '),
      closeButton: false,
      closeOnClick: this.options.closeOnClick,
      closeOnMove: this.options.closeOnMove,
    });
    this.maplibrePopup.on('close', this.popupCloseHandler.bind(this));
    const popupButtonHTML = `<flow-button class="popup-close-button" compact icon-button icon="fas-xmark"></flow-button>`;
    // Set popup content using Node or HTML string
    if (this.options.content instanceof Node) {
      // Set popup content from Node
      const template = document.createElement('template');
      template.innerHTML = popupButtonHTML;
      template.content.appendChild(this.options.content);
      this.maplibrePopup.setDOMContent(template.content);
    }
    else if (isStringFull(this.options.content)) {
      // Set popup content from HTML string
      this.maplibrePopup.setHTML(`${popupButtonHTML}${this.options.content}`);
    }
    else {
      this.map.context.emitError(new Error('FlowCoreMapPopup::open() - popup content must be specified.'));
      return;
    }
    // Add to map
    this.maplibrePopup.setMaxWidth(this.options.maxWidth).addTo(this.map.maplibreMap);
    this.moveTo(lngLat);
    // Add CSS part for content
    this.maplibrePopup.getElement().querySelector('.maplibregl-popup-content').setAttribute('part', 'popup-content');
    const closeButton = this.maplibrePopup.getElement().querySelector('flow-button');
    if (closeButton) {
      closeButton.addEventListener('click', this.close.bind(this));
    }
    if (isStringFull(this.options.trackLocation)) {
      this.trackLocation(this.options.trackLocation, this.options.fireTrackImmediately);
    }
  }
  /**
   * Close the popup.
   */
  close() {
    if (this.maplibrePopup) {
      this.maplibrePopup.remove();
      this.maplibrePopup = null;
    }
  }
  /**
   * Move the popup to a new location.
   *
   * @param lngLat New location.
   */
  moveTo(lngLat) {
    this.maplibrePopup.setLngLat(lngLat);
  }
  /**
   * Start tracking a Flo.w reactive expression to move the popup.
   *
   * Call with null `location` to stop tracking.
   *
   * <flow-badge pill>Reactive</flow-badge>
   *
   * @param location Flo.w reactive expression returning lng/lat as a 2-element array.
   * @param fireImmediately Update popup position immediately.
   */
  trackLocation(location, fireImmediately = false) {
    this.unsubscribe();
    if (location) {
      // Lng/Lat reactive expression - subscribe to changes
      const parsedLngLatExpression = this.map.context.parser.parseExpression(location);
      if (parsedLngLatExpression.status) {
        this.autorunDisposer = this.map.context.state.autorun(() => {
          try {
            const evaluatedLngLat = this.map.context.parser.evaluateExpression(parsedLngLatExpression.value) || [0, 0];
            // Optionally skip first update
            if (fireImmediately) {
              this.moveTo(evaluatedLngLat);
            }
            fireImmediately = true;
          }
          catch (e) {
            this.map.context.emitWarning(e.message);
          }
        }, `autorun: trackLocation(FlowCoreMapPopup.${this.map.id}.${this.options.id})`);
      }
      else {
        this.map.context.emitError(new Error(`Malformed lng/lat expression in popup - ${parsedLngLatExpression}`));
      }
    }
  }
  popupCloseHandler() {
    this.emit('close');
    this.unsubscribe();
    this.removeAllListeners();
  }
  unsubscribe() {
    if (this.autorunDisposer) {
      this.autorunDisposer();
      this.autorunDisposer = null;
    }
  }
}
