/*!
 * (C) Emu Analytics
 */
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
  var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
  if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
  else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
  return c > 3 && r && Object.defineProperty(target, key, r), r;
};
import { Map as MaplibreMap } from '@emuanalytics/maplibre-gl';
import { diff } from '@emuanalytics/maplibre-gl-style-spec';
import MapboxDraw from '@mapbox/mapbox-gl-draw';
import Debug from 'debug';
import throttle from 'lodash-es/throttle';
import { autorun, computed, observable, toJS } from 'mobx';
import EventEmitter from 'wolfy87-eventemitter';
import { isNil, isObject, isURL } from '../../utils/checks.util';
import { debugMapStyle } from '../../utils/debug';
import { globalContextRegistry } from '../context/context-registry';
import { MaplibreGLError } from '../errors/maplibregl-error';
import { isCompositeLayer } from './flow-core-map-composite-layer';
import { FlowCoreMapPopup } from './flow-core-map-popup';
import { MapExporter } from './map-exporter';
import { CircleMode, DirectMode, DragCircleMode, SimpleSelectMode } from './maplibre-gl-draw-circle';
/** @internal */
const debug = Debug('flow:flow-map-core');
/** @internal */
const MOUSE_MOVE_THROTTLE_DURATION = 200;
/** @internal */
const MAP_LOADING_POLL_INTERVAL = 500;
/** @internal */
const MapboxRegex = new RegExp('mapbox:\\/\\/styles/([^\\/]+)/([^\\?]+)\\?access_token=(.*)$');
/** @internal */
const defaultMapOptions = {
  boxZoom: false,
  doubleClickZoom: false
};
/** @internal */
const mapboxDrawEvents = [
  'draw.create',
  'draw.delete',
  'draw.combine',
  'draw.uncombine',
  'draw.update',
  'draw.selectionchange',
  'draw.modechange',
  // 'draw.render',
  'draw.actionable'
];
/**
 * Core functionality for Flo.w maps. Wrapped by the `<flow-map>` web component.
 *
 * @noInheritDoc
 */
export class FlowCoreMap extends EventEmitter {
  /**
   * Constructs a new `FlowCoreMap`.
   *
   * @param id Map ID.
   * @param flowContextId ID of associated Flo.w context object.
   */
  constructor(id, flowContextId) {
    super();
    this.id = id;
    this.flowContextId = flowContextId;
    this.initialSync = false;
    this.mapLoading = false;
    this.exclusiveDrawMode = false;
    this.dataSources = [];
    this.popupTemplates = [];
    this.cachedSourceIds = [];
    this.forwardEvent = (ev) => {
      this.emit(ev.type, ev);
    };
  }
  /**
   * Get the full expanded [MaplibreGL Style Specification](https://maplibre.org/maplibre-gl-js-docs/style-spec/) for this map.
   *
   * <flow-badge pill>Reactive</flow-badge>
   */
  get mapStyle() {
    // Save mapbox-draw-gl sources and layers to copy over to updated style
    const drawControlSources = {};
    let drawControlLayers = [];
    if (this.drawControl) {
      try {
        const currentStyle = this.maplibreMap.getStyle();
        if (currentStyle) {
          Object.keys(currentStyle.sources)
            .filter((sourceId) => sourceId.startsWith('mapbox-gl-draw'))
            .forEach((sourceId) => (drawControlSources[sourceId] = currentStyle.sources[sourceId]));
          drawControlLayers = currentStyle.layers.filter((layer) => layer.id.startsWith('gl-draw'));
        }
      }
      catch (e) {
        // Ignore errors from getStyle
      }
    }
    const mapStyle = toJS(this.baseStyle);
    if (this.dataSources) {
      this.dataSources.forEach((dataSource) => {
        mapStyle.sources[dataSource.id] = dataSource.maplibreSourceSpecification;
      });
    }
    this.cachedSourceIds = Object.keys(mapStyle.sources);
    // Add mapbox-gl-draw datasources back into style
    Object.keys(drawControlSources).forEach((sourceId) => {
      mapStyle.sources[sourceId] = drawControlSources[sourceId];
    });
    if (this.layers) {
      this.layers.forEach((layer) => {
        var _a;
        const layerStyleSpec = layer.maplibreStyleSpecification;
        // Add composite layer datasources to map style
        // Note that composite layer datasource IDs are not prefixed with composite layer ID
        // This is so that the same datasource can be used by multiple composite layers
        for (const [key, value] of Object.entries((_a = layerStyleSpec.datasources) !== null && _a !== void 0 ? _a : {})) {
          mapStyle.sources[key] = value;
        }
        // To support composite layers - each layer now can compose a style spec that includes
        // multiple datasources and layers. Each sublayer provided by a FlowCoreMapLayer is merged into a flat list
        // of maplibre layers here.
        layerStyleSpec.layers.forEach((layerDef) => {
          const baseLayer = mapStyle.layers.find((l) => l.id === layerDef.id);
          if (baseLayer) {
            // Merge paint and layout properties with existing layer
            baseLayer.paint = Object.assign(Object.assign({}, baseLayer.paint), layerDef.paint);
            baseLayer.layout = Object.assign(Object.assign({}, baseLayer.layout), layerDef.layout);
          }
          else {
            // Append new layer
            const sourceId = layerDef['source'];
            const source = mapStyle.sources[sourceId];
            if (!source) {
              // Source not found for layer
              this.context.emitError(new Error(`<flow-map id="${this.id}">: Map data source '${sourceId}' not found for layer '${layerDef.id}'`));
              return;
            }
            // Default source layer to 'geometry' if missing
            if (source.type === 'vector' && !layerDef['source-layer']) {
              layerDef['source-layer'] = 'geometry';
            }
            else if (source.type === 'geojson') {
              delete layerDef['source-layer'];
            }
            let layerIndex = mapStyle.layers.length;
            if (layer.options.above) {
              const aboveIndex = mapStyle.layers.findIndex((l) => l.id === layer.options.above);
              if (aboveIndex > -1) {
                layerIndex = aboveIndex + 1;
              }
            }
            else if (layer.options.below) {
              const belowIndex = mapStyle.layers.findIndex((l) => l.id === layer.options.below);
              if (belowIndex > -1) {
                layerIndex = belowIndex;
              }
            }
            else if (mapStyle.metadata && mapStyle.metadata.insertionPoints) {
              const belowIndex = mapStyle.layers.findIndex((l) => l.id === mapStyle.metadata.insertionPoints[layerDef.type]);
              if (belowIndex > -1) {
                layerIndex = belowIndex;
              }
            }
            mapStyle.layers.splice(layerIndex, 0, layerDef);
          }
        });
      });
    }
    // Add mapbox-gl-draw layers back into style
    mapStyle.layers = mapStyle.layers.concat(drawControlLayers);
    return mapStyle;
  }
  /**
   * @internal
   *
   * Wait for the associated Flo.w context to be initialize.
   *
   * This method is called automatically by the `flow-map` web component.
   */
  async initializeContext() {
    if (!this.context) {
      this.context = await globalContextRegistry.getContext(this.flowContextId);
    }
    return this.context;
  }
  /**
   * Initialize the map.
   *
   * This method is called automatically by the `flow-map` web component. If instantiating
   * a `FlowCoreMap` directly, the developer is responsible for calling this method.
   *
   * @param styleUrl Base map style URL
   * @param sync Specify `true` to synchronize browser address bar with map location and zoom level.
   * @param options MaplibrexGL instantiation options.
   */
  async initialize(styleUrl, sync, options) {
    this.options = Object.assign(Object.assign(Object.assign({}, defaultMapOptions), options), { transformRequest: this.transformMapRequest.bind(this) });
    await this.initializeContext();
    await this.updateBaseStyle(styleUrl);
    this.maplibreMap = new MaplibreMap(Object.assign(Object.assign({}, this.options), { style: this.baseStyle }));
    this.mapContainer =
      typeof options.container === 'string' ? document.getElementById(options.container) : options.container;
    // Re-order canvas classes to fix keyboard issue with mapbox-gl-draw
    this.maplibreMap.getCanvas().classList.remove('maplibregl-canvas');
    this.maplibreMap.getCanvas().classList.add('maplibregl-canvas');
    const debouncedMouseMoveHandler = throttle(this.mouseMoveHandler.bind(this), MOUSE_MOVE_THROTTLE_DURATION, {
      leading: false,
      trailing: true
    });
    this.maplibreMap.on('styleimagemissing', this.handleMissingImage.bind(this));
    this.maplibreMap.on('mousemove', debouncedMouseMoveHandler);
    this.maplibreMap.on('click', this.clickHandler('click'));
    this.maplibreMap.on('dblclick', this.clickHandler('dblClick'));
    this.maplibreMap.on('error', (e) => {
      this.context.emitError(new MaplibreGLError(e.error));
    });
    if (sync) {
      this.initialSync = true;
      this.maplibreMap.on('moveend', this.moveEndHandler.bind(this));
    }
    // Forward additional events to component
    ['load', 'moveEnd'].forEach((e) => this.maplibreMap.on(e.toLowerCase(), (...args) => this.emit(e, ...args)));
    this.context.maps.registerMap(this.id, this);
  }
  /**
   * Uninitialize the map.
   *
   * Note that this is not called automatically by the `flow-map` web component because
   * there is no way to reliably detect when the component is removed from the DOM for disposal
   * rather than being moved around.
   *
   * The developer is responsible for calling this method when the map is no longer needed.
   */
  uninitialize() {
    var _a;
    if (this.context) {
      this.context.maps.unregisterMap(this.id);
    }
    // Ensure data sources are detached
    this.setDataSources([]);
    this.removeAllListeners();
    this.disconnect();
    (_a = this.maplibreMap) === null || _a === void 0 ? void 0 : _a.remove();
  }
  /* Initialization to be done every time the map is connected to the DOM
   * Note that this can be called multiple times (paired with disconnect())
   * as the map is moved around in the DOM.
   */
  connect() {
    this.disposer = autorun(() => {
      try {
        const styleDiff = diff(this.maplibreMap.getStyle(), this.mapStyle);
        debugMapStyle(`${this.id}: updating map style`, styleDiff);
      }
      catch (e) {
        // Ignore error - style will be set from scratch
        debugMapStyle(`${this.id}: setting map style`, this.mapStyle);
      }
      this.maplibreMap.setStyle(this.mapStyle);
      this.emit('mapStyleChanged', this.mapStyle);
    }, { name: `autorun: setStyle(FlowMap.${this.id})` });
    this.mapLoadingPollTimer = setInterval(this.pollMapLoading.bind(this), MAP_LOADING_POLL_INTERVAL);
  }
  /* Clean-up to be done every time the map is disconnected to the DOM
   * Note that this can be called multiple times (paired with connect())
   * as the map is moved around in the DOM.
   */
  disconnect() {
    var _a;
    clearInterval(this.mapLoadingPollTimer);
    (_a = this.disposer) === null || _a === void 0 ? void 0 : _a.call(this);
  }
  /**
   * Update the base map style and redraw the map.
   *
   * @param styleUrlOrId Base map style URL or ID
   */
  async updateBaseStyle(styleUrlOrId) {
    this.hoveredFeature = null;
    const baseStyle = await this.loadBaseStyle(styleUrlOrId);
    this.baseStyle = baseStyle;
  }
  /**
   * Programmatically set feature state properties for a map feature.
   *
   * @param feature Feature to modify.
   * @param state Feature state properties to set.
   */
  setFeatureState(feature, state) {
    // Add default sourceLayer for vector sources if not specified
    const source = this.maplibreMap.getSource(feature.source);
    if (source && source.type === 'vector') {
      feature.sourceLayer = feature.sourceLayer || 'geometry';
    }
    this.maplibreMap.setFeatureState(feature, state);
  }
  /**
   * Programmatically remove feature state properties from a map feature.
   *
   * @param feature Feature to modify. Specify `null` to modify all map features.
   * @param key Feature state key to remove. Specify `null` to remove all feature state for specified feature(s).
   */
  removeFeatureState(feature, key) {
    // Add default sourceLayer for vector sources if not specified
    const source = this.maplibreMap.getSource(feature.source);
    if (source && source.type === 'vector') {
      feature.sourceLayer = feature.sourceLayer || 'geometry';
    }
    if (typeof feature.id === 'undefined' && typeof key !== 'undefined') {
      // Maplibre API does not support removing specified key from ALL features
      // This functionality is added here using maplibregl's private API
      // Note this may break if the maplibregl version is updated
      const style = this.maplibreMap.style;
      style._checkLoaded();
      const sourceId = feature.source;
      const sourceCache = style.sourceCaches[sourceId];
      if (sourceCache === undefined) {
        this.context.emitError(new Error(`The source '${sourceId}' does not exist in the map's style.`));
        return;
      }
      const sourceState = sourceCache._state;
      const sourceType = sourceCache.getSource().type;
      const sourceLayer = sourceType === 'vector' ? feature.sourceLayer : '_geojsonTileLayer';
      if (sourceType === 'vector' && !sourceLayer) {
        this.context.emitError(new Error(`The sourceLayer parameter must be provided for vector source types.`));
        return;
      }
      sourceState.deletedStates[sourceLayer] = sourceState.deletedStates[sourceLayer] || {};
      const sourceLayerDeleted = sourceState.deletedStates[sourceLayer] === null;
      if (sourceLayerDeleted) {
        return;
      }
      if (sourceState.state[sourceLayer]) {
        Object.keys(sourceState.state[sourceLayer]).forEach((feature) => {
          if (sourceState.deletedStates[sourceLayer][feature] !== null) {
            sourceState.deletedStates[sourceLayer][feature] = sourceState.deletedStates[sourceLayer][feature] || {};
            sourceState.deletedStates[sourceLayer][feature][key] = null;
          }
        });
      }
      this.maplibreMap._update();
    }
    else {
      // Use standard behavior
      this.maplibreMap.removeFeatureState(feature, key);
    }
  }
  /**
   * Zoom the map by the specified increment.
   *
   * @param increment Zoom increment. Specify positive values to zoom in and negative values to zoom out.
   */
  async zoomBy(increment) {
    const zoom = Math.round(this.maplibreMap.getZoom() + increment);
    this.maplibreMap.zoomTo(zoom);
  }
  /**
   * Export the current map view as a PNG image.
   *
   * The image will be downloaded by the browser.
   *
   * @param options Export options.
   *
   * @returns A promise that resolves with the filename when the map has been exported.
   */
  export(options) {
    const mapExporter = new MapExporter(this.id, this.maplibreMap, this.mapContainer, this.context);
    return mapExporter.export(options);
  }
  /**
   * Set map layers.
   *
   * This method is called internally by the `<flow-map>` web component. If modified programmatically,
   * any modification will be overwritten by `<flow-map>`.
   *
   * @param layers Map layers.
   */
  setLayers(layers) {
    this.layers = layers;
  }
  /**
   * Returns the specified map layer.
   *
   * @param id Map layer ID.
   */
  getLayer(id) {
    return this.layers.find((l) => {
      if (!l) {
        debug('Layer not found', id);
      }
      return l.id === id;
    });
  }
  /**
   * Set map data sources.
   *
   * This method is called internally by the `<flow-map>` web component. If modified programmatically,
   * any modification will be overwritten by `<flow-map>`.
   *
   * @param dataSources Map data sources.
   */
  setDataSources(dataSources) {
    const datasourcesToRemove = this.dataSources.filter((ds) => !dataSources.includes(ds));
    const datasourcesToAdd = dataSources.filter((ds) => !this.dataSources.includes(ds));
    datasourcesToRemove.forEach((ds) => {
      var _a;
      // Invalidate hovered feature if datasource is removed
      if (ds.id === ((_a = this.hoveredFeature) === null || _a === void 0 ? void 0 : _a.source)) {
        this.hoveredFeature = null;
      }
      ds.detach();
    });
    datasourcesToAdd.forEach((ds) => ds.attach(this));
    this.dataSources = dataSources;
  }
  /**
   * Returns the specified map data source.
   *
   * @param id Data source ID.
   */
  getDataSource(id) {
    if (!id) {
      return undefined;
    }
    // Check root datasources
    let datasource = this.dataSources.find((ds) => ds.id === id);
    // Check composite layers for matching datasource id
    if (!datasource) {
      for (const layer of this.layers) {
        if (!isCompositeLayer(layer)) {
          continue;
        }
        datasource = layer.datasources.find((ds) => ds.id === id);
        if (datasource)
          break;
      }
    }
    return datasource;
  }
  /**
   * Register named popup options. Used internally to display automatic popups.
   *
   * @param popupOptions Popup options.
   */
  registerPopupOptions(popupOptions) {
    this.popupTemplates = popupOptions;
  }
  /**
   * Returns the specified registered popup options.
   *
   * @param id Registered popup options ID.
   */
  getPopupOptions(id) {
    return this.popupTemplates.find((t) => t.id === id);
  }
  /**
   * Open a map popup at the specified location.
   *
   * @param lngLat Map location to open popup.
   * @param optionsOrPopupName Popup options or ID of registered map popup.
   */
  openPopup(lngLat, optionsOrPopupName) {
    const options = typeof optionsOrPopupName === 'string' ? this.getPopupOptions(optionsOrPopupName) : optionsOrPopupName;
    const popup = new FlowCoreMapPopup(this, options);
    popup.open(lngLat);
    return popup;
  }
  /**
   * Close all open popups for all data sources
   */
  closeAllPopups() {
    this.dataSources.forEach((ds) => ds.closeAllPopups());
  }
  startDrawing(options) {
    if (!this.maplibreMap) {
      throw new Error('Map not initialized');
    }
    if (this.drawControl) {
      return this.drawControl;
    }
    this.drawControl = new MapboxDraw(Object.assign(Object.assign({}, options), { userProperties: true, modes: Object.assign(Object.assign(Object.assign({}, MapboxDraw.modes), { draw_circle: CircleMode, drag_circle: DragCircleMode, direct_select: DirectMode, simple_select: SimpleSelectMode }), options.modes) }));
    this.maplibreMap.addControl(this.drawControl, options.controlPosition);
    // Add icons to buttons
    const buttons = {
      'mapbox-gl-draw_line': 
      // tslint:disable-next-line: quotemark
      "data:image/svg+xml,%3Csvg xmlns='http://www.w3.org/2000/svg' width='20' height='20' viewBox='0 0 20 20' %3E%3Cpath d='m13.5 3.5c-1.4 0-2.5 1.1-2.5 2.5 0 .3 0 .6.2.9l-3.8 3.8c-.3-.1-.6-.2-.9-.2-1.4 0-2.5 1.1-2.5 2.5s1.1 2.5 2.5 2.5 2.5-1.1 2.5-2.5c0-.3 0-.6-.2-.9l3.8-3.8c.3.1.6.2.9.2 1.4 0 2.5-1.1 2.5-2.5s-1.1-2.5-2.5-2.5z' fill='white'/%3E%3C/svg%3E",
      // tslint:disable-next-line: quotemark
      'mapbox-gl-draw_polygon': "data:image/svg+xml,%3Csvg xmlns='http://www.w3.org/2000/svg' width='20' height='20' viewBox='0 0 20 20' %3E%3Cpath d='m15 12.3v-4.6c.6-.3 1-1 1-1.7 0-1.1-.9-2-2-2-.7 0-1.4.4-1.7 1h-4.6c-.3-.6-1-1-1.7-1-1.1 0-2 .9-2 2 0 .7.4 1.4 1 1.7v4.6c-.6.3-1 1-1 1.7 0 1.1.9 2 2 2 .7 0 1.4-.4 1.7-1h4.6c.3.6 1 1 1.7 1 1.1 0 2-.9 2-2 0-.7-.4-1.4-1-1.7zm-8-.3v-4l1-1h4l1 1v4l-1 1h-4z'/%3E%3C/svg%3E",
      // tslint:disable-next-line: quotemark
      'mapbox-gl-draw_point': "data:image/svg+xml,%3Csvg xmlns='http://www.w3.org/2000/svg' width='20' height='20' viewBox='0 0 20 20'%3E%3Cpath d='m10 2c-3.3 0-6 2.7-6 6s6 9 6 9 6-5.7 6-9-2.7-6-6-6zm0 2c2.1 0 3.8 1.7 3.8 3.8 0 1.5-1.8 3.9-2.9 5.2h-1.7c-1.1-1.4-2.9-3.8-2.9-5.2-.1-2.1 1.6-3.8 3.7-3.8z' fill='white'/%3E%3C/svg%3E",
      // tslint:disable-next-line: quotemark
      'mapbox-gl-draw_trash': "data:image/svg+xml,%3Csvg xmlns='http://www.w3.org/2000/svg' width='20' height='20' viewBox='0 0 20 20'%3E%3Cpath d='M10,3.4 c-0.8,0-1.5,0.5-1.8,1.2H5l-1,1v1h12v-1l-1-1h-3.2C11.5,3.9,10.8,3.4,10,3.4z M5,8v7c0,1,1,2,2,2h6c1,0,2-1,2-2V8h-2v5.5h-1.5V8h-3 v5.5H7V8H5z' fill='white'/%3E%3C/svg%3E",
      // tslint:disable-next-line: quotemark
      'mapbox-gl-draw_combine': "data:image/svg+xml,%3Csvg xmlns='http://www.w3.org/2000/svg' width='20' height='20' viewBox='0 0 20 20'%3E%3Cpath d='M12.1,2c-0.3,0-0.5,0.1-0.7,0.3l-1,1c-0.4,0.4-0.4,1,0,1.4l4.9,4.9c0.4,0.4,1,0.4,1.4,0l1-1 c0.4-0.4,0.4-1,0-1.4l-4.9-4.9C12.6,2.1,12.3,2,12.1,2z M8,8C7,8,7,9,7.5,9.5c0.3,0.3,1,1,1,1l-1,1c0,0-0.5,0.5,0,1s1,0,1,0l1-1l1,1 C11,13,12,13,12,12V8H8z M4,10c-0.3,0-0.5,0.1-0.7,0.3l-1,1c-0.4,0.4-0.4,1,0,1.4l1,1c0.4,0.4,1,0.4,1.4,0l1-1c0.4-0.4,0.4-1,0-1.4 l-1-1C4.5,10.1,4.3,10,4,10z M8,14c-0.3,0-0.5,0.1-0.7,0.3l-1,1c-0.4,0.4-0.4,1,0,1.4l1,1c0.4,0.4,1,0.4,1.4,0l1-1 c0.4-0.4,0.4-1,0-1.4l-1-1C8.5,14.1,8.3,14,8,14z' fill='white'/%3E%3C/svg%3E",
      // tslint:disable-next-line: quotemark
      'mapbox-gl-draw_uncombine': "data:image/svg+xml,%3Csvg xmlns='http://www.w3.org/2000/svg' width='20' height='20' viewBox='0 0 20 20'%3E%3Cpath d='m12 2c-.3 0-.5.1-.7.3l-1 1c-.4.4-.4 1 0 1.4l1 1c.4.4 1 .4 1.4 0l1-1c.4-.4.4-1 0-1.4l-1-1c-.2-.2-.4-.3-.7-.3zm4 4c-.3 0-.5.1-.7.3l-1 1c-.4.4-.4 1 0 1.4l1 1c.4.4 1 .4 1.4 0l1-1c.4-.4.4-1 0-1.4l-1-1c-.2-.2-.4-.3-.7-.3zm-7 1c-1 0-1 1-.5 1.5.3.3 1 1 1 1l-1 1s-.5.5 0 1 1 0 1 0l1-1 1 1c.5.5 1.5.5 1.5-.5v-4zm-5 3c-.3 0-.5.1-.7.3l-1 1c-.4.4-.4 1 0 1.4l4.9 4.9c.4.4 1 .4 1.4 0l1-1c.4-.4.4-1 0-1.4l-4.9-4.9c-.1-.2-.4-.3-.7-.3z' fill='white'/%3E%3C/svg%3E"
    };
    Object.keys(buttons).forEach((btnClass) => {
      const elements = document.getElementsByClassName(btnClass);
      if (elements.length) {
        elements[0].innerHTML = `<flow-icon icon="${buttons[btnClass]}"></flow-icon>`;
      }
    });
    mapboxDrawEvents.forEach((eventName) => {
      this.maplibreMap.on(eventName, this.forwardEvent);
    });
    if (options.exclusive) {
      this.exclusiveDrawMode = true;
      if (this.hoveredFeature) {
        // Removed hovered feature
        this.maplibreMap.removeFeatureState(this.hoveredFeature, 'hover');
        this.hoveredFeature = null;
      }
    }
    return this.drawControl;
  }
  endDrawing() {
    if (!this.maplibreMap) {
      throw new Error('Map not initialized');
    }
    if (this.drawControl) {
      this.maplibreMap.removeControl(this.drawControl);
      this.drawControl = null;
      mapboxDrawEvents.forEach((eventName) => {
        this.maplibreMap.off(eventName, this.forwardEvent);
      });
      this.exclusiveDrawMode = false;
    }
  }
  pollMapLoading() {
    const mapLoading = this.cachedSourceIds.reduce((isBusy, source) => isBusy || !this.maplibreMap.isSourceLoaded(source), false);
    if (mapLoading !== this.mapLoading) {
      this.mapLoading = mapLoading;
      this.emit('maploading', this.mapLoading);
    }
  }
  async handleMissingImage(e) {
    // Append map namespace to image id
    const imageId = this.id ? `${this.id}.${e.id}` : e.id;
    const imageInfo = this.context.images.getImageInfo(imageId);
    if (imageInfo) {
      if (!imageInfo.image.complete) {
        // Image hasn't loaded yet - trigger an update once it has loaded
        imageInfo.image.onload = () => {
          this.maplibreMap.updateImage(e.id, imageInfo.image, { pixelRatio: 2 });
        };
      }
      this.maplibreMap.addImage(e.id, imageInfo.image, { pixelRatio: imageInfo.pixelDensity, sdf: imageInfo.recolor });
    }
  }
  mouseMoveHandler(event) {
    const layerIds = this.getLayerIds();
    const features = this.maplibreMap.queryRenderedFeatures(this.featureQueryBox(event), { layers: layerIds });
    this.emit('mousemove', event, features);
    if (this.exclusiveDrawMode) {
      // Do not handle hover if in exclusive draw mode
      return;
    }
    const feature = features[0];
    if (!this.compareFeatures(feature, this.hoveredFeature)) {
      if (this.hoveredFeature && this.maplibreMap.getSource(this.hoveredFeature.source)) {
        // Remove old hovered feature
        this.maplibreMap.removeFeatureState(this.hoveredFeature, 'hover');
        this.hoveredFeature = null;
      }
      if (feature && this.maplibreMap.getSource(feature.source) && !isNil(feature.id)) {
        // Set new hovered feature
        this.hoveredFeature = feature;
        this.maplibreMap.setFeatureState(this.hoveredFeature, { hover: true });
      }
    }
  }
  clickHandler(type) {
    return (event) => {
      // Clicks are ignored in exclusive draw mode
      if (this.exclusiveDrawMode) {
        return;
      }
      const layerIds = this.getLayerIds();
      const features = this.maplibreMap.queryRenderedFeatures(this.featureQueryBox(event), { layers: layerIds });
      const multiSelect = event.originalEvent.shiftKey || event.originalEvent.altKey || event.originalEvent.metaKey;
      if (features.length) {
        debug(`Feature ${type}ed`, features[0]);
      }
      if (type === 'click') {
        // Notify datsources to handle selection for 'click' event only
        const clickedPoint = [event.lngLat.lng, event.lngLat.lat];
        const clickedFeature = features.length && features[0];
        const clickedDataset = this.getDataSource(clickedFeature === null || clickedFeature === void 0 ? void 0 : clickedFeature.source);
        const selectionGroup = clickedDataset === null || clickedDataset === void 0 ? void 0 : clickedDataset.options.selectionGroup;
        // Forward click to datasources for selection handling
        this.dataSources.forEach((ds) => ds.featureClicked(clickedFeature, multiSelect, selectionGroup, clickedPoint));
        // Forward to composite layer datasources
        this.layers
          .filter((layer) => isCompositeLayer(layer))
          .forEach((layer) => {
          layer.datasources.forEach((ds) => ds.featureClicked(features.length && features[0], multiSelect, selectionGroup, clickedPoint));
        });
      }
      this.emit(type, Object.assign(Object.assign({}, event), { features }));
    };
  }
  featureQueryBox(event) {
    const queryBox = this.options.featureClickTolerance
      ? [
        [event.point.x - this.options.featureClickTolerance, event.point.y - this.options.featureClickTolerance],
        [event.point.x + this.options.featureClickTolerance, event.point.y + this.options.featureClickTolerance]
      ]
      : event.point;
    return queryBox;
  }
  moveEndHandler(event) {
    if (this.initialSync) {
      // Sync map state from URL
      this.initialSync = false;
      const searchParams = new URLSearchParams(window.location.search);
      const hash = searchParams.get(this.id);
      if (hash) {
        this.maplibreMap.jumpTo(this.hashToMapState(hash));
      }
    }
    else {
      // Sync map state to URL
      const searchParams = new URLSearchParams(window.location.search);
      searchParams.set(this.id, this.mapStateToHash());
      const newUrl = `${location.pathname}?${searchParams.toString()}`;
      window.history.replaceState({}, '', newUrl);
    }
  }
  mapStateToHash() {
    const state = {
      z: this.maplibreMap.getZoom().toFixed(3),
      x: this.maplibreMap.getCenter().lng.toFixed(3),
      y: this.maplibreMap.getCenter().lat.toFixed(3),
      pitch: this.maplibreMap.getPitch().toFixed(1),
      bearing: this.maplibreMap.getBearing().toFixed(1)
    };
    return `${state.z}_${state.x}_${state.y}_${state.pitch}_${state.bearing}`;
  }
  hashToMapState(hash) {
    const [zoom, x, y, pitch, bearing] = hash.split('_');
    return {
      center: [+x, +y],
      zoom: +zoom,
      pitch: +pitch,
      bearing: +bearing
    };
  }
  transformMapRequest(url, resourceType) {
    if (this.context.flowClient.accessToken && url.startsWith(this.context.host)) {
      return {
        url,
        headers: { Authorization: `Bearer ${this.context.flowClient.accessToken}` }
      };
    }
  }
  loadBaseStyle(styleUrl) {
    /* No longer supported in maplibre
    const mapboxMatch = styleUrl.match(MapboxRegex);
    if (mapboxMatch) {
      // Expand mapbox-style url mapbox://styles/<user>/<map>?access_token=<token>'
      // and store access token
      styleUrl = `https://api.mapbox.com/styles/v1/${mapboxMatch[1]}/${mapboxMatch[2]}?access_token=${mapboxMatch[3]}`;
      this.mapboxAccessToken = mapboxMatch[3];
    }
    */
    if (isURL(styleUrl)) {
      // URL base style
      return fetch(styleUrl).then((resp) => resp.json());
    }
    else {
      // Flo.w base style
      return this.context.flowClient.maps.get(styleUrl);
    }
  }
  compareFeatures(f1, f2) {
    if (f1 === f2) {
      return true;
    }
    if (isObject(f1) && isObject(f2)) {
      return f1.id == f2.id && f1.source === f2.source && f1.sourceLayer == f2.sourceLayer;
    }
    return false;
  }
  getLayerIds(filterInteractive = true) {
    const flatLayers = this.layers
      .map((layer) => {
      if (isCompositeLayer(layer) && (!filterInteractive || layer.options.interactive)) {
        return layer.layers;
      }
      else {
        return [layer];
      }
    })
      .flat();
    return flatLayers.filter((layer) => !filterInteractive || layer.options.interactive).map((layer) => layer.id);
  }
}
__decorate([
  observable
], FlowCoreMap.prototype, "baseStyle", void 0);
__decorate([
  observable
], FlowCoreMap.prototype, "layers", void 0);
__decorate([
  observable
], FlowCoreMap.prototype, "dataSources", void 0);
__decorate([
  computed
], FlowCoreMap.prototype, "mapStyle", null);
