/*!
 * (C) Emu Analytics
 */
import { randomId } from '../../utils/random-id';
// Removed because of dependency on file-saver git repo https://github.com/MrRio/jsPDF/pull/2203
// import jsPDF from 'jspdf';
import maplibregl from '@emuanalytics/maplibre-gl';
import saveAs from 'file-saver';
import { isFunction } from '../../utils/checks.util';
/**
 * @internal
 */
export class MapExporter {
  constructor(mapId, map, hostElement, flowContext) {
    this.mapId = mapId;
    this.map = map;
    this.hostElement = hostElement;
    this.flowContext = flowContext;
    this.nativePixelRatio = window.devicePixelRatio;
  }
  async export(options) {
    options = Object.assign(Object.assign({}, MapExporter.defaultOptions), options);
    const [container, mapCopy] = await this.createMapCopy(options);
    return new Promise((resolve, reject) => {
      const fileName = `${options.name}.${options.format}`;
      const mapCanvas = mapCopy.getCanvas();
      const exportCanvas = this.createExportCanvas(container, mapCanvas);
      const exportContext = exportCanvas.getContext('2d');
      // Callback function supplied to client to add an overlay to the export
      // Calling this function finalizes the export
      const drawOverlayCallback = (err = null) => {
        if (err) {
          // overlay callback reported an error
          this.destroyMapCopy(container, mapCopy);
          reject(err);
        }
        else {
          switch (options.format) {
            case 'png':
              exportCanvas.toBlob((blob) => saveAs(blob, fileName));
              break;
            case 'pdf':
              throw new Error('Export to PDF is not currently supported');
              // this.buildPdf(canvas2d, fileName, options.width, options.height);
              break;
          }
          this.destroyMapCopy(container, mapCopy);
          resolve(fileName);
        }
      };
      // Draw map to export canvas
      const mapImage = new Image();
      mapImage.onload = () => {
        exportContext.drawImage(mapImage, 0, 0);
        // Draw overlay on top of map required
        if (isFunction(options.drawOverlayFn)) {
          options.drawOverlayFn(exportContext, exportCanvas.width, exportCanvas.height, options.dpmm, drawOverlayCallback);
        }
        else {
          // Finish without calling out to overlay handler
          drawOverlayCallback();
        }
      };
      // Setting image src triggers map to render to canvas
      // onLoad handler (above) will be called once complete to finalise the export
      mapImage.src = mapCopy.getCanvas().toDataURL();
    });
  }
  createMapCopy(options) {
    // Geographical coordinates of desired export area
    const geoBounds = options.bounds || this.map.getBounds().toArray().flat();
    const geoBottomLeft = [geoBounds[0], geoBounds[1]];
    const geoTopRight = [geoBounds[2], geoBounds[3]];
    const geoCenter = [(geoBottomLeft[0] + geoTopRight[0]) / 2, (geoBottomLeft[1] + geoTopRight[1]) / 2];
    // Projected pixel coordinates of desired export area
    const pxBottomLeft = this.map.project(geoBottomLeft);
    const pxTopRight = this.map.project(geoTopRight);
    const pxWidth = pxTopRight.x - pxBottomLeft.x;
    const pxHeight = pxBottomLeft.y - pxTopRight.y; // Pixel coordinates are inverted
    const exportWidth = options.width * options.dpmm;
    const exportHeight = options.height * options.dpmm;
    const aspectRatios = {
      width: pxWidth / exportWidth,
      height: pxHeight / exportHeight
    };
    const coverRatio = Math.max(aspectRatios.width, aspectRatios.height);
    // const containRatio = Math.min(aspectRatios.width, aspectRatios.height);
    const copyWidth = exportWidth * coverRatio;
    const copyHeight = exportHeight * coverRatio;
    const pixelRatio = exportWidth / copyWidth;
    this.setDevicePixelRatio(pixelRatio);
    // Create container for map copy
    const container = document.createElement('div');
    container.id = randomId('export');
    container.style.width = `${copyWidth}px`;
    container.style.height = `${copyHeight}px`;
    this.hostElement.insertBefore(container, this.hostElement.firstChild);
    // Create map copy to use for export
    const mapCopy = new maplibregl.Map({
      container,
      center: geoCenter,
      zoom: this.map.getZoom(),
      style: Object.assign(Object.assign({}, this.map.getStyle()), { transition: { delay: 0, duration: 0 } }),
      bearing: this.map.getBearing(),
      pitch: this.map.getPitch(),
      interactive: false,
      attributionControl: false,
      fadeDuration: 0,
      preserveDrawingBuffer: true,
      transformRequest: this.transformMapRequest.bind(this)
    });
    mapCopy.on('styleimagemissing', this.handleMissingImage.bind(this));
    return new Promise((resolve) => {
      mapCopy.once('load', () => {
        // copy feature data to mapCopy
        Object.keys(this.map.style.sourceCaches).forEach((sourceId) => {
          mapCopy.style.sourceCaches[sourceId]._state.stateChanges = this.map.style.sourceCaches[sourceId]._state.state;
        });
        mapCopy._update();
        setTimeout(() => resolve([container, mapCopy]), 500);
      });
    });
  }
  destroyMapCopy(container, map) {
    this.setDevicePixelRatio(this.nativePixelRatio);
    map.remove();
    container.parentNode.removeChild(container);
  }
  createExportCanvas(container, mapCanvas) {
    const canvasContainer = container.getElementsByClassName('maplibregl-canvas-container')[0];
    const overlayCanvas = document.createElement('canvas');
    overlayCanvas.id = randomId('overlay');
    overlayCanvas.width = mapCanvas.width;
    overlayCanvas.height = mapCanvas.height;
    overlayCanvas.style.position = 'absolute';
    overlayCanvas.style.width = mapCanvas.style.width;
    overlayCanvas.style.height = mapCanvas.style.height;
    canvasContainer.appendChild(overlayCanvas);
    return overlayCanvas;
  }
  setDevicePixelRatio(pixelRatio) {
    Object.defineProperty(window, 'devicePixelRatio', {
      get: () => pixelRatio
    });
  }
  async handleMissingImage(e) {
    const map = e.target;
    // Append map namespace to image id
    const imageId = this.mapId ? `${this.mapId}.${e.id}` : e.id;
    const imageInfo = this.flowContext.images.getImageInfo(imageId);
    if (imageInfo) {
      if (!imageInfo.image.complete) {
        // Image hasn't loaded yet - trigger an update once it has loaded
        imageInfo.image.onload = () => {
          map.updateImage(e.id, imageInfo.image, { pixelRatio: 2 });
        };
      }
      map.addImage(e.id, imageInfo.image, { pixelRatio: imageInfo.pixelDensity, sdf: imageInfo.recolor });
    }
  }
  transformMapRequest(url, resourceType) {
    if (this.flowContext.flowClient.accessToken && url.startsWith(this.flowContext.host)) {
      return {
        url,
        headers: { Authorization: `Bearer ${this.flowContext.flowClient.accessToken}` }
      };
    }
  }
  buildPdf(canvas, name, width, height) {
    /*const pdf = new jsPDF({
      orientation: width > height ? 'l' : 'p',
      unit: 'mm',
      // Format must be expressed in 'pt' in v1.53 of jsPDF
      format: [width * 2.834, height * 2.834],
      compress: true
    });

    pdf.addImage(canvas.toDataURL('image/png'), 'png', 0, 0, width, height, null, 'FAST');

    pdf.save(name);
    */
  }
}
MapExporter.defaultOptions = {
  name: 'map',
  width: 297,
  height: 210,
  dpmm: 12,
  format: 'png',
  drawOverlayFn: null
};
