/*!
 * (C) Emu Analytics
 */
import { isArray, isBoolean, isNumber, isString } from '../../../utils/checks.util';
export class MapLayerStyleBuilder {
  constructor() {
    this._styles = new Map();
  }
  static registerStyleBuilder(layerType, builder) {
    this._styleBuilders.set(layerType, builder);
  }
  static createStyleBuilder(layerType) {
    const builder = this._styleBuilders.get(layerType);
    if (!builder) {
      throw new Error(`MapLayerStyleBuilder: Unknown layer type '${layerType}'`);
    }
    return new builder();
  }
  /**
   * Whether this layer is displayed.
   *
   * @param value - visible value (default `true`)
   */
  visibility(value) {
    this.addStyle('visibility', value);
    return this;
  }
  /**
   * Create a style specification string.
   *
   * Call this method after setting all style properties
   * on the style builder.
   *
   * @returns Layer style specification string
   */
  build() {
    const styles = [];
    for (const [prop, value] of this._styles.entries()) {
      styles.push(`${prop}: ${value};`);
    }
    return styles.join('\n');
  }
  /**
   * Format and add style property to the style builder.
   */
  addStyle(property, value) {
    this._styles.set(property, this.formatValue(value));
  }
  formatValue(value) {
    if (value instanceof StyleExpression) {
      return value.toString();
    }
    else if (isString(value)) {
      return `'${value.toString()}'`;
    }
    else if (isNumber(value)) {
      return value.toString();
    }
    else if (isBoolean(value)) {
      return (!!value).toString();
    }
    else if (isArray(value)) {
      return `[${value.map((v) => this.formatValue(v)).join(', ')}]`;
    }
    else {
      return value.toString();
    }
  }
}
MapLayerStyleBuilder._styleBuilders = new Map();
// Wrapper to allow style expressions to be passed to style builder methods
// of non-string types.
export class StyleExpression {
  constructor(_expr) {
    this._expr = _expr;
  }
  toString() {
    return this._expr;
  }
}
export function expr(expr) {
  return new StyleExpression(expr);
}
