/*!
 * (C) Emu Analytics
 */
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
  var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
  if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
  else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
  return c > 3 && r && Object.defineProperty(target, key, r), r;
};
import { debugQuery } from '../../utils/debug';
import { computed, createAtom } from 'mobx';
import { nonenumerable } from '../../utils/non-enumeable';
import { isNil, isPlainObject } from '../../utils/checks.util';
import { QueryTimeoutError } from '../errors/query-timeout-error';
/**
 * A `FlowQuery` defines a Flo.w reactive dataset query that is executed in response to changes
 * in application state properties.
 *
 * To register a Flo.w reactive dataset query call the [[StateStore.registerQuery]] method.
 *
 * The query is defined by a supplied query definition function that takes the current application state as an
 * argument and should return a [[FlowQueryDefinition]]. Changes in application state properties referenced in
 * the function will trigger the query to execute. For example,
 *
 * ```javascript
 * (state) => {
 *  return {
 *    datasetId: 'airquality_with_params',
 *    limit: 10,
 *    params: {no2_filter: state.filterValue}
 *  };
 * }
 * ```
 *
 * To transform the returned data specify a result transform function, which accepts the raw query results
 * as an array and should return a transformed value. For example,
 *
 * ```javascript
 * (results) => {
 *  // Return the first results object from the results array
 *  if (results && results.length) {
 *    return results[0];
 *  } else {
 *    return null;
 *  }
 * }
 * ```
 *
 * `FlowQuery` exposes two reactive properties:
 *
 * - [[busy]] - indicates that the query is currently executing.
 * - [[results]] - the query results.
 *
 * @typeParam TResults Type of returned data (including transformation, if any).
 * @typeParam TState Type of context state.
 * ```
 */
export class FlowQuery {
  /**
   * Constructs a `FlowQuery` instance.
   *
   * @param context Flo.w context.
   * @param queryId Query ID.
   * @param queryFn Query definition.
   * @param resultTransformFn Result transform function.
   * @param delay Debounce delay (milliseconds)
   */
  constructor(context, queryId, queryFn, resultTransformFn, delay = 0) {
    this.context = context;
    this.queryId = queryId;
    this.queryFn = queryFn;
    this.resultTransformFn = resultTransformFn;
    this.delay = delay;
    this.running = false;
    this._error = null;
    nonenumerable(this, 'busyAtom');
    nonenumerable(this, 'context');
    nonenumerable(this, 'asyncCompute');
    this.busyAtom = createAtom(`${queryId}-busy`);
    this.errorAtom = createAtom(`${queryId}-error`);
    this.asyncCompute = this.context.state.asyncCompute((state) => this.runQuery(state), null, this.delay, `async-computed: run(FlowQuery.${queryId})`);
  }
  /**
   * Manually run the query. This method is provided for debugging purposes and should not generally
   * be used. Rely on reactivity to run queries when required.
   */
  runQueryImmediate() {
    let results;
    this.context.state.autorun((state) => {
      results = this.runQuery(state);
    }, `autorun: runQueryImmediate(FlowQuery.${this.queryId})`)();
    return results;
  }
  async runQuery(state) {
    this.running = true;
    this.busyAtom.reportChanged();
    this._error = null;
    this.errorAtom.reportChanged();
    const query = await this.queryFn(state);
    debugQuery(`Running query '${this.queryId}'`, query);
    let results = null;
    if (isNil(query)) {
      // Null returned from queryFn
      // Set results to null and bypass running query
      results = null;
    }
    else if (isPlainObject(query)) {
      const datsetId = query.datasetId;
      delete query.datasetId;
      const params = Object.assign(Object.assign({}, query.params), { format: query.format || 'json' });
      results = await this.context.flowClient.datasets.query(datsetId, query, params);
    }
    else {
      this.context.emitError(new Error(`Unrecognised query definition for ${this.queryId}`));
    }
    if (this.resultTransformFn) {
      results = this.resultTransformFn(results);
    }
    this.running = false;
    this.busyAtom.reportChanged();
    return results;
  }
  /**
   * `true` if the query is currently executing and awaiting results.
   *
   * Use this property to indicate busy state in the UI.
   *
   * <flow-badge pill>Reactive</flow-badge>
   */
  get busy() {
    this.busyAtom.reportObserved();
    return this.running;
  }
  /**
   * If non-null, indicates that a query error occurred
   *
   * Use this property to indicate error state in the UI.
   *
   * <flow-badge pill>Reactive</flow-badge>
   */
  get error() {
    this.errorAtom.reportObserved();
    return this._error;
  }
  /**
   * The query results.
   *
   * <flow-badge pill>Reactive</flow-badge>
   */
  get results() {
    try {
      return this.asyncCompute.get();
    }
    catch (err) {
      this.running = false;
      this.busyAtom.reportChanged();
      this._error = this.normalizeError(err);
      if (this.errorAtom.isBeingObserved) {
        debugQuery(`Query '${this.queryId}' failed`, this._error);
        this.errorAtom.reportChanged();
        return null;
      }
      else {
        // Error is not being observed - throw exception
        throw this._error;
      }
    }
  }
  /**
   * Lazy version of `results` that does not keepAlive.
   *
   * <flow-badge pill>Reactive</flow-badge>
   */
  get lazyResults() {
    try {
      return this.asyncCompute.get();
    }
    catch (err) {
      this.running = false;
      this.busyAtom.reportChanged();
      this._error = this.normalizeError(err);
      if (this.errorAtom.isBeingObserved) {
        debugQuery(`Query '${this.queryId}' failed`, this._error);
        this.errorAtom.reportChanged();
        return null;
      }
      else {
        // Error is not being observed - throw exception
        throw this._error;
      }
    }
  }
  /**
   * @internal
   *
   * Get query results in non-reactive context
   *
   */
  get resultsNonReactive() {
    return this.asyncCompute.getNonReactive();
  }
  /**
   * Cancel the currently running query.
   */
  cancel() {
    this.running = false;
    this.busyAtom.reportChanged();
    this.asyncCompute.cancel();
  }
  normalizeError(err) {
    if (err.message === 'canceling statement due to statement timeout') {
      err = new QueryTimeoutError(this.queryId);
    }
    return err;
  }
}
__decorate([
  computed
], FlowQuery.prototype, "busy", null);
__decorate([
  computed({ keepAlive: true })
], FlowQuery.prototype, "results", null);
__decorate([
  computed({ keepAlive: false })
], FlowQuery.prototype, "lazyResults", null);
