/*!
 * (C) Emu Analytics
 */
import Debug from 'debug';
const debug = Debug('flow:image-registry');
/**
 * The `ImageRegistry` is a container for map icon images.
 *
 * Images can be registered and retrieved by ID.
 */
export class ImageRegistry {
  /**
   * Constructs a new `ImageRegistry`.
   *
   * @param context The parent Flo.w context object.
   */
  constructor(context) {
    this.context = context;
    this.registry = new Map();
  }
  /**
   * Register a map icon image.
   *
   * @param id Image ID.
   * @param src Image source URL.
   * @param width Image display width.
   * @param height Image display height.
   * @param pixelDensity Image pixel density. Defaults to `2` - the source image should be @2x resolution to enable better quality rendering on high-resolution displays.
   * @param recolor Set to `true` to allow recoloring of map icon image using map layer style properties.
   */
  registerImage(id, src, width, height, pixelDensity = 2, recolor = false) {
    debug(`Registering image '${id}'`);
    const entry = { id, src, width, height, pixelDensity, recolor };
    this.registry.set(id, entry);
    return entry;
  }
  /**
   * Un-register a map icon image.
   *
   * @param id Image ID.
   */
  unregisterImage(id) {
    debug(`Unregistering image '${id}'`);
    this.registry.delete(id);
  }
  /**
   * Get a map icon image.
   *
   * @param id Image ID.
   * @returns The map icon image as an `HTMLImageElement`.
   */
  getImage(id) {
    const registryEntry = this.getImageInfo(id);
    return registryEntry === null || registryEntry === void 0 ? void 0 : registryEntry.image;
  }
  /**
   * Get an image registry entry.
   *
   * @param id Image ID.
   * @returns The image registry entry.
   */
  getImageInfo(id) {
    debug(`Retrieving image '${id}'`);
    if (!this.registry.has(id)) {
      const [namespace, globalId] = this.splitNamespace(id);
      if (namespace) {
        // Try looking up in global namespace
        return this.getImageInfo(globalId);
      }
      else {
        // Not found
        return null;
      }
    }
    const registryEntry = this.registry.get(id);
    if (!registryEntry.image) {
      registryEntry.image = this.context.imageLoader.loadImage(registryEntry.src, registryEntry.width, registryEntry.height, registryEntry.pixelDensity);
    }
    return registryEntry;
  }
  /**
   * Splits an id of the form 'namespace.basename' and returns both components. The split point is
   * the first '.' so namespaces may not contain dots.
   *
   * @returns namespace, basename
   */
  splitNamespace(id) {
    const dotPos = id.indexOf('.');
    if (dotPos === -1) {
      return [null, id];
    }
    return [id.substring(0, dotPos), id.substring(dotPos + 1)];
  }
}
