/*!
 * (C) Emu Analytics
 */
import Debug from 'debug';
import { deferred } from '../utils/deferred';
const debug = Debug('flow:map-registry');
/**
 * The `MapRegistry` is a container for [[FlowCoreMap]] instances.
 *
 * Maps can be registered and retrieved by ID.
 */
export class MapRegistry {
  constructor() {
    this.maps = new Map();
  }
  /**
   * Register a Flo.w map.
   *
   * @param id Map ID.
   * @param map Map to register.
   */
  registerMap(id, map) {
    debug(`Registering map '${id}'`);
    if (!this.maps.get(id)) {
      this.maps.set(id, deferred());
    }
    const deferredMap = this.maps.get(id);
    map.once('load', () => {
      debug(`Registered map '${id}'`);
      deferredMap.resolve(map);
    });
  }
  /**
   * Un-register a Flo.w map.
   *
   * @param id Map ID.
   */
  unregisterMap(id) {
    debug(`Unregistering map '${id}'`);
    this.maps.delete(id);
  }
  /**
   * Retrieve the specified map.
   *
   * This method returns a promise that resolves when the map has been initialized.
   *
   * @param id Map ID.
   */
  getMap(id) {
    if (!this.maps.get(id)) {
      // Map hasn't been registered yet. A promise is created that will
      // eventually resolve when the map is registered.
      this.maps.set(id, deferred());
    }
    const deferredMap = this.maps.get(id);
    return deferredMap;
  }
}
