/*!
 * (C) Emu Analytics
 */
import colorbrewer from 'colorbrewer';
import { observable } from 'mobx';
/**
 * A `Palette` defines 5 colors used for producing color scales.
 */
export class Palette {
  /**
   * Constructs a new instance of `Palette`.
   *
   * @param id Palette ID.
   * @param colors Array of CSS-format colors (color name, hex or color function).
   */
  constructor(id, colors) {
    this.id = id;
    this.colors = colors;
  }
}
/**
 * A `PaletteRegistry` is a container for [[Palette]] instances.
 *
 * Palettes can be registered and retrieved by ID.
 *
 * A set of standard palettes is registered automatically on creation of the `PaletteRegistry`. These match
 * the 5-color variants of the [Color Brewer](https://colorbrewer2.org/#type=sequential&scheme=BuGn&n=5) palettes.
 * The palette IDs match the Color Brewer IDs. For example, `BuGn` is the ID for the 5-color blue-green sequential
 * Color Brewer palette.
 */
export class PaletteRegistry {
  /**
   * Constructs a new instance of `PaletteRegistry`.
   *
   * The standard Color Brewer palettes are automatically registered.
   */
  constructor() {
    this.palettes = observable.map({}, { name: 'StateStore' });
    this.registerColorBrewerPalettes();
  }
  /**
   *
   * @param paletteId Palette ID
   * @param colors Array of CSS-format colors (color name, hex or color function).
   */
  register(paletteId, colors) {
    this.palettes.set(paletteId.toLowerCase(), new Palette(paletteId, colors));
  }
  /**
   * Get the specified palette.
   *
   * @param paletteId Palette ID.
   */
  get(paletteId) {
    return this.palettes.get(paletteId.toLowerCase());
  }
  /**
   * Returns `true` if the specified palette has been registered.
   *
   * @param paletteId Palette ID.
   */
  has(paletteId) {
    return this.palettes.has(paletteId.toLowerCase());
  }
  registerColorBrewerPalettes() {
    Object.keys(colorbrewer)
      .filter(key => key !== 'schemeGroups')
      .forEach(key => this.register(key, colorbrewer[key][5]));
  }
}
