/*!
 * (C) Emu Analytics
 */
/**
 * Helper functions for the style parser to support operator parsing
 * See: https://github.com/jneen/parsimmon/blob/master/examples/math.js
 */
/**
 * Copyright © 2020 Emu Analytics
 */
import P from 'parsimmon';
/**
 * @internal
 */
// Takes a parser for the prefix operator, and a parser for the base thing being
// parsed, and parses as many occurrences as possible of the prefix operator.
// Note that the parser is created using `P.lazy` because it's recursive. It's
// valid for there to be zero occurrences of the prefix operator.
export function PREFIX(operatorsParser, nextParser) {
  const parser = P.lazy(() => {
    return P.seq(operatorsParser, parser).or(nextParser);
  });
  return parser;
}
/**
 * @internal
 */
// Ideally this function would be just like `PREFIX` but reordered like
// `P.seq(parser, operatorsParser).or(nextParser)`, but that doesn't work. The
// reason for that is that Parsimmon will get stuck in infinite recursion, since
// the very first rule. Inside `parser` is to match parser again. Alternatively,
// you might think to try `nextParser.or(P.seq(parser, operatorsParser))`, but
// that won't work either because in a call to `.or` (aka `P.alt`), Parsimmon
// takes the first possible match, even if subsequent matches are longer, so the
// parser will never actually look far enough ahead to see the postfix
// operators.
export function POSTFIX(operatorsParser, nextParser) {
  // Because we can't use recursion like stated above, we just match a flat list
  // of as many occurrences of the postfix operator as possible, then use
  // `.reduce` to manually nest the list.
  //
  // Example:
  //
  // INPUT  :: "4!!!"
  // PARSE  :: [4, "factorial", "factorial", "factorial"]
  // REDUCE :: ["factorial", ["factorial", ["factorial", 4]]]
  return P.seqMap(nextParser, operatorsParser.many(), (x1, suffixes) => suffixes.reduce((acc, x2) => [x2, acc], x1));
}
/**
 * @internal
 */
// Takes a parser for all the operators at this precedence level, and a parser
// that parsers everything at the next precedence level, and returns a parser
// that parses as many binary operations as possible, associating them to the
// right. (e.g. 1^2^3 is 1^(2^3) not (1^2)^3)
export function BINARY_RIGHT(operatorsParser, nextParser) {
  const parser = P.lazy(() => nextParser.chain(next => P.seq(operatorsParser, P.of(next), parser).or(P.of(next))));
  return parser;
}
/**
 * @internal
 */
export function TERNARY_RIGHT(operatorsParser, nextParser) {
  const parser = P.lazy(() => nextParser.chain(next => P.seqMap(operatorsParser, P.of(next), parser, P.string(':').trim(P.optWhitespace), parser, (op, cond, val1, _colon, val2) => [op, ['to-boolean', cond], val1, val2]).or(P.of(next))));
  return parser;
}
/**
 * @internal
 */
export function BINARY_LEFT(operatorsParser, nextParser) {
  // We run into a similar problem as with the `POSTFIX` parser above where we
  // can't recurse in the direction we want, so we have to resort to parsing an
  // entire list of operator chunks and then using `.reduce` to manually nest
  // them again.
  //
  // Example:
  //
  // INPUT  :: "1+2+3"
  // PARSE  :: [1, ["+", 2], ["+", 3]]
  // REDUCE :: ["+", ["+", 1, 2], 3]
  return P.seqMap(nextParser, P.seq(operatorsParser, nextParser).many(), (first, rest) => {
    return rest.reduce((acc, ch) => {
      const [op, another] = ch;
      return [op, acc, another];
    }, first);
  });
}
