/*!
 * (C) Emu Analytics
 */
/**
 *
 * A parser for converting Flow.js to MaplibreGL style expressions.
 *
 * Individual expressions can be parsed e.g.:
 * 1 + 2 * 3 -> ['+', ['*', 2, 3]]
 *
 * A whole 'style program' can also be parsed e.g.:
 * circle-color: @featureParam * 10;
 * circle-radius: min(@featureParam * 2, 10);
 *
 * Reactive state lookup and palette expansion is also supported. Expressions are
 * parsed once to MaplibreGL format and then expansion is performed as a separate step
 * in a reactive context to avoid having to re-parse when reactive values change.
 *
 * See: https://github.com/jneen/parsimmon
 */
/**
 * Copyright © 2020 Emu Analytics
 */
import P from 'parsimmon';
import { PREFIX, BINARY_RIGHT, BINARY_LEFT, TERNARY_RIGHT } from './parser-helpers';
import { MaplibreGLSpecHelper } from '../maplibre-spec-helper';
import { isNil } from '../../utils/checks.util';
/**
 * @internal
 */
function isFunctionExpression(expr) {
  return Array.isArray(expr);
}
/**
 * @internal
 */
export class StyleParser {
  constructor(context) {
    this.context = context;
    this.language = this.createLanguage();
  }
  /**
   * Parse an expression
   *
   * @param expr Expression to parse
   */
  parseExpression(expr) {
    return this.language.Expr.parse(expr);
  }
  /**
   * Parse a complete assignment program of the form:
   * a: expr1;
   * b: expr2;
   *
   * @param program Program to parse
   */
  parseProgram(program) {
    return this.language.Program.parse(program);
  }
  /**
   * Evaluate an expression.
   *
   * Reactive values are expanded as part of the evaluation.
   *
   * @returns evaluated result
   */
  evaluateExpression(expr) {
    const expandedExpression = this.expandReactiveExpression(expr);
    return MaplibreGLSpecHelper.instance.evaluateExpression(expandedExpression);
  }
  /**
   * Expand reactive values in the given expression (e.g. state lookup, palette expansion).
   *
   * This is performed as a second step after parsing to avoid the need to re-parse expressions
   * when reactive values change.
   *
   * @returns fully expanded expression ready for hand-off to MaplibreGL
   */
  expandReactiveExpression(expr) {
    if (isFunctionExpression(expr)) {
      // Expression is a 'function-type' expression
      let func = expr[0];
      const args = expr.slice(1);
      if (func === 'state') {
        // Expand state lookup
        const stateValue = this.context.state.get(args);
        if (isNil(stateValue)) {
          return null;
        }
        else if (typeof stateValue === 'object') {
          return ['literal', stateValue];
        }
        else {
          return stateValue;
        }
      }
      else if (func === 'from-palette') {
        // Expand palette lookup
        return this.expandFromPalette(expr);
      }
      else if (func === 'regexi') {
        // Add collator for case-insensitive regex
        func = '~';
        args.push(['collator', { caseSensitive: false }]);
      }
      else if (func === 'equali') {
        // Add collator for case-insensitive string equality
        func = '==';
        args.push(['collator', { caseSensitive: false }]);
      }
      // Recursively apply to  arguments
      return [func].concat(args.map(this.expandReactiveExpression.bind(this)));
    }
    return expr;
  }
  createLanguage() {
    // This table defines operator precedence and associativity - higher precedence first
    const operatorTable = [
      { type: PREFIX, ops: this.operators({ '-': '-', '!': '!' }) },
      { type: BINARY_RIGHT, ops: this.operators({ '^': '**' }) },
      { type: BINARY_LEFT, ops: this.operators({ '*': '*', '/': '/', '%': '%' }) },
      { type: BINARY_LEFT, ops: this.operators({ '+': '+', '-': '-' }) },
      { type: BINARY_LEFT, ops: this.operators({ '>=': '>=', '<=': '<=' }) },
      { type: BINARY_LEFT, ops: this.operators({ '<': '<', '>': '>' }) },
      { type: BINARY_LEFT, ops: this.operators({ regexi: '~*', equali: '=*', '==': '==', '!=': '!=', '~': '~' }) },
      { type: BINARY_LEFT, ops: this.operators({ all: '&&' }) },
      { type: BINARY_LEFT, ops: this.operators({ any: '||' }) },
      { type: TERNARY_RIGHT, ops: this.operators({ case: '?' }) }
    ];
    return P.createLanguage({
      Whitespace: () => P.regexp(/[ \n]*/).desc('whitespace'),
      NotNewline: () => P.regexp(/[^\n]*/).desc('non-newline'),
      LineComment: r => r.NotNewline.wrap(P.string('//'), P.alt(P.string('\n'), P.eof)),
      BlockComment: () => P.regexp(/\/\*[\s\S]*?\*\//),
      Comment: r => P.alt(r.LineComment, r.BlockComment).desc('comment'),
      End: () => P.alt(P.string(';'), P.eof),
      _: r => r.Comment.sepBy(r.Whitespace).trim(r.Whitespace),
      Integer: () => {
        return P.regexp(/-?[0-9]+/).map(Number);
      },
      Decimal: () => {
        return P.regexp(/-?[0-9]+\.[0-9]+/).map(Number);
      },
      Number: r => {
        return P.alt(r.Decimal, r.Integer).desc('number');
      },
      Boolean: () => {
        return P.alt(P.string('true'), P.string('false'))
          .map(val => val === 'true')
          .desc('boolean');
      },
      Null: () => {
        return P.string('null')
          .result(null)
          .desc('null');
      },
      DoubleQuoteString: () => {
        return P.regexp(/"((?:\\.|.)*?)"/, 1).desc('string');
      },
      SingleQuoteString: () => {
        return P.regexp(/'((?:\\.|.)*?)'/, 1).desc('string');
      },
      UnquotedString: () => {
        return P.regexp(/[a-zA-Z_][a-zA-Z0-9_-]*/).desc('string');
      },
      HexColor: () => {
        return P.regexp(/#(?:[0-9a-f]{3}){1,2}/i).desc('hex color');
      },
      String: r => {
        return P.alt(r.SingleQuoteString, r.DoubleQuoteString, r.HexColor, r.UnquotedString).map(s => s.replace(/\\n/gm, '\n'));
      },
      Symbol: () => {
        return P.regexp(/[a-zA-Z_][a-zA-Z0-9_-]*/).desc('symbol');
      },
      PartialSymbol: () => {
        return P.regexp(/[a-zA-Z0-9_-]+/).desc('symbol');
      },
      Feature: r => {
        return P.seqMap(P.string('@'), r.PartialSymbol.sepBy(P.string('.')), (_feat, symbols) => ['get'].concat(symbols));
      },
      FeatureState: r => {
        return P.seqMap(P.string('@@'), r.PartialSymbol.sepBy(P.string('.')), (_feat, symbols) => this.expandFeatureState(symbols));
      },
      Array: r => P.string('[')
        .trim(r._)
        .then(r.Value.trim(r._).sepBy(P.string(',')))
        .skip(P.string(']')),
      State: r => {
        return P.seqMap(P.string('$'), r.PartialSymbol.sepBy(P.string('.')), (_state, symbols) => ['state'].concat(symbols));
      },
      Paren: r => {
        return P.seqMap(P.string('('), r.Expr.trim(r._), P.string(')'), (_lb, expr, _rb) => expr);
      },
      Func: r => {
        return P.seqMap(r.Symbol.trim(r._), P.string('('), r.Expr.trim(r._).sepBy(P.string(',')), P.string(')'), (func, _lb, args, _rb) => [func, ...args]);
      },
      Value: r => {
        return P.alt(r.Paren, r.Array, r.Func, r.Boolean, r.Null, r.Number, r.String, r.FeatureState, r.Feature, r.State);
      },
      Expr: r => {
        return this.tableParser(operatorTable, r.Value);
      },
      PropertyName: () => {
        return P.regexp(/[a-zA-Z_-][a-zA-Z0-9_-]*/).desc('symbol');
      },
      Assignment: r => {
        return P.seqMap(P.index, r.PropertyName, P.string(':').trim(r._), r.Expr.trim(r._), r.End, (index, key, _colon, expr) => [key, { expr, index }]);
      },
      Program: r => {
        return r.Assignment.trim(r._)
          .many()
          .map(statements => {
          return statements
            .filter(s => s !== '')
            .reduce((acc, s) => {
            acc[s[0]] = s[1];
            return acc;
          }, {});
        });
      }
    });
  }
  /**
   * Helper function to create a parser that parsers a set of operators at the same precedence level
   *
   * @param ops Operator table
   */
  operators(ops) {
    // Remove sort - process keys in order of definition (for regex support)
    // const keys = Object.keys(ops).sort();
    const keys = Object.keys(ops);
    const ps = keys.map(k => P.string(ops[k])
      .trim(P.optWhitespace)
      .result(k));
    return P.alt.apply(null, ps);
  }
  /**
   * Helper function to create a table-driver operator parser
   *
   */
  tableParser(operatorTable, valueParser) {
    return operatorTable.reduce((acc, level) => level.type(level.ops, acc), valueParser);
  }
  /**
   * Expands the from-palette custom function to a form that MaplibreGL can process.
   *
   * Expression is of form:
   *  ['from-palette', palette_id, property, min, max, reverse] OR
   *  ['from-palette', palette_id, property, stop1, stop2, stop3, stop4, stop5, reverse]
   *
   */
  expandFromPalette(expr) {
    const paletteId = this.evaluateExpression(this.expandReactiveExpression(expr[1]));
    const property = this.expandReactiveExpression(expr[2]);
    let colorStops = expr.slice(3).map(c => this.evaluateExpression(c));
    let reversePalette = false;
    switch (colorStops.length) {
      case 2:
        break;
      case 3:
        reversePalette = !!colorStops[2];
        colorStops = colorStops.slice(0, 2);
        break;
      case 5:
        break;
      case 6:
        reversePalette = !!colorStops[5];
        colorStops = colorStops.slice(0, 5);
        break;
      default:
        throw new Error(`Incorrect color stop format for '${paletteId}'`);
    }
    const palette = this.context.palettes.has(paletteId)
      ? this.context.palettes.get(paletteId)
      : this.context.palettes.get('greys');
    if (colorStops.length === 2) {
      // Convert min/max to 5 linear values
      const min = colorStops[0];
      const max = colorStops[1];
      colorStops = [];
      const step = (max - min) / 4;
      for (let i = 0; i < 5; i++) {
        colorStops.push(min + i * step);
      }
    }
    expr = ['interpolate-lab', ['linear'], property];
    for (let i = 0; i < colorStops.length; i++) {
      expr.push(colorStops[i]);
      expr.push(reversePalette ? palette.colors[colorStops.length - i - 1] : palette.colors[i]);
    }
    return expr;
  }
  expandFeatureState(symbols) {
    let expandedFeatureState = ['feature-state', symbols.shift()];
    symbols.forEach(s => expandedFeatureState = ['get', s, expandedFeatureState]);
    return expandedFeatureState;
  }
}
