/*!
 * (C) Emu Analytics
 */
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
  var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
  if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
  else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
  return c > 3 && r && Object.defineProperty(target, key, r), r;
};
import { action, autorun, comparer, computed, isObservableArray, isObservableMap, observable, remove, toJS } from 'mobx';
import { toStream } from 'mobx-utils';
import { isArray } from '../utils/checks.util';
import { debugQuery } from '../utils/debug';
import { removeUndefined } from '../utils/remove-undefined';
import * as ComputedAsync from './async-computed';
import { warnOnce } from './errors';
import { FlowQuery } from './flow-queries/flow-query';
// Re-export mobx types
export { computed as flowComputed, observable as flowObservable } from 'mobx';
/**
 * A `StateStore` is a container of reactive application state properties.
 *
 * Application state properties define key application state and are the core of Flo.w RDF reactivity.
 * Changes to application state properties trigger reactive updates of UI elements and other components
 * that depend on them.
 *
 * Application state properties are identified by property paths. The root of a property path is the application
 * `StateStore` object. Paths can have arbitrary depth allowing the developer to organize and group application
 * state properties.
 *
 * In JavaScript, property paths are specified as strings using dotted notation. For example:
 *
 * - `'prop1'`
 * - `'subGroup.prop2'`
 *
 * To take advantage of type-safety in Typescript, property paths should be specified as arrays of strings.
 * For example:
 *
 * - `['prop1']`
 * - `['subGroup', 'prop2']`
 *
 *  Using this syntax allows IDEs to provide code-completion and
 * ensures type errors are detected during development.
 *
 * ### State Initialization Methods
 *
 * - [[initialize]]
 *
 * ### State accessor Methods
 *
 * - [[get]] - Get a state property value.
 * - [[set]] - Set a state property value.
 * - [[merge]] - Set multiple state property values.
 * - [[increment]] - Increment a state property value.
 * - [[decrement]] - Decrement a state property value.
 * - [[toggle]] - Toggle a state property value.
 *
 * ### Computed State Methods
 * - [[compute]] - Register a computed state property.
 * - [[asyncCompute]] - Create an asynchronous computed state property.
 *
 * ### Flo.w Dataset Query Methods
 *
 * - [[registerQuery]] - Register a Flo.w reactive dataset query.
 *
 * ### Serialization Methods
 * - [[serialize]] - Configure state serialization with the browser URL.
 *
 * ### Other Reactive State Methods
 *
 * - [[autorun]] - Run a function in a reactive context when application state changes.
 * - [[toStream]] - Create an observable stream of application state changes.
 *
 * @typeParam T  The type of the application state object. This enables code-completion and type-safety when using Flo.w RDF with Typescript.
 */
export class StateStore {
  /**
   * Constructs a new `StateStore` instance.
   *
   * @param context The parent Flo.w context.
   */
  constructor(context) {
    this.context = context;
    this.state = observable({});
  }
  /**
   * Initializes the `StateStore` with the supplied initial state.
   *
   * @param state State object.
   */
  initialize(state) {
    this.state = observable(state);
  }
  /**
   * Run multiple state-modification methods atomically in a single transation.
   * Reactive changes will be coalesced to avoid multiple updates.
   *
   * e.g.
   *
   * ```javascript
   * state.transaction(() => {
   *   state.set('prop1', 1);
   *   state.set('prop2', 2);
   *   state.increment('prop3');
   * });
   * ```
   *
   * @param path Application state property path.
   */
  transaction(func) {
    if (func) {
      func();
    }
  }
  /**
   * Get the current value of an application state property.
   *
   * @param path Application state property path.
   */
  get(path) {
    let pathStr = path;
    // Support for legacy path array
    if (isArray(path)) {
      pathStr = path.join('.');
    }
    if (pathStr.startsWith('$')) {
      pathStr = pathStr.slice(1);
    }
    let value = this.state;
    const p = pathStr.split('.');
    while (p.length > 0) {
      if (value == null) {
        value = undefined;
        break;
      }
      const key = p.shift();
      if (isObservableArray(value[key])) {
        // Unwrap array
        value = toJS(value[key]);
      }
      else if (isObservableMap(value[key])) {
        // Unwrap map
        value = toJS(value[key]);
      }
      else if (value[key] && typeof value[key].get === 'function') {
        // Property is a computed value - get it
        value = value[key].get();
      }
      else {
        value = value[key];
      }
    }
    if (value === undefined) {
      warnOnce(`Accessing undefined state '${pathStr}'`);
    }
    if (value instanceof FlowQuery) {
      // Return query instance intact
      return value;
    }
    else {
      // Unwrap value before return
      return toJS(value);
    }
  }
  /**
   * Set the value of an application state property.
   *
   * @param path Application state property path.
   * @param val New value to set.
   */
  set(path, val) {
    this.baseSet(path, val);
  }
  /**
   * @deprecated
   *
   * Merge the supplied partial state into the `StateStore`.
   *
   * Use this method to update multiple state properties in one transaction.
   *
   * @param patch Partial state object.
   */
  merge(patch) {
    this.baseMerge(this.state, patch);
  }
  /**
   * Toggle the specified boolean state property.
   *
   * The state property will be coerced to a boolean if it is not already.
   *
   * @param path The state property path.
   */
  toggle(path) {
    const val = !this.get(path);
    this.set(path, val);
    return val;
  }
  /**
   * Increment the specified numeric state property.
   *
   * The state property will be coerced to a number if it is not already.
   *
   * @param path Application state property path.
   */
  increment(path) {
    const val = +this.get(path) + 1;
    this.set(path, val);
    return val;
  }
  /**
   * Decrement the specified numeric application state property.
   *
   * The state property will be coerced to a number if it is not already.
   *
   * @param path Application state property path.
   */
  decrement(path) {
    const val = +this.get(path) - 1;
    this.set(path, val);
    return val;
  }
  /**
   * Register a computed application state property.
   *
   * The computed property will be recalculated when state properties referenced in `computeFn` change.
   *
   * @param path Application state path.
   * @param computeFn Function to compute value from other state properties.
   * @param options Reactive options.
   */
  compute(path, computeFn, options) {
    // Property getter will be created that calls computeFn and passes in 'state'
    const wrappedFunction = () => computeFn(this.state);
    wrappedFunction.options = options;
    this.set(path, wrappedFunction);
  }
  /**
   * Create an asynchronous compute function.
   *
   * @param computeFn Asyncronous compute function
   * @param initialValue Initial result value. Specifies the computed value before the asynchronous operation has completed.
   * @param delay Debounce delay (milliseconds).
   * @param name Optional name displayed in debugging messages.
   */
  asyncCompute(computeFn, initialValue = null, delay, name) {
    return ComputedAsync.asyncComputed(initialValue, delay, () => {
      return computeFn(this.state);
    }, name);
  }
  /**
   * Expose an application state property as an observable stream of values.
   *
   * The observable stream will emit a value whenever the application state property changes.
   *
   * Subscribe to the returned stream to receive values. Call the `unsubscribe` method of the subscription
   * when the subscription is no longer required.
   *
   * @param path Application state property path
   * @param fireImmediately Set to `true` to emit a value immediately. Otherwise wait for first change.
   */
  toStream(path, fireImmediately = true) {
    return toStream(() => this.get(path), fireImmediately);
  }
  /**
   * Configure serialization/deserialization of application state properties to/from browser URL query parameters.
   *
   * This allows deep-linking and sharing of web application URLs that specify application state.
   *
   * @param serializeFn Function to serialize application state properties to browser URL query parameters.
   * @param deserializeFn Function to deserialize application state properties from browser URL query parameters.
   */
  serialize(serializeFn, deserializeFn) {
    var _a;
    if (deserializeFn) {
      const searchParams = new URLSearchParams(window.location.search);
      const vals = {};
      searchParams.forEach((val, key) => (vals[key] = val));
      const deserializedStatePatch = deserializeFn(vals);
      removeUndefined(deserializedStatePatch);
      this.merge(deserializedStatePatch);
    }
    const disposer = autorun(() => {
      const searchParams = new URLSearchParams(window.location.search);
      const serializedState = serializeFn(this.state);
      removeUndefined(serializedState);
      Object.keys(serializedState).forEach((key) => searchParams.set(key, serializedState[key].toString()));
      const queryString = searchParams.toString();
      const newUrl = queryString.length ? `${location.pathname}?${queryString}` : location.pathname;
      window.history.replaceState(serializedState, '', newUrl);
    }, { name: `autorun: serializeUrl(FlowContext.${(_a = this.context) === null || _a === void 0 ? void 0 : _a.id})` });
    return disposer;
  }
  /**
   * Defines a function that runs in a reactive context when state properties that are referenced
   * in the function change.
   *
   * @param autorunFn Function to execute when referenced state properties change.
   * @param name Optional name displayed in debugging messages.
   */
  autorun(autorunFn, name) {
    return autorun(() => autorunFn(this.state), { name });
  }
  /**
   * Registers a Flo.w reactive query that executes when application state properties referenced in
   * the supplied query definition function are changed.
   *
   * See [[FlowQuery]] for more details.
   *
   * @param path Application state path.
   * @param queryFn Query definition function.
   * @param resultTransformFn Results transorm function.
   */
  registerQuery(path, queryFn, resultTransformFn) {
    const queryId = isArray(path) ? path.join('.') : path;
    debugQuery(`Registering query '${queryId}'`);
    const flowQuery = new FlowQuery(this.context, queryId, queryFn, resultTransformFn);
    this.set(path, flowQuery);
    return flowQuery;
  }
  baseMerge(obj, patch) {
    if (typeof patch === 'object') {
      // Array of patches
      if (Array.isArray(patch)) {
        for (const p of patch) {
          this.baseMerge(obj, p);
        }
      }
      else {
        for (const k of Object.keys(patch)) {
          const val = patch[k];
          if (typeof val === 'function') {
            // Function-based update
            obj[k] = val(obj[k]);
          }
          else if (val === undefined) {
            // Removal of property
            remove(obj, k);
          }
          else if (val === null || typeof val !== 'object' || Array.isArray(val)) {
            // Value set
            obj[k] = val;
          }
          else if (typeof obj[k] === 'object') {
            // Recursive patch set
            this.baseMerge(obj[k], val);
          }
          else {
            // Add new property
            obj[k] = val;
          }
        }
      }
    }
    return obj;
  }
  baseSet(keyOrPath, val) {
    var _a, _b;
    if (typeof keyOrPath === 'string') {
      if (keyOrPath.startsWith('$')) {
        keyOrPath = keyOrPath.slice(1);
      }
      this.baseSet(keyOrPath.split('.'), val);
    }
    else {
      let obj = this.state;
      while (keyOrPath.length > 1) {
        const key = keyOrPath.shift();
        if (typeof obj[key] !== 'object') {
          obj[key] = {};
        }
        obj = obj[key];
      }
      if (obj[keyOrPath[0]] !== val) {
        if (typeof val === 'function') {
          // Setting a property to a function creates a 'computed' property
          let equals;
          switch ((_a = val.options) === null || _a === void 0 ? void 0 : _a.equals) {
            case 'deep':
              equals = comparer.structural;
              break;
            case 'shallow':
              equals = comparer.shallow;
              break;
            case 'identity':
            default:
              equals = comparer.default;
              break;
          }
          const name = ((_b = val.options) === null || _b === void 0 ? void 0 : _b.name) || `computed-${keyOrPath.join('.')}`;
          const computedProp = computed(() => val(), { equals, name });
          obj[keyOrPath[0]] = computedProp;
        }
        else {
          obj[keyOrPath[0]] = val;
        }
      }
    }
  }
}
__decorate([
  observable
], StateStore.prototype, "state", void 0);
__decorate([
  action
], StateStore.prototype, "transaction", null);
__decorate([
  action
], StateStore.prototype, "set", null);
__decorate([
  action
], StateStore.prototype, "merge", null);
__decorate([
  action
], StateStore.prototype, "serialize", null);
