/*!
 * (C) Emu Analytics
 */
import { proxyCustomElement, HTMLElement, createEvent, h, Host } from '@stencil/core/internal/client';
import { r as randomId } from './random-id.js';
import { d as debugLifecycle, f as formatEl } from './debug.js';
import { S as Selection, r as root, f as format, i as interpolateNumber } from './defaultLocale.js';
import { g as globalContextRegistry } from './context-registry.js';
import { k as isUndefined, m as isNull } from './EventEmitter.js';

function select(selector) {
  return typeof selector === "string"
      ? new Selection([[document.querySelector(selector)]], [document.documentElement])
      : new Selection([[selector]], root);
}

function quadInOut(t) {
  return ((t *= 2) <= 1 ? t * t : --t * (2 - t) + 1) / 2;
}

const dataValueCss = ":host{line-height:1;display:inline-flex;min-width:0}span{white-space:nowrap;overflow:hidden;text-overflow:ellipsis}";

const FlowValue = /*@__PURE__*/ proxyCustomElement(class extends HTMLElement {
  constructor() {
    super();
    this.__registerHost();
    this.__attachShadow();
    this.flowDataValueChanged = createEvent(this, "flowDataValueChanged", 7);
    this.initialized = false;
    /**
     * Specifies the transition duration in milliseconds for the animation between numeric values. Set
     * to zero to disable animation.
     */
    this.duration = 250;
    /**
     * The Flo.w context ID to use for application state.
     */
    this.flowContext = 'default';
  }
  async connectedCallback() {
    debugLifecycle(`${formatEl(this.el)} connectedCallback`);
    if (this.initialized) {
      // Handle reconnect
      this.connect();
    }
  }
  async componentDidLoad() {
    debugLifecycle(`${formatEl(this.el)} did load`);
    if (!this.el.id) {
      this.el.setAttribute('id', randomId('value'));
    }
    this.initialized = true;
    this.connect();
  }
  async disconnectedCallback() {
    debugLifecycle(`${formatEl(this.el)} disconnectedCallback`);
    this.disconnect();
  }
  async connect() {
    this.context = await globalContextRegistry.getContext(this.flowContext);
    this.updateNumberFormatter();
    this.subscribe();
  }
  async disconnect() {
    this.unsubscribe();
  }
  updateNumberFormatter() {
    if (!this.format || this.format === 'icon') {
      this.numberFormatter = n => n.toString(10);
    }
    else {
      this.numberFormatter = format(this.format);
    }
    // update display
    this.setValue(this.internalValue);
  }
  subscribe() {
    var _a;
    this.unsubscribe();
    if (isUndefined(this.value)) {
      this.context.emitWarning(`${formatEl(this.el)} value attribute undefined`);
    }
    else if (isNull(this.value)) {
      // Don't attempt to evaluate null value
      this.setValue(this.value);
      this.flowDataValueChanged.emit(this.value);
    }
    else {
      const parsedDataExpression = this.context.parser.parseExpression(this.value);
      if (parsedDataExpression.status) {
        this.autorunDisposer = this.context.state.autorun(() => {
          try {
            const newValue = this.context.parser.evaluateExpression(parsedDataExpression.value);
            this.setValue(newValue);
            this.flowDataValueChanged.emit(newValue);
          }
          catch (e) {
            this.context.emitError(e);
          }
        }, `autorun: update(FlowDataValue.${(_a = this.el) === null || _a === void 0 ? void 0 : _a.id})`);
      }
      else {
        this.context.emitError(new Error(`Malformed expression - ${parsedDataExpression}`));
      }
    }
  }
  unsubscribe() {
    if (this.autorunDisposer) {
      this.autorunDisposer();
    }
  }
  setValue(newValue) {
    const selection = select(this.el.shadowRoot).select('span');
    const node = selection.node();
    selection.interrupt(this.el.id);
    if (!this.duration || typeof newValue !== 'number' || typeof this.internalValue !== 'number') {
      // Cannot animate so just set value
      this.internalValue = newValue;
      node.innerHTML = this.formatValue(newValue);
    }
    else {
      const oldValue = isNaN(this.internalValue) ? 0 : this.internalValue;
      this.internalValue = newValue;
      // Going from number to number - animate
      selection
        .transition(this.el.id)
        .duration(this.duration)
        .ease(quadInOut)
        .tween('update', () => {
        const interp = interpolateNumber(oldValue, newValue);
        return (t) => {
          node.innerHTML = this.formatValue(interp(t)).replace(' ', '&nbsp;');
        };
      });
    }
  }
  formatValue(value) {
    if (this.format === 'icon') {
      return `<flow-icon icon="${value || 'far-circle-question'}"></flow-icon>`;
    }
    if (typeof value === 'number') {
      return this.numberFormatter(value);
    }
    if (typeof value === 'undefined' || value === null) {
      return '--';
    }
    return value;
  }
  render() {
    return (h(Host, { class: "flow-value" }, h("span", { part: "content" })));
  }
  get el() { return this; }
  static get watchers() { return {
    "format": ["updateNumberFormatter"],
    "value": ["subscribe"]
  }; }
  static get style() { return dataValueCss; }
}, [1, "flow-value", {
    "duration": [2],
    "flowContext": [1, "flow-context"],
    "format": [1],
    "value": [1]
  }]);
function defineCustomElement() {
  if (typeof customElements === "undefined") {
    return;
  }
  const components = ["flow-value"];
  components.forEach(tagName => { switch (tagName) {
    case "flow-value":
      if (!customElements.get(tagName)) {
        customElements.define(tagName, FlowValue);
      }
      break;
  } });
}

export { FlowValue as F, defineCustomElement as d };
