/*!
 * (C) Emu Analytics
 */
import { proxyCustomElement, HTMLElement, h, Host } from '@stencil/core/internal/client';
import { A as AncestorElement } from './component.utils.js';
import { r as randomId } from './random-id.js';
import { d as debugLifecycle, f as formatEl } from './debug.js';
import { g as globalContextRegistry } from './context-registry.js';
import { i as isNil, a as isArray } from './EventEmitter.js';

/**
 * Core functionality for chart data. This class is wrapped by the `<flow-chart-data>` web component.
 */
class FlowCoreChartData {
  /**
   * Constructs a new `FlowCoreChartData` instance.
   *
   * @param flowChart Parent Flo.w chart.
   */
  constructor(flowChart) {
    this.flowChart = flowChart;
  }
  /**
   * Subscribe to reactive data updates.
   *
   * This method is called internally by the `<flow-chart-data>` component.
   *
   * @param options Chart data options
   */
  async subscribe(options) {
    this.unsubscribe();
    if (isNil(options.data)) {
      // set data to empty array
      this.flowChart.setData(options.name, [], options.xProperty, options.yProperty);
    }
    else if (isArray(options.data)) {
      this.flowChart.setData(options.name, options.data, options.xProperty, options.yProperty);
    }
    else {
      const context = await globalContextRegistry.getContext(this.flowChart.contextId);
      const parsedDataExpression = context.parser.parseExpression(options.data);
      if (parsedDataExpression.status) {
        this.autorunDisposer = context.state.autorun(() => {
          try {
            const data = context.parser.evaluateExpression(parsedDataExpression.value);
            this.flowChart.setData(options.name, data, options.xProperty, options.yProperty);
          }
          catch (e) {
            context.emitError(e);
          }
        }, `autorun: update(FlowChartData.${this.flowChart.id}.${options.name})`);
      }
      else {
        context.emitError(new Error(`Malformed expression - ${parsedDataExpression}`));
      }
    }
  }
  /**
   * Unsubscribe from reactive data updates.
   *
   * This method is called internally by the `<flow-chart-data>` component.
   */
  unsubscribe() {
    if (this.autorunDisposer) {
      this.autorunDisposer();
    }
  }
}

var __decorate = (undefined && undefined.__decorate) || function (decorators, target, key, desc) {
  var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
  if (typeof Reflect === "object" && typeof Reflect.decorate === "function")
    r = Reflect.decorate(decorators, target, key, desc);
  else
    for (var i = decorators.length - 1; i >= 0; i--)
      if (d = decorators[i])
        r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
  return c > 3 && r && Object.defineProperty(target, key, r), r;
};
const FlowChartData$1 = /*@__PURE__*/ proxyCustomElement(class extends HTMLElement {
  constructor() {
    super();
    this.__registerHost();
    this.initialized = false;
  }
  connectedCallback() {
    if (this.initialized) {
      // Handle re-connect to DOM
      this.connect();
    }
  }
  async componentWillLoad() {
    if (!this.el.id) {
      this.el.setAttribute('id', randomId('chartdata'));
    }
    debugLifecycle(`${formatEl(this.el)} will load`);
    this._chartData = new FlowCoreChartData(this.flowChart._chart);
  }
  componentDidLoad() {
    debugLifecycle(`${formatEl(this.el)} did load`);
    this.initialized = true;
    this.connect();
  }
  disconnectedCallback() {
    this.disconnect();
  }
  connect() {
    this.subscribe();
  }
  disconnect() {
    this._chartData.unsubscribe();
  }
  subscribe() {
    this._chartData.subscribe({
      name: this.name,
      data: this.data,
      xProperty: this.x,
      yProperty: this.y
    });
  }
  render() {
    return h(Host, null);
  }
  get el() { return this; }
  static get watchers() { return {
    "data": ["subscribe"],
    "x": ["subscribe"],
    "y": ["subscribe"]
  }; }
}, [0, "flow-chart-data", {
    "_chartData": [16],
    "data": [1],
    "name": [1],
    "x": [1],
    "y": [1]
  }]);
__decorate([
  AncestorElement({ supportAttr: false })
], FlowChartData$1.prototype, "flowChart", void 0);
function defineCustomElement$1() {
  if (typeof customElements === "undefined") {
    return;
  }
  const components = ["flow-chart-data"];
  components.forEach(tagName => { switch (tagName) {
    case "flow-chart-data":
      if (!customElements.get(tagName)) {
        customElements.define(tagName, FlowChartData$1);
      }
      break;
  } });
}

const FlowChartData = FlowChartData$1;
const defineCustomElement = defineCustomElement$1;

export { FlowChartData, defineCustomElement };
