/*!
 * (C) Emu Analytics
 */
import { proxyCustomElement, HTMLElement, h, Host } from '@stencil/core/internal/client';
import { A as AncestorElement } from './component.utils.js';
import { r as randomId } from './random-id.js';
import { d as debugLifecycle, f as formatEl } from './debug.js';
import { g as globalContextRegistry } from './context-registry.js';

/**
 * Core functionality for chart signals. This class is wrapped by the `<flow-chart-signal>` web component.
 */
class FlowCoreChartSignal {
  /**
   * Constructs a new `FlowCoreChartSignal` instance.
   *
   * @param flowChart Parent Flo.w chart.
   */
  constructor(flowChart) {
    this.flowChart = flowChart;
  }
  /**
   * Subscribe to reactive signal updates.
   *
   * This method is called internally by the `<flow-chart-signal>` component.
   *
   * @param options Chart signal options
   */
  async subscribe(options) {
    this.unsubscribe();
    const context = await globalContextRegistry.getContext(this.flowChart.contextId);
    const parsedValueExpression = context.parser.parseExpression(options.value);
    if (parsedValueExpression.status) {
      this.autorunDisposer = context.state.autorun(() => {
        try {
          const value = context.parser.evaluateExpression(parsedValueExpression.value);
          this.flowChart.setSignal(options.name, value);
        }
        catch (e) {
          context.emitError(e);
        }
      }, `autorun: update(FlowChartSignal.${this.flowChart.id}.${options.name})`);
    }
    else {
      context.emitError(new Error(`Malformed expression - ${parsedValueExpression}`));
    }
  }
  /**
   * Unsubscribe from reactive signal updates.
   *
   * This method is called internally by the `<flow-chart-signal>` component.
   */
  unsubscribe() {
    if (this.autorunDisposer) {
      this.autorunDisposer();
    }
  }
}

var __decorate = (undefined && undefined.__decorate) || function (decorators, target, key, desc) {
  var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
  if (typeof Reflect === "object" && typeof Reflect.decorate === "function")
    r = Reflect.decorate(decorators, target, key, desc);
  else
    for (var i = decorators.length - 1; i >= 0; i--)
      if (d = decorators[i])
        r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
  return c > 3 && r && Object.defineProperty(target, key, r), r;
};
const FlowChartSignal$1 = /*@__PURE__*/ proxyCustomElement(class extends HTMLElement {
  constructor() {
    super();
    this.__registerHost();
    this.initialized = false;
  }
  connectedCallback() {
    if (this.initialized) {
      // Handle re-connect to DOM
      this.connect();
    }
  }
  componentWillLoad() {
    if (!this.el.id) {
      this.el.setAttribute('id', randomId('chartsignal'));
    }
    debugLifecycle(`${formatEl(this.el)} will load`);
    this._chartSignal = new FlowCoreChartSignal(this.flowChart._chart);
  }
  componentDidLoad() {
    this.initialized = true;
    this.connect();
  }
  disconnectedCallback() {
    this.disconnect();
  }
  connect() {
    this.subscribe();
  }
  disconnect() {
    this._chartSignal.unsubscribe();
  }
  subscribe() {
    this._chartSignal.subscribe({
      name: this.name,
      value: this.value
    });
  }
  render() {
    return h(Host, null);
  }
  get el() { return this; }
  static get watchers() { return {
    "value": ["subscribe"]
  }; }
}, [0, "flow-chart-signal", {
    "_chartSignal": [16],
    "name": [1],
    "value": [1]
  }]);
__decorate([
  AncestorElement({ supportAttr: false })
], FlowChartSignal$1.prototype, "flowChart", void 0);
function defineCustomElement$1() {
  if (typeof customElements === "undefined") {
    return;
  }
  const components = ["flow-chart-signal"];
  components.forEach(tagName => { switch (tagName) {
    case "flow-chart-signal":
      if (!customElements.get(tagName)) {
        customElements.define(tagName, FlowChartSignal$1);
      }
      break;
  } });
}

const FlowChartSignal = FlowChartSignal$1;
const defineCustomElement = defineCustomElement$1;

export { FlowChartSignal, defineCustomElement };
