/*!
 * (C) Emu Analytics
 */
import { proxyCustomElement, HTMLElement, createEvent, h, Host } from '@stencil/core/internal/client';
import { g as globalContextRegistry } from './context-registry.js';
import { a as isArray } from './EventEmitter.js';

const checkboxGroupCss = ":host{position:relative;box-sizing:border-box}:host *,:host *:before,:host *:after{box-sizing:inherit}:host{display:flex;flex-direction:column}";

const CheckboxGroup = /*@__PURE__*/ proxyCustomElement(class extends HTMLElement {
  constructor() {
    super();
    this.__registerHost();
    this.__attachShadow();
    this.flowCheckboxGroupChange = createEvent(this, "flowCheckboxGroupChange", 7);
    this.initialized = false;
    /** The value attribute of the checkbox group. */
    this.value = [];
    /**
     * Set to `true` to disable interaction with the checkbox group.
     */
    this.disabled = false;
    /**
     * The Flo.w context ID to use for application state.
     */
    this.flowContext = 'default';
  }
  async updateValue(value, oldValue) {
    if (!this.valuesEqual(oldValue, value)) {
      if (!isArray(value)) {
        this.getAllCheckboxes().forEach(radio => (radio.checked = false));
      }
      else {
        // Select radio button matching value
        this.getAllCheckboxes().forEach(radio => (radio.checked = this.value.includes(radio.value)));
      }
      // Update state if subscribed
      if (this.selectionState) {
        const context = await globalContextRegistry.getContext(this.flowContext);
        context.state.set(this.selectionState, value);
      }
    }
  }
  updateDisabled(disabled) {
    this.getAllCheckboxes().forEach(checkbox => (checkbox.disabled = disabled));
  }
  connectedCallback() {
    if (this.initialized) {
      // Handle re-connect to DOM
      this.connect();
    }
  }
  componentDidLoad() {
    this.initialized = true;
    this.connect();
  }
  disconnectedCallback() {
    this.disconnect();
  }
  connect() {
    this.updateDisabled(this.disabled);
    this.subscribe();
  }
  disconnect() {
    this.unsubscribe();
  }
  /**
   * Update group value when radio is clicked
   */
  onFlowChange(ev) {
    const target = ev.target;
    if (target.tagName === 'FLOW-CHECKBOX') {
      this.value = this.getAllCheckboxes()
        .map(checkbox => (checkbox.checked ? checkbox.value : null))
        .filter(val => val !== null);
      this.flowCheckboxGroupChange.emit();
    }
  }
  async subscribe() {
    this.unsubscribe();
    if (this.selectionState) {
      const context = await globalContextRegistry.getContext(this.flowContext);
      this.subscription = context.state.toStream(this.selectionState, true).subscribe(value => {
        this.value = value;
      });
    }
  }
  unsubscribe() {
    if (this.subscription) {
      this.subscription.unsubscribe();
      this.subscription = undefined;
    }
  }
  /**
   * Return all child radio elements
   */
  getAllCheckboxes() {
    const radios = this.host.querySelectorAll('flow-checkbox');
    return Array.from(radios);
  }
  valuesEqual(val1, val2) {
    if (!isArray(val1) || !isArray(val2)) {
      return false;
    }
    return val1.length === val2.length && val1.every((value, index) => value === val2[index]);
  }
  render() {
    const hostClasses = {
      'flow-checkbox-group': true
    };
    return (h(Host, { class: hostClasses }, h("slot", null)));
  }
  get host() { return this; }
  static get watchers() { return {
    "value": ["updateValue"],
    "disabled": ["updateDisabled"],
    "selectionState": ["subscribe"]
  }; }
  static get style() { return checkboxGroupCss; }
}, [1, "flow-checkbox-group", {
    "value": [1040],
    "disabled": [1540],
    "selectionState": [1, "selection-state"],
    "flowContext": [1, "flow-context"]
  }, [[0, "flowChange", "onFlowChange"]]]);
function defineCustomElement$1() {
  if (typeof customElements === "undefined") {
    return;
  }
  const components = ["flow-checkbox-group"];
  components.forEach(tagName => { switch (tagName) {
    case "flow-checkbox-group":
      if (!customElements.get(tagName)) {
        customElements.define(tagName, CheckboxGroup);
      }
      break;
  } });
}

const FlowCheckboxGroup = CheckboxGroup;
const defineCustomElement = defineCustomElement$1;

export { FlowCheckboxGroup, defineCustomElement };
