/*!
 * (C) Emu Analytics
 */
import { proxyCustomElement, HTMLElement, h, Host } from '@stencil/core/internal/client';
import { A as AncestorElement } from './component.utils.js';
import { r as randomId } from './random-id.js';
import { t as toJS, k as debugMapStyle, o as observable, c as computed, l as action, d as debugLifecycle, f as formatEl } from './debug.js';
import { P as ParseError } from './parse-error.js';

var __decorate$1 = (undefined && undefined.__decorate) || function (decorators, target, key, desc) {
  var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
  if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
  else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
  return c > 3 && r && Object.defineProperty(target, key, r), r;
};
/**
 * Core functionality for Flo.w map parameters. This class is wrapped by the `<flow-map-parameter>` web component.
 *
 * Map parameters are used to specify additional options for `FlowCoreMapDataSource` and `FlowCoreMapDataJoin` instances.
 *
 * @noInheritDoc
 */
class FlowCoreParameter {
  /**
   * Constructs a new `FlowCoreParameter` instance.
   *
   * @param name Parameter name.
   * @param flowMap Parent Flo.w map.
   * @param required Parameter is required.
   */
  constructor(name, flowMap, required = false) {
    this.name = name;
    this.flowMap = flowMap;
    this.required = required;
  }
  /**
   * Return the parameter definition.
   *
   * Parameters whose value is defined by a Flo.w reactive expression are evaluated and expanded.
   *
   * <flow-badge pill>Reactive</flow-badge>
   */
  get parameterDef() {
    let value = null;
    if (this.parsedValue) {
      value = this.flowMap.context.parser.expandReactiveExpression(toJS(this.parsedValue));
    }
    return { name: this.name, value, required: this.required };
  }
  /**
   * Set the parameter value.
   *
   * Values are specified using Flo.w reactive expressions.
   *
   * @param value Parameter value.
   */
  setValue(value) {
    debugMapStyle(`setting map parameter ${this.name}=${value}`);
    if (!value) {
      // Remove property
      this.parsedValue = null;
      return;
    }
    const parseResult = this.flowMap.context.parser.parseExpression(value);
    if (!parseResult.status) {
      this.flowMap.context.emitError(new ParseError(value, parseResult));
    }
    else {
      this.parsedValue = parseResult.value;
    }
  }
}
__decorate$1([
  observable
], FlowCoreParameter.prototype, "parsedValue", void 0);
__decorate$1([
  computed
], FlowCoreParameter.prototype, "parameterDef", null);
__decorate$1([
  action
], FlowCoreParameter.prototype, "setValue", null);

var __decorate = (undefined && undefined.__decorate) || function (decorators, target, key, desc) {
  var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
  if (typeof Reflect === "object" && typeof Reflect.decorate === "function")
    r = Reflect.decorate(decorators, target, key, desc);
  else
    for (var i = decorators.length - 1; i >= 0; i--)
      if (d = decorators[i])
        r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
  return c > 3 && r && Object.defineProperty(target, key, r), r;
};
const FlowMapParameter$1 = /*@__PURE__*/ proxyCustomElement(class extends HTMLElement {
  constructor() {
    super();
    this.__registerHost();
    /**
     * Set to `true` to disable data joining if this parameter is `null`.
     */
    this.required = false;
  }
  async componentWillLoad() {
    if (!this.el.id) {
      this.el.setAttribute('id', randomId('parameter'));
    }
    debugLifecycle(`${formatEl(this.el)} will load`);
    await this.flowMap._map.initializeContext();
    this._parameter = new FlowCoreParameter(this.name, this.flowMap._map, this.required);
    this.setValue(this.value);
  }
  setValue(value) {
    if (this._parameter) {
      this._parameter.setValue(value);
    }
  }
  async componentDidLoad() {
    debugLifecycle(`${formatEl(this.el)} did load`);
  }
  async disconnectedCallback() {
    debugLifecycle(`${formatEl(this.el)} did unload`);
  }
  render() {
    return h(Host, null);
  }
  get el() { return this; }
  static get watchers() { return {
    "value": ["setValue"]
  }; }
}, [0, "flow-map-parameter", {
    "_parameter": [1040],
    "name": [1],
    "value": [1],
    "required": [4]
  }]);
__decorate([
  AncestorElement({ supportAttr: false })
], FlowMapParameter$1.prototype, "flowMap", void 0);
function defineCustomElement$1() {
  if (typeof customElements === "undefined") {
    return;
  }
  const components = ["flow-map-parameter"];
  components.forEach(tagName => { switch (tagName) {
    case "flow-map-parameter":
      if (!customElements.get(tagName)) {
        customElements.define(tagName, FlowMapParameter$1);
      }
      break;
  } });
}

const FlowMapParameter = FlowMapParameter$1;
const defineCustomElement = defineCustomElement$1;

export { FlowCoreParameter as F, FlowMapParameter, defineCustomElement };
