/*!
 * (C) Emu Analytics
 */
import { getAssetPath, proxyCustomElement, HTMLElement, Build, h, Host } from '@stencil/core/internal/client';
import { c as createColorClasses } from './theme.js';

let CACHED_MAP;
const getIconMap = () => {
  if (!CACHED_MAP) {
    const win = window;
    win.Ionicons = win.Ionicons || {};
    CACHED_MAP = win.Ionicons.map = win.Ionicons.map || new Map();
  }
  return CACHED_MAP;
};
const getUrl = (i) => {
  let url = getName(null, i.icon);
  if (url) {
    return getNamedUrl(url);
  }
  if (i.icon) {
    url = getSrc(i.icon);
    if (url) {
      return url;
    }
  }
  return null;
};
const getNamedUrl = (iconName) => {
  const url = getIconMap().get(iconName);
  if (url) {
    return url;
  }
  return getAssetPath(`assets/svg/${iconName}.svg`);
};
const getName = (iconName, icon) => {
  // if an icon was passed in using the ios or md attributes
  // set the iconName to whatever was passed in
  if (!iconName && icon && !isSrc(icon)) {
    iconName = icon;
  }
  if (isStr(iconName)) {
    iconName = toLower(iconName);
  }
  if (!isStr(iconName) || iconName.trim() === '') {
    return null;
  }
  // only allow alpha characters and dash
  const invalidChars = iconName.replace(/[a-z]|-|\d/gi, '');
  if (invalidChars !== '') {
    return null;
  }
  return iconName;
};
const getSrc = (src) => {
  if (isStr(src)) {
    src = src.trim();
    if (isSrc(src)) {
      return src;
    }
  }
  return null;
};
const isSrc = (str) => str.length > 0 && /(\/|\.)/.test(str);
const isStr = (val) => typeof val === 'string';
const toLower = (val) => val.toLowerCase();

const validateContent = (svgContent) => {
  if (svgContent) {
    const div = document.createElement('div');
    div.innerHTML = svgContent;
    // setup this way to ensure it works on our buddy IE
    for (let i = div.childNodes.length - 1; i >= 0; i--) {
      if (div.childNodes[i].nodeName.toLowerCase() !== 'svg') {
        div.removeChild(div.childNodes[i]);
      }
    }
    // must only have 1 root element
    const svgElm = div.firstElementChild;
    if (svgElm && svgElm.nodeName.toLowerCase() === 'svg') {
      const svgClass = svgElm.getAttribute('class') || '';
      svgElm.setAttribute('class', (svgClass + ' s-ion-icon').trim());
      // root element must be an svg
      // lets double check we've got valid elements
      // do not allow scripts
      if (isValid(svgElm)) {
        return div.innerHTML;
      }
    }
  }
  return '';
};
const isValid = (elm) => {
  if (elm.nodeType === 1) {
    if (elm.nodeName.toLowerCase() === 'script') {
      return false;
    }
    /* tslint:disable:prefer-for-of */
    for (let i = 0; i < elm.attributes.length; i++) {
      const val = elm.attributes[i].value;
      if (isStr(val) && val.toLowerCase().indexOf('on') === 0) {
        return false;
      }
    }
    for (let i = 0; i < elm.childNodes.length; i++) {
      if (!isValid(elm.childNodes[i])) {
        return false;
      }
    }
  }
  return true;
};

const ioniconContent = new Map();
const requests = new Map();
const getSvgContent = (url) => {
  // see if we already have a request for this url
  let req = requests.get(url);
  if (!req) {
    // we don't already have a request
    req = fetch(url).then(rsp => {
      if (rsp.ok) {
        return rsp.text().then(svgContent => {
          ioniconContent.set(url, validateContent(svgContent));
        });
      }
      ioniconContent.set(url, '');
    });
    // cache for the same requests
    requests.set(url, req);
  }
  return req;
};

const iconCss = ":host{display:inline-block;width:1em;height:1em;contain:strict;box-sizing:content-box !important;fill:var(--color);--color:currentColor}:host(.flow-color){--color:var(--flow-color-base)}:host .ionicon{stroke:var(--color)}.ionicon-fill-none{fill:none}.ionicon-stroke-width{stroke-width:32px;stroke-width:var(--flowicon-stroke-width, 32px)}.icon-inner,.ionicon,svg{display:block;height:100%;width:100%}:host(.flip-rtl) .icon-inner{transform:scaleX(-1)}:host(.icon-x-small){font-size:8px !important}:host(.icon-small){font-size:12px !important}:host(.icon-regular){font-size:16px !important}:host(.icon-large){font-size:24px !important}:host(.icon-x-large){font-size:32px !important}:host(.icon-xx-large){font-size:46px !important}";

const FlowIcon = /*@__PURE__*/ proxyCustomElement(class extends HTMLElement {
  constructor() {
    super();
    this.__registerHost();
    this.__attachShadow();
    this.isVisible = false;
    /**
     * If enabled, the icon will be loaded lazily when it is visible in the viewport.
     */
    this.lazy = false;
  }
  connectedCallback() {
    // purposely do not return the promise here because loading
    // the svg file should not hold up loading the app
    // only load the svg if it's visible
    this.waitUntilVisible(this.el, '50px', () => {
      this.isVisible = true;
      this.loadIcon();
    });
  }
  disconnectedCallback() {
    if (this.io) {
      this.io.disconnect();
      this.io = undefined;
    }
  }
  waitUntilVisible(el, rootMargin, cb) {
    if (Build.isBrowser && this.lazy && typeof window !== 'undefined' && window.IntersectionObserver) {
      const io = (this.io = new window.IntersectionObserver((data) => {
        if (data[0].isIntersecting) {
          io.disconnect();
          this.io = undefined;
          cb();
        }
      }, { rootMargin }));
      io.observe(el);
    }
    else {
      // browser doesn't support IntersectionObserver
      // so just fallback to always show it
      cb();
    }
  }
  loadIcon() {
    if (Build.isBrowser && this.isVisible) {
      const url = getUrl(this);
      if (url) {
        if (ioniconContent.has(url)) {
          // sync if it's already loaded
          this.svgContent = ioniconContent.get(url);
        }
        else {
          // async if it hasn't been loaded
          getSvgContent(url).then(() => (this.svgContent = ioniconContent.get(url)));
        }
      }
    }
    if (!this.ariaLabel) {
      const label = getName(null, this.icon);
      // user did not provide a label
      // come up with the label based on the icon name
      if (label) {
        this.ariaLabel = label.replace(/\-/g, ' ');
      }
    }
  }
  render() {
    return (h(Host, { role: "img", class: Object.assign(Object.assign({}, createColorClasses(this.color)), { [`icon-${this.size}`]: !!this.size }) }, Build.isBrowser && this.svgContent ? (h("div", { class: "icon-inner", innerHTML: this.svgContent })) : (h("div", { class: "icon-inner" }))));
  }
  static get assetsDirs() { return ["assets/svg"]; }
  get el() { return this; }
  static get watchers() { return {
    "icon": ["loadIcon"]
  }; }
  static get style() { return iconCss; }
}, [1, "flow-icon", {
    "ariaLabel": [1537, "aria-label"],
    "color": [1],
    "icon": [520],
    "size": [1],
    "lazy": [4],
    "svgContent": [32],
    "isVisible": [32]
  }]);
function defineCustomElement() {
  if (typeof customElements === "undefined") {
    return;
  }
  const components = ["flow-icon"];
  components.forEach(tagName => { switch (tagName) {
    case "flow-icon":
      if (!customElements.get(tagName)) {
        customElements.define(tagName, FlowIcon);
      }
      break;
  } });
}

export { FlowIcon as F, defineCustomElement as d };
