/*!
 * (C) Emu Analytics
 */
const DASH_REGEX = /[A-Z\u00C0-\u00D6\u00D8-\u00DE]/g;
function toDashCase(str) {
  return str.replace(DASH_REGEX, function (match) {
    return '-' + match.toLowerCase();
  });
}

/**
 * Helper to get injected Stencil @Element. Throws an exception if the injected host element is not defined.
 */
function getEl(obj) {
  const el = obj && obj.el;
  if (!el) {
    throw new Error(`Property 'el' required!`);
  }
  return el;
}
/**
 * Decorator to inject an ancestor element of specified type. The tag name of the ancestor can be specified
 * as an argument (e.g. 'flow-context') or, if omitted, the decorated property name will be used
 * (converted to kebab case). Note that the decorator works by hooking 'componentDidLoad'
 * so the injected ancestor will be null until the componentDidLoad lifecycle event has been fired.
 *
 * if `allowParentAttr` is true then the ancestor element can be specified directly using an attribute. This
 * allows an ancestor to be 'linked' rather than relying on DOM hierarchy.
 *
 * e.g.
 * // By hierarchy:
 * <flow-context><my-element></my-element></flow-context>
 *
 * // By 'link'
 * <flow-context id="my-context"></flow-context>
 * <my-element flow-context="my-context"></my-element>
 *
 * @param tagName tag name of ancestor to inject
 * @param allowParentAttr
 */
function AncestorElement(options = {}) {
  // Apply defaults
  options = Object.assign({ optional: false, tagName: null, supportAttr: true }, options);
  return function (prototype, propertyName) {
    const tagName = (options.tagName && options.tagName.toUpperCase()) || toDashCase(propertyName).toUpperCase();
    const attrName = tagName.toLowerCase();
    function findAncestor(obj) {
      const el = getEl(obj);
      let ancestorElement = el;
      do {
        if (options.supportAttr && ancestorElement.hasAttribute(attrName)) {
          ancestorElement = document.getElementById(ancestorElement.getAttribute(attrName));
          break;
        }
        else {
          if (ancestorElement.tagName === tagName) {
            break;
          }
        }
      } while ((ancestorElement = ancestorElement.parentElement));
      if (!options.optional && !ancestorElement) {
        throw new Error(`Cannot find ancestor ${tagName} for <${el.localName} id="${el.id}">`);
      }
      return ancestorElement;
    }
    if (delete prototype[propertyName]) {
      Object.defineProperty(prototype, propertyName, {
        get() {
          // TODO - possibly cache ancestor??
          return findAncestor(this);
        },
        enumerable: true,
        configurable: true
      });
    }
  };
}

export { AncestorElement as A };
