/*!
 * (C) Emu Analytics
 */
import { r as registerInstance, h, e as Host, g as getElement } from './index-abc3ebe6.js';
import { A as AncestorElement } from './component.utils-3df331de.js';
import { r as randomId } from './random-id-d84012dc.js';
import { T as debugLifecycle, U as formatEl } from './debug-eda7ee1f.js';
import { g as globalContextRegistry } from './context-registry-d4e711d5.js';
import { i as isNil, b as isArray } from './EventEmitter-1b1a10d1.js';
import './flow-core-context-291ff9b4.js';
import './maplibre-spec-helper-6abb210f.js';
import './constants-34bf9e9d.js';

/**
 * Core functionality for chart data. This class is wrapped by the `<flow-chart-data>` web component.
 */
class FlowCoreChartData {
  /**
   * Constructs a new `FlowCoreChartData` instance.
   *
   * @param flowChart Parent Flo.w chart.
   */
  constructor(flowChart) {
    this.flowChart = flowChart;
  }
  /**
   * Subscribe to reactive data updates.
   *
   * This method is called internally by the `<flow-chart-data>` component.
   *
   * @param options Chart data options
   */
  async subscribe(options) {
    this.unsubscribe();
    if (isNil(options.data)) {
      // set data to empty array
      this.flowChart.setData(options.name, [], options.xProperty, options.yProperty);
    }
    else if (isArray(options.data)) {
      this.flowChart.setData(options.name, options.data, options.xProperty, options.yProperty);
    }
    else {
      const context = await globalContextRegistry.getContext(this.flowChart.contextId);
      const parsedDataExpression = context.parser.parseExpression(options.data);
      if (parsedDataExpression.status) {
        this.autorunDisposer = context.state.autorun(() => {
          try {
            const data = context.parser.evaluateExpression(parsedDataExpression.value);
            this.flowChart.setData(options.name, data, options.xProperty, options.yProperty);
          }
          catch (e) {
            context.emitError(e);
          }
        }, `autorun: update(FlowChartData.${this.flowChart.id}.${options.name})`);
      }
      else {
        context.emitError(new Error(`Malformed expression - ${parsedDataExpression}`));
      }
    }
  }
  /**
   * Unsubscribe from reactive data updates.
   *
   * This method is called internally by the `<flow-chart-data>` component.
   */
  unsubscribe() {
    if (this.autorunDisposer) {
      this.autorunDisposer();
    }
  }
}

var __decorate = (undefined && undefined.__decorate) || function (decorators, target, key, desc) {
  var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
  if (typeof Reflect === "object" && typeof Reflect.decorate === "function")
    r = Reflect.decorate(decorators, target, key, desc);
  else
    for (var i = decorators.length - 1; i >= 0; i--)
      if (d = decorators[i])
        r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
  return c > 3 && r && Object.defineProperty(target, key, r), r;
};
const FlowChartData = class {
  constructor(hostRef) {
    registerInstance(this, hostRef);
    this.initialized = false;
  }
  connectedCallback() {
    if (this.initialized) {
      // Handle re-connect to DOM
      this.connect();
    }
  }
  async componentWillLoad() {
    if (!this.el.id) {
      this.el.setAttribute('id', randomId('chartdata'));
    }
    debugLifecycle(`${formatEl(this.el)} will load`);
    this._chartData = new FlowCoreChartData(this.flowChart._chart);
  }
  componentDidLoad() {
    debugLifecycle(`${formatEl(this.el)} did load`);
    this.initialized = true;
    this.connect();
  }
  disconnectedCallback() {
    this.disconnect();
  }
  connect() {
    this.subscribe();
  }
  disconnect() {
    this._chartData.unsubscribe();
  }
  subscribe() {
    this._chartData.subscribe({
      name: this.name,
      data: this.data,
      xProperty: this.x,
      yProperty: this.y
    });
  }
  render() {
    return h(Host, null);
  }
  get el() { return getElement(this); }
  static get watchers() { return {
    "data": ["subscribe"],
    "x": ["subscribe"],
    "y": ["subscribe"]
  }; }
};
__decorate([
  AncestorElement({ supportAttr: false })
], FlowChartData.prototype, "flowChart", void 0);

export { FlowChartData as flow_chart_data };
