/*!
 * (C) Emu Analytics
 */
import { B as keys, t as toJS, Q as debugMapStyle, k as remove, o as observable, e as computed, f as action } from './debug-eda7ee1f.js';
import { P as ParseError } from './parse-error-4f01ea15.js';
import { F as FlowCoreDataSource } from './flow-core-data-source-21977c39.js';
import { c as computedFn, M as MaplibreGLSpecHelper } from './maplibre-spec-helper-6abb210f.js';
import { E as ERRORS } from './constants-34bf9e9d.js';

const isUndefined = (val) => typeof val === 'undefined';
const isNull = (val) => val === null;
const isNil = (val) => isUndefined(val) || isNull(val);
const isDefined = (val) => !isNil(val);
const isString = (val) => typeof val === 'string';
const hasLength = (val) => isDefined(val) && val.length > 0;
const isStringFull = (val) => isString(val) && hasLength(val);
const isArray = (val) => Array.isArray(val);
const isArrayFull = (val) => Array.isArray(val) && hasLength(val);
const isArrayStrings = (val) => isArrayFull(val) && val.every((v) => isStringFull(v));
const isObject = (val) => typeof val === 'object' && !isNull(val);
const isObjectFull = (val) => isObject(val) && hasLength(Object.keys(val));
const isNumber = (val) => typeof val === 'number' && !Number.isNaN(val) && Number.isFinite(val);
const isFalse = (val) => val === false;
const isTrue = (val) => val === true;
const isIn = (val, arr) => isArrayFull(arr) && arr.some((o) => val === o);
const isBoolean = (val) => typeof val === 'boolean';
const isNumeric = (val) => isString(val) && /^[+-]?([0-9]*[.])?[0-9]+$/.test(val);
const isDate = (val) => val instanceof Date;
const isValue = (val) => isStringFull(val) || isNumber(val) || isBoolean(val) || isDate(val);
const hasValue = (val) => isArrayFull(val) ? val.every((o) => isValue(o)) : isValue(val);
// eslint-disable-next-line @typescript-eslint/ban-types
const isFunction = (val) => typeof val === 'function';
function isPlainObject(o) {
  if (!isObject(o)) {
    return false;
  }
  // If has modified constructor
  const ctor = o.constructor;
  /* istanbul ignore if */
  if (ctor === undefined) {
    return true;
  }
  // If has modified prototype
  const prot = ctor.prototype;
  /* istanbul ignore if */
  if (isObject(prot) === false) {
    return false;
  }
  // If constructor does not have an Object-specific method
  /* istanbul ignore if */
  // eslint-disable-next-line no-prototype-builtins
  if (prot.hasOwnProperty('isPrototypeOf') === false) {
    return false;
  }
  // Most likely a plain Object
  return true;
}
const urlRegex = new RegExp('^(https?:\\/\\/)?' + // protocol
  '((([a-z\\d]([a-z\\d-]*[a-z\\d])*)\\.)+[a-z]{2,}|' + // domain name
  '((\\d{1,3}\\.){3}\\d{1,3}))' + // OR ip (v4) address
  '(\\:\\d+)?(\\/[-a-z\\d%_.~+]*)*' + // port and path
  '(\\?[;&a-z\\d%_.~+=-]*)?' + // query string
  '(\\#[-a-z\\d_]*)?$', // fragment locator
'i');
function isURL(val) {
  return isString(val) && !!urlRegex.test(val);
}

/* Minimal example of a composite layer definition - no datasources and single layer */
const testSimpleLayer = {
  id: 'airquality_layer',
  layers: [
    {
      // Layer 1
      id: 'test',
      source: 'airquality',
      type: 'circle',
      style: {
        type: 'flow',
        fillColor: '#222222',
        strokeColor: 'white',
        strokeWidth: 2,
        labelText: '@name',
        labelColor: 'white',
        radius: 10
      }
    }
  ]
};
const testSimpleLayer2 = {
  id: 'airquality_layer',
  dataSources: [
    {
      id: 'airquality',
      type: 'vector',
      src: 'airquality'
    }
  ],
  layers: [
    {
      // Layer 1
      id: 'test',
      source: 'airquality',
      type: 'circle',
      style: {
        type: 'flow',
        fillColor: {
          type: 'expression',
          value: 'rgba(0, 0, 0, 0.5)'
        },
        strokeColor: {
          type: 'data-driven',
          attribute: 'name',
          default: 'gray',
          values: [
            ['^A', 'red', { regexp: true }],
            ['^B', 'green']
          ]
        },
        radius: {
          type: 'data-driven',
          attribute: 'no2',
          default: 10,
          values: [
            [0, 10],
            [5, 15],
            [100, 20]
          ]
        }
      }
    }
  ]
};
/**
 * Takes a single composite layer definition and expands it into a list of Maplibre-style layers.
 *
 * @param compositeLayer
 */
function expandCompositeLayer(compositeLayer, layerPrefix) {
  // Style is already a Maplibre style definition
  if (compositeLayer.style.type === 'maplibre') {
    return [compositeLayer];
  }
  else if (compositeLayer.style.type === 'flow') {
    const expandedLayers = [];
    switch (compositeLayer.type) {
      case 'circle':
        expandedLayers.push(...expandCompositeCircleLayer(compositeLayer, layerPrefix));
        break;
      case 'fill':
        expandedLayers.push(...expandCompositeFillLayer(compositeLayer, layerPrefix));
        break;
      case 'line':
        expandedLayers.push(...expandCompositeLineLayer(compositeLayer, layerPrefix));
        break;
      default:
        throw new Error('expandCompositeLayer: Unsupported layer type');
    }
    if (isStringFull(compositeLayer.style.labelText)) {
      expandedLayers.push(labelLayerForCompositeLayer(compositeLayer, layerPrefix));
    }
    return expandedLayers;
  }
  throw new Error(`expandCompositeLayer: Unsupported style type '${compositeLayer.style.type}}'`);
}
function addLayerPrefix(layerId, layerPrefix) {
  return layerPrefix ? `${layerPrefix}__${layerId}` : layerId;
}
function expandCompositeCircleLayer(layer, layerPrefix) {
  if (layer.style.type !== 'flow') {
    throw new Error('expandCompositeCircleLayer: Unsupported style type');
  }
  const styleRules = [];
  if (layer.style.fillColor) {
    styleRules.push(`circle-color: ${expandPropertyValue(layer.style.fillColor)};`);
  }
  if (layer.style.strokeColor) {
    styleRules.push(`circle-stroke-color: ${expandPropertyValue(layer.style.strokeColor)};`);
  }
  if (layer.style.strokeWidth) {
    styleRules.push(`circle-stroke-width: ${expandStrokeWidth(layer.style.strokeWidth)};`);
  }
  if (layer.style.radius) {
    styleRules.push(`circle-radius: ${expandCircleRadius(layer.style.radius)};`);
  }
  if (layer.style.opacity) {
    styleRules.push(`circle-opacity: ${expandPropertyValue(layer.style.opacity)};`);
    styleRules.push(`circle-stroke-opacity: ${expandPropertyValue(layer.style.opacity)};`);
  }
  if (isDefined(layer.style.visibility)) {
    styleRules.push(`visibility: ${expandPropertyValue(layer.style.visibility)};`);
  }
  const maplibreLayer = Object.assign(Object.assign({}, layer), { id: addLayerPrefix(layer.id, layerPrefix), interactive: isBoolean(layer.interactive) ? layer.interactive : true, type: 'circle', style: {
      type: 'maplibre',
      style: styleRules.join('\n')
    } });
  return [maplibreLayer];
}
function expandCompositeFillLayer(layer, layerPrefix) {
  if (layer.style.type !== 'flow' || layer.type !== 'fill') {
    throw new Error('expandCompositeFillLayer: Unsupported style type');
  }
  // Fill layer
  let fillStyleRules = [];
  if (layer.style.fillColor) {
    fillStyleRules.push(`fill-color: ${expandPropertyValue(layer.style.fillColor)};`);
  }
  if (layer.style.opacity) {
    fillStyleRules.push(`fill-opacity: ${expandPropertyValue(layer.style.opacity)};`);
  }
  if (isDefined(layer.style.visibility)) {
    fillStyleRules.push(`visibility: ${expandPropertyValue(layer.style.visibility)};`);
  }
  const fillLayer = Object.assign(Object.assign({}, layer), { id: addLayerPrefix(layer.id, layerPrefix), type: 'fill', interactive: isBoolean(layer.interactive) ? layer.interactive : true, style: {
      type: 'maplibre',
      style: fillStyleRules.join('\n')
    } });
  // Line layer (outline of polygon)
  let lineStyleRules = [];
  if (layer.style.strokeColor) {
    lineStyleRules.push(`line-color: ${expandPropertyValue(layer.style.strokeColor)};`);
  }
  if (layer.style.strokeWidth) {
    lineStyleRules.push(`line-width: ${expandStrokeWidth(layer.style.strokeWidth)};`);
  }
  if (layer.style.opacity) {
    lineStyleRules.push(`line-opacity: ${expandPropertyValue(layer.style.opacity)};`);
  }
  if (isDefined(layer.style.visibility)) {
    lineStyleRules.push(`visibility: ${expandPropertyValue(layer.style.visibility)};`);
  }
  const lineLayer = Object.assign(Object.assign({}, layer), { id: `${addLayerPrefix(layer.id, layerPrefix)}__stroke`, type: 'line', interactive: isBoolean(layer.interactive) ? layer.interactive : true, style: {
      type: 'maplibre',
      style: lineStyleRules.join('\n')
    } });
  return [fillLayer, lineLayer];
}
function expandCompositeLineLayer(layer, layerPrefix) {
  if (layer.style.type !== 'flow' || layer.type !== 'line') {
    throw new Error('expandCompositeLineLayer: Unsupported style type');
  }
  const styleRules = [];
  if (layer.style.strokeColor) {
    styleRules.push(`line-color: ${expandPropertyValue(layer.style.strokeColor)};`);
  }
  if (layer.style.strokeWidth) {
    styleRules.push(`line-width: ${expandStrokeWidth(layer.style.strokeWidth)};`);
  }
  if (isDefined(layer.style.visibility)) {
    styleRules.push(`visibility: ${expandPropertyValue(layer.style.visibility)};`);
  }
  const maplibreLayer = Object.assign(Object.assign({}, layer), { id: addLayerPrefix(layer.id, layerPrefix), type: 'line', interactive: isBoolean(layer.interactive) ? layer.interactive : true, style: {
      type: 'maplibre',
      style: styleRules.join('\n')
    } });
  return [maplibreLayer];
}
function expandCompositeLayerFlowStyle(layerType, style) {
  let styleRules = [];
  switch (layerType) {
    case 'circle':
      if (style.fillColor) {
        styleRules.push(`circle-color: ${expandPropertyValue(style.fillColor)};`);
      }
      if (style.strokeColor) {
        styleRules.push(`circle-stroke-color: ${expandPropertyValue(style.strokeColor)};`);
      }
      if (style.strokeWidth) {
        styleRules.push(`circle-stroke-width: ${expandStrokeWidth(style.strokeWidth)};`);
      }
      if (style.radius) {
        styleRules.push(`circle-radius: ${expandCircleRadius(style.radius)};`);
      }
      if (style.opacity) {
        styleRules.push(`circle-opacity: ${expandPropertyValue(style.opacity)};`);
      }
      break;
    case 'line':
      if (style.strokeColor) {
        styleRules.push(`line-color: ${expandPropertyValue(style.strokeColor)};`);
      }
      if (style.strokeWidth) {
        styleRules.push(`line-width: ${expandStrokeWidth(style.strokeWidth)};`);
      }
      if (style.opacity) {
        styleRules.push(`line-opacity: ${expandPropertyValue(style.opacity)};`);
      }
      break;
    case 'fill':
      if (style.fillColor) {
        styleRules.push(`fill-color: ${expandPropertyValue(style.fillColor)};`);
      }
      if (style.opacity) {
        styleRules.push(`fill-opacity: ${expandPropertyValue(style.opacity)};`);
      }
      break;
    default:
      throw new Error('expandCompositeLayerFlowStyle: Unsupported layer type');
  }
  if (isDefined(style.visibility)) {
    styleRules.push(`visibility: ${expandPropertyValue(style.visibility)};`);
  }
  return {
    type: 'maplibre',
    style: styleRules.join('\n')
  };
}
function labelLayerForCompositeLayer(layer, layerPrefix) {
  if (layer.style.type !== 'flow' || !isStringFull(layer.style.labelText)) {
    return;
  }
  const styleRules = [];
  styleRules.push(`text-field: ${expandPropertyValue(layer.style.labelText)};`);
  if (layer.style.labelColor) {
    styleRules.push(`text-color: ${expandPropertyValue(layer.style.labelColor)};`);
  }
  if (layer.style.labelHaloColor) {
    styleRules.push(`text-halo-color: ${expandPropertyValue(layer.style.labelHaloColor)};`);
    styleRules.push(`text-halo-blur: 1;`);
    styleRules.push(`text-halo-width: 2;`);
  }
  if (layer.style.labelSize) {
    styleRules.push(`text-size: ${expandLabelSize(layer.style.labelSize)};`);
  }
  if (layer.style.opacity) {
    styleRules.push(`text-opacity: ${expandPropertyValue(layer.style.opacity)};`);
  }
  if (isDefined(layer.style.visibility)) {
    styleRules.push(`visibility: ${expandPropertyValue(layer.style.visibility)};`);
  }
  if (layer.type === 'circle') {
    styleRules.push(`text-variable-anchor: ['top'];`);
    styleRules.push(`text-radial-offset: 1.2;`);
  }
  const labelLayer = Object.assign(Object.assign({}, layer), { id: `${addLayerPrefix(layer.id, layerPrefix)}__label`, interactive: isBoolean(layer.interactive) ? layer.interactive : false, type: 'symbol', style: {
      type: 'maplibre',
      style: styleRules.join('\n')
    } });
  return labelLayer;
}
function expandPropertyValue(value) {
  if (isString(value)) {
    // Support short-hand feature property expression
    return value.startsWith('@') ? value : `'${value}'`;
  }
  if (isNumber(value) || isBoolean(value)) {
    return value.toString();
  }
  if (isLiteralProperty(value)) {
    if (isString(value.value)) {
      return `'${value.value}'`;
    }
    else {
      return value.value.toString();
    }
  }
  if (isExpressionProperty(value)) {
    return value.value;
  }
  if (isDataDrivenProperty(value)) {
    return expandDataDrivenPropertyValue(value);
  }
  throw new Error('expandPropertyValue: Unsupported property value type');
}
function isLiteralProperty(value) {
  return isPlainObject(value) && value.type === 'literal';
}
function isExpressionProperty(value) {
  return isPlainObject(value) && value.type === 'expression';
}
function isDataDrivenProperty(value) {
  return isPlainObject(value) && value.type === 'data-driven';
}
function isSupportedLayerType(type) {
  return ['fill', 'line', 'circle'].includes(type);
}
function expandDataDrivenPropertyValue(value) {
  // check values
  const types = value.values.map((v) => typeof v[0]);
  // Check if all values are of the same type
  if (!types.every((t) => t === types[0])) {
    throw new Error('expandPropertyValue: All case values in a data-driven property must be of the same type');
  }
  switch (value.mode) {
    case 'off':
      // return fixed default value
      return expandPropertyValue(value.default);
    case 'interpolate':
      return expandDataDrivenInterpolateModePropertyValue(value);
    default:
      return expandDataDrivenMatchModePropertyValue(value);
  }
}
function expandDataDrivenInterpolateModePropertyValue(value) {
  const cases = value.values
    .map((v) => {
    const inputVal = v[0];
    const outputVal = v[1];
    if (!isNumber(inputVal)) {
      throw new Error('expandPropertyValue: Input values for interpolated data-driven property must be numeric');
    }
    return `${expandPropertyValue(inputVal)}, ${expandPropertyValue(outputVal)}`;
  })
    .join(', ');
  return `typeof(get("${value.attribute}")) == 'number' ? interpolate(linear(), get("${value.attribute}"), ${cases}) : ${expandPropertyValue(value.default)}`;
}
function expandDataDrivenMatchModePropertyValue(value) {
  const propertyType = typeof value.values[0][0];
  if (propertyType === 'string') {
    const cases = value.values
      .map((v) => {
      let operator;
      const caseMatch = v[0];
      const caseValue = v[1];
      const options = v[2];
      if (options === null || options === void 0 ? void 0 : options.regexp) {
        operator = options.caseSensitive ? '~' : '~*';
        // Check if the regex is valid
        try {
          new RegExp(caseValue);
        }
        catch (e) {
          // Invalid regex
          throw new Error('expandPropertyValue: Invalid regex in data-driven property');
        }
      }
      else {
        operator = (options === null || options === void 0 ? void 0 : options.caseSensitive) ? '==' : '=*';
      }
      return `get("${value.attribute}") ${operator} '${escapeMaplibreString(caseMatch)}', ${expandPropertyValue(caseValue)}`;
    })
      .join(', ');
    return `case(${cases}, ${expandPropertyValue(value.default)})`;
  }
  else if (propertyType === 'number') {
    const cases = orderByArrayIndex(value.values, [0], ['desc'])
      .map((v) => {
      const caseMatch = v[0];
      const caseValue = v[1];
      return `get("${value.attribute}") >= ${caseMatch}, ${expandPropertyValue(caseValue)}`;
    })
      .join(', ');
    return `case(get("${value.attribute}") == null, ${expandPropertyValue(value.default)}, ${cases}, ${expandPropertyValue(value.default)})`;
  }
  else {
    return expandPropertyValue(value.default);
  }
}
function escapeMaplibreString(value) {
  return value.replace(/['\\]/g, '\\$&'); // $& means the whole matched string
}
function expandStrokeWidth(strokeWidth) {
  if (isString(strokeWidth) && !strokeWidth.startsWith('$')) {
    switch (strokeWidth) {
      case 'x-small':
        // Hairline
        strokeWidth = 0.5;
        break;
      case 'small':
        strokeWidth = { type: 'expression', value: 'interpolate(linear(), zoom(), 0, 1, 10, 1, 20, 2)' };
        break;
      case 'medium':
        strokeWidth = { type: 'expression', value: 'interpolate(linear(), zoom(), 0, 1, 10, 2, 20, 4)' };
        break;
      case 'large':
        strokeWidth = { type: 'expression', value: 'interpolate(linear(), zoom(), 0, 2, 10, 4, 20, 8)' };
        break;
      case 'x-large':
        strokeWidth = { type: 'expression', value: 'interpolate(linear(), zoom(), 0, 4, 10, 8, 20, 16)' };
        break;
      default:
        throw new Error(`expandStrokeWidth: Unsupported stroke width '${strokeWidth}'`);
    }
  }
  return expandPropertyValue(strokeWidth);
}
function expandLabelSize(labelSize) {
  if (isString(labelSize) && !labelSize.startsWith('$')) {
    switch (labelSize) {
      case 'x-small':
        labelSize = 8;
        break;
      case 'small':
        labelSize = { type: 'expression', value: 'interpolate(linear(), zoom(), 0, 4, 10, 8, 20, 15)' };
        break;
      case 'medium':
        labelSize = { type: 'expression', value: 'interpolate(linear(), zoom(), 0, 5, 10, 10, 20, 20)' };
        break;
      case 'large':
        labelSize = { type: 'expression', value: 'interpolate(linear(), zoom(), 0, 8, 10, 14, 20, 30)' };
        break;
      case 'x-large':
        labelSize = { type: 'expression', value: 'interpolate(linear(), zoom(), 0, 10, 10, 18, 20, 36)' };
        break;
      default:
        throw new Error(`expandStrokeWidth: Unsupported label size '${labelSize}'`);
    }
  }
  return expandPropertyValue(labelSize);
}
function expandCircleRadius(circleRadius) {
  if (isString(circleRadius) && !circleRadius.startsWith('$')) {
    switch (circleRadius) {
      case 'x-small':
        circleRadius = { type: 'expression', value: 'interpolate(linear(), zoom(), 0, 1, 10, 2, 20, 8)' };
        break;
      case 'small':
        circleRadius = { type: 'expression', value: 'interpolate(linear(), zoom(), 0, 1, 10, 3, 20, 12)' };
        break;
      case 'medium':
        circleRadius = { type: 'expression', value: 'interpolate(linear(), zoom(), 0, 2, 10, 5, 20, 20)' };
        break;
      case 'large':
        circleRadius = { type: 'expression', value: 'interpolate(linear(), zoom(), 0, 3, 10, 7, 20, 28)' };
        break;
      case 'x-large':
        circleRadius = { type: 'expression', value: 'interpolate(linear(), zoom(), 0, 4, 10, 10, 20, 40)' };
        break;
      default:
        throw new Error(`expandStrokeWidth: Unsupported stroke width '${circleRadius}'`);
    }
  }
  return expandPropertyValue(circleRadius);
}
function orderByArrayIndex(array, indexes, directions) {
  const sorted = [...array];
  sorted.sort((a, b) => {
    for (let i = 0; i < indexes.length; i++) {
      const index = indexes[i];
      const direction = directions[i];
      if (a[index] < b[index]) {
        return direction === 'asc' ? -1 : 1;
      }
      else if (a[index] > b[index]) {
        return direction === 'asc' ? 1 : -1;
      }
    }
    return 0;
  });
  return sorted;
}

/**
 * @internal
 */
class ValidationError extends Error {
  constructor(message, source, errors) {
    super(ValidationError.formatMessage(message, source, errors));
    this.name = ERRORS.ValidationError;
    this.source = source;
    this.errors = errors;
  }
  static formatMessage(message, _source, errors) {
    return [message].concat(errors).join('\n');
  }
}

var __decorate$1 = (undefined && undefined.__decorate) || function (decorators, target, key, desc) {
  var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
  if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
  else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
  return c > 3 && r && Object.defineProperty(target, key, r), r;
};
/**
 * Core functionality for Flo.w map layers. Wrapped by the `<flow-map-layer>` web component.
 *
 * @noInheritDoc
 */
class FlowCoreMapLayer {
  /**
   * Constructs a new `FlowCoreMapLayer` instance.
   *
   * @param options Map layer options.
   */
  constructor(options) {
    this.minZoom = 0;
    this.maxZoom = 18;
    this.parsedStyle = {};
    /**
     * Computed function to collect layer properties from the parsed style base on type 'paint' or 'layout'
     * Also calls expandLayerProperty on each property to expand state variables etc.
     *
     */
    this.collectLayerProperties = computedFn(function collectLayerProperties(propertyType) {
      const props = keys(this.parsedStyle)
        .filter((propertyKey) => MaplibreGLSpecHelper.instance.getPropertyType(propertyKey, this.options.type) === propertyType)
        .reduce((acc, propertyKey) => {
        acc[propertyKey] = this.expandLayerProperty(propertyKey);
        return acc;
      }, {});
      return Object.keys(props).length ? props : null;
    }, { name: 'collectLayersProperties' });
    /**
     * Computed function to retrieve expanded property expression from parsed style.
     *
     * 'Expansion' includes:
     *    - replacing state variables
     *    - converting visibility form boolean expression to 'none'/'visible' string literal
     *
     */
    this.expandLayerProperty = computedFn(function expandLayerProperty(propertyKey) {
      // Clone and return plain non-observable property
      let property = toJS(this.parsedStyle[propertyKey].expr, { detectCycles: false });
      property = this.options.flowMap.context.parser.expandReactiveExpression(property);
      // Visibility is treated as a boolean property but needs to be converted to string property
      // for maplibre to handle. State variables have already been expanded above so any state changes
      // will re-trigger computing the visibility property.
      if (propertyKey === 'visibility') {
        try {
          property = MaplibreGLSpecHelper.instance.evaluateExpression(property) ? 'visible' : 'none';
        }
        catch (e) {
          this.options.flowMap.context.emitError(e);
        }
      }
      return property;
    });
    this.id = options.id;
    this.options = Object.assign({ minZoom: 0, maxZoom: 18 }, options);
    this.minZoom = this.options.minZoom;
    this.maxZoom = this.options.maxZoom;
  }
  get maplibreStyleSpecification() {
    return {
      datasources: {},
      layers: [this.maplibreLayerDef]
    };
  }
  /**
   * Get computed MaplibreGL style specification for this layer.
   *
   * <flow-badge pill>Reactive</flow-badge>
   */
  get maplibreLayerDef() {
    const layer = {
      id: this.options.id
    };
    if (typeof this.minZoom !== 'undefined') {
      layer.minzoom = this.minZoom;
    }
    if (typeof this.maxZoom !== 'undefined') {
      layer.maxzoom = this.maxZoom;
    }
    if (this.options.type) {
      layer.type = this.options.type;
    }
    if (this.options.source) {
      layer.source = this.options.source;
    }
    if (this.options.sourceLayer) {
      layer['source-layer'] = this.options.sourceLayer;
    }
    if (this.options.metadata) {
      layer.metadata = this.options.metadata;
    }
    const paintProps = this.collectLayerProperties('paint');
    const layoutProps = this.collectLayerProperties('layout');
    if (this.parsedFilter) {
      layer.filter = this.options.flowMap.context.parser.expandReactiveExpression(toJS(this.parsedFilter));
    }
    if (paintProps) {
      layer.paint = paintProps;
    }
    if (layoutProps) {
      layer.layout = layoutProps;
    }
    return layer;
  }
  /**
   * Set the layer style.
   *
   * @param style Style as a Flo.w layer style specification string.
   */
  setStyle(style) {
    style = style || '';
    if (isString(style)) {
      this.style = {
        type: 'maplibre',
        style
      };
    }
    else {
      this.style = style;
    }
    let maplibreStyle;
    if (this.style.type === 'maplibre') {
      maplibreStyle = this.style;
    }
    else {
      if (!isSupportedLayerType(this.options.type)) {
        throw new Error(`Unsupported flow-style layer type '${this.options.type}'`);
      }
      maplibreStyle = expandCompositeLayerFlowStyle(this.options.type, this.style);
    }
    const parseResult = this.options.flowMap.context.parser.parseProgram(maplibreStyle.style);
    if (parseResult.status) {
      // Validate all style properties and collect errors
      const errors = Object.keys(parseResult.value).reduce((acc, prop) => {
        const errs = this.validateParsedProperty(prop, parseResult.value[prop]);
        if (errs) {
          acc = acc.concat(errs);
        }
        return acc;
      }, []);
      if (errors && errors.length) {
        this.options.flowMap.context.emitError(new ValidationError('Validation errors in style:', maplibreStyle.style, errors));
      }
      const parsedStyle = Object.keys(parseResult.value).reduce((acc, prop) => {
        acc[prop] = parseResult.value[prop];
        return acc;
      }, {});
      this.parsedStyle = parsedStyle;
    }
    else {
      this.options.flowMap.context.emitError(new ParseError(maplibreStyle.style, parseResult));
    }
  }
  /**
   * Set an individual style property.
   *
   * @param property Property name.
   * @param value Property style expression.
   */
  setStyleProperty(property, value) {
    debugMapStyle('setting style property', property, value);
    const errors = this.validateParsedProperty(property, { expr: value, index: null });
    if (errors && errors.length) {
      this.options.flowMap.context.emitError(new ValidationError(`Validation error:`, property, errors));
      return;
    }
    if (value === null) {
      // Remove property
      remove(this.parsedStyle, property);
      return;
    }
    const parseResult = this.options.flowMap.context.parser.parseExpression(value);
    if (!parseResult.status) {
      this.options.flowMap.context.emitError(new ParseError(value, parseResult));
      return;
    }
    else {
      this.parsedStyle[property] = { expr: parseResult.value, index: null };
    }
  }
  /**
   * This is an internal method used by FlowCoreMapCompositeLayer to set a parsed style property on its sub-layers.
   * @param property
   * @param value
   * @returns
   */
  setParsedStyleProperty(property, value) {
    debugMapStyle('setting style property', property, value);
    const errors = this.validateParsedProperty(property, { expr: value, index: null });
    if (errors && errors.length) {
      this.options.flowMap.context.emitError(new ValidationError(`Validation error:`, property, errors));
      return;
    }
    if (value === null) {
      // Remove property
      remove(this.parsedStyle, property);
      return;
    }
    this.parsedStyle[property] = { expr: value, index: null };
  }
  setSource(source) {
    if (!isString(source)) {
      this.options.flowMap.context.emitError(new Error('FlowCoreMapLayer: layer source must be a string datasource identifier.'));
      return;
    }
    this.options.source = source;
  }
  /**
   * Set the layer filter.
   *
   * @param filter Layer filter expression.
   */
  setFilter(filter) {
    const errors = this.validateFilter(filter);
    if (errors && errors.length) {
      this.options.flowMap.context.emitError(new ValidationError('Validaton error in filter:', filter, errors));
      return;
    }
    if (!filter) {
      // Remove property
      this.parsedFilter = null;
      return;
    }
    const parseResult = this.options.flowMap.context.parser.parseExpression(filter);
    if (!parseResult.status) {
      this.options.flowMap.context.emitError(new ParseError(filter, parseResult));
      return;
    }
    else {
      this.parsedFilter = parseResult.value;
    }
  }
  /**
   * Set multiple layer style properties in a single transaction.
   *
   * @param properties Property name/value pairs.
   */
  setStyleProperties(properties) {
    Object.keys(properties).forEach((p) => this.setStyleProperty(p, properties[p]));
  }
  setInteractive(interactive = true) {
    this.options.interactive = interactive;
  }
  /**
   * Validate parsed property expression. Returns errors as an array of error messages.
   *
   */
  validateParsedProperty(prop, expr) {
    const errors = [];
    const metadata = MaplibreGLSpecHelper.instance.getPropertyMetadata(prop, this.options.type);
    if (!metadata) {
      const index = (expr.index && ` at line ${expr.index.line}, column ${expr.index.column}`) || '';
      errors.push(`Unrecognized layer property '${prop}'${index}`);
    }
    return errors;
  }
  validateFilter(_filter) {
    const errors = [];
    // TODO
    return errors;
  }
}
__decorate$1([
  observable
], FlowCoreMapLayer.prototype, "minZoom", void 0);
__decorate$1([
  observable
], FlowCoreMapLayer.prototype, "maxZoom", void 0);
__decorate$1([
  observable
], FlowCoreMapLayer.prototype, "style", void 0);
__decorate$1([
  observable
], FlowCoreMapLayer.prototype, "parsedStyle", void 0);
__decorate$1([
  observable
], FlowCoreMapLayer.prototype, "parsedFilter", void 0);
__decorate$1([
  computed
], FlowCoreMapLayer.prototype, "maplibreStyleSpecification", null);
__decorate$1([
  computed
], FlowCoreMapLayer.prototype, "maplibreLayerDef", null);
__decorate$1([
  action
], FlowCoreMapLayer.prototype, "setStyle", null);
__decorate$1([
  action
], FlowCoreMapLayer.prototype, "setStyleProperty", null);
__decorate$1([
  action
], FlowCoreMapLayer.prototype, "setParsedStyleProperty", null);
__decorate$1([
  action
], FlowCoreMapLayer.prototype, "setSource", null);
__decorate$1([
  action
], FlowCoreMapLayer.prototype, "setFilter", null);
__decorate$1([
  action
], FlowCoreMapLayer.prototype, "setStyleProperties", null);
function isMaplibreLayer(layer) {
  return layer.options.type !== 'composite';
}

var __decorate = (undefined && undefined.__decorate) || function (decorators, target, key, desc) {
  var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
  if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
  else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
  return c > 3 && r && Object.defineProperty(target, key, r), r;
};
/**
 * Core functionality for Flo.w map layers. Wrapped by the `<flow-map-layer>` web component.
 *
 * @noInheritDoc
 */
class FlowCoreMapCompositeLayer {
  /**
   * Constructs a new `FlowCoreMapLayer` instance.
   *
   * @param options Map layer options.
   */
  constructor(options) {
    this.parsedStyle = {};
    this.minZoom = 0;
    this.maxZoom = 18;
    this.layers = [];
    this.id = options.id;
    this.options = Object.assign({ minZoom: 0, maxZoom: 18 }, options);
    this.setSource(this.options.source);
  }
  get maplibreStyleSpecification() {
    const datasources = this.datasources.reduce((datasources, datasource) => {
      datasources[datasource.id] = datasource.maplibreSourceSpecification;
      return datasources;
    }, {});
    const layers = this.layers.map((layer) => layer.maplibreStyleSpecification.layers).flat();
    return {
      datasources,
      layers
    };
  }
  setSource(source) {
    if (isString(source)) {
      this.options.flowMap.context.emitError(new Error('FlowCoreMapCompositeLayer: layer source must be a CompositeLayerSpecification.'));
      return;
    }
    this.source = source;
    this.createSubLayers();
  }
  setStyle(style) {
    style = style || '';
    const parseResult = this.options.flowMap.context.parser.parseProgram(style);
    if (parseResult.status) {
      this.parsedStyle = parseResult.value;
      this.createSubLayers();
    }
    else {
      this.options.flowMap.context.emitError(new ParseError(style, parseResult));
    }
  }
  setFilter(filter) {
    if (!filter) {
      // Remove property
      this.parsedFilter = null;
      return;
    }
    const parseResult = this.options.flowMap.context.parser.parseExpression(filter);
    if (!parseResult.status) {
      this.options.flowMap.context.emitError(new ParseError(filter, parseResult));
      return;
    }
    else {
      this.parsedFilter = parseResult.value;
    }
  }
  /**
   * Creates sub-layers and data sources from the composite layer specification.
   */
  createSubLayers() {
    var _a, _b;
    this.layers = [];
    // The composite layer's visibility is applied to all sub-layers if specified in the layer's style.
    const compositeLayerVisibility = (_a = this.parsedStyle.visibility) === null || _a === void 0 ? void 0 : _a.expr;
    for (const layer of this.source.layers) {
      const expandedLayers = expandCompositeLayer(layer, this.options.id);
      for (const expandedLayer of expandedLayers) {
        const subLayer = new FlowCoreMapLayer(Object.assign(Object.assign({}, expandedLayer), { metadata: Object.assign(Object.assign({}, this.options.metadata), { composite: true, compositeParentLayer: this.id }), flowMap: this.options.flowMap }));
        if (expandedLayer.filter) {
          subLayer.setFilter(expandedLayer.filter);
        }
        subLayer.setStyle(expandedLayer.style);
        // Apply composite layer visibility to sub-layers if specified
        if (isDefined(compositeLayerVisibility)) {
          subLayer.setParsedStyleProperty('visibility', compositeLayerVisibility);
        }
        this.layers.push(subLayer);
      }
    }
    // Create datasources from the composite layer specification.
    this.datasources = ((_b = this.source.dataSources) !== null && _b !== void 0 ? _b : []).map((datasource) => new FlowCoreDataSource(Object.assign(Object.assign({}, datasource), { flowMap: this.options.flowMap })));
  }
}
__decorate([
  observable
], FlowCoreMapCompositeLayer.prototype, "parsedFilter", void 0);
__decorate([
  observable
], FlowCoreMapCompositeLayer.prototype, "minZoom", void 0);
__decorate([
  observable
], FlowCoreMapCompositeLayer.prototype, "maxZoom", void 0);
__decorate([
  observable
], FlowCoreMapCompositeLayer.prototype, "layers", void 0);
__decorate([
  observable
], FlowCoreMapCompositeLayer.prototype, "datasources", void 0);
__decorate([
  computed
], FlowCoreMapCompositeLayer.prototype, "maplibreStyleSpecification", null);
__decorate([
  action
], FlowCoreMapCompositeLayer.prototype, "setSource", null);
__decorate([
  action
], FlowCoreMapCompositeLayer.prototype, "setStyle", null);
function isCompositeLayer(layer) {
  return layer.options.type === 'composite';
}

export { FlowCoreMapCompositeLayer as F, expandCompositeCircleLayer as a, expandCompositeFillLayer as b, expandCompositeLineLayer as c, expandCompositeLayerFlowStyle as d, expandCompositeLayer as e, expandPropertyValue as f, isExpressionProperty as g, isDataDrivenProperty as h, isLiteralProperty as i, isSupportedLayerType as j, escapeMaplibreString as k, labelLayerForCompositeLayer as l, expandStrokeWidth as m, expandLabelSize as n, expandCircleRadius as o, orderByArrayIndex as p, isCompositeLayer as q, FlowCoreMapLayer as r, isMaplibreLayer as s };
