/*!
 * (C) Emu Analytics
 */
import { r as registerInstance, h, e as Host, g as getElement } from './index-abc3ebe6.js';
import { A as AncestorElement } from './component.utils-3df331de.js';
import { r as randomId } from './random-id-d84012dc.js';
import { t as toJS, V as debugDataJoin, a as autorun, o as observable, e as computed, T as debugLifecycle, U as formatEl } from './debug-eda7ee1f.js';
import { c as FLOW_MAP_PARAMETER_TAG } from './constants-a1f3922c.js';
import { F as FlowCoreDataSource } from './flow-core-data-source-21977c39.js';
import { i as isNil, h as isURL } from './EventEmitter-1b1a10d1.js';
import './index-15bb9065.js';
import './ensure-array-ffbb7493.js';

var __decorate$1 = (undefined && undefined.__decorate) || function (decorators, target, key, desc) {
  var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
  if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
  else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
  return c > 3 && r && Object.defineProperty(target, key, r), r;
};
/**
 * Core functionality for Flo.w map data joins. Wrapped by the `<flow-map-data-join>` web component.
 *
 * @noInheritDoc
 */
class FlowCoreDataJoin {
  /**
   * Constructs a new `FlowCoreDataJoin` instance.
   *
   * @param options Map data join options.
   */
  constructor(options) {
    this.options = Object.assign(Object.assign({}, FlowCoreDataSource.defaultOptions), options);
    this.id = options.id;
    this.src = options.src;
  }
  /**
   * Return the data source for the map data join.
   *
   * If `src` is a Flo.w reactive expression, the expression will be expanded to
   * return an array of data objects.
   *
   * <flow-badge pill>Reactive</flow-badge>
   *
   * @returns array of data, dataset ID or external URL.
   */
  get dataSrc() {
    if (typeof this._src === 'string' && this._src.startsWith('$')) {
      // src is a state variable - should contain an array of data objects
      const context = this.options.flowMap.context;
      return toJS(context.state.get(this._src));
    }
    else {
      // src is an array, dataset ID or external URL
      return this._src;
    }
  }
  /**
   * Return the parameters associated with the map data join.
   *
   * <flow-badge pill>Reactive</flow-badge>
   */
  get parameters() {
    return this._parameters.map(p => {
      return p.parameterDef;
    });
  }
  /**
   * Start the reactive map data join.
   */
  start() {
    debugDataJoin(`Starting data join: ${this.id}`);
    this.stop();
    this.disposer = autorun(async () => {
      const src = this.dataSrc;
      const params = this.parameters;
      const missingParams = !!params.find(p => p.required && isNil(p.value));
      if (src === null || missingParams) {
        // Null source or required params missing
        debugDataJoin(`Skipping data join: ${this.id} - src ${src} missing ${missingParams}`);
        return;
      }
      debugDataJoin(`Performing data join: ${this.id}`);
      let data;
      if (typeof src === 'string') {
        // Load data from dataset or url
        try {
          data = await this.loadData(src, params);
        }
        catch (e) {
          this.options.flowMap.context.emitWarning(`Failed to load data for data join '${this.id}': ${e.message}`);
          data = [];
        }
      }
      else {
        data = src;
      }
      this.doDataJoin(data);
    }, { name: `autorun: dataJoin(FlowDataJoin.${this.options.flowMap.id}.${this.id})` });
  }
  /**
   * Stop the reactive map data join.
   */
  stop() {
    if (this.disposer) {
      debugDataJoin(`Stopping data join: ${this.id}`);
      this.disposer();
      this.disposer = null;
    }
  }
  /**
   * Set the `src` property. One of
   *
   * - array of data objects.
   * - Flo.w reactive expression.
   * - dataset ID.
   * - external URL.
   */
  set src(src) {
    this._src = src;
  }
  /**
   * Get the `src` property.
   */
  get src() {
    return this._src;
  }
  /**
   * Set parameters associated with this data join.
   *
   * Called internally by the `flow-map-data-join` web component. If set programmatically,
   * it will be overwritten by the `flow-map-data-join` web component.
   *
   * @param parameters Data join parameters.
   */
  setParameters(parameters) {
    this._parameters = parameters;
  }
  loadData(datasetOrUrl, paramDefs) {
    if (isURL(datasetOrUrl)) {
      const url = new URL(datasetOrUrl);
      // Add datajoin parameters to querystring
      const params = new URLSearchParams(url.searchParams);
      paramDefs.forEach(p => {
        params.append(p.name, p.value);
      });
      url.search = params.toString();
      const headers = new Headers({ Accept: 'application/json' });
      const request = new Request(url.toString(), {
        method: 'GET',
        mode: 'cors',
        headers,
        cache: 'default'
      });
      return fetch(request).then(response => {
        if (!response.ok) {
          throw new Error(`status=${response.status} ${response.statusText}`);
        }
        return response.json();
      });
    }
    else {
      // Dataset - run query using flowClient
      const context = this.options.flowMap.context;
      const params = paramDefs.reduce((accum, p) => {
        accum[p.name] = p.value;
        return accum;
      }, {});
      return context.flowClient.datasets.query(datasetOrUrl, {}, params);
    }
  }
  doDataJoin(data) {
    const { id, foreignKey, clear, flowDataSource, flowMap } = this.options;
    const source = flowDataSource.id;
    const sourceLayer = flowDataSource.type === 'vector' ? this.options.sourceLayer || 'geometry' : null;
    if (clear) {
      flowMap.removeFeatureState({ source, sourceLayer }, id);
    }
    if (data) {
      data.forEach(d => flowMap.setFeatureState({ id: d[foreignKey], source, sourceLayer }, { [id]: d }));
    }
  }
}
FlowCoreDataJoin.defaultOptions = {
  foreignKey: 'id'
};
__decorate$1([
  observable
], FlowCoreDataJoin.prototype, "_parameters", void 0);
__decorate$1([
  observable
], FlowCoreDataJoin.prototype, "_src", void 0);
__decorate$1([
  computed
], FlowCoreDataJoin.prototype, "dataSrc", null);
__decorate$1([
  computed
], FlowCoreDataJoin.prototype, "parameters", null);

var __decorate = (undefined && undefined.__decorate) || function (decorators, target, key, desc) {
  var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
  if (typeof Reflect === "object" && typeof Reflect.decorate === "function")
    r = Reflect.decorate(decorators, target, key, desc);
  else
    for (var i = decorators.length - 1; i >= 0; i--)
      if (d = decorators[i])
        r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
  return c > 3 && r && Object.defineProperty(target, key, r), r;
};
const FlowMapDataJoin = class {
  constructor(hostRef) {
    registerInstance(this, hostRef);
    /** The foreign key property of the joined data. Data will be joined to features
     * where the foreign key equals the feature ID.
     */
    this.foreignKey = 'id';
    /**
     * Set to `true` to clear all current feature state before applying the data join.
     */
    this.clear = false;
  }
  updateSrc(src) {
    this._datajoin.src = src;
  }
  async componentWillLoad() {
    var _a;
    if (!this.el.id) {
      this.el.setAttribute('id', (_a = this.name) !== null && _a !== void 0 ? _a : randomId('datajoin'));
    }
    debugLifecycle(`${formatEl(this.el)} will load`);
    await this.flowMap._map.initializeContext();
    this._datajoin = new FlowCoreDataJoin({
      id: this.el.id,
      src: this.src,
      sourceLayer: this.sourceLayer,
      foreignKey: this.foreignKey,
      clear: this.clear,
      flowDataSource: this.flowMapDataSource._datasource,
      flowMap: this.flowMap._map
    });
    this.updateSrc(this.src);
  }
  async componentDidLoad() {
    debugLifecycle(`${formatEl(this.el)} did load`);
    this.setDataJoinParameters();
    this.mutationObserver = new MutationObserver((ev) => {
      this.setDataJoinParameters();
    });
    // this._datajoin.start();
    this.mutationObserver.observe(this.el, {
      childList: true
    });
    if (this.flowMap._map.maplibreMap) {
      this.flowMap._map.maplibreMap.once('idle', () => this._datajoin.start());
    }
    else {
      this.flowMap._map.once('load', () => this._datajoin.start());
    }
  }
  async disconnectedCallback() {
    debugLifecycle(`${formatEl(this.el)} did unload`);
    if (this._datajoin) {
      this._datajoin.stop();
      this._datajoin = undefined;
    }
    if (this.mutationObserver) {
      this.mutationObserver.disconnect();
      this.mutationObserver = undefined;
    }
  }
  setDataJoinParameters() {
    // Get direct child parameters
    const params = Array.prototype.filter
      .call(this.el.children, (el) => el.tagName === FLOW_MAP_PARAMETER_TAG)
      .map((element) => element._parameter);
    this._datajoin.setParameters(params);
  }
  render() {
    return h(Host, null);
  }
  get el() { return getElement(this); }
  static get watchers() { return {
    "src": ["updateSrc"]
  }; }
};
__decorate([
  AncestorElement({ supportAttr: false })
], FlowMapDataJoin.prototype, "flowMapDataSource", void 0);
__decorate([
  AncestorElement({ supportAttr: false })
], FlowMapDataJoin.prototype, "flowMap", void 0);

export { FlowMapDataJoin as flow_map_data_join };
