/*!
 * (C) Emu Analytics
 */
import { r as registerInstance, f as createEvent, h, e as Host, g as getElement } from './index-abc3ebe6.js';
import { r as randomId } from './random-id-d84012dc.js';
import { T as debugLifecycle, U as formatEl } from './debug-eda7ee1f.js';
import { f as format, s as select, q as quadInOut, i as interpolateNumber } from './index-b11e5989.js';
import { g as globalContextRegistry } from './context-registry-d4e711d5.js';
import { c as isUndefined, m as isNull } from './EventEmitter-1b1a10d1.js';
import './flow-core-context-291ff9b4.js';
import './maplibre-spec-helper-6abb210f.js';
import './constants-34bf9e9d.js';

const dataValueCss = ":host{line-height:1;display:inline-flex;min-width:0}span{white-space:nowrap;overflow:hidden;text-overflow:ellipsis}";

const FlowValue = class {
  constructor(hostRef) {
    registerInstance(this, hostRef);
    this.flowDataValueChanged = createEvent(this, "flowDataValueChanged", 7);
    this.initialized = false;
    /**
     * Specifies the transition duration in milliseconds for the animation between numeric values. Set
     * to zero to disable animation.
     */
    this.duration = 250;
    /**
     * The Flo.w context ID to use for application state.
     */
    this.flowContext = 'default';
  }
  async connectedCallback() {
    debugLifecycle(`${formatEl(this.el)} connectedCallback`);
    if (this.initialized) {
      // Handle reconnect
      this.connect();
    }
  }
  async componentDidLoad() {
    debugLifecycle(`${formatEl(this.el)} did load`);
    if (!this.el.id) {
      this.el.setAttribute('id', randomId('value'));
    }
    this.initialized = true;
    this.connect();
  }
  async disconnectedCallback() {
    debugLifecycle(`${formatEl(this.el)} disconnectedCallback`);
    this.disconnect();
  }
  async connect() {
    this.context = await globalContextRegistry.getContext(this.flowContext);
    this.updateNumberFormatter();
    this.subscribe();
  }
  async disconnect() {
    this.unsubscribe();
  }
  updateNumberFormatter() {
    if (!this.format || this.format === 'icon') {
      this.numberFormatter = n => n.toString(10);
    }
    else {
      this.numberFormatter = format(this.format);
    }
    // update display
    this.setValue(this.internalValue);
  }
  subscribe() {
    var _a;
    this.unsubscribe();
    if (isUndefined(this.value)) {
      this.context.emitWarning(`${formatEl(this.el)} value attribute undefined`);
    }
    else if (isNull(this.value)) {
      // Don't attempt to evaluate null value
      this.setValue(this.value);
      this.flowDataValueChanged.emit(this.value);
    }
    else {
      const parsedDataExpression = this.context.parser.parseExpression(this.value);
      if (parsedDataExpression.status) {
        this.autorunDisposer = this.context.state.autorun(() => {
          try {
            const newValue = this.context.parser.evaluateExpression(parsedDataExpression.value);
            this.setValue(newValue);
            this.flowDataValueChanged.emit(newValue);
          }
          catch (e) {
            this.context.emitError(e);
          }
        }, `autorun: update(FlowDataValue.${(_a = this.el) === null || _a === void 0 ? void 0 : _a.id})`);
      }
      else {
        this.context.emitError(new Error(`Malformed expression - ${parsedDataExpression}`));
      }
    }
  }
  unsubscribe() {
    if (this.autorunDisposer) {
      this.autorunDisposer();
    }
  }
  setValue(newValue) {
    const selection = select(this.el.shadowRoot).select('span');
    const node = selection.node();
    selection.interrupt(this.el.id);
    if (!this.duration || typeof newValue !== 'number' || typeof this.internalValue !== 'number') {
      // Cannot animate so just set value
      this.internalValue = newValue;
      node.innerHTML = this.formatValue(newValue);
    }
    else {
      const oldValue = isNaN(this.internalValue) ? 0 : this.internalValue;
      this.internalValue = newValue;
      // Going from number to number - animate
      selection
        .transition(this.el.id)
        .duration(this.duration)
        .ease(quadInOut)
        .tween('update', () => {
        const interp = interpolateNumber(oldValue, newValue);
        return (t) => {
          node.innerHTML = this.formatValue(interp(t)).replace(' ', '&nbsp;');
        };
      });
    }
  }
  formatValue(value) {
    if (this.format === 'icon') {
      return `<flow-icon icon="${value || 'far-circle-question'}"></flow-icon>`;
    }
    if (typeof value === 'number') {
      return this.numberFormatter(value);
    }
    if (typeof value === 'undefined' || value === null) {
      return '--';
    }
    return value;
  }
  render() {
    return (h(Host, { class: "flow-value" }, h("span", { part: "content" })));
  }
  get el() { return getElement(this); }
  static get watchers() { return {
    "format": ["updateNumberFormatter"],
    "value": ["subscribe"]
  }; }
};
FlowValue.style = dataValueCss;

export { FlowValue as flow_value };
