import EventEmitter from 'wolfy87-eventemitter';
import { View } from 'vega';
import { ChartSpec } from './char-spec-registry';
/**
 * [[FlowCoreChart]] options.
 */
export interface FlowCoreChartOptions {
  /** Container element to host chart. */
  container: HTMLElement;
  /** Vega/Vega-lite chart specification. */
  spec: ChartSpec;
  /** Width of chart in pixels. */
  width?: number;
  /** Height of chart in pixels. */
  height?: number;
  /** Set to `true` to enable enhanced tooltips. */
  tooltip?: boolean;
}
/**
 * Core functionality for Flo.w charts. This class is wrapped by the `<flow-chart>` web component.
 *
 * Charts are defined using [Vega/Vega-Lite](https://vega.github.io/) chart specifications.
 *
 * @noInheritDoc
 */
export declare class FlowCoreChart extends EventEmitter {
  readonly id: string;
  readonly contextId: string;
  private vegaView;
  private boundSignalHandler;
  private pendingSignals;
  private pendingData;
  private vegaFinalizeFn;
  private _initialized;
  private options;
  /**
   * Returns `true` if the chart has been initialized.
   */
  get initialized(): boolean;
  /**
   * Constructs a new `FlowCoreChart` instance.
   *
   * @param id Chart ID.
   * @param contextId Associated Flo.w context ID.
   */
  constructor(id: string, contextId: string);
  /**
   * Initialize the chart.
   *
   * This method is called internally by the `<flow-chart>` web component.
   *
   * @param options Chart options.
   */
  initialize(options: FlowCoreChartOptions): void;
  /**
   * Uninitialize the chart.
   *
   * This method is called internally by the `<flow-chart>` component. If instantiated programmatically,
   * calling this method is the responsibility of the developer.
   */
  uninitialize(): void;
  /**
   * Set a Vega chart signal.
   *
   * @param signal Signal name.
   * @param value Signal value.
   * @param run Run chart update immediately.
   */
  setSignal(signal: string, value: string, run?: boolean): Promise<void>;
  /**
   * Set chart data.
   *
   * This method is called internally by the `<flow-chart>` web component.
   *
   * @param name Data name.
   * @param vals Data values.
   * @param xProp Name of X property in data.
   * @param yProp Name of Y property in data.
   * @param run Run update immediately.
   */
  setData(name: string, vals: any, xProp?: string, yProp?: string, run?: boolean): Promise<void>;
  /**
   * Trigger a resize of the chart.
   */
  resize(): Promise<View>;
  private initializeSignals;
  private initializeData;
  private cleanUpSignals;
  private signalHandler;
}
