import { Client as FlowClient } from '@emuanalytics/flow-engine-client';
import { MapRegistry } from '../map-registry';
import { ImageRegistry } from '../image-registry';
import { ImageLoader } from '../image-loader';
import { StateStore } from '../state-store';
import EventEmitter from 'wolfy87-eventemitter';
import { PaletteRegistry } from '../palette-registry';
import { StyleParser } from '../parser/style-parser';
import { ChartSpecRegistry } from '../charts/char-spec-registry';
import { Debugger } from '../debugger/debugger';
/** @internal */
export declare const FLOW_CONTEXT_TAG = "FLOW-CONTEXT";
/** @internal */
export declare const FLOW_CONTEXT_ATTR = "context";
/**
 * `FlowCoreContext` if the root of the Flo.w RDF JavaScript API.
 *
 * Properties of this class expose functionality to consumers of the Flo.w RDF library.
 *
 * Use the global [[ContextRegistry]] (exposed as `flow` by the library) to initialize
 * a Flo.w RDF context in your application start-up code.
 *
 * @typeParam TState  Type of the reactive application state object.
 * @noInheritDoc
 */
export declare class FlowCoreContext<TState = any> extends EventEmitter {
  readonly id: string;
  readonly host: string;
  readonly apiKey: string;
  /** A Flo.w Engine client instance for performing low-level requests to Flo.w Engine.
   *
   * See [Flo.w Client Library](/docs/flow-engine/flow-client-lib) for more details.
   */
  readonly flowClient: FlowClient;
  /** The map registry. */
  readonly maps: MapRegistry;
  /** The Flo.w debugger. Available as `window.FDBG` in the browser's web developer console. */
  readonly debugger: Debugger;
  /** The map icon registry. */
  readonly images: ImageRegistry;
  /** The map icon image loader. */
  readonly imageLoader: ImageLoader;
  /** Reactive application state. */
  readonly state: StateStore<TState>;
  /** The palette registry. */
  readonly palettes: PaletteRegistry;
  /** @internal */
  readonly parser: StyleParser;
  /** The chart specification registry. */
  readonly chartSpecs: ChartSpecRegistry;
  /**
   * Constructs a new `FlowCoreContext`.
   *
   * Use [[ContextRegistry.initializeContext]] to create an instance of `FlowCoreContext`.
   *
   * @param id The ID of the context.
   * @param host The Flo.w Engine host URL.
   * @param apiKey The Flo.w Engine API key.
   * @param enableDebug Set to `true` to enable the debugger.
   */
  constructor(id: string, host: string, apiKey: string, enableDebug?: boolean);
  /**
   * Emit an error.
   *
   * Attach an error handler to respond to errors.
   *
   * ```javascript
   * context.on('error', (error: Error) => {...handle error...});
   * ```
   *
   * If no error handler is attached an exception will be thrown.
   */
  emitError(error: Error): void;
  /**
   * Emit a warning
   *
   * Attach a warning handler to respond to warnings.
   *
   * ```javascript
   * context.on('warn', (warning: string) => {...handle warning...});
   * ```
   *
   * If no warning handler is attached, the warning will be logged to the console.
   */
  emitWarning(warning: string): void;
}
