export interface CompositeLayerSpecification {
  id: string;
  layers: CompositeLayer[];
  dataSources?: (CompositeVectorDataSource | CompositeGeojsonDataSource | CompositeRasterDataSource)[];
}
export interface CompositeBaseDataSource {
  id: string;
  type: 'vector' | 'geojson' | 'raster';
  src: string;
  featureId?: string;
}
export interface CompositeVectorDataSource extends CompositeBaseDataSource {
  type: 'vector';
}
export interface CompositeGeojsonDataSource extends CompositeBaseDataSource {
  type: 'geojson';
  cluster?: boolean;
  clusterRadius?: number;
  clusterMaxZoom?: number;
  clusterMinPoints?: number;
}
export interface CompositeRasterDataSource extends CompositeBaseDataSource {
  type: 'raster';
  src: string;
}
export interface CompositeLayer {
  id: string;
  type: CompositeLayerType;
  source: string;
  interactive?: boolean;
  minZoom?: number;
  maxZoom?: number;
  above?: string;
  below?: string;
  filter?: string;
  style: CompositeLayerFlowStyle | CompositeLayerMaplibreStyle;
}
export interface CompositeLayerFlowStyle {
  type: 'flow';
  opacity?: CompositeLayerProperty<number>;
  fillColor?: CompositeLayerProperty<string>;
  strokeColor?: CompositeLayerProperty<string>;
  strokeWidth?: CompositeLayerProperty<CompositeLayerSize>;
  labelText?: CompositeLayerProperty<string>;
  labelColor?: CompositeLayerProperty<string>;
  labelHaloColor?: CompositeLayerProperty<string>;
  labelSize?: CompositeLayerProperty<CompositeLayerSize>;
  radius?: CompositeLayerProperty<CompositeLayerSize>;
  visibility?: CompositeLayerProperty<boolean>;
}
export declare type CompositeLayerSize = number | 'x-small' | 'small' | 'medium' | 'large' | 'x-large';
export interface CompositeLayerMaplibreStyle {
  type: 'maplibre';
  style: string;
}
export declare type CompositeLayerProperty<TProp> = TProp | string | CompositeLayerLiteralProperty<TProp> | CompositeLayerExpressionProperty | CompositeLayerDataDrivenProperty<TProp>;
export interface CompositeLayerLiteralProperty<TProp> {
  type: 'literal';
  value: TProp;
}
export interface CompositeLayerExpressionProperty {
  type: 'expression';
  value: string;
}
export interface CompositeLayerDataDrivenProperty<TProp> {
  type: 'data-driven';
  attribute: string;
  default: TProp;
  mode?: 'off' | 'match' | 'interpolate';
  values: [match: string | number, value: TProp, options?: {
    regexp?: boolean;
    caseSensitive?: boolean;
  }][];
}
export declare type CompositeLayerType = 'fill' | 'line' | 'symbol' | 'circle';
/**
 * Takes a single composite layer definition and expands it into a list of Maplibre-style layers.
 *
 * @param compositeLayer
 */
export declare function expandCompositeLayer(compositeLayer: CompositeLayer, layerPrefix?: string): CompositeLayer[];
export declare function expandCompositeCircleLayer(layer: CompositeLayer, layerPrefix?: string): CompositeLayer[];
export declare function expandCompositeFillLayer(layer: CompositeLayer, layerPrefix?: string): CompositeLayer[];
export declare function expandCompositeLineLayer(layer: CompositeLayer, layerPrefix?: string): CompositeLayer[];
export declare function expandCompositeLayerFlowStyle(layerType: CompositeLayerType, style: CompositeLayerFlowStyle): CompositeLayerMaplibreStyle;
export declare function labelLayerForCompositeLayer(layer: CompositeLayer, layerPrefix?: string): CompositeLayer | undefined;
export declare function expandPropertyValue(value: CompositeLayerProperty<any>): string;
export declare function isLiteralProperty<T>(value: CompositeLayerProperty<T>): value is CompositeLayerLiteralProperty<T>;
export declare function isExpressionProperty(value: CompositeLayerProperty<unknown>): value is CompositeLayerExpressionProperty;
export declare function isDataDrivenProperty<T>(value: CompositeLayerProperty<T>): value is CompositeLayerDataDrivenProperty<T>;
export declare function isSupportedLayerType(type: string): type is CompositeLayerType;
export declare function escapeMaplibreString(value: string): string;
export declare function expandStrokeWidth(strokeWidth: CompositeLayerProperty<CompositeLayerSize>): string;
export declare function expandLabelSize(labelSize: CompositeLayerProperty<CompositeLayerSize>): string;
export declare function expandCircleRadius(circleRadius: CompositeLayerProperty<CompositeLayerSize>): string;
export declare function orderByArrayIndex<TValue extends Array<any>>(array: TValue[], indexes: number[], directions: ('asc' | 'desc')[]): TValue[];
