import { FlowCoreMap } from './flow-core-map';
import { FlowCoreParameter, ParameterDef } from './flow-core-parameter';
import { FlowCoreDataSource } from './flow-core-data-source';
/**
 * [[FlowCoreDataJoin]] options.
 */
export interface FlowCoreMapDataJoinOptions {
  /** Data join ID. */
  id: string;
  /** Data join source: dataset ID, external URL or array of data. */
  src: string | any[];
  /** Data source layer to join. */
  sourceLayer: string;
  /** Parent map. */
  flowMap: FlowCoreMap;
  /** Parent map data source. */
  flowDataSource: FlowCoreDataSource;
  /** Data join foreign key. */
  foreignKey: string;
  /** Set to `true` to clear feature state before applying data join. */
  clear: boolean;
}
/**
 * Core functionality for Flo.w map data joins. Wrapped by the `<flow-map-data-join>` web component.
 *
 * @noInheritDoc
 */
export declare class FlowCoreDataJoin {
  static defaultOptions: Partial<FlowCoreMapDataJoinOptions>;
  /** Map data join ID. */
  id: string;
  private options;
  private disposer;
  /**
   * Constructs a new `FlowCoreDataJoin` instance.
   *
   * @param options Map data join options.
   */
  constructor(options: FlowCoreMapDataJoinOptions);
  private _parameters;
  private _src;
  /**
   * Return the data source for the map data join.
   *
   * If `src` is a Flo.w reactive expression, the expression will be expanded to
   * return an array of data objects.
   *
   * <flow-badge pill>Reactive</flow-badge>
   *
   * @returns array of data, dataset ID or external URL.
   */
  get dataSrc(): string | any[] | null;
  /**
   * Return the parameters associated with the map data join.
   *
   * <flow-badge pill>Reactive</flow-badge>
   */
  get parameters(): ParameterDef[];
  /**
   * Start the reactive map data join.
   */
  start(): void;
  /**
   * Stop the reactive map data join.
   */
  stop(): void;
  /**
   * Set the `src` property. One of
   *
   * - array of data objects.
   * - Flo.w reactive expression.
   * - dataset ID.
   * - external URL.
   */
  set src(src: string | any[]);
  /**
   * Get the `src` property.
   */
  get src(): string | any[];
  /**
   * Set parameters associated with this data join.
   *
   * Called internally by the `flow-map-data-join` web component. If set programmatically,
   * it will be overwritten by the `flow-map-data-join` web component.
   *
   * @param parameters Data join parameters.
   */
  setParameters(parameters: FlowCoreParameter[]): void;
  private loadData;
  private doDataJoin;
}
