import EventEmitter from 'wolfy87-eventemitter';
import { FlowCoreMap, FlowMapGeoJSONFeature, FlowMapSourceSpecification } from './flow-core-map';
import { FlowCoreMapPopupOptions } from './flow-core-map-popup';
import { FlowCoreParameter } from './flow-core-parameter';
/**
 * User-provided function to provide a popup definition for display when a feature is selected.
 */
export declare type SelectionPopupFn = (feature: FlowMapGeoJSONFeature) => FlowCoreMapPopupOptions | string;
/**
 * [[FlowCoreDataSource]] options.
 */
export interface FlowCoreMapDataSourceOptions {
  /** Data source ID. */
  id: string;
  /** Data source: dataset ID, external URL or GeoJSON FeatureCollection. */
  src: string | GeoJSON.FeatureCollection;
  /**
   * The vector source layer to display. Defaults to 'geometry' for vector-type data sources.
   * Omit for GeoJSON-type data sources.
   */
  sourceLayer?: string;
  /** Parent map. */
  flowMap: FlowCoreMap;
  /** Data source type: 'vector', 'geojson' , 'raster'. */
  type?: string;
  /** Enable GeoJSON point clustering. */
  cluster?: boolean;
  /** GeoJSON point clustering radius. */
  clusterRadius?: number;
  /** GeoJSON point clustering maximum zoom level. */
  clusterMaxZoom?: number;
  /** Data source limit - applied to dataset queries. */
  limit?: number;
  /** Specify feature property to use as unique ID. */
  featureId?: string;
  /** Set to override application-wide API key for this data source */
  apiKey?: string;
  /**
   * Enable automatic feature selection support for single or multiple features.
   */
  selectionMode?: 'none' | 'single' | 'multi';
  /**
   * Set to bind feature selection to a Flo.w application state property. The application state property
   * is the selected feature ID ('single' mode) or an array of selected feature IDs ('multi' mode).
   *
   * Binding is two-way. Set the associated application state property to programmatically modify feature
   * selection for this data source.
   *
   * @reactive
   */
  selectionState?: string;
  /**
   * Set to bind feature selection to a Flo.w application state property. The application state property
   * is:
   *
   * - single mode: an object containing the selected feature's properties.
   * - multi mode: an object mapping selected feature IDs to feature properties.
   *
   * Binding is one-way. The application state property is updated automatically when map feature selection changes.
   * Setting the application state property programmatically has no effect.
   *
   * @reactive
   */
  selectionFeatureState?: string;
  /**
   * Set to `true` to clear the selected feature whenever the map is clicked (empty area or a feature from another data source).
   *
   * By default, selection is only cleared when clicking on an empty area of the map.
   */
  selectionClear?: boolean | 'group';
  /**
   * Set to enable cross-dataset selection management. When set and `clear` mode is set to `group`,
   * then clicking on a feature from another data source in the same group will clear the selection
   * for this data source.
   */
  selectionGroup?: string;
  /**
   * Set to automatically open a popup when a map feature from this data source is clicked. The popup can be configured
   * by specifying the ID of a registered `<flow-map-popup>` or programmatically by specifying a function that returns a
   * `FlowCoreMapPopupOptions` object.
   */
  selectionPopup?: string | SelectionPopupFn;
}
export declare class FlowCoreDataSource extends EventEmitter {
  static defaultOptions: Partial<FlowCoreMapDataSourceOptions>;
  private subscription;
  private _options;
  private selectedFeatureIDs;
  private _selectedFeatures;
  private boundMapSourceDataHandler;
  private boundMapSourceDataLoadingHandler;
  private dataLoading;
  private popups;
  id: string;
  type: string;
  get options(): Readonly<FlowCoreMapDataSourceOptions>;
  constructor(options: FlowCoreMapDataSourceOptions);
  private parameters;
  private _src;
  get maplibreSourceSpecification(): FlowMapSourceSpecification;
  set src(src: string | GeoJSON.FeatureCollection);
  get src(): string | GeoJSON.FeatureCollection;
  /**
   * @internal
   *
   * Called internally from MapDataSource to update options
   *
   * @param parameters Data source parameters
   */
  setParameters(parameters: FlowCoreParameter[]): void;
  /**
   * @internal
   *
   * Called internally from MapDataSource to update options
   *
   * @param options Data source options
   */
  updateOptions(options: FlowCoreMapDataSourceOptions): void;
  /**
   * Called internally to attach a data source to its parent map.
   *
   * @param map Parent map
   */
  attach(map: FlowCoreMap): void;
  /**
   * Called internally to detach a data source from its parent map.
   */
  detach(): void;
  /**
   * @internal
   *
   * Subscribe to reactive updates from application state
   */
  subscribe(): void;
  /**
   * @internal
   *
   * Unsubscribe from reactive updates from application state
   */
  unsubscribe(): void;
  /**
   * @internal
   *
   * Register a click on a feature. Called internally by FlowCoreMap to support automatic feature selection and popups.
   *
   * @param feature Clicked feature
   * @param multiSelect True if alt/ctrl/meta click
   * @param clickedPoint Lng/Lat of clicked point
   */
  featureClicked(feature: FlowMapGeoJSONFeature, multiSelect: boolean, clickedGroup?: string, clickedPoint?: [number, number]): void;
  /**
   * Programmatically select one or more features.
   *
   * Call with an empty array to clear selection.
   *
   * @param features Single feature ID or array of feature IDs to select
   * @returns
   */
  selectFeatures(features: string | number | (string | number)[]): void;
  /**
   * Open a selection popup for the specified feature.
   *
   * For point features, clickedPoint is ignored and the popup is opened at the feature point.
   *
   * For LineStrings, the popup is opened at the closest point on the line to the clicked point.
   *
   * For Polygons, the popup is opened at the clicked point.
   *
   * If clicked point is not specified then the feature centroid is used.
   *
   * @param feature Map feature
   * @param clickedPoint Lng/Lat of clicked point.
   */
  openPopup(feature: FlowMapGeoJSONFeature, clickedPoint?: [number, number]): void;
  /**
   * Close the selection popup for the specified feature.
   *
   * @param featureId Feature ID
   */
  closePopup(featureId: string | number): void;
  /**
   * Close all selection popups.
   */
  closeAllPopups(): void;
  /**
   * Convert a src parameter to final URL
   *
   * @param src Data source url/dataset/tilesource ID
   * @param type Data source type
   * @returns URL
   */
  private expandSrc;
  /**
   * Query map source data for specified features.
   *
   * @param featureIDs Array of features IDs to query
   * @returns Array of features
   */
  private queryMapFeatures;
  /**
   * Update map `selected` feature state to reflect current selection
   */
  private updateMapSelectionState;
  /**
   * Update bound app state property from current selection. This will trigger reactive updates.
   *
   * selectionMode determines how property is updated:
   *
   * - single-id: Feature ID of selected features
   * - multi-id: Array of selected feature IDs
   * - single-feature: Object containing feature properties
   * - multi-feature: Map of feature IDs to feature properties
   */
  private updateApplicationSelectionState;
  private updateApplicationFeatureSelectionState;
  /**
   * Attach map idle handler when source data starts loading as a fail-safe in case
   * sourceLoaded is not detected properly. This ensures selected feature state will be updated in all cases.
   *
   * (sourcedata event firing is unreliable)
   */
  private handleMapSourceDataLoading;
  /**
   * Handle sourcedata events:
   *
   * - 'visibility' events generated when source is no longer visible (all associated layers hidden)
   * - 'tile' events signalling that data is loaded.
   *
   * Note that the last 'tile' event should be sent with isSourceLoaded=true signalling that the source is fully loaded.
   * However, it seems that this is not always the case. Handling the 'idle' event above is a fail-safe to ensure that
   * feature data is always updated. (Updating via 'idle' introduces a visible delay for moving popups so updating on a 'tile'
   * event is preferable if it is actually sent).
   */
  private handleMapSourceData;
  /**
   * Perform updates when data source has finished loading data
   *
   * - update bound app state property with new feature properties
   * - move popup to new location if required
   * - deselect features that are no longer available in map data
   */
  private updateFromMapData;
}
