import { FlowCoreMap, FlowMapLngLatLike } from './flow-core-map';
import EventEmitter from 'wolfy87-eventemitter';
/**
 * [[FlowCoreMapPopup]] options.
 */
export interface FlowCoreMapPopupOptions {
  /**
   * Unique ID of registered popup options
   *
   * @internal
   */
  id?: string;
  /** Enable 'close' button. */
  closeButton?: boolean;
  /** Enable closing popup on click on map. */
  closeOnClick?: boolean;
  /** Enable closing pop when map is moved. */
  closeOnMove?: boolean;
  /** Maximum width of popup in CSS units. */
  maxWidth?: string;
  /**
   * Move popup reactively.
   *
   * Either:
   * - Reactive expression returning lng/lat as a 2-element array.
   * - `true` to enable auto-tracking of map features
   * - `false` to disable auto-tracking (default)
   */
  trackLocation?: string | boolean;
  /** Move popup to `trackLocation` immediately. */
  fireTrackImmediately?: boolean;
  /** Specify popup content as HTML string or HTML node. */
  content: string | Node;
  /** Specify a CSS class to add to the popup. */
  class?: string;
}
/**
 * Core functionality for Flo.w map popups.
 *
 * @noInheritDoc
 */
export declare class FlowCoreMapPopup extends EventEmitter {
  readonly map: FlowCoreMap;
  readonly options: FlowCoreMapPopupOptions;
  private maplibrePopup;
  private autorunDisposer;
  /**
   * Constructs a new `FlowCoreMapPopup` instance.
   *
   * This is called by the `FlowCoreMap::openPopup` method.
   *
   * @param map Parent Flo.w map.
   * @param options Popup options.
   */
  constructor(map: FlowCoreMap, options: FlowCoreMapPopupOptions);
  /**
   * Open the popup at the specified location.
   *
   * @param lngLat Popup location.
   */
  open(lngLat: FlowMapLngLatLike): void;
  /**
   * Close the popup.
   */
  close(): void;
  /**
   * Move the popup to a new location.
   *
   * @param lngLat New location.
   */
  moveTo(lngLat: FlowMapLngLatLike): void;
  /**
   * Start tracking a Flo.w reactive expression to move the popup.
   *
   * Call with null `location` to stop tracking.
   *
   * <flow-badge pill>Reactive</flow-badge>
   *
   * @param location Flo.w reactive expression returning lng/lat as a 2-element array.
   * @param fireImmediately Update popup position immediately.
   */
  trackLocation(location: string, fireImmediately?: boolean): void;
  private popupCloseHandler;
  private unsubscribe;
}
