/// <reference types="mapbox__mapbox-gl-draw" />
import { Map as MaplibreMap, MapOptions } from '@emuanalytics/maplibre-gl';
import MapboxDraw from '@mapbox/mapbox-gl-draw';
import EventEmitter from 'wolfy87-eventemitter';
import { FlowCoreContext } from '../context/flow-core-context';
import { FlowCoreDataSource } from './flow-core-data-source';
import { FlowCoreMapLayerBase } from './flow-core-map-layer-base';
import { FlowCoreMapPopup, FlowCoreMapPopupOptions } from './flow-core-map-popup';
import { FlowMapExportOptions } from './map-exporter';
export declare type FlowMap = maplibregl.Map;
export declare type FlowMapLngLatLike = maplibregl.LngLatLike;
export declare type FlowMapGeoJSONFeature = maplibregl.MapGeoJSONFeature;
export declare type FlowMapFeatureIdentifier = maplibregl.FeatureIdentifier;
export declare type FlowMapSourceSpecification = maplibregl.SourceSpecification;
export declare type FlowMapSourceDataEvent = maplibregl.MapSourceDataEvent;
/**
 * FlowCoreMap constructor options.
 *
 * @noInheritDoc
 */
export interface FlowCoreMapOptions extends Omit<MapOptions, 'style'> {
  /** Radius in pixels of area around clicked point to search for map features. */
  featureClickTolerance?: number;
}
export interface FlowCoreMapDrawOptions {
  exclusive?: boolean;
  displayControlsDefault?: boolean | undefined;
  keybindings?: boolean | undefined;
  touchEnabled?: boolean | undefined;
  boxSelect?: boolean | undefined;
  clickBuffer?: number | undefined;
  touchBuffer?: number | undefined;
  controls?: MapboxDraw.MapboxDrawControls | undefined;
  styles?: object[] | undefined;
  modes?: {
    [modeKey: string]: MapboxDraw.DrawMode | MapboxDraw.DrawCustomMode;
  } | undefined;
  defaultMode?: string | undefined;
  userProperties?: boolean | undefined;
  controlPosition?: 'top-right' | 'top-left' | 'bottom-right' | 'bottom-left';
}
/**
 * Map feature state properties.
 */
interface FeatureStateProperties {
  [key: string]: any;
}
/**
 * Core functionality for Flo.w maps. Wrapped by the `<flow-map>` web component.
 *
 * @noInheritDoc
 */
export declare class FlowCoreMap extends EventEmitter {
  readonly id: string;
  readonly flowContextId: string;
  private initialSync;
  private mapLoading;
  private mapLoadingPollTimer;
  private hoveredFeature;
  private options;
  private drawControl;
  private exclusiveDrawMode;
  /**
   * The underlying Maplibre GL JS map instance. Use this property to access the full [MaplibreGL JS map API](https://maplibre.org/maplibre-gl-js-docs/api/).
   */
  maplibreMap: MaplibreMap;
  /**
   * The associated Flo.w Context object.
   */
  context: FlowCoreContext;
  private baseStyle;
  private layers;
  private dataSources;
  private popupTemplates;
  private cachedSourceIds;
  /**
   * Get the full expanded [MaplibreGL Style Specification](https://maplibre.org/maplibre-gl-js-docs/style-spec/) for this map.
   *
   * <flow-badge pill>Reactive</flow-badge>
   */
  get mapStyle(): any;
  private mapContainer;
  private disposer;
  /**
   * Constructs a new `FlowCoreMap`.
   *
   * @param id Map ID.
   * @param flowContextId ID of associated Flo.w context object.
   */
  constructor(id: string, flowContextId: string);
  /**
   * @internal
   *
   * Wait for the associated Flo.w context to be initialize.
   *
   * This method is called automatically by the `flow-map` web component.
   */
  initializeContext(): Promise<FlowCoreContext<any>>;
  /**
   * Initialize the map.
   *
   * This method is called automatically by the `flow-map` web component. If instantiating
   * a `FlowCoreMap` directly, the developer is responsible for calling this method.
   *
   * @param styleUrl Base map style URL
   * @param sync Specify `true` to synchronize browser address bar with map location and zoom level.
   * @param options MaplibrexGL instantiation options.
   */
  initialize(styleUrl: string, sync: boolean, options: FlowCoreMapOptions): Promise<void>;
  /**
   * Uninitialize the map.
   *
   * Note that this is not called automatically by the `flow-map` web component because
   * there is no way to reliably detect when the component is removed from the DOM for disposal
   * rather than being moved around.
   *
   * The developer is responsible for calling this method when the map is no longer needed.
   */
  uninitialize(): void;
  connect(): void;
  disconnect(): void;
  /**
   * Update the base map style and redraw the map.
   *
   * @param styleUrlOrId Base map style URL or ID
   */
  updateBaseStyle(styleUrlOrId: string): Promise<void>;
  /**
   * Programmatically set feature state properties for a map feature.
   *
   * @param feature Feature to modify.
   * @param state Feature state properties to set.
   */
  setFeatureState(feature: maplibregl.FeatureIdentifier | FlowMapGeoJSONFeature, state: FeatureStateProperties): void;
  /**
   * Programmatically remove feature state properties from a map feature.
   *
   * @param feature Feature to modify. Specify `null` to modify all map features.
   * @param key Feature state key to remove. Specify `null` to remove all feature state for specified feature(s).
   */
  removeFeatureState(feature: maplibregl.FeatureIdentifier | FlowMapGeoJSONFeature, key?: string): void;
  /**
   * Zoom the map by the specified increment.
   *
   * @param increment Zoom increment. Specify positive values to zoom in and negative values to zoom out.
   */
  zoomBy(increment: number): Promise<void>;
  /**
   * Export the current map view as a PNG image.
   *
   * The image will be downloaded by the browser.
   *
   * @param options Export options.
   *
   * @returns A promise that resolves with the filename when the map has been exported.
   */
  export(options: FlowMapExportOptions): Promise<string>;
  /**
   * Set map layers.
   *
   * This method is called internally by the `<flow-map>` web component. If modified programmatically,
   * any modification will be overwritten by `<flow-map>`.
   *
   * @param layers Map layers.
   */
  setLayers(layers: FlowCoreMapLayerBase[]): void;
  /**
   * Returns the specified map layer.
   *
   * @param id Map layer ID.
   */
  getLayer(id: string): FlowCoreMapLayerBase;
  /**
   * Set map data sources.
   *
   * This method is called internally by the `<flow-map>` web component. If modified programmatically,
   * any modification will be overwritten by `<flow-map>`.
   *
   * @param dataSources Map data sources.
   */
  setDataSources(dataSources: FlowCoreDataSource[]): void;
  /**
   * Returns the specified map data source.
   *
   * @param id Data source ID.
   */
  getDataSource(id: string): FlowCoreDataSource;
  /**
   * Register named popup options. Used internally to display automatic popups.
   *
   * @param popupOptions Popup options.
   */
  registerPopupOptions(popupOptions: FlowCoreMapPopupOptions[]): void;
  /**
   * Returns the specified registered popup options.
   *
   * @param id Registered popup options ID.
   */
  getPopupOptions(id: string): FlowCoreMapPopupOptions;
  /**
   * Open a map popup at the specified location.
   *
   * @param lngLat Map location to open popup.
   * @param optionsOrPopupName Popup options or ID of registered map popup.
   */
  openPopup(lngLat: FlowMapLngLatLike, optionsOrPopupName?: FlowCoreMapPopupOptions | string): FlowCoreMapPopup;
  /**
   * Close all open popups for all data sources
   */
  closeAllPopups(): void;
  startDrawing(options?: FlowCoreMapDrawOptions): MapboxDraw;
  endDrawing(): void;
  private pollMapLoading;
  private handleMissingImage;
  private mouseMoveHandler;
  private clickHandler;
  private featureQueryBox;
  private moveEndHandler;
  private mapStateToHash;
  private hashToMapState;
  private transformMapRequest;
  private loadBaseStyle;
  private compareFeatures;
  private forwardEvent;
  private getLayerIds;
}
export {};
