import { FlowCoreContext } from '../context/flow-core-context';
/** Result format of a [[FlowQuery]]. */
export declare type FlowQueryResultFormat = 'json' | 'geojson' | 'csv';
/** [[FlowQueryDefinition]] 'attributes' entry. */
export declare type FlowQueryAttribute = string | [any, string];
/**
 * [[FlowQueryDefinition]] 'where' clauses.
 */
export interface FlowQueryWhereClauses {
  [attribute: string]: any;
}
/**
 * [[FlowQueryDefinition]] extra parameters.
 */
export interface FlowQueryExtraParams {
  [key: string]: string | number;
}
/**
 * [[FlowQueryDefinition]] 'order' clause.
 *
 * A field name or [field, asc/desc] tuple.
 */
export declare type FlowQueryOrderClause = string | number | [string | number, 'asc' | 'desc'];
export interface FlowQuerySampleOptions {
  percentage: number;
  method?: 'system' | 'bernoulli';
  seed?: number;
}
/**
 * Definition of a Flo.w reactive dataset query.
 */
export interface FlowQueryDefinition {
  /** ID of dataset to query. */
  datasetId: string;
  /** Attributes to return in results. */
  attributes?: FlowQueryAttribute[];
  /** DISTINCT ON attributes */
  distinct?: string | string[];
  /** Filter results using 'where' clauses. */
  where?: FlowQueryWhereClauses;
  /** Sort results by attributes. */
  order?: FlowQueryOrderClause[];
  /** Group results by attributes. */
  group?: (string | number)[];
  /** Limit number of returned results */
  limit?: number;
  /** Format of returned data. */
  format?: FlowQueryResultFormat;
  /** Additional query parameters. */
  params?: FlowQueryExtraParams;
  /** Sampling options. */
  sample?: FlowQuerySampleOptions;
}
/**
 * [[FlowQuery]] query definition function.
 *
 * @typeParam TState  Type of application state.
 */
export declare type FlowQueryDefinitionFn<TState> = (state: TState) => FlowQueryDefinition | Promise<FlowQueryDefinition> | null;
/**
 * [[FlowQuery]] result transform function.
 *
 * @typeParam T  Type of transformed query results.
 */
export declare type FlowQueryResultTransformFn<T = any> = (results: any) => T;
/**
 * A `FlowQuery` defines a Flo.w reactive dataset query that is executed in response to changes
 * in application state properties.
 *
 * To register a Flo.w reactive dataset query call the [[StateStore.registerQuery]] method.
 *
 * The query is defined by a supplied query definition function that takes the current application state as an
 * argument and should return a [[FlowQueryDefinition]]. Changes in application state properties referenced in
 * the function will trigger the query to execute. For example,
 *
 * ```javascript
 * (state) => {
 *  return {
 *    datasetId: 'airquality_with_params',
 *    limit: 10,
 *    params: {no2_filter: state.filterValue}
 *  };
 * }
 * ```
 *
 * To transform the returned data specify a result transform function, which accepts the raw query results
 * as an array and should return a transformed value. For example,
 *
 * ```javascript
 * (results) => {
 *  // Return the first results object from the results array
 *  if (results && results.length) {
 *    return results[0];
 *  } else {
 *    return null;
 *  }
 * }
 * ```
 *
 * `FlowQuery` exposes two reactive properties:
 *
 * - [[busy]] - indicates that the query is currently executing.
 * - [[results]] - the query results.
 *
 * @typeParam TResults Type of returned data (including transformation, if any).
 * @typeParam TState Type of context state.
 * ```
 */
export declare class FlowQuery<TResults, TState = any> {
  private readonly context;
  readonly queryId: string;
  private readonly queryFn;
  private readonly resultTransformFn?;
  private readonly delay;
  private asyncCompute;
  private busyAtom;
  private errorAtom;
  private running;
  private _error;
  /**
   * Constructs a `FlowQuery` instance.
   *
   * @param context Flo.w context.
   * @param queryId Query ID.
   * @param queryFn Query definition.
   * @param resultTransformFn Result transform function.
   * @param delay Debounce delay (milliseconds)
   */
  constructor(context: FlowCoreContext<TState>, queryId: string, queryFn: FlowQueryDefinitionFn<TState>, resultTransformFn?: FlowQueryResultTransformFn<TResults>, delay?: number);
  /**
   * Manually run the query. This method is provided for debugging purposes and should not generally
   * be used. Rely on reactivity to run queries when required.
   */
  runQueryImmediate(): Promise<TResults>;
  private runQuery;
  /**
   * `true` if the query is currently executing and awaiting results.
   *
   * Use this property to indicate busy state in the UI.
   *
   * <flow-badge pill>Reactive</flow-badge>
   */
  get busy(): boolean;
  /**
   * If non-null, indicates that a query error occurred
   *
   * Use this property to indicate error state in the UI.
   *
   * <flow-badge pill>Reactive</flow-badge>
   */
  get error(): Error;
  /**
   * The query results.
   *
   * <flow-badge pill>Reactive</flow-badge>
   */
  get results(): TResults;
  /**
   * Lazy version of `results` that does not keepAlive.
   *
   * <flow-badge pill>Reactive</flow-badge>
   */
  get lazyResults(): TResults;
  /**
   * @internal
   *
   * Get query results in non-reactive context
   *
   */
  get resultsNonReactive(): TResults;
  /**
   * Cancel the currently running query.
   */
  cancel(): void;
  private normalizeError;
}
