import { FlowCoreContext } from './context/flow-core-context';
/**
 * @internal
 */
interface ImageRegistryEntry {
  id: string;
  src: string;
  width?: number;
  height?: number;
  pixelDensity?: number;
  recolor?: boolean;
  image?: HTMLImageElement;
}
/**
 * The `ImageRegistry` is a container for map icon images.
 *
 * Images can be registered and retrieved by ID.
 */
export declare class ImageRegistry {
  private readonly context;
  private registry;
  /**
   * Constructs a new `ImageRegistry`.
   *
   * @param context The parent Flo.w context object.
   */
  constructor(context: FlowCoreContext);
  /**
   * Register a map icon image.
   *
   * @param id Image ID.
   * @param src Image source URL.
   * @param width Image display width.
   * @param height Image display height.
   * @param pixelDensity Image pixel density. Defaults to `2` - the source image should be @2x resolution to enable better quality rendering on high-resolution displays.
   * @param recolor Set to `true` to allow recoloring of map icon image using map layer style properties.
   */
  registerImage(id: string, src: string, width?: number, height?: number, pixelDensity?: number, recolor?: boolean): ImageRegistryEntry;
  /**
   * Un-register a map icon image.
   *
   * @param id Image ID.
   */
  unregisterImage(id: string): void;
  /**
   * Get a map icon image.
   *
   * @param id Image ID.
   * @returns The map icon image as an `HTMLImageElement`.
   */
  getImage(id: string): HTMLImageElement;
  /**
   * Get an image registry entry.
   *
   * @param id Image ID.
   * @returns The image registry entry.
   */
  getImageInfo(id: string): ImageRegistryEntry;
  /**
   * Splits an id of the form 'namespace.basename' and returns both components. The split point is
   * the first '.' so namespaces may not contain dots.
   *
   * @returns namespace, basename
   */
  private splitNamespace;
}
export {};
