/**
 *
 * A parser for converting Flow.js to MaplibreGL style expressions.
 *
 * Individual expressions can be parsed e.g.:
 * 1 + 2 * 3 -> ['+', ['*', 2, 3]]
 *
 * A whole 'style program' can also be parsed e.g.:
 * circle-color: @featureParam * 10;
 * circle-radius: min(@featureParam * 2, 10);
 *
 * Reactive state lookup and palette expansion is also supported. Expressions are
 * parsed once to MaplibreGL format and then expansion is performed as a separate step
 * in a reactive context to avoid having to re-parse when reactive values change.
 *
 * See: https://github.com/jneen/parsimmon
 */
/**
 * Copyright © 2020 Emu Analytics
 */
import P from 'parsimmon';
import { FlowCoreContext } from '../context/flow-core-context';
/**
 * @internal
 */
export interface Program {
  [key: string]: ExpressionWithIndex;
}
/**
 * @internal
 */
export declare type FunctionExpression = [string, ...any[]];
/**
 * @internal
 */
export declare type Expression = number | string | FunctionExpression;
/**
 * @internal
 */
export interface ExpressionWithIndex {
  expr: Expression;
  index: ParserIndex;
}
/**
 * @internal
 */
export interface ParserIndex {
  offset: number;
  line: number;
  column: number;
}
/**
 * @internal
 */
export declare class StyleParser {
  private readonly context;
  private readonly language;
  constructor(context: FlowCoreContext);
  /**
   * Parse an expression
   *
   * @param expr Expression to parse
   */
  parseExpression(expr: string): P.Result<Expression>;
  /**
   * Parse a complete assignment program of the form:
   * a: expr1;
   * b: expr2;
   *
   * @param program Program to parse
   */
  parseProgram(program: string): P.Result<Program>;
  /**
   * Evaluate an expression.
   *
   * Reactive values are expanded as part of the evaluation.
   *
   * @returns evaluated result
   */
  evaluateExpression(expr: Expression): any;
  /**
   * Expand reactive values in the given expression (e.g. state lookup, palette expansion).
   *
   * This is performed as a second step after parsing to avoid the need to re-parse expressions
   * when reactive values change.
   *
   * @returns fully expanded expression ready for hand-off to MaplibreGL
   */
  expandReactiveExpression(expr: Expression): Expression;
  private createLanguage;
  /**
   * Helper function to create a parser that parsers a set of operators at the same precedence level
   *
   * @param ops Operator table
   */
  private operators;
  /**
   * Helper function to create a table-driver operator parser
   *
   */
  private tableParser;
  /**
   * Expands the from-palette custom function to a form that MaplibreGL can process.
   *
   * Expression is of form:
   *  ['from-palette', palette_id, property, min, max, reverse] OR
   *  ['from-palette', palette_id, property, stop1, stop2, stop3, stop4, stop5, reverse]
   *
   */
  private expandFromPalette;
  expandFeatureState(symbols: string[]): any[];
}
