import { IDisposer, IStreamObserver, ISubscription } from 'mobx-utils';
import { DeepPartial } from 'ts-essentials';
import { Path, PathValue } from '../utils/typescript-paths';
import * as ComputedAsync from './async-computed';
import { FlowCoreContext } from './context/flow-core-context';
import { FlowQuery, FlowQueryDefinitionFn, FlowQueryResultTransformFn } from './flow-queries/flow-query';
declare global {
  interface SymbolConstructor {
    readonly observable: symbol;
  }
}
export { computed as flowComputed, observable as flowObservable, IComputedValue } from 'mobx';
export { IDisposer, ISubscription } from 'mobx-utils';
declare type Subscribable<T> = {
  subscribe(observer: Partial<IStreamObserver<T>>): ISubscription;
  [Symbol.observable]: () => Subscribable<T>;
};
export declare type IObservableStream<T> = Subscribable<T> & {
  subscribe(observer: (value: T) => void): ISubscription;
};
/**
 * A computed application state property.
 *
 * @typeParam T  Type of property.
 */
export interface FlowComputed<T> {
  /** Get the value of the computed property. */
  get(): T;
}
/**
 * Reactive options for `compute` method.
 */
export interface ReactiveOptions {
  /** Equality method for change detection. Default is `identity`. */
  equals?: 'identity' | 'deep' | 'shallow';
  /** Name for debug output. Defaults to state property path. */
  name?: string;
}
declare type ExtractQueryResultType<T> = T extends FlowQuery<infer TResults, any> ? TResults : never;
/**  Compute function for computed properties.
 *
 * @typeParam TState  Type of application state.
 * @typeParam TResult  Type of computed property.
 */
declare type ComputeFn<TState = any, TResult = any> = (state: TState) => TResult;
/**
 * Compute function for asynchronous computed properties.
 *
 * @typeParam TState  Type of application state.
 * @typeParam TResult  Type of asynchronous computed property.
 */
declare type AsyncComputeFn<TState = any, TResult = any> = (state: TState) => TResult | PromiseLike<TResult>;
/**
 * Query parameter key/value pairs.
 */
export interface QueryParams {
  [key: string]: string;
}
/**
 * Application state serialization function.
 *
 * @typeParam T  Type of application state.
 */
declare type SerializeFn<T> = (state: T) => QueryParams;
/**
 * Application state deserialization function.
 *
 * @typeParam T  Type of application state.
 */
declare type DeserializeFn<T> = (queryParams: QueryParams) => DeepPartial<T>;
/**
 * A `StateStore` is a container of reactive application state properties.
 *
 * Application state properties define key application state and are the core of Flo.w RDF reactivity.
 * Changes to application state properties trigger reactive updates of UI elements and other components
 * that depend on them.
 *
 * Application state properties are identified by property paths. The root of a property path is the application
 * `StateStore` object. Paths can have arbitrary depth allowing the developer to organize and group application
 * state properties.
 *
 * In JavaScript, property paths are specified as strings using dotted notation. For example:
 *
 * - `'prop1'`
 * - `'subGroup.prop2'`
 *
 * To take advantage of type-safety in Typescript, property paths should be specified as arrays of strings.
 * For example:
 *
 * - `['prop1']`
 * - `['subGroup', 'prop2']`
 *
 *  Using this syntax allows IDEs to provide code-completion and
 * ensures type errors are detected during development.
 *
 * ### State Initialization Methods
 *
 * - [[initialize]]
 *
 * ### State accessor Methods
 *
 * - [[get]] - Get a state property value.
 * - [[set]] - Set a state property value.
 * - [[merge]] - Set multiple state property values.
 * - [[increment]] - Increment a state property value.
 * - [[decrement]] - Decrement a state property value.
 * - [[toggle]] - Toggle a state property value.
 *
 * ### Computed State Methods
 * - [[compute]] - Register a computed state property.
 * - [[asyncCompute]] - Create an asynchronous computed state property.
 *
 * ### Flo.w Dataset Query Methods
 *
 * - [[registerQuery]] - Register a Flo.w reactive dataset query.
 *
 * ### Serialization Methods
 * - [[serialize]] - Configure state serialization with the browser URL.
 *
 * ### Other Reactive State Methods
 *
 * - [[autorun]] - Run a function in a reactive context when application state changes.
 * - [[toStream]] - Create an observable stream of application state changes.
 *
 * @typeParam T  The type of the application state object. This enables code-completion and type-safety when using Flo.w RDF with Typescript.
 */
export declare class StateStore<T = any> {
  readonly context: FlowCoreContext<T>;
  private state;
  /**
   * Constructs a new `StateStore` instance.
   *
   * @param context The parent Flo.w context.
   */
  constructor(context: FlowCoreContext<T>);
  /**
   * Initializes the `StateStore` with the supplied initial state.
   *
   * @param state State object.
   */
  initialize(state: T): void;
  /**
   * Run multiple state-modification methods atomically in a single transation.
   * Reactive changes will be coalesced to avoid multiple updates.
   *
   * e.g.
   *
   * ```javascript
   * state.transaction(() => {
   *   state.set('prop1', 1);
   *   state.set('prop2', 2);
   *   state.increment('prop3');
   * });
   * ```
   *
   * @param path Application state property path.
   */
  transaction(func: () => void): void;
  /**
   * Get the current value of an application state property.
   *
   * @param path Application state property path.
   */
  get<P extends Path<T>>(path: P): PathValue<T, P>;
  /**
   * Set the value of an application state property.
   *
   * @param path Application state property path.
   * @param val New value to set.
   */
  set<P extends Path<T>>(path: P, val: PathValue<T, P>): void;
  /**
   * @deprecated
   *
   * Merge the supplied partial state into the `StateStore`.
   *
   * Use this method to update multiple state properties in one transaction.
   *
   * @param patch Partial state object.
   */
  merge(patch: DeepPartial<T>): void;
  /**
   * Toggle the specified boolean state property.
   *
   * The state property will be coerced to a boolean if it is not already.
   *
   * @param path The state property path.
   */
  toggle<P extends Path<T>>(path: P): boolean;
  /**
   * Increment the specified numeric state property.
   *
   * The state property will be coerced to a number if it is not already.
   *
   * @param path Application state property path.
   */
  increment<P extends Path<T>>(path: P): number;
  /**
   * Decrement the specified numeric application state property.
   *
   * The state property will be coerced to a number if it is not already.
   *
   * @param path Application state property path.
   */
  decrement<P extends Path<T>>(path: P): number;
  /**
   * Register a computed application state property.
   *
   * The computed property will be recalculated when state properties referenced in `computeFn` change.
   *
   * @param path Application state path.
   * @param computeFn Function to compute value from other state properties.
   * @param options Reactive options.
   */
  compute<P extends Path<T>>(path: P, computeFn: ComputeFn<T, PathValue<T, P>>, options?: ReactiveOptions): void;
  /**
   * Create an asynchronous compute function.
   *
   * @param computeFn Asyncronous compute function
   * @param initialValue Initial result value. Specifies the computed value before the asynchronous operation has completed.
   * @param delay Debounce delay (milliseconds).
   * @param name Optional name displayed in debugging messages.
   */
  asyncCompute<TResult>(computeFn: AsyncComputeFn<T, TResult>, initialValue: TResult | null, delay: number, name?: string): ComputedAsync.PromisedComputedValue<TResult>;
  /**
   * Expose an application state property as an observable stream of values.
   *
   * The observable stream will emit a value whenever the application state property changes.
   *
   * Subscribe to the returned stream to receive values. Call the `unsubscribe` method of the subscription
   * when the subscription is no longer required.
   *
   * @param path Application state property path
   * @param fireImmediately Set to `true` to emit a value immediately. Otherwise wait for first change.
   */
  toStream<P extends Path<T>>(path: P, fireImmediately?: boolean): IObservableStream<PathValue<T, P>>;
  /**
   * Configure serialization/deserialization of application state properties to/from browser URL query parameters.
   *
   * This allows deep-linking and sharing of web application URLs that specify application state.
   *
   * @param serializeFn Function to serialize application state properties to browser URL query parameters.
   * @param deserializeFn Function to deserialize application state properties from browser URL query parameters.
   */
  serialize(serializeFn: SerializeFn<T>, deserializeFn: DeserializeFn<T>): import("mobx").IReactionDisposer;
  /**
   * Defines a function that runs in a reactive context when state properties that are referenced
   * in the function change.
   *
   * @param autorunFn Function to execute when referenced state properties change.
   * @param name Optional name displayed in debugging messages.
   */
  autorun(autorunFn: (state: T) => void, name?: string): IDisposer;
  /**
   * Registers a Flo.w reactive query that executes when application state properties referenced in
   * the supplied query definition function are changed.
   *
   * See [[FlowQuery]] for more details.
   *
   * @param path Application state path.
   * @param queryFn Query definition function.
   * @param resultTransformFn Results transorm function.
   */
  registerQuery<P extends Path<T>>(path: P, queryFn: FlowQueryDefinitionFn<T>, resultTransformFn?: FlowQueryResultTransformFn<ExtractQueryResultType<PathValue<T, P>>>): FlowQuery<ExtractQueryResultType<PathValue<T, P>>, T>;
  private baseMerge;
  private baseSet;
}
